package org.pasoa.accessors;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * The interface for all managers of data accessors.  Each implementation must
 * supply these core operations on a particular type of data accessor, which
 * allow provenance queries to be performed.  Data accessor managers can be
 * registered in a DataAccessorRegistry.
 */
public interface DataAccessorManager {
    /** Converts the DOM element content of a data accessor to an XPath representation */
    XPath   convertToXPath         (Element accessor) throws AccessorException;
    /** Determines whether two data accessors (using given documentation styles) are equal */
    boolean equalDataAccessors     (Element accessor1, String style1, Element accessor2, String style2) throws AccessorException;
    /** Retrieves the data accessor for the data item in the content of a p-assertion */
    Element getDataAccessorForItem (Node dataItem)    throws AccessorException ;
}
