/********************************************************************************
 * Created on 2006 February 28
 *
 * Copyright (c) 2005 University of Southampton
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *******************************************************************************/
package org.pasoa.accessors.javacall;

import java.util.HashMap;
import javax.xml.namespace.QName;
import org.pasoa.accessors.AccessorException;
import org.pasoa.accessors.DataAccessorManager;
import org.pasoa.accessors.DataAccessorRegistry;
import org.pasoa.accessors.XPath;
import org.pasoa.simpledom.DOMSerialiser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class JavaCallXPathManager implements DataAccessorManager {
    public static String _namespace   = "http://www.pasoa.org/schemas/bp/version01/JavaCallAccessor.xsd";
    public static String _elementName = "javaCall";
    
    public Element createResultObjectAccessor () {
        DOMSerialiser writer = new DOMSerialiser (_namespace);
        
        writer.single (_elementName, "/java/object/void[@property='result']/object");
        
        return ((Document) writer.getRoot ()).getDocumentElement ();
    }
    
    public Element createInvocationArrayAccessor () {
        DOMSerialiser writer = new DOMSerialiser (_namespace);
        
        writer.single (_elementName, "/java/object/void[@property='args']/array");
        
        return ((Document) writer.getRoot ()).getDocumentElement ();
    }
    
    public Element createInvocationArrayElementAccessor (int index) {
        DOMSerialiser writer = new DOMSerialiser (_namespace);
        
        writer.single (_elementName, "/java/object/void[@property='args']/array/void[index='" + index + "']");
        
        return ((Document) writer.getRoot ()).getDocumentElement ();
    }
    
    public XPath convertToXPath (Element accessor) {
        return new XPath (accessor.getTextContent (), new HashMap ());
    }
    
    public boolean equalDataAccessors (Element accessor1, String style1, Element accessor2, String style2) {
//System.out.println ("COMPARING " +accessor1.getTextContent () + " AND " + accessor2.getTextContent ());
        return accessor1.getTextContent ().equals (accessor2.getTextContent ()) ||
                accessor1.getTextContent ().substring (1).equals (accessor2.getTextContent ()) ||
                accessor1.getTextContent ().equals (accessor2.getTextContent ().substring (1));
    }
    
    public Element getDataAccessorForItem (Node dataItem) throws AccessorException {
        if (dataItem.getLocalName ().equals ("void")) {
            return createInvocationArrayElementAccessor (Integer.valueOf (((Element) dataItem).getAttribute ("index")));
        }
        if (dataItem.getLocalName ().equals ("array")) {
            if (((Element) dataItem.getParentNode ()).getAttribute ("property").equals ("args")) {
                return createInvocationArrayAccessor ();
            } else {
                return createResultObjectAccessor ();
            }
        }
        if (dataItem.getLocalName ().equals ("object")) {
            return createResultObjectAccessor ();
        }
        throw new AccessorException ("Cannot create java call data accessor for element of name " + dataItem.getLocalName ());
    }
    
    public void register (DataAccessorRegistry registry, boolean useAsDefault) {
        registry.register (new QName (_namespace, _elementName), this, useAsDefault);
        registry.register (new QName (null,       "xPath"),      this, false);
        registry.register (new QName ("http://www.pasoa.org/accessors/javaCallXPath", "javaCallXPath"), this, false);
    }

}
