package org.pasoa.preserv.factory;

import com.sleepycat.je.DatabaseException;
import java.io.UnsupportedEncodingException;
import javax.xml.parsers.ParserConfigurationException;
import org.pasoa.preserv.processor.PlugInRegistry;
import org.pasoa.preserv.pquery.ProvenanceQueryPlugin;
import org.pasoa.preserv.xquery.XQueryPlugIn;
import org.pasoa.preserv.record.RecordPlugIn;
import org.pasoa.preserv.storage.BackendStore;
import org.pasoa.preserv.storage.GeneralStorageSystem;
import org.pasoa.preserv.storage.StorageSystem;

public class ProvenanceStoreFactory 
{
    
    
    public static ProvenanceStore newInstance (BackendStore backend, String xqueryImpl) throws DatabaseException, UnsupportedEncodingException, ParserConfigurationException {
        StorageSystem storageSystem = new GeneralStorageSystem (backend, xqueryImpl);
        
        RecordPlugIn rpi = new RecordPlugIn ();
        XQueryPlugIn xqpi = new XQueryPlugIn ();
        ProvenanceQueryPlugin pqpi = new ProvenanceQueryPlugin ();
        
        PlugInRegistry pir = new PlugInRegistry ();
        try {
            pir.registerPlugIn ("/record", rpi);
            pir.registerPlugIn ("/xquery", xqpi);
            pir.registerPlugIn ("/pquery", pqpi);
        } catch (Exception e) {
            System.out.println ("Unable to register plug-ins");
            return null;
        }
        
        ProvenanceStore pStore = new ProvenanceStore (pir, storageSystem);
        
        if (pStore == null) {
            System.out.println ("Factory: pStore is null");
        }
        
        return pStore;
    }
}
