package org.pasoa.preserv.index;

import java.util.Iterator;
import java.util.List;

import org.pasoa.common.Constants;
import org.pasoa.preserv.storage.Retrieval;
import org.pasoa.preserv.index.MyIndex;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.PAssertion;
import org.pasoa.pstructure.View;
import org.pasoa.simpledom.NormalisingDOMWriter;
import org.w3c.dom.Element;

/**
 * A basic but slow implementation of Index that trawls the database via the
 * Retrieval interface.  Not recommended for production systems.
 */
public class DefaultIndex implements MyIndex, Index {
    private Retrieval _navigate;
    
    public DefaultIndex (Retrieval navigate) {
        _navigate = navigate;
    }
    
    public InteractionRecord findInteractionRecord (InteractionKey key)
    throws IndexProblem {
        InteractionRecord record;
        
        try {
            for (long index = 0; index < _navigate
                    .getNumberOfInteractionRecords (); index += 1) {
                record = _navigate.getInteractionRecord (index);
                if (record.getInteractionKey ().equals (key)) {
                    return record;
                }
            }
        } catch (Exception unspecifiedProblem) {
            throw new IndexProblem ("Problem in retrieving from back-end",
                    unspecifiedProblem);
        }
        throw new IndexProblem ("Reference to unknown interaction: " + key);
    }
    
    public PAssertion findPAssertion (
            GlobalPAssertionKey gpak
            ) throws IndexProblem {
        View view = findView (gpak.getInteractionKey (), gpak.getViewKind ());
        Iterator eachPA;
        PAssertion assertion;
        
        eachPA = view.getActorStatePAssertions ().iterator ();
        while (eachPA.hasNext ()) {
            assertion = (PAssertion) eachPA.next ();
            if (assertion.getLocalPAssertionID ().equals (
                    gpak.getLocalPAssertionID ())) {
                return assertion;
            }
        }
        eachPA = view.getInteractionPAssertions ().iterator ();
        while (eachPA.hasNext ()) {
            assertion = (PAssertion) eachPA.next ();
            if (assertion.getLocalPAssertionID ().equals (
                    gpak.getLocalPAssertionID ())) {
                return assertion;
            }
        }
        
        eachPA = view.getRelationshipPAssertions ().iterator ();
        while (eachPA.hasNext ()) {
            assertion = (PAssertion) eachPA.next ();
            if (assertion.getLocalPAssertionID ().equals (
                    gpak.getLocalPAssertionID ())) {
                return assertion;
            }
        }
        
        
        throw new IndexProblem ("Reference to unknown p-assertion: " + gpak);
    }
    
    //    public View findView (InteractionKey key, String viewKind) throws IndexProblem {
    //        InteractionRecord interaction = findInteractionRecord (key);
    //
    //        if (viewKind.equals (Constants.SENDER_VIEW_TYPE)) {
    //            return interaction.getSenderView ();
    //        }
    //        if (viewKind.equals (Constants.RECEIVER_VIEW_TYPE)) {
    //            return interaction.getReceiverView ();
    //        }
    //
    //        throw new IndexProblem ("Unrecognised view kind: " + viewKind);
    //    }
    
    public View findView (InteractionKey key, String viewKind) throws IndexProblem {
        return findViews (key, viewKind).get (0);
    }
    
    
    public List<View> findViews (
            InteractionKey key,
            String viewKind
            ) throws IndexProblem {
        InteractionRecord interaction = findInteractionRecord (key);
        
        String vk = viewKind.intern ();
        if (vk == Constants.SENDER_VIEW_TYPE) {
            return interaction.getSenderViews ();
        } else if (vk == Constants.RECEIVER_VIEW_TYPE) {
            return interaction.getReceiverViews ();
        } else if (vk == Constants.INFERRED_VIEW_TYPE) {
            return interaction.getInferredViews ();
        }
        
        throw new IndexProblem ("Unrecognised view kind: " + viewKind);
    }
    
    public View findView (
            InteractionKey key,
            String viewKind,
            Element asserter
            ) throws IndexProblem {
        InteractionRecord interaction = findInteractionRecord (key);
        
        String vk = viewKind.intern ();
        if (vk == Constants.SENDER_VIEW_TYPE) {
            return findViewByAsserter (interaction.getSenderViews (), asserter);
        } else if (vk == Constants.RECEIVER_VIEW_TYPE) {
            return findViewByAsserter (interaction.getReceiverViews (), asserter);
        } else if (vk == Constants.INFERRED_VIEW_TYPE) {
            return findViewByAsserter (interaction.getInferredViews (), asserter);
        }
        
        throw new IndexProblem ("Unrecognised view kind: " + viewKind);
    }
    
    
    private View findViewByAsserter (List<View> views, Element asserter) throws IndexProblem {
        String strAsserter = NormalisingDOMWriter.writeToString (asserter);
        for (View view: views) {
            if (NormalisingDOMWriter.writeToString (
                    view.getAsserter ()).equals (strAsserter)) {
                return view;
            }
        }
        throw new IndexProblem ("View not found");
    }
    
}
