package org.pasoa.preserv.index;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.pasoa.accessors.AccessorException;
import org.pasoa.common.BestPractice;
import org.pasoa.preserv.pquery.*;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.pstructure.View;
import org.pasoa.pstructure.deserialiser.PStructureDeserialiser;
import org.pasoa.accessors.DataAccessorManager;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.ExposedInteractionMetadata;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.PAssertion;
import org.pasoa.pstructure.PAssertionDataKey;
import org.pasoa.pstructure.RelationshipPAssertion;
import org.pasoa.storeclient.proxy.StoreCatalogue;
import org.w3c.dom.Element;

public class RelationshipIndex {
    private DataAccessorManager _compare;
    private StoreCatalogue      _stores;
    
    public RelationshipIndex (StoreCatalogue stores, DataAccessorManager compare) {
        _compare = compare;
        _stores  = stores;
    }
    
    private Element findPAssertionBody (Index storeIndex, GlobalPAssertionKey gpak) throws IndexProblem {
        PAssertion assertion;
        Element    content;
        
        assertion = storeIndex.findPAssertion (gpak);
        
        if (assertion == null) {
            throw new IndexProblem ("Cannot find p-assertion for relationship p-assertion object in store: " + gpak);
        }
        if (assertion instanceof ActorStatePAssertion) {
            content = ((ActorStatePAssertion) assertion).getContent ();
            if (content == null) {
                throw new IndexProblem ("Actor state p-assertion with null content at GPAK: " + gpak);
            }
            return content;
        }
        if (assertion instanceof InteractionPAssertion) {
            content = ((InteractionPAssertion) assertion).getContent ();
            if (content == null) {
                throw new IndexProblem ("Interaction p-assertion with null content at GPAK: " + gpak);
            }
            return content;
        }

        throw new IndexProblem ("GPAK in relationship p-assertion object does not refer to an interaction or actor state p-assertion: " + gpak);
    }
    
    public Collection getRelationships (String store, Object effectKey) throws IndexProblem {
        Index                  storeIndex    = _stores.getStoreProxy (store).getIndexInterface ();
        Collection             relationships = new LinkedList ();
        InteractionKey         key;
        InteractionRecord      record;
        Iterator               eachView;
        PAssertionDataKey      instantiated;
        Iterator               eachRelate;
        RelationshipPAssertion relation;
        View                   view;
        
        if (effectKey instanceof PAssertionDataKey) {
            key          = ((PAssertionDataKey) effectKey).getGlobalPAssertionKey ().getInteractionKey ();
            instantiated = (PAssertionDataKey) effectKey;
        } else {
            key = ((PAssertionDataKeyTemplate) effectKey).getGlobalPAssertionKey ().getInteractionKey ();
            instantiated = null;
        }
        record   = storeIndex.findInteractionRecord (key);
        eachView = getViews (record).iterator ();
        while (eachView.hasNext ()) {
            view = (View) eachView.next ();
            eachRelate = view.getRelationshipPAssertions ().iterator ();
            while (eachRelate.hasNext ()) {
                relation = (RelationshipPAssertion) eachRelate.next ();
                //if (relation.getSubject ().getLocalPAssertionID ().equals (effectKey.getGlobalPAssertionKey ().getLocalPAssertionID ()) &&
                try {
                    if (instantiated == null) {
                        instantiated = ((PAssertionDataKeyTemplate) effectKey).instantiate (relation.getSubject ().getDataAccessor ());
                    }
                    if (_compare.equalDataAccessors (relation.getSubject ().getDataAccessor (),
                            BestPractice.VERBATIM_STYLE, instantiated.getDataAccessor (),
                            BestPractice.VERBATIM_STYLE)) {
                        relationships.add (relation);
                    }
                } catch (AccessorException problem) {
                    System.err.println ("Problem parsing data accessor in relationship p-assertion");
                    problem.printStackTrace ();
                }
            }
        }
        
        return relationships;
    }
    
    public Collection getRelationshipTargets (RelationshipPAssertion relationship, String fromStore) throws IndexProblem {
        Collection targets    = new LinkedList ();
        Iterator   eachObject = relationship.getObjectIDs ().iterator ();
        ObjectID   object;
        Index      index;
        
        while (eachObject.hasNext ()) {
            object = (ObjectID) eachObject.next ();
            index  = null;
            if (object.getLink () != null) {
                //try {
                    index = _stores.getStoreProxy (object.getLink ().getStoreURL ()).getIndexInterface ();
                /*} catch (MalformedURLException ex) {
                    System.err.println ("Could not load object from store " + object.getLink ().getStoreURL () + " in relationship with lpaid " + relationship.getLocalPAssertionID ());
                    ex.printStackTrace ();
                }*/
            } else {
                index = _stores.getStoreProxy (fromStore).getIndexInterface ();
            }
            if (index != null) {
                try {
                targets.add (new RelationshipTarget (object, fromStore.toString (), relationship.getRelation (),
                        findPAssertionBody (index, object.getGlobalPAssertionKey ())));
                } catch (PQueryProblem problem) {
                    problem.printStackTrace ();
                }
            }
        }
        
        return targets;
    }
    
    public List getLinkedViews (InteractionKey key, View view) throws IndexProblem {
        List                       views       = new LinkedList ();
        PStructureDeserialiser     pstruct     = new PStructureDeserialiser ();
        Iterator                   eachExposed = view.getExposedInteractionMetadata ().iterator ();
        ExposedInteractionMetadata exposed;
        Iterator                   eachMetadata;
        Element                    metadata;
        String                     url;
        Index                      index;
        InteractionRecord          remote;
        
        while (eachExposed.hasNext ()) {
            exposed      = ((ExposedInteractionMetadata) eachExposed.next ());
            eachMetadata = exposed.getInteractionMetadata ().iterator ();
            while (eachMetadata.hasNext ()) {
                metadata = (Element) eachMetadata.next ();
                if (metadata.getNodeName ().equals ("viewLink")) {
                    url = pstruct.deserialiseLink (metadata).getStoreURL ();
                    //try {
                        index  = _stores.getStoreProxy (url).getIndexInterface ();
                        remote = index.findInteractionRecord (key);
                        views.addAll (remote.getSenderViews ());
                        views.addAll (remote.getReceiverViews ());
                        views.addAll (remote.getInferredViews ());
                    /*} catch (MalformedURLException ex) {
                        System.err.println ("Could not load object from store " + url + " found in view link");
                        ex.printStackTrace ();
                    }*/
                }
            }
        }
        
        return views;
    }
    
    private List getViews (InteractionRecord record) throws IndexProblem {
        List views = new LinkedList ();
        
        if (record.getSenderView () != null) {
            views.add (record.getSenderView ());
            views.addAll (getLinkedViews (record.getInteractionKey (), record.getSenderView ()));
        }
        if (record.getReceiverView () != null) {
            views.add (record.getReceiverView ());
            views.addAll (getLinkedViews (record.getInteractionKey (), record.getReceiverView ()));
        }
        views.addAll (record.getInferredViews ());
        
        return views;
    }
}
