package org.pasoa.preserv.ontology.sesame;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import org.openrdf.model.URI;
import org.openrdf.model.impl.URIImpl;
import org.openrdf.sesame.Sesame;
import org.openrdf.sesame.admin.AdminMsgCollector;
import org.openrdf.sesame.config.AccessDeniedException;
import org.openrdf.sesame.config.ConfigurationException;
import org.openrdf.sesame.constants.RDFFormat;
import org.openrdf.sesame.repository.local.LocalRepository;
import org.pasoa.preserv.ontology.Ontology;
import org.pasoa.preserv.ontology.OntologyException;

public class OntologySesame implements Ontology {
    public static URI rdfsSubClassOf;
    
    public OntologySesame () {
        rdfsSubClassOf = new URIImpl ("http://www.w3.org/2000/01/rdf-schema#subClassOf");
    }
    
    public boolean isSubclassOf (String subResource, String superResource) throws OntologyException {
        try {
            LocalRepository repository  = Sesame.getService ().createRepository ("preserv.respository", true);
            URL             targetAsURL = new URL (subResource);
            String          baseURI     = subResource.substring (0, subResource.length () - targetAsURL.getRef ().length ());
            URL             ontologyURL = new URL (baseURI.substring (0, baseURI.length () - 1));

            repository.addData (ontologyURL, baseURI, RDFFormat.TURTLE, false, new AdminMsgCollector ());

            return repository.getGraph ().contains (new URIImpl (subResource), rdfsSubClassOf, new URIImpl (superResource));
        } catch (MalformedURLException badURL) {
            throw new OntologyException (badURL);
        } catch (ConfigurationException badConfig) {
            throw new OntologyException (badConfig);
        } catch (IOException cannotAccessURL) {
            throw new OntologyException (cannotAccessURL);
        } catch (AccessDeniedException cannotAccessURL) {
            throw new OntologyException (cannotAccessURL);
        }  
    }
}
