package org.pasoa.preserv.pquery;

import java.util.List;
import org.pasoa.accessors.AccessorException;
import org.pasoa.accessors.DataAccessorManager;
import org.pasoa.common.Constants;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.PAssertionDataKey;
import org.pasoa.pstructure.deserialiser.PStructureDeserialiser;
import org.pasoa.preserv.xquery.QueryProblem;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.simpledom.DOMTraverser;
import org.pasoa.accessors.XPath;
//import org.pasoa.accessors.snxpath.SingleNodeXPath;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class PAssertionDataItem {
    private Node                _item;
    private DataAccessorManager _manager;
    
    public PAssertionDataItem (DataAccessorManager manager, Node item) {
        _manager = manager;
        _item    = item;
    }
    
/*    private Element getAncestorNode (String[] localNames) throws QueryProblem {
        List locals  = Arrays.asList (localNames);
        Node current = _item;
 
        while (current != null && !matchesName (locals, current.getNamespaceURI (), current.getLocalName (), current.getNodeName ())) {
            current = current.getParentNode ();
        }
 
        if (current == null) {
            throw new QueryProblem ("Cannot find ancestor node of name " + locals + " from current node " + _item.getNodeName ());
        }
 
        return (Element) current;
    }*/
    
    private boolean matchesName (List matches, String namespace, String local, String whole) {
        if (namespace != null && local != null) {
            return namespace.equals (Constants.PSTRUCT_NS) && matches.contains (local);
        }
        if (whole == null) {
            return false;
        }
        return matches.contains (whole.substring (whole.indexOf (":") + 1));
    }
    
    public Element getDataAccessor () throws QueryProblem {
        try {
            return _manager.getDataAccessorForItem (_item);
        } catch (AccessorException ex) {
            throw new QueryProblem ("Cannot parse data accessor", ex);
        }
        //return getXPathDataAccessor ().toXML ();
    }
    
    public XPath getXPathDataAccessor () throws QueryProblem {
        try {
            return _manager.convertToXPath (_manager.getDataAccessorForItem (_item));
        } catch (AccessorException ex) {
            throw new QueryProblem ("Cannot parse data accessor", ex);
        }
        //return SingleNodeXPath.generate (_item, getAncestorNode (new String[] {"content", "relationshipPAssertion"}));
    }
    
    public InteractionKey getInteractionKey () throws QueryProblem {
        Element record = DOMTraverser.getAncestorNode (_item, Constants.PSTRUCT_NS, new String[] {"interactionRecord"});
        Element key    = new DOMDeserialiser (Constants.PSTRUCT_NS).single (record, "interactionKey");
        
        return new PStructureDeserialiser ().deserialiseInteractionKey (key);
    }
    
    public GlobalPAssertionKey getGlobalPAssertionKey () throws QueryProblem {
        return new GlobalPAssertionKey (getInteractionKey (), getViewKind (), getLocalPAssertionID ());
    }
    
    public String getLocalPAssertionID () throws QueryProblem {
        Element assertion = DOMTraverser.getAncestorNode (_item, Constants.PSTRUCT_NS, new String[] {"interactionPAssertion", "actorStatePAssertion", "relationship"});
        
        return new DOMDeserialiser (Constants.PSTRUCT_NS).text (assertion, "localPAssertionId");
    }
    
    public PAssertionDataKey getPAssertionDataKey () throws QueryProblem {
        return new PAssertionDataKey (getGlobalPAssertionKey (), getDataAccessor ());
    }
    
    public String getViewKind () throws QueryProblem {
        if (DOMTraverser.getAncestorNode (_item, Constants.PSTRUCT_NS, new String[] {"sender", "receiver"}).getLocalName ().equals ("sender")) {
            return Constants.SENDER_VIEW_TYPE;
        } else {
            return Constants.RECEIVER_VIEW_TYPE;
        }
    }
}
