package org.pasoa.preserv.pquery;

import org.pasoa.accessors.AccessorException;
import org.pasoa.accessors.DataAccessorRegistry;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.PAssertionDataKey;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * A p-assertion data key for an item where the data accessor type is unknown:
 * it can be resolved when the type of data accessor is known.
 */
public class PAssertionDataKeyTemplate {
    private GlobalPAssertionKey  _gpak;
    private DataAccessorRegistry _registry;
    private Node                 _item;
    private PAssertionDataKey    _instance;
    
    public PAssertionDataKeyTemplate (GlobalPAssertionKey gpak, Node item, DataAccessorRegistry registry) {
        _gpak     = gpak;
        _registry = registry;
        _item     = item;
    }
    
    public GlobalPAssertionKey getGlobalPAssertionKey () {
        return _gpak;
    }
    
    public PAssertionDataKey getInstantiation () throws AccessorException {
        if (_instance == null) {
            return new PAssertionDataKey (_gpak, _registry.getDefault ().getDataAccessorForItem (_item));
        } else {
            return _instance;
        }
    }
    
    public PAssertionDataKey instantiate (String accessorNamespace, String accessorElementName) throws AccessorException {
        _instance = new PAssertionDataKey (_gpak, _registry.lookup (accessorNamespace, accessorElementName).
                getDataAccessorForItem (_item));
        return _instance;
    }
    
    public PAssertionDataKey instantiate (Element otherAccessor) throws AccessorException {
        if (otherAccessor == null) {
            return new PAssertionDataKey (_gpak, null);
        } else {
            return instantiate (otherAccessor.getNamespaceURI (), otherAccessor.getLocalName ());
        }
    }
}
