package org.pasoa.preserv.pquery;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import org.pasoa.common.Constants;
import org.pasoa.preserv.storage.Retrieval;
import org.pasoa.preserv.xquery.QueryProblem;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.pstructure.deserialiser.PStructureDeserialiser;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.simpledom.DOMWriter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class ProvenanceQueryDeserialiser {
    public ProvenanceQueryRequest deserialiseProvenanceQuery (Element serialised, Retrieval retrieval, String thisStoreURL, Document data) throws PQueryProblem {
        if (serialised == null) {
            //System.err.println ("ProvenanceQueryDeserialiser.deserialiseProvenanceQuery: Element to be deserialised is null");
            return null;
        }
        
        DOMDeserialiser          parse  = new DOMDeserialiser (Constants.PQUERY_NS);
        QueryDataHandle          qdh    = deserialiseQueryDataHandle (parse.single (serialised, "queryDataHandle"), retrieval, thisStoreURL, data);
        RelationshipTargetFilter filter = deserialiseRelationshipTargetFilter (parse.single (serialised, "relationshipTargetFilter"));
        
        return new ProvenanceQueryRequest (qdh, filter);
    }
    
    public ProvenanceQueryResults deserialiseProvenanceQueryResults (Element serialised) throws PQueryProblem {
        DOMDeserialiser parse = new DOMDeserialiser (Constants.PQUERY_NS);
        
        if (serialised == null) {
            return null;
        }
        if (serialised.getLocalName ().equals ("Envelope")) {
            serialised = parse.single (serialised, Constants.SOAP_NS, "Body");
            serialised = parse.single (serialised, Constants.PQUERY_NS, "provenanceQueryResult");
        }
        PStructureDeserialiser pstruct  = new PStructureDeserialiser ();
        Element         start             = parse.single (serialised, "start");
        Iterator        eachStart         = parse.iterator (start, Constants.PSTRUCT_NS, "pAssertionDataKey");
        Iterator        eachFull          = parse.iterator (serialised, "fullRelationship");
        Collection      starts            = new LinkedList ();
        Collection      fullRelationships = new LinkedList ();
        
        while (eachStart.hasNext ()) {
            try {
                starts.add (pstruct.deserialisePAssertionDataKey ((Element) eachStart.next ()));
            } catch (QueryProblem ex) {
                throw new PQueryProblem ("Cannot deserialise p-assertion data key", ex);
            }
        }
        while (eachFull.hasNext ()) {
            fullRelationships.add (deserialiseFullRelationship ((Element) eachFull.next ()));
        }
        
        return new ProvenanceQueryResults (starts, fullRelationships);
    }
    
    public FullRelationship deserialiseFullRelationship (Element serialised) throws PQueryProblem {
        if (serialised == null) {
            return null;
        }
        
        DOMDeserialiser        parse    = new DOMDeserialiser (Constants.PQUERY_NS);
        PStructureDeserialiser pstruct  = new PStructureDeserialiser ();
        ObjectID               subject  = pstruct.deserialiseObjectID (parse.single (serialised, "fullSubjectId"));
        String                 relation = parse.text (serialised, "relation");
        String                 lpaid    = parse.text   (serialised, "localPAssertionId");
        ObjectID               object   = pstruct.deserialiseObjectID (parse.single (serialised, "fullObjectId"));
        
        return new FullRelationship (subject, relation, lpaid, object);
    }
    
/*    public PAssertionDataKey deserialisePAssertionDataKey (Element serialised) throws PQueryProblem {
        if (serialised == null) {
            return null;
        }
 
        try {
            DOMDeserialiser        parse       = new DOMDeserialiser (Constants.PSTRUCT_NS);
            PStructureDeserialiser pstruct     = new PStructureDeserialiser ();
            InteractionKey         interaction = pstruct.deserialiseInteractionKey (parse.single (serialised, "interactionKey"));
            String                 viewKind    = parse.text   (serialised, "viewKind");
            String                 lpaid       = parse.text   (serialised, "localPAssertionId");
            Element                accessor    = parse.single (serialised, "dataAccessor");
 
            return new PAssertionDataKey (new GlobalPAssertionKey (interaction, viewKind, lpaid), accessor);
        } catch (QueryProblem pstructProblem) {
            throw new PQueryProblem ("Could not deserialise p-assertion data key", pstructProblem);
        }
    }*/
    
    public QueryDataHandle deserialiseQueryDataHandle (Element serialised, Retrieval retrieval, String thisStoreURL, Document data) throws PQueryProblem {
        if (serialised == null) {
            //System.err.println ("ProvenanceQueryDeserialiser.deserialiseProvenanceQuery: No query data handle specified in query");
            return null;
        }
        
        DOMDeserialiser     parse      = new DOMDeserialiser (Constants.PQUERY_NS);
        Element             search     = parse.single (serialised, "search");
        Element             xpath      = parse.single (search, "xpathSearch");
        Element             padk       = parse.single (search, Constants.PSTRUCT_NS, "pAssertionDataKey");
        String              reference;
        Map                 namespaces;
        PStructureReference searchOver;
        
        if (xpath != null) {
            reference  = parse.text (xpath, "path");
            namespaces = deserialiseNamespaceMappings (xpath);
            searchOver = deserialisePStructureReference (parse.single (serialised, "pStructureReference"), retrieval, thisStoreURL, data);
            return new QueryDataHandle (reference, namespaces, "", searchOver);
        }
        if (padk != null) {
            try {
                return new QueryDataHandle (new PStructureDeserialiser ().deserialisePAssertionDataKey (padk));
            } catch (QueryProblem padkProblem) {
                throw new PQueryProblem ("Malformed p-assertion data key in query data handle", padkProblem);
            }
        }
        
        throw new PQueryProblem ("Cannot find a query data handle in provenance query request");
    }
    
    public RelationshipTargetFilter deserialiseRelationshipTargetFilter (Element serialised) {
        if (serialised == null) {
            System.err.println ("ProvenanceQueryDeserialiser.deserialiseProvenanceQuery: No relationship target filter specified in query");
            return null;
        }
        DOMDeserialiser parse      = new DOMDeserialiser (Constants.PQUERY_NS);
        Element         search     = parse.single (parse.single (serialised, "check"), "xpathSearch");
        String          reference  = parse.text (search, "path");
        Map             namespaces = deserialiseNamespaceMappings (search);
        
        return new RelationshipTargetFilter (reference, namespaces, "");
    }
    
    public PStructureReference deserialisePStructureReference (Element serialised, Retrieval retrieval, String thisStoreURL, Document data) throws PQueryProblem {
        if (serialised == null) {
            return null;
        }
        
        DOMDeserialiser        parse    = new DOMDeserialiser (Constants.PQUERY_NS);
        boolean                contents = parse.single (serialised, "storeContents") != null;
        ProvenanceQueryRequest pQuery   = deserialiseProvenanceQuery (parse.single (serialised, "provenanceQuery"), retrieval, thisStoreURL, data);
        //TransitiveClosure
        
        if (contents) {
            return new ProvenanceStoreContents (data);
        }
        if (pQuery != null) {
            return pQuery;
        }
        return null;
    }
    
    public Map deserialiseNamespaceMappings (Element serialised) {
        if (serialised == null) {
            return null;
        }
        
        Map             mappings = new HashMap ();
        DOMDeserialiser parse    = new DOMDeserialiser (Constants.PQUERY_NS);
        Iterator        eachMap  = parse.iterator (serialised, "namespaceMapping");
        
        while (eachMap.hasNext ()) {
            deserialiseNamespaceMapping ((Element) eachMap.next (), mappings);
        }
        
        return mappings;
    }
    
    public void deserialiseNamespaceMapping (Element serialised, Map addTo) {
        if (serialised == null) {
            return;
        }
        
        DOMDeserialiser parse     = new DOMDeserialiser (Constants.PQUERY_NS);
        String          prefix    = parse.text (serialised, "prefix");
        String          namespace = parse.text (serialised, "namespace");
        
        addTo.put (prefix, namespace);
    }
}
