package org.pasoa.preserv.pquery;

import java.util.Iterator;
import java.util.Map;
import org.pasoa.accessors.AccessorException;
import org.pasoa.common.Constants;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.pstructure.PAssertionDataKey;
import org.pasoa.pstructure.serialiser.PStructureSerialiser;
import org.pasoa.simpledom.DOMSerialiser;
import org.pasoa.accessors.XPath;
import org.w3c.dom.Document;

public class ProvenanceQuerySerialiser {
    private Document serialiseFullRelationship (FullRelationship fullRelationship) {
        DOMSerialiser        build   = new DOMSerialiser ();
        PStructureSerialiser pstruct = new PStructureSerialiser ();
        
        build.namespace (Constants.PQUERY_NS);
        build.start ("fullRelationship");
        build.append (serialiseFullSubjectOrObject (true, fullRelationship.getSubject ()));
        build.single ("relation", fullRelationship.getRelation ());
        build.namespace (Constants.PSTRUCT_NS);
        build.single ("localPAssertionId", fullRelationship.getLocalPAssertionID ());
        build.namespace (Constants.PQUERY_NS);
        build.append (serialiseFullSubjectOrObject (false, fullRelationship.getObject ()));
        
        return (Document) build.getRoot ();
    }
    
    public Document serialiseFullSubjectOrObject (boolean isSubject, ObjectID objectID) {
        DOMSerialiser        build   = new DOMSerialiser ();
        PStructureSerialiser pstruct = new PStructureSerialiser ();
        
        build.namespace (Constants.PQUERY_NS);
        if (isSubject) {
            build.start ("fullSubjectId");
        } else  {
            build.start ("fullObjectId");
        }
        build.namespace (Constants.PSTRUCT_NS);
        build.append (pstruct.serialiseInteractionKey (objectID.getGlobalPAssertionKey ().getInteractionKey ()));
        build.append (pstruct.serialiseViewKind (objectID.getGlobalPAssertionKey ().getViewKind ()));
        build.single ("localPAssertionId", objectID.getGlobalPAssertionKey ().getLocalPAssertionID ());
        build.start ("dataAccessor");
        build.append (objectID.getDataAccessor ());
        build.end ();
        build.single ("parameterName", objectID.getParameterName ());
        build.start ("objectLink");
        if (objectID.getLink () != null) {
            build.single ("provenanceStoreRef", objectID.getLink ().getStoreURL ());
        }
        
        return (Document) build.getRoot ();
    }
    
    public Document serialiseProvenanceQueryRequest (ProvenanceQueryRequest request) throws PQueryProblem {
        DOMSerialiser build = new DOMSerialiser (Constants.PQUERY_NS);
        
        build.start  ("provenanceQuery");
        build.append (serialiseQueryDataHandle          (request.getQueryDataHandle ()));
        build.append (serialiseRelationshipTargetFilter (request.getRelationshipTargetFilter ()));
        
        return (Document) build.getRoot ();
    }
    
    public Document serialiseProvenanceQueryResults (ProvenanceQueryResults results) throws PQueryProblem {
        DOMSerialiser        build     = new DOMSerialiser ();
        PStructureSerialiser pstruct   = new PStructureSerialiser ();
        Iterator             eachStart = results.getStarts ().iterator ();
        Iterator             eachRel   = results.getFullRelationships ().iterator ();
        PAssertionDataKey    padk;
        
        build.namespace (Constants.PQUERY_NS);
        build.start ("provenanceQueryResult");
        build.start ("start");
        while (eachStart.hasNext ()) {
            try {
                padk = ((PAssertionDataKeyTemplate) eachStart.next ()).getInstantiation ();
            } catch (AccessorException accessorProblem) {
                throw new PQueryProblem ("Problem instantiating data accessor for start", accessorProblem);
            }
            if (padk != null) {
                build.append (pstruct.serialisePAssertionDataKey (padk));
            } else {
                System.err.println ("Null start produced");
            }
        }
        build.end ();
        while (eachRel.hasNext ()) {
            build.append (serialiseFullRelationship ((FullRelationship) eachRel.next ()));
        }
        
        return (Document) build.getRoot ();
    }
    
    public Document serialiseQueryDataHandle (QueryDataHandle qdh) throws PQueryProblem {
        XPath             path = qdh.getXPath ();
        PAssertionDataKey padk = qdh.getPAssertionDataKey ();
        
        if (path != null) {
            return serialiseQueryDataHandle (path.getPath (), path.getNamespaces ());
        }
        if (padk != null) {
            return serialiseQueryDataHandle (padk);
        }
        
        throw new PQueryProblem ("Unknown query data handle content");
    }
    
    public Document serialiseQueryDataHandle (String queryDataHandleXPath, Map qdhNamespaces) {
        DOMSerialiser builder       = new DOMSerialiser (Constants.PQUERY_NS);
        Iterator      eachQDHPrefix = qdhNamespaces.keySet ().iterator ();
        String        prefix;
        
        builder.start  ("queryDataHandle");
        builder.start  ("search");
        builder.start  ("xpathSearch");
        builder.single ("path", queryDataHandleXPath);
        while (eachQDHPrefix.hasNext ()) {
            prefix = (String) eachQDHPrefix.next ();
            builder.start  ("namespaceMapping");
            builder.single ("prefix",    prefix);
            builder.single ("namespace", (String) qdhNamespaces.get (prefix));
            builder.end    ();
        }
        builder.end ();
        builder.end ();
        builder.start  ("pStructureReference");
        builder.single ("storeContents");
        
        return (Document) builder.getRoot ();
    }
    
    public Document serialiseQueryDataHandle (PAssertionDataKey padk) {
        DOMSerialiser        builder = new DOMSerialiser (Constants.PQUERY_NS);
        PStructureSerialiser pstruct = new PStructureSerialiser ();
        
        builder.start  ("queryDataHandle");
        builder.start  ("search");
        builder.append (pstruct.serialisePAssertionDataKey (padk));
        builder.start  ("pStructureReference");
        builder.single ("storeContents");
        
        return (Document) builder.getRoot ();
    }
    
    public Document serialiseRelationshipTarget (RelationshipTarget relationshipTarget) {
        DOMSerialiser        build   = new DOMSerialiser ();
        PStructureSerialiser pstruct = new PStructureSerialiser ();
        
        build.namespace (Constants.PQUERY_NS);
        build.start ("relationshipTarget");
        build.namespace (Constants.PSTRUCT_NS);
        build.append (pstruct.serialiseInteractionKey (relationshipTarget.getGlobalPAssertionKey ().getInteractionKey ()));
        build.single ("viewKind", relationshipTarget.getGlobalPAssertionKey ().getViewKind ());
        build.single ("localPAssertionId", relationshipTarget.getGlobalPAssertionKey ().getLocalPAssertionID ());
        if (relationshipTarget.getDataAccessor () != null) {
            build.copy (relationshipTarget.getDataAccessor ());
        }
        build.single ("parameterName", relationshipTarget.getParameterName ());
        build.start ("objectLink");
        if (relationshipTarget.getLink () != null) {
            build.single ("provenanceStoreRef", relationshipTarget.getLink ().getStoreURL ());
        }
        build.end ();
        build.single ("relation", relationshipTarget.getRelation ());
        build.copy (relationshipTarget.getAssertion ());
        return (Document) build.getRoot ();
    }
    
    public Document serialiseRelationshipTargetFilter (RelationshipTargetFilter scope) {
        return serialiseRelationshipTargetFilter (scope.getXPath ().getPath (), scope.getXPath ().getNamespaces ());
    }
    
    public Document serialiseRelationshipTargetFilter (String scopingXPath, Map rtfNamespaces) {
        DOMSerialiser builder       = new DOMSerialiser (Constants.PQUERY_NS);
        Iterator      eachRTFPrefix = rtfNamespaces.keySet ().iterator ();
        String        prefix;
        
        builder.start  ("relationshipTargetFilter");
        builder.start  ("check");
        builder.start  ("xpathSearch");
        builder.single ("path", scopingXPath);
        while (eachRTFPrefix.hasNext ()) {
            prefix = (String) eachRTFPrefix.next ();
            builder.start  ("namespaceMapping");
            builder.single ("prefix",    prefix);
            builder.single ("namespace", (String) rtfNamespaces.get (prefix));
            builder.end    ();
        }
        
        return (Document) builder.getRoot ();
    }
}
