package org.pasoa.preserv.pquery;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import org.pasoa.preserv.xquery.QueryProblem;
import org.pasoa.pstructure.PAssertionDataKey;
import org.pasoa.accessors.DataAccessorRegistry;
import org.pasoa.accessors.XPath;
import org.pasoa.laxquery.XPathUtilities;
import org.pasoa.pstructure.dom.util.PAssertionDataItemHelper;
import org.w3c.dom.Node;

public class QueryDataHandle {
    private String              _reference;
    private Map                 _namespaces;
    private String              _xmlMapping;
    private PStructureReference _searchOver;
    private PAssertionDataKey   _pAssertionDataKey;
    
    public QueryDataHandle (String reference, Map namespaces, String xmlMapping, PStructureReference searchOver) {
        _reference  = reference;
        _namespaces = namespaces;
        _xmlMapping = xmlMapping;
        _searchOver = searchOver;
    }
    
    public QueryDataHandle (PAssertionDataKey padk) {
        _pAssertionDataKey = padk;
    }
    
    public PAssertionDataKey getPAssertionDataKey () {
        return _pAssertionDataKey;
    }
    
    public PStructureReference getSearchSpace () {
        return _searchOver;
    }
    
    public XPath getXPath () {
        if (_reference != null) {
            return new XPath (_reference, _namespaces);
        } else {
            return null;
        }
    }
    
    public String getXMLMapping () {
        return _xmlMapping;
    }
    
    /**
     * @return A set of p-assertion data keys
     */
    public Collection resolve (DataAccessorRegistry accessors) throws PQueryProblem {
        if (_reference != null) {
            return resolveXPath (accessors);
        } else {
            return Collections.singleton (_pAssertionDataKey);
        }
    }

    private Collection resolveXPath (DataAccessorRegistry accessors) throws PQueryProblem {
        PAssertionDataItemHelper itemHelper = new PAssertionDataItemHelper (accessors);
        Collection               results    = new HashSet ();
        Iterator                 eachDataItem;
        Node                     item;

        if (_searchOver == null) {
            throw new PQueryProblem ("No search space specified for query data handle");
        }
        try {
            eachDataItem = XPathUtilities.resolveNodeSet (_searchOver.getPStructure (), _reference, _namespaces).iterator ();
            while (eachDataItem.hasNext ()) {
                item = (Node) eachDataItem.next ();
                results.add (new PAssertionDataKeyTemplate (itemHelper.getGlobalPAssertionKey (item), item, accessors));
            }
        } catch (QueryProblem ex) {
            throw new PQueryProblem ("Cannot resolve query data handle", ex);
        }

        return results;
    }
    
    public void setSearchSpace (PStructureReference searchSpace) {
        _searchOver = searchSpace;
    }
    
    public String toString () {
        if (_reference != null) {
            return "Path: " + _reference + ", Namespaces: " + _namespaces;
        } else {
            return _pAssertionDataKey.toString ();
        }
    }
}
