package org.pasoa.preserv.record;

import java.io.StringReader;
import java.util.Iterator;
import java.util.Map;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.pasoa.common.Constants;
import org.pasoa.preserv.factory.PlugIn;
import org.pasoa.preserv.storage.StorageSystem;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.deserialiser.PStructureDeserialiser;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.simpledom.DOMWriter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public class RecordPlugIn implements PlugIn {
    public  static final String NO_LPID      = "nolocalpassertionid";
    
    private boolean _sync;
    private DocumentBuilderFactory _dbf;
    private DocumentBuilder _docBuilder;
    private DOMDeserialiser _parser;
    private PStructureDeserialiser _depstruct;
    
    public RecordPlugIn () {
        _parser = new DOMDeserialiser (Constants.PSTRUCT_NS);
        _depstruct = new PStructureDeserialiser ();
        _dbf = DocumentBuilderFactory.newInstance ();
        _dbf.setNamespaceAware (true);
        
        try {
            _docBuilder = _dbf.newDocumentBuilder ();
        } catch (ParserConfigurationException e) {
            e.printStackTrace ();
            
        }
        // Default is to receive dispatches asynchrounsly
        _sync = false;
    }
    
    public Document process (Document document, StorageSystem ss) throws Exception {
        
        
        StringBuffer ackStr = new StringBuffer ("<pr:recordAck xmlns:pr=\"" + Constants.PRECORD_NS + "\">");
        
        NodeList nl = document.getDocumentElement ().getChildNodes ();
        Element soapBody = null;
        for (int i = 0; i < nl.getLength (); i++) {
            if (nl.item (i) instanceof Element) {
                if (((Element) nl.item (i)).getTagName ().endsWith ("Body")) {
                    soapBody = (Element) nl.item (i);
                    break;
                }
            }
        }
        
        NodeList idContents = soapBody.getElementsByTagNameNS (Constants.PRECORD_NS, "identifiedContent");
        
        for (int i = 0; i < idContents.getLength (); i++) {
            Element idc = (Element) idContents.item (i);
            Element identifiers = _parser.single (idc, "interactionKey");
            InteractionKey irid = _depstruct.deserialiseInteractionKey (identifiers);
            Element asserter = _parser.single (_parser.single (idc, "asserter"));
            Element viewKindElem = _parser.single (idc, "viewKind");
            String viewKind = _depstruct.deserialiseViewKind (viewKindElem);
            Iterator eachContent = _parser.iterator (idc, Constants.PRECORD_NS, "content");
            
            while (eachContent.hasNext ()) {
                Element content     = (Element) eachContent.next ();
                String  contentName = _parser.single (content).getLocalName ();
                String  localPAssertionId;
                if (contentName.equalsIgnoreCase ("submissionFinished") ||
                    contentName.equalsIgnoreCase ("exposedInteractionMetaData")) {
                    localPAssertionId = NO_LPID;
                } else {
                    localPAssertionId = _parser.text (_parser.single(content), "localPAssertionId");
                }
                GlobalPAssertionKey gpid = new GlobalPAssertionKey (irid, viewKind, localPAssertionId);
                
                // Record the assertion in the underlying storage system
                ss.record (gpid, asserter, _parser.single (content), contentName);
                
                // Make the acknowledgment document
                
                ackStr.append ("<pr:ack xmlns:pr=\"" + Constants.PRECORD_NS + "\" xmlns:ps=\"" + Constants.PSTRUCT_NS + "\">");
                ackStr.append ("<pr:contentName>");
                ackStr.append (contentName);
                ackStr.append ("</pr:contentName>");
                ackStr.append ("<ps:interactionKey>");
                
                ackStr.append ("<ps:messageSource>");
                ackStr.append (DOMWriter.writeToString (irid.getMessageSource ()));
                ackStr.append ("</ps:messageSource>");
                
                ackStr.append ("<ps:messageSink>");
                ackStr.append (DOMWriter.writeToString (irid.getMessageSink ()));
                ackStr.append ("</ps:messageSink>");
                
                ackStr.append ("<pr:interactionId>");
                ackStr.append (irid.getInteractionID ());
                ackStr.append ("</pr:interactionId>");
                
                ackStr.append ("</ps:interactionKey>");
                ackStr.append ("</pr:ack>");
            }
            
        }
        ackStr.append ("</pr:recordAck>");
        
        Document retDoc = _dbf.newDocumentBuilder().parse (new InputSource (new StringReader (ackStr.toString ())));
        
        return retDoc;
    }
    
    public void setReceiveDipatchesSynchronously (boolean sync) {
        _sync = sync;
    }
    
    public boolean receiveDispatchesSyncronously () {
        return _sync;
    }
    
    public String getDefaultResponse () {
        return "<pr:recordAck xmlns:pr=\"" + Constants.PRECORD_NS + "\"><pr:synch_ack/></pr:recordAck>";
    }
    
    public Map getConfiguration () {
        // TODO Auto-generated method stub
        return null;
    }
    
    public void initialize (Map configuration) {
        // TODO Auto-generated method stub
        
    }
    
}
