package org.pasoa.preserv.servlet.management;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class DynamicClassFinder {

	private Collection<Class> cache;
	private String pkg;
	private Class needle;

	public DynamicClassFinder(String pkg, Class needle) throws IOException {
		this.pkg = pkg;
		this.needle = needle;
		reload();
	}

	public void reload() throws IOException {

		String pkgPath = pkg.replace('.', '/');

		Enumeration<URL> resource;
		resource = Thread.currentThread().getContextClassLoader().getResources(pkgPath);
		while (resource.hasMoreElements()) {
			URL url = resource.nextElement();
			System.out.println(url);
			cache = traversePackage(new File(url.getFile()), pkg);
		}
	}
	
	public Collection<Class> getClasses() {
		return cache;
	}

	private Set<Class> traversePackage(File parent, String currentPackage) {
		Set<Class> list = new HashSet<Class>();

		File files [] = parent.listFiles();
		for(File f: files) {
			String fname = f.getName();
			if (f.isDirectory()) {
				list.addAll(traversePackage(f, currentPackage + "." + fname));
			} else if (fname.endsWith(".class")) {
				try {
					Class cl = Class.forName(currentPackage + "." + fname.substring(0, fname.length() - 6));
					if (needle.isAssignableFrom(cl) && !cl.isInterface()) {
						list.add(cl);
						System.out.println(cl.getCanonicalName());
					}
				} catch(ClassNotFoundException ex) {
					System.err.println("Class not found " + currentPackage + "." + fname.substring(0, fname.length() - 6));
				}
			}
		}

		return list;
	}
}
