package org.pasoa.preserv.servlet.management;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.pasoa.common.Constants;
import org.pasoa.preserv.factory.ProvenanceStore;
import org.pasoa.preserv.factory.ProvenanceStoreFactory;
import org.pasoa.preserv.factory.ProvenanceStoreRegistry;
import org.pasoa.preserv.processor.Dispatcher;
import org.pasoa.preserv.servlet.ServletInitialConfig;
import org.pasoa.preserv.storage.BackendStore;
import org.pasoa.preserv.storage.InitialConfig;
import org.pasoa.preserv.xquery.XQueryPlugIn;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.simpledom.DOMWriter;
import org.pasoa.storeclient.messages.XQueryRequestDocument;
import org.pasoa.util.httpsoap.DefaultDocumentClient;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import com.sleepycat.je.DatabaseException;

public class ProvenanceServiceManagement extends HttpServlet
{
	public static String STORE_PATH = "StorePath";
	public static String BACKEND_KIND = "Backend";
	public static String XQUERY_IMPL = "XQueryImpl";
	public static String BERKLEY_DB = "BerkeleyJDB";
	public static String MYSQL_DB = "MySQL";
	public static String STORE_PACKAGE = "org.pasoa.preserv.storage";
//	private BackendStore _backend;
//	private String _storagePath;
//	private long _cacheWaitTime;
	private ProvenanceStoreRegistry _pStoreRegistry;

	private String defaultXQueryEngine;
//	private DocumentBuilder docBuilder;

	private boolean storeListActive = true;

	private final static String PREF_CONFIGURATION = "Configuration";
	private final static String PREF_XQUERY = "XQueryImplementation";
	private final static String PREF_STORES = "Stores";
	private final static String PREF_BACKEND = "Backend";

//	private void initBuilder() {
//	try {
//	DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
//	docBuilderFactory.setNamespaceAware(true);
//	docBuilder = docBuilderFactory.newDocumentBuilder();
//	} catch (ParserConfigurationException ex) {
//	throw new RuntimeException(ex);
//	}
//	}

	private DynamicClassFinder clFinder;

	public void init() throws ServletException
	{
//		initBuilder();
		System.out.println("initialize");

		try {
			clFinder = new DynamicClassFinder(STORE_PACKAGE, Class.forName("org.pasoa.preserv.storage.BackendStore"));
			storeListActive = true;
		} catch (ClassNotFoundException ex) {
			storeListActive = false;
			ex.printStackTrace();
		} catch (IOException ex) {
			storeListActive = false;
			ex.printStackTrace();
		}

		_pStoreRegistry = new ProvenanceStoreRegistry();
		InitialConfig iConfig = new ServletInitialConfig(getServletConfig());
		String backendKind = getInitParameter(BACKEND_KIND);
		String xqueryUtilities = getInitParameter(XQUERY_IMPL);
		defaultXQueryEngine = xqueryUtilities;
		BackendStore backend = null;
		try {
			Class backendClass = Class.forName(backendKind);
			Class iConfigClass = Class
			.forName("org.pasoa.preserv.storage.InitialConfig");
			Class[] constructorArgs = { iConfigClass };
			boolean initable = true;
			Constructor construct = null;
			try {
				construct = backendClass.getConstructor(constructorArgs);
			} catch (NoSuchMethodException nsme) {
				initable = false;
				nsme.printStackTrace();
			}
			if (initable) {
				Object[] params = { iConfig };
				backend = (BackendStore) construct.newInstance(params);
			} else {
				backend = (BackendStore) backendClass.newInstance();
			}
		} catch (Exception backendException) {
			System.out.println("PReServ: Could not create backend store");
			backendException.printStackTrace();
			throw new ServletException(backendException);
		}
		try {
			ProvenanceStore pStore = ProvenanceStoreFactory.newInstance(
					backend, xqueryUtilities);
			_pStoreRegistry.registerPlugIn("/default", pStore);

		} catch (Exception regException) {
			System.out
			.println("PReServ: Could not create or register default provenance store");
			regException.printStackTrace();
		}
		// TODO get the servlet port at initialisation time.
//		initStores();
		System.out.println("PReServ: Done with initialization");
	}

	private boolean initStoresEx = false;

	private void initStores(HttpServletRequest request){
		System.out.println("initStores");
		initStoresEx = true;
		try {
			Preferences pref = getServletPrefRoot(request).node(PREF_STORES);
			String children[] = pref.childrenNames();
			for (String ch: children) {
				try {
					Preferences chpref = pref.node(ch);
					String backend = chpref.get(PREF_BACKEND, null);
					Class b = Class.forName(backend);
					BackendStore bstore = (BackendStore) 
					b.getConstructor(new Class[] {InitialConfig.class})
					.newInstance(new Object[] {toInitialConfig(chpref.node(PREF_CONFIGURATION))});
					_pStoreRegistry.registerPlugIn("/" + ch, ProvenanceStoreFactory.newInstance(bstore, chpref.get(PREF_XQUERY, null)));
				} catch (Exception e) {
					e.printStackTrace();
				}

			}
		} catch(BackingStoreException ex) {
			ex.printStackTrace();
		}
	}

	private InitialConfig toInitialConfig(Preferences pref) {
		try {
			String keys[] = pref.childrenNames();
			DefaultInitialConfigImpl conf = new DefaultInitialConfigImpl();
			for(String key: keys){
				conf.setInitParameter(key, pref.get(key, null));
			}
			return conf;
		} catch (BackingStoreException ex) {
			ex.printStackTrace();
			return null;
		}
	}

	private void printHtmlHeader(HttpServletResponse response) throws IOException {
		response.setContentType("text/html;charset=UTF-8");
		PrintWriter pw = response.getWriter();
		pw.println("<html><head>" +
				"<LINK REL=StyleSheet HREF=\"/css/manage.css\" TYPE=\"text/css\" MEDIA=screen>" +
		"<title>PReServ Store Management</title></head><body>");
	}
	private void printHtmlFooter(PrintWriter out) {
		out.println("</body></html>");
	}
	public Element xquery (String xquery, Dispatcher dispatch) throws UnsupportedEncodingException, Exception {
		XQueryRequestDocument request = new XQueryRequestDocument (xquery);
		System.out.println(DOMWriter.writeToString(request));
		return new DefaultDocumentClient ().invoke (dispatch, "/xquery", request).getDocumentElement ();
	}
//	private InputStream createXQ(String query) {
//	try {
//	return new ByteArrayInputStream (("<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">" +
//	"<soapenv:Header>" +
//	"</soapenv:Header>" + 
//	"<soapenv:Body>" +
//	"<xq:query xmlns:xq = \"" + Constants.WSXQUERY_NS + "\">" +
//	"<xq:xquery>" +
//	query +
//	"</xq:xquery>" +
//	"</xq:query>" +
//	"</soapenv:Body>" +
//	"</soapenv:Envelope>").getBytes("UTF8"));
//	} catch (UnsupportedEncodingException ex) {
//	ex.printStackTrace();
//	return null;
//	}
//	}

	public static final String DELETE_ACTION = "delete";
	public static final String CONTENTS_ACTION = "contents";
	public static final String COPY_ACTION = "copy";
	public static final String NEW_ACTION = "new";
	public static final String XQUERY_ACTION = "xquery";
	public static final String REMOVE_ACTION = "remove";

	/*private String getServletURL (HttpServletRequest request) {
		String scheme = request.getScheme(); // http
		String serverName = request.getServerName(); // hostname.com
		int serverPort = request.getServerPort(); // 80
		String contextPath = request.getContextPath(); // /mywebapp
		return scheme + "://" + serverName + ":" + serverPort + contextPath;
	}*/
	private String getServletURL (HttpServletRequest request) {
		return request.getContextPath();
	}

	private Preferences getServletPrefRoot(HttpServletRequest request) {
		return Preferences.systemNodeForPackage(
				this.getClass()).node(request.getServerPort() + getServletURL(request));
	}

	private String getManageURL(HttpServletRequest request) {
		return getServletURL(request) + "/manage";
	}

//	private Document executeXQ(String xq, ProvenanceStore store) throws Exception {
//	ByteArrayOutputStream out = new ByteArrayOutputStream();
//	store.getDispatcher().processAndDispatch(
//	createXQ(xq),	out, "/" + Constants.XQUERY_CONTEXT);
//	return docBuilder.parse(new ByteArrayInputStream(out.toByteArray()));
//	}

	private void printTablePage(
			String url,
//			URL xqueryURL,
			HttpServletResponse response) throws IOException {

//		"for $n in $ps: pstruct/ps:interactionRecord/* return count($n)"
//		"for $n in $ps: pstruct/ps:interactionRecord/* return
//		<result>{fn:count($n)}</result>"
//		PrintStream out = new PrintStream(outs);
		PrintWriter out = response.getWriter();
		printHtmlHeader(response);
		out.println("<form action='" + url + "/result.xml' method=get name=postform target=result>" +
				"<input type=hidden name=store>" +
				"<input type=hidden name=dest>" +
				"<input type=hidden name=action>" +
		"</form>");
		printRedirectionForm(out, url);

		out.println("<script language=javascript>" +
//				"function update() {" +
//				"window.location=window.location;" +
//				"}" +
				"function askQuery(lnk) {" +
				"var query = prompt('Please enter the XQuery');" +
				"if (query == null) { " +
				"return false;" +
				"} else {" +
				"lnk.href = lnk.href + query;" +
				"return true;" +
				"}" +
				"}" +
//				"function setValues_2(_form, _store, _action) {" +
//				"_form.store.value = _store; _form.action.value=_action; _form.submit();" +
//				"setRedirect_2(_store, _action);" +
////				"result.close();" +
//				"}" +
//				"function setValues_3(_form, _store, _action, _msg) {" +
//				"_form.store.value=_store; _form.action.value=_action;" +
//				"var _dest = prompt(_msg);" +
//				"if (_dest != null) {" +
//				"_form.dest.value=_dest;" +
//				"_form.submit();" +
//				"setRedirect_2(_store, _action);" +
////				"result.close();" +
//				"} else {" +
//				"_form.store.value = \"\"; _form.action.value=\"\";" +
//				"}" +
//				"}" +
//				"function setRedirect_2(_store, _action) {" +
//				"redirection.store.value=_store;" +
//				"redirection.action.value=_action;" +
//				"redirection.submit();" +
//				"}" +
		"</script>");
		out.println("<table border = 0>\n");

		out.println("<tr>" +
				"<td><a href=\"" + url + "?action=" + NEW_ACTION + "\">New Store</a></td>" +
				"<td><a href=\"" + url + "\">Refresh</a></td>" +
		"</tr>");

		out.println("<tr><th>Store</th><th>Number of InteractionRecords</th></tr>");

		Enumeration<String> keys = _pStoreRegistry.storeNames();
		while (keys.hasMoreElements()) {
			String next = keys.nextElement();
			long count = 0;
			try {
				ProvenanceStore store = _pStoreRegistry.lookup(next);
				count = store.getRetrievalInterface().getNumberOfInteractionRecords();
			} catch (Exception ex) {
				ex.printStackTrace();
			}
			out.println("<tr><td>" + next + "</td><td><span>" + count
					+ "</span>" +
					"<td><a href=\"" + url + "?store=" + next + 
					"&action=" + DELETE_ACTION + 
					"\")'>Delete</a></td>" +
					"<td><a href=\"" + url + "/result.xml?store=" + next + 
					"&action=" + CONTENTS_ACTION + 
					"\">Contents</a></td>" +
					"<td><a href=\"" + url + "/result.xml?store=" + next + 
					"&action=" + XQUERY_ACTION + 
					"&xquery=\" onclick='askQuery(this);'>XQuery</a></td>" +
					"<td><a href = \"" + url + "?action=" + REMOVE_ACTION + 
					"&store=" + next + "\"> Remove </a></td>" +
//					"<td><span onclick = 'setValues_3(postform, \"" + next + 
//					"\", \"" + COPY_ACTION + 
//					"\", \"Please enter the name for the new store\")'>Copy</span></td>" +
			"</tr>");

		}
		out.println("<tr>" +
				"<td><a href=\"" + url + "?action=" + NEW_ACTION + "\">New Store</a></td>" +
				"<td><a href=\"" + url + "\">Refresh</a></td>" +
		"</tr>");
		out.println("</table>");
		printHtmlFooter(out);
	}
	private void performXQuery(
			HttpServletRequest request,
			HttpServletResponse response, 
			String xquery, 
			ProvenanceStore pstore
	) throws Exception {
		//		String servletURL = getServletURL(request);
		PrintWriter pw = response.getWriter();
		printSOAPHeader(response);
		System.out.println("Getting contents");
		//		System.out.println(servletURL + "/xquery");
		Element result = xquery(xquery, pstore.getDispatcher());
		DOMDeserialiser des = new DOMDeserialiser(Constants.PSTRUCT_NS);
		Element env = des.single(result);
		pw.write(DOMWriter.writeToString(env));
		printSOAPFooter(response);
	}
	private void handleWebManagement(HttpServletRequest request, HttpServletResponse response) {
		printRequest(request);
		String action = request.getParameter("action").intern();
		String store = request.getParameter("store");
//		String servletURL = getServletURL(request);
		try {
			ProvenanceStore pstore = null;
			if (store != null) pstore = _pStoreRegistry.lookup(store);
			PrintWriter pw = response.getWriter();
			if (action == DELETE_ACTION) {
				System.out.println("Deleting");
				pstore.delete();
				printTablePage(getManageURL(request), response);
			} else if (action == CONTENTS_ACTION) {
				try {
					performXQuery(request, response, "$ps:pstruct", pstore);					
				} catch (Exception ex) {
					ex.printStackTrace();
					pw.println("XQuery could not be performed");
				}
			} else if (action == COPY_ACTION) {
				// TODO Implement the copy
				System.out.println("Copying contents");
			} else if (action == NEW_ACTION) {
				try {
					createNewStoreWeb(request, response);
				} catch (Exception ex) {
					ex.printStackTrace();
					printHtmlHeader(response);
//					pw.println("<p>Could not create the store.");
					printHtmlFooter(pw);
				}
			} else if (action == REMOVE_ACTION) {
				removeStore(store, request);
				printTablePage(getManageURL(request), response);
			} else if (action == XQUERY_ACTION) {
				try {
					performXQuery(request, response, request.getParameter("xquery"), pstore);					
				} catch (Exception ex) {
					pw.println("XQuery could not be performed");
				}
			} else {
				System.out.println("Unrecognised action");
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		//		PrintWriter pw = response.getWriter();
	}
//	private Preferences getDefaultsForBackend(Class backend) {
//	Preferences pref = Preferences.systemNodeForPackage(backend);
//	return pref;
//	}
	private void removeStore(String store, HttpServletRequest request) throws Exception {
		System.out.println("Removing store " + store);
		_pStoreRegistry.deregisterPlugIn(store);
		Preferences pref = getServletPrefRoot(request).node(PREF_STORES);
		Preferences chpref = pref.node(store.substring(1));
		chpref.clear();
		chpref.removeNode();
		pref.flush();
//		System.out.println("Node exists: " + pref.nodeExists(store))

	}

	private void displayNewStoreForm(HttpServletRequest request, PrintWriter pw, String storeType, String servletURL){
		if (storeListActive) {
			String xqueryEngine = request.getParameter("xqueryengine");
			String servlet = request.getParameter("servlet");

			Collection<Class> backends = clFinder.getClasses();

			String createDis = "";
			if (storeType == null || storeType == "") {
				createDis = "disabled";
			}

			pw.println("<table><form action=\"" + servletURL + "/manage\" method=get name=newform>" +
					"<tr><td>Store Type</td><td><select name=storetype onchange='this.form.submit();'>" +
			"<option value = ''>--- Select the type ---</option>");


			pw.flush();
			Class selectedBackend;
			if (storeType == null) { 
				selectedBackend = null;
			} else {
				try {
					selectedBackend = Class.forName(storeType);
				} catch (ClassNotFoundException e) {
					selectedBackend = null;
				}
			}
			for (Class cl: backends) {
				String clName = cl.getCanonicalName();
//				pw.println("Looking at " + clName);
				try {
//					pw.println("Constructor of " + clName);
					cl.getConstructor(new Class[] {InitialConfig.class});
//					pw.println("Defaults method of " + clName);
					cl.getMethod("getDefaultInitConfig");
				} catch (SecurityException e) {
//					pw.println("Security Exception with " + clName);
					e.printStackTrace();
					continue;
				} catch (NoSuchMethodException e) {
//					pw.println("No Such Method Exception with " + clName);
					e.printStackTrace();
					continue;
				}
				pw.print("<option");
				if (selectedBackend == cl) {
					pw.println(" selected ");
				}
				pw.println(">" + clName + "</option>\n");

			}

			pw.println("</select></td></tr>");
			pw.println("<input type=hidden name=action value=\"" + NEW_ACTION + "\">");

			if (xqueryEngine == null || xqueryEngine.length() == 0) xqueryEngine = defaultXQueryEngine;
			if (servlet == null || servlet.length() == 0) servlet = findServletName();

			pw.println("<tr><td>Servlet name</td><td>" +
					"<input size=70 type=text name=servlet value=\"" + servlet + "\"></td></tr>");
			if (selectedBackend != null) {
//				Preferences pref = Preferences.systemNodeForPackage(selectedBackend);
				try {
					InitialConfig defaults = (InitialConfig)selectedBackend.getMethod("getDefaultInitConfig").invoke(null, new Object[] {});
//					String[] keys;
//					keys = pref.keys();
//					System.out.println("There are keys: " + keys);
//					for (String key: keys) {
					Enumeration<String> confEnum = defaults.getInitParameterNames();
					while(confEnum.hasMoreElements()){
						String key = confEnum.nextElement();
						String param = request.getParameter(key);
						param = param == null ? defaults.getInitParameter(key) : param;
						pw.println("<tr><td>" + key + "</td><td>" +
								"<input size=70 type=text name='" + key + "' value=\"" + param + "\"></td></tr>");
					}
				} catch (Exception ex ) {
					ex.printStackTrace();
					pw.println("<P> There was an error while fetching the default values</p>");
				}
			} else {
				System.out.println("Backend Class is null");
			}
			pw.println("<tr><td>XQuery engine </td><td>" +
					"<input size=70 type=text name=xqueryengine value=\"" + xqueryEngine + "\"></td></tr>");
			pw.println("<tr><td></td><td>" +
					"<input " + createDis + " type=submit name=create value=Create></td></tr>" +
			"</form></table>");
		} else {
			pw.println("<p>The list of availible backends could not be loaded. " +
			"You can try and refresh the list. ");
		}
	}

	private void createNewStoreWeb(HttpServletRequest request, HttpServletResponse response) throws Exception {
		String servletURL = getServletURL(request);
		PrintWriter pw = response.getWriter();
		System.out.println("New Store");
		printHtmlHeader(response);
//		Collection<Class> backends = clFinder.getClasses();
		String storeType = request.getParameter("storetype");
		String xqueryEngine = request.getParameter("xqueryengine");
		String servlet = request.getParameter("servlet");
		if (
				xqueryEngine != null && 
				storeType != null && 
				xqueryEngine.length() != 0 && 
				storeType.length() != 0 &&
				request.getParameter("create") != null
		) {
			Class backendClass = Class.forName(storeType);
			Enumeration<String> defaultNames = ((InitialConfig)backendClass.getMethod("getDefaultInitConfig").invoke(null, new Object[] {})).getInitParameterNames();

			DefaultInitialConfigImpl conf = new DefaultInitialConfigImpl();
			while (defaultNames.hasMoreElements()) {
				String key = defaultNames.nextElement();
				String value = request.getParameter(key);
				conf.setInitParameter(key, value);
			}
	
			createStore(storeType, request, conf, servlet, xqueryEngine);
			printTablePage(getManageURL(request), response);
		} else {
			displayNewStoreForm(request, pw, storeType, servletURL);
		}
		printHtmlFooter(pw);
	}

	private String findServletName() {
		Enumeration<String> names = _pStoreRegistry.storeNames();
		HashSet<String> nameSet = new HashSet<String>();
		while (names.hasMoreElements()) {
			nameSet.add(names.nextElement());
		}
		for (int i = 1; ; i++) {
			String servlet = "/store" + i;
			if (!nameSet.contains(servlet))
				return servlet;
		}
	}

	private void createStore(String storeType, HttpServletRequest request, InitialConfig conf, String repositoryName, String xqueryEngine) throws Exception {
		Class backendClass = Class.forName(storeType);
//		Preferences prefDefault = Preferences.systemNodeForPackage(backendClass);
		Preferences pref = getServletPrefRoot(request).node(PREF_STORES + repositoryName);
		pref.put(PREF_BACKEND, storeType);
		pref.put(PREF_XQUERY, xqueryEngine);
		pref = pref.node(PREF_CONFIGURATION);
		pref.clear();
		Enumeration<String> names = conf.getInitParameterNames();
		while (names.hasMoreElements()) {
			String key = names.nextElement();
			pref.put(key, conf.getInitParameter(key));
		}
		pref.flush();
		Constructor cons = backendClass.getConstructor(new Class[] {InitialConfig.class});
		BackendStore backend = (BackendStore) cons.newInstance(new Object[] {conf}); 
		ProvenanceStore pStore = ProvenanceStoreFactory.newInstance(
				backend, xqueryEngine);
		_pStoreRegistry.registerPlugIn(repositoryName, pStore);
	}
	
	private void printRedirectionForm(PrintWriter pw, String url) {
		pw.println("<form action=\"" + url + "\" method=post name=redirection>");
		pw.println("<input type=hidden name=\"store\" >");
		pw.println("<input type=hidden name=\"action\" >");
		pw.println("<input type=hidden name=\"redr\" value='y' >");
		pw.println("</form>");
		pw.flush();
	}
	private void printSOAPHeader(HttpServletResponse response) throws IOException {
		response.setContentType ("application/soap+xml; charset=utf-8");
		PrintWriter pw = response.getWriter();
		pw.print ("<env:Envelope xmlns:env=\"http://schemas.xmlsoap.org/soap/envelope/\">");
		pw.print ("<env:Header>");
		pw.print ("</env:Header>");
		pw.print ("<env:Body>");
		pw.flush();
	}
	private void printSOAPFooter(HttpServletResponse response) throws IOException {
		PrintWriter pw = response.getWriter();
		pw.print ("</env:Body>");
		pw.print ("</env:Envelope>");
		pw.flush();
	}
	private String getRootURL (String path, String servletURL) {
		if ("/".equals(servletURL)) {
			return path;
		} else {
			String pathURL = path.replace('\\', '/');
			System.out.println("pathURL: " + pathURL);
			int i = pathURL.indexOf(servletURL);
			System.out.println("Paths match at " + i);
			return pathURL.substring(0, i);
		}
	}

	private String rootURL = null;

//	private boolean findFile(String servletURL, OutputStream os) throws IOException {
//	File f = new File (rootURL, servletURL);
//	if (!f.exists()) return false;
//	try {
//	FileInputStream fis = new FileInputStream(f);
//	int chars = fis.available();
//	for (int i = 0; i < chars; i++) {
//	os.write(fis.read());
//	}
//	return true;
//	} catch (FileNotFoundException ex) {
//	ex.printStackTrace();
//	return false;
//	}
//	}

	public void doGet(HttpServletRequest request, HttpServletResponse response)
	{
		if (!initStoresEx) {
			initStores(request);
		}
//		System.out.println("Get: " + request.getQueryString());
//		File f = new File("css/manage.css");

//		System.out.println(f.getAbsolutePath());
		try {
			int i = 1;
			System.out.println((i++) + ": " + request.getContextPath());
			System.out.println((i++) + ": " + request.getServletPath());
			System.out.println((i++) + ": " + request.getPathInfo());
			System.out.println((i++) + ": " + request.getPathTranslated());
			System.out.println((i++) + ": " + request.getQueryString());
			System.out.println((i++) + ": " + request.getLocalName());
			System.out.println((i++) + ": " + request.getLocalAddr());
			System.out.println((i++) + ": " + request.getRemoteAddr());
			System.out.println((i++) + ": " + request.getRemoteHost());
			System.out.println((i++) + ": " + request.getScheme());
			System.out.println((i++) + ": " + request.getServerName());
//			String scheme = request.getScheme(); // http

//			String serverName = request.getServerName(); // hostname.com

//			int serverPort = request.getServerPort(); // 80

//			String contextPath = request.getContextPath(); // /mywebapp

//			String servletPath = request.getServletPath(); // /servlet/MyServlet

//			String queryStr = request.getQueryString();

//			String pathInfo = request.getPathInfo();

//			// Reconstruct original requesting URL

//			String servletURL = scheme + "://" + serverName + ":" + serverPort
//			+ contextPath;
			String servletURL = getServletURL(request);
//			System.out.println(servletPath);
			System.out.println(servletURL);
//			System.out.println(pathInfo);
			if (rootURL == null) {
				rootURL = getRootURL(request.getPathTranslated(), servletURL);
			}

			System.out.println("RootURL: " + rootURL);
			System.out.println("PReServ: Loading Management Page");
			response.setStatus(HttpServletResponse.SC_OK);
//			if (!findFile(servletURL, response.getOutputStream())) {
//			PrintWriter pw = response.getWriter();
//			System.out.println("The management version");

			if (request.getParameter("redr") == null && request.getParameter("action") != null) {
				handleWebManagement(request, response);
			} else {
				printTablePage(getManageURL(request), response);
			}
//			pw.write("<html>");
//			pw.write("<body>");
//			pw.write("<img src=\"" + servletURL + "/PReServLogo.gif\"/>");
//			pw.write("<br>");
//			pw.write("</body>");
//			pw.write("</html>");
//			pw.close();
//			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	private void printRequest(HttpServletRequest request) {
		Map<String, String[]> req = request.getParameterMap();
		Set<String> keys = req.keySet();
		System.out.println("handleWebManagement");
		for (String key: keys) 
		{
			System.out.print(key + " => [");
			for (String val: req.get(key)) {
				System.out.print(val + ", ");
			}
			System.out.println("]");
		}
	}
//	public void handleManagement(InputStream is, PrintWriter pw)
//	throws Exception

//	{

//	int n = is.available();
//	printHtmlHeader(pw);
//	pw.print("Result: ");
//	for (int i = 0; i < n; i++) {
//	pw.print(is.read());
//	}
//	printHtmlFooter(pw);
//	printTablePage(pw);
	// DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	//
	// dbf.setNamespaceAware(true);
	//
	// DocumentBuilder docBuilder = dbf.newDocumentBuilder();
	//
	// Document doc = docBuilder.parse(is);
	//
	//        
	//
	// StringBuffer ackStrBuf = new StringBuffer();
	//
	// NodeList nlToUse = null;
	//
	//        
	//
	// NodeList nl =
	// doc.getDocumentElement().getElementsByTagNameNS(Constants.PMANAGE_NS,
	// "createRepository");
	//
	// boolean createRepository = false;
	//
	// if (nl.getLength() > 0) {
	//
	// createRepository = true;
	//
	// nlToUse = nl;
	//
	// }
	//
	//        
	//
	// nl =
	// doc.getDocumentElement().getElementsByTagNameNS(Constants.PMANAGE_NS,
	// "deleteRepository");
	//
	// boolean deleteRepository = false;
	//
	// if (nl.getLength() > 0) {
	//
	// deleteRepository = true;
	//
	// nlToUse = nl;
	//
	// }
	//
	//        
	//
	// nl =
	// doc.getDocumentElement().getElementsByTagNameNS(Constants.PMANAGE_NS,
	// "listRepositories");
	//
	// boolean list = false;
	//
	// if (nl.getLength() > 0) {
	//
	// list = true;
	//
	// nlToUse = nl;
	//
	// }
	//
	//        
	//
	//        
	//
	// if (createRepository) {
	//
	// boolean created = true;
	//
	// Node createRepositoryNode = nlToUse.item(0);
	//
	// String repositoryName = createRepositoryNode.getChildNodes ().item
	// (0).getNodeValue ();
	//
	// if (repositoryName.indexOf("/") == -1) {
	//
	// created = false;
	//
	// } else {
	//
	// // ProvenanceStore pStore =
	// ProvenanceStoreFactory.newInstance(_storagePath + "/" +
	// repositoryName, _cacheWaitTime);
	//
	// // _pStoreRegistry.registerPlugIn("/" + repositoryName, pStore);
	//
	//                   
	//
	// }
	//
	// ackStrBuf.append("<pm:createRepositoryAck xmlns:pm=\"" +
	// Constants.PMANAGE_NS + "\">");
	//
	// ackStrBuf.append("<pm:repository>");
	//
	// ackStrBuf.append("<pm:repositoryName>");
	//
	// ackStrBuf.append(repositoryName);
	//
	// ackStrBuf.append("</pm:repositoryName>");
	//
	// ackStrBuf.append("</pm:repository>");
	//
	// ackStrBuf.append("<pm:created>");
	//
	// ackStrBuf.append(created);
	//
	// ackStrBuf.append("</pm:created>");
	//
	// ackStrBuf.append("</pm:createRepositoryAck>");
	//
	// }
	//
	//        
	//
	//        
	//
	// if (deleteRepository) {
	//
	// boolean deleted = true;
	//
	//            
	//
	// Element deleteRepositoryNode = (Element)nlToUse.item(0);
	//
	// if (deleteRepositoryNode == null) {
	//
	// System.out.println("The node is null");
	//
	// } else {
	//
	// System.out.println(DOMWriter.writeToString(deleteRepositoryNode,
	// true));
	//
	// }
	//
	//            
	//
	// System.out.println(deleteRepositoryNode.getTextContent());
	//
	// String repositoryName = deleteRepositoryNode.getTextContent();
	//
	// System.out.println("ProvenanceService: Deleting contents of
	// repository: " + repositoryName);
	//
	// ProvenanceStore pStore = _pStoreRegistry.lookup("/" +
	// repositoryName);
	//
	// //_pStoreRegistry.deregisterPlugIn("/" + repositoryName);
	//
	// pStore.delete();
	//
	//                   
	//
	//            
	//
	// ackStrBuf.append("<pm:deleteRepositoryAck xmlns:pm=\"" +
	// Constants.PMANAGE_NS + "\">");
	//
	// ackStrBuf.append("<pm:repository>");
	//
	// ackStrBuf.append("<pm:repositoryName>");
	//
	// ackStrBuf.append(repositoryName);
	//
	// ackStrBuf.append("</pm:repositoryName>");
	//
	// ackStrBuf.append("</pm:repository>");
	//
	// ackStrBuf.append("<pm:deleted>");
	//
	// ackStrBuf.append(deleted);
	//
	// ackStrBuf.append("</pm:deleted>");
	//
	// ackStrBuf.append("</pm:deleteRepositoryAck>");
	//
	// }
	//
	//        
	//
	// if (list) {
	//
	//            
	//
	// ackStrBuf.append("<pm:listRepositoriesAck xmlns:pm=\"" +
	// Constants.PMANAGE_NS + "\">");
	//
	//            
	//
	// Enumeration names = _pStoreRegistry.storeNames();
	//
	//            
	//
	// while (names.hasMoreElements()) {
	//
	// ackStrBuf.append("<pm:repository>");
	//
	// ackStrBuf.append("<pm:repositoryName>");
	//
	// ackStrBuf.append(names.nextElement());
	//
	// ackStrBuf.append("</pm:repositoryName>");
	//
	// ackStrBuf.append("</pm:repository>");
	//
	// }
	//
	//            
	//
	// ackStrBuf.append("</pm:listRepositoriesAck>");
	//
	// }
	//
	//
	//
	//        
	//
	//
	//
	// Document responseDoc = docBuilder.parse (new InputSource (new
	// StringReader (ackStrBuf.toString ())));
	//
	// DOMWriter domWriter = new DOMWriter(os);
	//
	// domWriter.write(responseDoc);

//	}
	public void destroy()
	{
		try {
			System.out.println("Calling dispatcher shutdown");
			_pStoreRegistry.shutdownAll();
		} catch (Exception e) {
			System.out.println("PReServ: Problems during shutdown");
			e.printStackTrace();
		}
	}
	/*private void processManageSetString (String queryStr) throws Exception
	    {
	        // Example query string
	        // backend=berkeleydb&storepath=%2FUsers%2Fpgroth%2FDevelop%2FStore&cachepausetime=1000
	        String decodedQueryStr = URLDecoder.decode (queryStr, "UTF-8");
	        System.out.println("Decoded String: " + decodedQueryStr);
	        // Example query string after decoding
	        // backend=berkeleydb&storepath=/Users/pgroth/Develop/store&cachepausetime=200
	        StringTokenizer st = new StringTokenizer (decodedQueryStr, "&");
	        // Get the backend store
	        String backend = st.nextToken();
	        int index = backend.indexOf("=");
	        backend = backend.substring(index+1);
	        System.out.println (backend);
	        // Get the store path
	        String storePath = st.nextToken();
	        index = storePath.indexOf("=");
	        storePath = storePath.substring(index+1);
	        System.out.println (storePath);
	        // Get the store path
	        String cachePauseTime = st.nextToken();
	        index = cachePauseTime.indexOf("=");
	        cachePauseTime = cachePauseTime.substring(index+1);
	        System.out.println (cachePauseTime);
	        String oldStoragePath = _storagePath;
	        if (!(storePath.equals(""))) {
	            _storagePath = storePath;
	        }
	        if (backend.equals ("memory")) {
	            if (!(_backend instanceof MemoryStorage)) {
	                _backend = new MemoryStorage();
	            }
	        }
	        else if (backend.equals ("berkeleydb")) {
	            if (!(_backend instanceof BerkeleyDBJEStorage)) {
	                _backend = new BerkeleyDBJEStorage (_storagePath);
	            } 
	            else if (!(_storagePath.equals (oldStoragePath))) {
	            }
	        }
	    }*/
	/* private void generateManagementHTML()
	    {
	        if (servletPath.equalsIgnoreCase("/management")) {
	            if (queryStr != null) {
	                processManageSetString (queryStr);
	            }
	            System.out.println ("PReServ: Loading Management Page");
	            response.setStatus (HttpServletResponse.SC_OK);
	            response.setContentType("text/html;charset=UTF-8");
	            PrintWriter pw = response.getWriter();
	            pw.write("<html>");
	            pw.write("<body>");
	            pw.write("<img src=\"" + servletURL + "/PReServLogo.gif\"/>");
	            pw.write("<br>");
	            pw.write("<b>Manage</b>");
	            String action = "management";
	            pw.write("<FORM name=\"input\" action=\"" + action + "\" method=\"get\">");
	            pw.write("<P>");
	            if (_backend instanceof MemoryStorage) {
	                pw.write("<input type=\"radio\" checked name=\"backend\" value=\"memory\"> In-Memory Store");
	            }
	            else {
	                pw.write("<input type=\"radio\" name=\"backend\" value=\"memory\"> In-Memory Store");
	            }
	            pw.write("<br>");
	            if (_backend instanceof BerkeleyDBJEStorage) {
	                pw.write("<input type=\"radio\" checked name=\"backend\" value=\"berkeleydb\">Java Object Database");
	            } 
	            else {
	                pw.write("<input type=\"radio\" name=\"backend\" value=\"berkeleydb\">Java Object Database");
	            }
	            pw.write("<br>");
	            pw.write("<LABEL for=\"path\">Store Path: </LABEL>");
	            pw.write("<INPUT type=\"text\" name=\"storepath\" value=\"" + _storagePath  + "\"><BR>");
	            pw.write("<br>");
	            pw.write("<LABEL for=\"processTime\">Pause time before processing cached items (in milliseconds): </LABEL>");
	            pw.write("<INPUT type=\"text\" name=\"cachepausetime\" value=\"" + _dispatcher.getProcessCacheItemsWaitTime() + "\"><BR>");
	            pw.write("<INPUT type=\"submit\" value=\"Submit\">");
	            pw.write("</P></FORM>");
	            pw.write("</body>");
	            pw.write("</html>");
	            pw.close();
	            // Things to set
	            // - which backend
	            // - path for preserv to put stuff at
	            // - available plug ins
	            // - whether a plug in uses caching
	            // - time when the thread runs to process cached items
	        }
	    }*/
	public void doPost (HttpServletRequest request, HttpServletResponse response) throws ServletException
	{
		if (!initStoresEx) {
			initStores(request);
		}

		//System.out.println ("Post");
		try {
			int i = 1;
			System.out.println((i++) + ": " + request.getContextPath());
			System.out.println((i++) + ": " + request.getServletPath());
			System.out.println((i++) + ": " + request.getPathInfo());
			System.out.println((i++) + ": " + request.getPathTranslated());
			System.out.println((i++) + ": " + request.getQueryString());
			System.out.println((i++) + ": " + request.getLocalName());
			System.out.println((i++) + ": " + request.getLocalAddr());
			System.out.println((i++) + ": " + request.getRemoteAddr());
			System.out.println((i++) + ": " + request.getRemoteHost());
			System.out.println((i++) + ": " + request.getScheme());
			System.out.println((i++) + ": " + request.getServerName());

			/*String store = request.getServletPath();
			System.out.println ("PReServ: Store " + store);
			String servlet = request.getPathInfo(); 
			System.out.println ("PReServ: Servlet " + servlet);
			 */

			// Parsing string that looks like /store/operation
			// where store is optional

			String path = request.getPathInfo();
			if (path.endsWith("/")) {
				path = path.substring(0, path.length()-1);
			}
			String[] paths = path.split("/");

			String servlet = "/" + paths[paths.length-1];
			String store = "/default";
			if (paths.length == 3) {
				store = "/" + paths[1];
			} else {

				System.out.println ("PReServ: No store given");
				System.out.println ("PReServ: Using default store");
			}

			System.out.println("Store: " + store);
			System.out.println("Operation: " + servlet);
			boolean compression = false;
			String compressionStr = request.getHeader("Content-Encoding");
			//System.out.println(compressionStr);
			if (compressionStr != null && compressionStr.equalsIgnoreCase("gzip")) {
				//System.out.println("Compression is on");
				compression = true;
			}
			response.setStatus (HttpServletResponse.SC_OK);
			response.setContentType ("application/soap+xml; charset=utf-8");
			PrintWriter pw = new PrintWriter (response.getOutputStream());
			pw.print ("<env:Envelope xmlns:env=\"http://schemas.xmlsoap.org/soap/envelope/\">");
			pw.print ("<env:Header>");
			pw.print ("</env:Header>");
			pw.print ("<env:Body>");
			pw.flush();
			if (servlet.equalsIgnoreCase("/manage")) {
				System.out.println ("Handle Management");
				this.handlePOSTManagement(request, response.getOutputStream());
			}
			else {
				try {
					ProvenanceStore pStore = _pStoreRegistry.lookup(store);
					pStore.getDispatcher().processAndDispatch(request.getInputStream(), 
							response.getOutputStream(), 
							servlet, 
							compression);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			pw.print ("</env:Body>");
			pw.print ("</env:Envelope>");
			pw.close();
			System.out.println("Replied");
			/*
	           Enumeration attributes = request.getHeaderNames();
	           while (attributes.hasMoreElements()) {
	               String headerName = (String)attributes.nextElement();
	               System.out.println(headerName);
	               System.out.println(request.getHeader(headerName));
	           }
	           System.out.println ("user " + request.getRemoteUser());
	           System.out.println("AuthType " + request.getAuthType());
	           System.out.println ("protocol " + request.getProtocol());
	           if (request.isSecure()) {
	               System.out.println ("Request is secure");
	           }
	           String cipherSuite = (String) request.getAttribute("javax.servlet.request.cipher_suite");
	           System.out.println("Cipher Suite: " + cipherSuite);
	           if (cipherSuite != null) {
	                //System.out.println(request.getAttribute("org.apache.coyote.request.X509Certificate"));
	           }
			 */
		} catch (Exception e) {
			e.printStackTrace();
			throw new ServletException (e);
		}
	}

	private void handlePOSTManagement(HttpServletRequest request, OutputStream out) throws Exception{
		InputStream in = request.getInputStream();
		System.out.println("handlePOSTManagement \n");
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setNamespaceAware(true);
		DocumentBuilder docBuilder = dbf.newDocumentBuilder();
		Document doc = docBuilder.parse(in);
		Element el = doc.getDocumentElement();
		StringBuffer ackStrBuf = new StringBuffer();

		boolean success = false;
		String content = null;
		String operation = "not known";

		// Delete        
		NodeList nl = el.getElementsByTagNameNS(Constants.PMANAGE_NS, "deleteRepository");
		if (nl.getLength() > 0) {
			operation = "deleteRepository";

			Element deleteRepositoryNode = (Element)nl.item(0);

			if (deleteRepositoryNode == null) {
				System.out.println("The node is null");
			} else {
				String store = unwrapRepository(deleteRepositoryNode);
				if (store != null) {
					ProvenanceStore pstore = _pStoreRegistry.lookup(store);
					if (pstore != null) {
						pstore.delete();
						success = true;
					} else {
						System.out.println("Store not found");
					}
				}
				else System.out.println("StoreName == null");
			}
		}
		// Remove
		nl = el.getElementsByTagNameNS(Constants.PMANAGE_NS, "removeRepository");
		if (nl.getLength() > 0) {
			operation = "removeRepository";
			Element repositoryNode = (Element)nl.item(0);

			if (repositoryNode == null) {
				System.out.println("The node is null");
			} else {
				String store = unwrapRepository(repositoryNode);
				if (store != null) {
					removeStore(store, request);
					success = true;
				}
				else System.out.println("StoreName == null ");
			}
		}
		// Contents
		nl = el.getElementsByTagNameNS(Constants.PMANAGE_NS, "contentsOfRepository");
		if (nl.getLength() > 0) {
			operation = "contentsOfRepository";
			Element repositoryNode = (Element)nl.item(0);

			if (repositoryNode == null) {
				System.out.println("The node is null");
			} else {
				String store = unwrapRepository(repositoryNode);
				if (store != null) {
					try {
						ProvenanceStore pstore = _pStoreRegistry.lookup(store);
						if (pstore != null) {
							content = DOMWriter.writeToString(xquery("$ps:pstruct", pstore.getDispatcher()));
							success = true;
						} else {
							System.out.println("Store not found");
						}
					} catch (Exception ex) {
						ex.printStackTrace();
						System.out.println("XQuery could not be performed");
					}
				}
				else System.out.println("StoreName == null ");
			}
		}

		// List
		nl = el.getElementsByTagNameNS(Constants.PMANAGE_NS, "listRepositories");
		if (nl.getLength() > 0) {
			operation = "listRepositories";
			Enumeration<String> names = _pStoreRegistry.storeNames();
			while (names.hasMoreElements()) {
				content += wrapRepository(names.nextElement());
			}
			success = true;
		}

		// Create
		nl = el.getElementsByTagNameNS(Constants.PMANAGE_NS, "createRepository");
		if (nl.getLength() > 0) {
			operation = "createRepository";
			Element repositoryNode = (Element)nl.item(0);

			if (repositoryNode == null) {
				System.out.println("The node is null");
			} else {
				String store = unwrapRepository(repositoryNode);
				if (store != null) {
					String xqueryEngine = repositoryNode.getElementsByTagNameNS(Constants.PMANAGE_NS, "xqueryEngine").item(0).getTextContent();
					String className = repositoryNode.getElementsByTagNameNS(Constants.PMANAGE_NS, "className").item(0).getTextContent();
					Element options = (Element)repositoryNode.getElementsByTagNameNS(Constants.PMANAGE_NS, "initialConfig").item(0);
					NodeList pairs = options.getElementsByTagNameNS(Constants.PMANAGE_NS, "pair");
					int length = pairs.getLength();
					DefaultInitialConfigImpl conf = new DefaultInitialConfigImpl();
					for (int i = 0; i < length; i++) {
						String key = ((Element)pairs.item(i)).getElementsByTagNameNS(Constants.PMANAGE_NS, "key").item(0).getTextContent();
						String value = ((Element)pairs.item(i)).getElementsByTagNameNS(Constants.PMANAGE_NS, "value").item(0).getTextContent();
						conf.setInitParameter(key, value);
//						System.out.println("key: " + key);
//						System.out.println("value: " + value);
//						System.out.println("---------------");
					}
//					System.out.println("className: " + className);
//					System.out.println("store: " + store);
//					System.out.println("xqueryEngine: " + xqueryEngine);
					createStore(className, request, conf, store, xqueryEngine);
					success = true;
				}
				else System.out.println("StoreName == null");
			}
		}
		String soap = "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">" +
		"<soapenv:Header>" +
		"</soapenv:Header>" + 
		"<soapenv:Body>" +
		"<pm:managementResponse xmlns:pm=\"" + Constants.PMANAGE_NS + "\">" +
		"<pm:success>" + success + "</pm:success>" +
		"<pm:operation>" + operation + "</pm:operation>" +
		"<pm:content>" + content + "</pm:content>" +
		"</pm:managementResponse>" + 
		"</soapenv:Body>" +
		"</soapenv:Envelope>";
//		System.out.println(soap);
		Document responseDoc = docBuilder.parse (new InputSource (new StringReader ( soap )));
        DOMWriter domWriter = new DOMWriter(out);
        domWriter.write(responseDoc);
	}

	private String unwrapRepository(Element el) {
		Node storeNameEl = el.getElementsByTagNameNS(Constants.PMANAGE_NS, "repositoryName").item(0);
		if (storeNameEl == null) {
			System.out.println("repositoryName null");
			return null;
		} else {
			return storeNameEl.getTextContent();
		}
	}
	
	private String wrapRepository(String name) {
		return "<pm:repository>" +
				"<pm:repositoryName>" +
				name +
				"</pm:repositoryName>" +
				"</pm:repository>";
	}
}
