package org.pasoa.preserv.storage;

import java.io.InputStream;
import java.io.OutputStream;
import org.pasoa.dommap.UnderlyingModelException;
import org.pasoa.preserv.index.DefaultIndex;
import org.pasoa.preserv.index.Index;
import org.pasoa.preserv.record.CacheableItem;
import org.pasoa.preserv.index.MyIndex;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.preserv.xquery.QueryProblem;
import org.pasoa.preserv.xquery.XQueryProcessor;
import org.pasoa.preserv.xquery.XQueryResolver;
import org.pasoa.pstructure.dom.PStructureJavaModel;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class GeneralStorageSystem implements StorageSystem {
    private MyIndex        _index;
    private BackendStore   _store;
    private XQueryResolver _engine;
    
    public GeneralStorageSystem (BackendStore store, String resolverClass) {
        _store = store;
        if (_store instanceof MyIndex) {
            _index = (MyIndex) _store;
        } else {
            _index = new DefaultIndex (_store);
        }
        try {
            _engine = (XQueryResolver) Class.forName (resolverClass).newInstance ();
        } catch (Exception problem) {
            throw new RuntimeException ("Fatal error: " + problem);
        }
    }
    
    public void record (GlobalPAssertionKey gpid, Element asserter, Element assertion,
            String kindOfAssertion) throws Exception {
        _store.record (gpid, asserter, assertion, kindOfAssertion);
    }
    
    public Document resolve (String query) throws QueryProblem {
        try {
            return XQueryProcessor.resolve (query, PStructureJavaModel.getPStructureDocument (_store), _engine);
        } catch (UnderlyingModelException ex) {
            throw new QueryProblem ("Problem in accessing database for XQuery", ex);
        }
    }

    public void resolve (String query, OutputStream output) throws QueryProblem {
        try {
            XQueryProcessor.resolve (query, PStructureJavaModel.getPStructureDocument (_store), _engine, output);
        } catch (UnderlyingModelException ex) {
            throw new QueryProblem ("Problem in accessing database for XQuery", ex);
        }
    }
   
    public boolean cacheEnabled () {
        return _store.cacheEnabled ();
    }
    
    public void cache (InputStream is, String context, boolean compression) throws Exception
    {
        _store.cache(is, context, compression);    
    }
    
    public void cache (CacheableItem ci) throws Exception {
        _store.cache (ci);
    }
    
    public CacheableItem getCachedItem () throws Exception {
        return _store.getCachedItem ();
    }
    
    
    public MyIndex getIndexInterface () {
        return _index;
    }
    
    public Retrieval getRetrievalInterface () {
        return _store;
    }
    
    public void shutdown () throws Exception {
        _store.shutdown ();
    }
    
    public void delete () throws Exception {
        _store.delete ();
    }
    
    public void cacheShutdown() {
        _store.cacheShutdown();
    }
    
    public BackendStore getBackend()
    {
        return _store;
    }
}
