package org.pasoa.preserv.storage.je;

import com.sleepycat.bind.EntryBinding;
import com.sleepycat.bind.serial.SerialBinding;
import com.sleepycat.bind.serial.StoredClassCatalog;
import com.sleepycat.bind.tuple.LongBinding;
import com.sleepycat.bind.tuple.StringBinding;
import com.sleepycat.bind.tuple.TupleBinding;
import com.sleepycat.je.Cursor;
import com.sleepycat.je.Database;
import com.sleepycat.je.DatabaseConfig;
import com.sleepycat.je.DatabaseEntry;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.DeadlockException;
import com.sleepycat.je.Environment;
import com.sleepycat.je.EnvironmentConfig;
import com.sleepycat.je.LockMode;
import com.sleepycat.je.OperationStatus;
import com.sleepycat.je.SecondaryConfig;
import com.sleepycat.je.SecondaryCursor;
import com.sleepycat.je.SecondaryDatabase;
import com.sleepycat.je.Sequence;
import com.sleepycat.je.SequenceConfig;
import com.sleepycat.je.Transaction;
import com.sleepycat.util.FastInputStream;
import com.sleepycat.util.FastOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.util.LinkedList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.pasoa.preserv.index.Index;
import org.pasoa.preserv.index.IndexProblem;
import org.pasoa.preserv.record.CacheShutdownException;
import org.pasoa.preserv.record.CacheableItem;
import org.pasoa.common.Constants;
import org.pasoa.preserv.index.MyIndex;
import org.pasoa.preserv.xquery.QueryProblem;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.ExposedInteractionMetadata;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.RelationshipPAssertion;
import org.pasoa.pstructure.View;
import org.pasoa.pstructure.PAssertion;
import org.pasoa.pstructure.deserialiser.PStructureDeserialiser;
import org.pasoa.pstructure.serialiser.PStructureSerialiser;
import org.pasoa.preserv.storage.BackendStore;
import org.pasoa.preserv.storage.InitialConfig;
import org.pasoa.preserv.storage.NoInteractionRecordForKeyProblem;
import org.pasoa.preserv.storage.UnhandledAssertionTypeProblem;
import org.pasoa.preserv.storage.UnhandledViewProblem;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
 
public class BerkeleyDBJEStorage2 implements BackendStore, Index, MyIndex
{
    public static final String HOME_DIR_CONFIG_KEY             = "StorePath";
    public static final String NO_SYNC_CONFIG_KEY              = "BDBJENoSyncToDisk";
    public static final String USE_COMPRESSION_KEY             = "UseCompression";
    public static final String TOTAL_NUM_RECORDS_KEY           = "totalnumberofrecordskey";
    public static final String PA_SEQUENCE_KEY                 = "pasequencekey";
    public static final String INTERACTION_KEY_DB_NAME         = "interactionkeydb";
    public static final String GPAK_TO_INTERACTION_KEY_DB_NAME = "gpakToIKDB";
    public static final String CACHE_DB_NAME                   = "cache";
    public static final String PASSERTION_DB_NAME              = "passertionDB";
    public static final String INTERACTION_RECORD_DB_NAME      = "interactionrecordDB";
    public static final String RECORD_COUNTER_DB_NAME          = "countDb";
    public static final String ASSERTER_DB_NAME                = "asserterDB";
    public static final String SUBMISSION_FINISHED_DB_NAME     = "submissionFinishedDB";
    public static final String INTERACTION_METADATA_DB_NAME    = "interactionMetadataDB";
    public static final String CACHE_KEY_STR                   = "cache";
    
    private static int INT_PA = 1;
    private static int AS_PA  = 2;
    private static int REL_PA = 3;
    private static int SF     = 4;
    private static int IMDATA = 5;
    
    private static int SENDER_VIEW = 1;
    private static int RECEIVER_VIEW = 2;
    
    private Database          _cacheDB;               // the cache database
    private Database          _paDB;                  // map from a light gpak with p-assertion kind to a p-assertion
    private Database          _irDB;                  // map from a local interaction key to a normalised interaction key
    private Database          _countDB;               // a count of all interaction records as identified by interaction keys
    private Database          _asserterDB;            // map from local view key to an asserter 
    private Database          _subFinishedDB;         // map from local view key to submission finished data
    private Database          _interactionMetadataDB; // map from local view key to interaction metadata
    private SecondaryDatabase _ikDB;                  // map from a normalised interaction key to a local interaction key
    private SecondaryDatabase _ikToPADB;              // map from a local interaction key to p-assertions
    private Environment       _theEnvironment;
    
    private File              _homeDir;
    private DatabaseEntry     _theCacheKey;
    
    private ByteArrayCachableItemBinding _cacheItemBinding;
    
    private EntryBinding      _elementBinding; 
    private EntryBinding      _interactionKeyBinding;
    private EntryBinding      _gpakLightBinding;
    private EntryBinding      _localViewKeyBinding;
    
    private PrintWriter       _entrySizeLog;
    private PrintWriter       _recordTimeLog;
    
    private PStructureDeserialiser    _psd;
    private PStructureSerialiser      _pss;
    private DocumentBuilder           _domBuilder;
    private boolean                   _sync;
    private boolean                   _cacheStop;
    private boolean                   _compressed;
     

    public BerkeleyDBJEStorage2 (InitialConfig config) throws DatabaseException, UnsupportedEncodingException, ParserConfigurationException 
    {
        System.out.println("Creating BBerkeleyDBJEStorage2 using InitialConfig");
        String home = config.getInitParameter(HOME_DIR_CONFIG_KEY);
        if (home == null) {
            System.out.println("Must configure the " + HOME_DIR_CONFIG_KEY +  " parameter");
                   
            throw new NullPointerException ("No " + HOME_DIR_CONFIG_KEY +  " parameter");
        }
        String noSyncToDisk = config.getInitParameter(NO_SYNC_CONFIG_KEY);
        boolean synch = false;
        if (noSyncToDisk != null) {
            synch = Boolean.getBoolean(noSyncToDisk);
        }
        String  useCompression = config.getInitParameter (USE_COMPRESSION_KEY);
        boolean compressed     = true;
        if (useCompression != null) {
            compressed = Boolean.valueOf (compressed);
        }

        File homeDir = new File (home);
        if (!homeDir.exists ()) {
             homeDir.mkdir ();
        }
        
        this.init(homeDir, synch, compressed);
    }
    
    public BerkeleyDBJEStorage2 (String home, boolean compress) throws DatabaseException, UnsupportedEncodingException, ParserConfigurationException 
    {
        this (home, false, compress);
    }
    
    /**
     * Constructor
     * @param home  home directory where the database stores its files
     * @param sync  true if you want the database NOT to guarantee writes to the database after every commit
     * @throws DatabaseException
     * @throws UnsupportedEncodingException
     * @throws ParserConfigurationException
     */
    public BerkeleyDBJEStorage2 (String home, boolean noSyncToDisk, boolean compress) 
        throws DatabaseException, UnsupportedEncodingException, ParserConfigurationException 
    {
        File homeDir = new File (home);
        if (!homeDir.exists ()) {
             homeDir.mkdir ();
        }
        
        this.init(homeDir, noSyncToDisk, compress);
            
    
    }
    
    private void init (File homeDir, boolean sync, boolean compress) 
        throws DatabaseException, UnsupportedEncodingException, ParserConfigurationException 
    {
        _entrySizeLog  = null;
        _recordTimeLog = null;
        _homeDir = homeDir;
        _sync = sync;
        _cacheStop = false;
        _compressed = compress;
        
        // Create the bindings we will use, these convert various objects into byte arrays
        _elementBinding        = new DomElementBinding(compress);
        _interactionKeyBinding = new InteractionKeyBinding();
        _cacheItemBinding      = new ByteArrayCachableItemBinding();
        _gpakLightBinding      = new GPAKLightBinding();
        _localViewKeyBinding   = new LocalViewKeyBinding();
      
        // When setting up the cache we also define the key used to store and retrieve
        // cached items
        _theCacheKey = new DatabaseEntry (CACHE_KEY_STR.getBytes ("UTF-8"));
        
        
        // Open the database environment
        EnvironmentConfig envConfig = new EnvironmentConfig ();
        envConfig.setAllowCreate (true);
        envConfig.setTransactional (true);
        envConfig.setTxnWriteNoSync(sync);
        _theEnvironment = new Environment (homeDir, envConfig);
        
        // Open the databases we are using 
        DatabaseConfig cacheConfig = new DatabaseConfig ();
        cacheConfig.setAllowCreate (true);
        cacheConfig.setSortedDuplicates (true);
        cacheConfig.setTransactional (true);
       
        Transaction cacheDbTxn = _theEnvironment.beginTransaction (null, null);
        _cacheDB = _theEnvironment.openDatabase (cacheDbTxn, CACHE_DB_NAME, cacheConfig);
        cacheDbTxn.commit ();
        
      
        
        DatabaseConfig dbConfig = new DatabaseConfig ();
        dbConfig.setAllowCreate (true);
        dbConfig.setTransactional (true);

        Transaction psTxn = _theEnvironment.beginTransaction (null, null);
        _paDB = _theEnvironment.openDatabase (psTxn, PASSERTION_DB_NAME, dbConfig);
        psTxn.commit ();
        
        Transaction irTxn = _theEnvironment.beginTransaction (null, null);
        _irDB = _theEnvironment.openDatabase (irTxn, INTERACTION_RECORD_DB_NAME, dbConfig);
        irTxn.commit ();
        
        Transaction countTxn = _theEnvironment.beginTransaction (null, null);
        _countDB = _theEnvironment.openDatabase (countTxn, RECORD_COUNTER_DB_NAME, dbConfig);
        countTxn.commit ();
        
        Transaction asserterTxn = _theEnvironment.beginTransaction (null, null);
        _asserterDB = _theEnvironment.openDatabase (asserterTxn, ASSERTER_DB_NAME, dbConfig);
        asserterTxn.commit ();

        Transaction subTxn = _theEnvironment.beginTransaction (null, null);
        _subFinishedDB = _theEnvironment.openDatabase (subTxn, SUBMISSION_FINISHED_DB_NAME, dbConfig);
        subTxn.commit();
        
        dbConfig.setSortedDuplicates(true);
        Transaction imTxn = _theEnvironment.beginTransaction (null, null);
        _interactionMetadataDB = _theEnvironment.openDatabase (imTxn, INTERACTION_METADATA_DB_NAME, dbConfig);
        imTxn.commit();
        
        //Open Secondary databases, i.e. other views of the main database
        IKSecondaryKeyCreator ikskc = new IKSecondaryKeyCreator (_interactionKeyBinding);
        SecondaryConfig irSecConfig = new SecondaryConfig ();
        irSecConfig.setKeyCreator (ikskc);
        irSecConfig.setAllowCreate (true);
        irSecConfig.setTransactional (true);
        
        Transaction secDbTxn = _theEnvironment.beginTransaction (null, null);
        _ikDB = _theEnvironment.openSecondaryDatabase (secDbTxn, INTERACTION_KEY_DB_NAME, _irDB, irSecConfig);
        secDbTxn.commit ();
        
        GPAKSecondaryKeyCreator gskc = new GPAKSecondaryKeyCreator ();
        irSecConfig = new SecondaryConfig ();
        irSecConfig.setKeyCreator (gskc);
        irSecConfig.setAllowCreate (true);
        irSecConfig.setTransactional (true);
        irSecConfig.setSortedDuplicates(true);
        
        secDbTxn = _theEnvironment.beginTransaction (null, null);
        _ikToPADB = _theEnvironment.openSecondaryDatabase (secDbTxn, GPAK_TO_INTERACTION_KEY_DB_NAME, _paDB, irSecConfig);
        secDbTxn.commit ();
     
       
        // create the various serialisers and deserialisers necessary
        _psd = new PStructureDeserialiser();
        _pss = new PStructureSerialiser();
        
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        _domBuilder = dbf.newDocumentBuilder();
        
    }
    
    public void record (GlobalPAssertionKey gpak, Element asserter, Element assertion, String kindOfAssertion) 
        throws DatabaseException, UnsupportedEncodingException, UnhandledViewProblem, UnhandledAssertionTypeProblem
    {
        // Timing information for tests
        long startTime = System.currentTimeMillis(); 
        
        // Get the local interaction key (localIK) for this interaction key from the database
        // create it if it doesn't already exist
        long localIK = acquireLocalInteractionKey(gpak.getInteractionKey());
        
        // Get the local assertion type and view kind
        int localViewKind = getLocalViewKind(gpak.getViewKind());
        int assertionType = getAssertionType(kindOfAssertion);
        
        // Based on the assertion type record the correct data
        if (assertionType == SF) { 
            recordSubmissionFinished(localIK, localViewKind, assertion);
        }
        else if (assertionType == IMDATA) {
            recordInteractionMetaData(localIK, localViewKind, assertion);
        }
        else {
            recordPAssertion(localIK, localViewKind, gpak.getLocalPAssertionID(), assertionType, assertion);
        }
        
        // Check to see if the asserter already exists for this view
        // as identified by a local view key. If it does NOT add
        // the given asserter
        recordAsserter(asserter, localIK, localViewKind);
        
        // Print out timing information
        long endTime = System.currentTimeMillis();
        if (_recordTimeLog != null) {
            _recordTimeLog.print("" + (endTime - startTime));
        }
    }
    
    public InteractionRecord getInteractionRecord (String normalisedInteractionKey) 
        throws DatabaseException, NoInteractionRecordForKeyProblem, QueryProblem, SAXException, IOException
    {
        DatabaseEntry ikKey = new DatabaseEntry();
        StringBinding.stringToEntry(normalisedInteractionKey, ikKey);
        DatabaseEntry irNum = new DatabaseEntry();
        DatabaseEntry data  = new DatabaseEntry();

        Transaction txn = _theEnvironment.beginTransaction(null, null);
        try {
            if (_ikDB.get(txn, ikKey, irNum, data, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
               txn.commit();
            } else {
                txn.commit();
                throw new NoInteractionRecordForKeyProblem ("BerkeleyDBJEStorage2: No interaction record for the given normalised interaction key");
            }
        } catch (DatabaseException dbe) {
            txn.abort();
            throw dbe;
        }

        return getInteractionRecord (irNum);
     }
    
    public InteractionRecord getInteractionRecord (long index) throws Exception 
    {
        DatabaseEntry irNum  = new DatabaseEntry ();
        LongBinding.longToEntry(index, irNum);
        return getInteractionRecord (irNum);
        
    }
    
    public long getNumberOfInteractionRecords () 
        throws DatabaseException, UnsupportedEncodingException 
    {
        DatabaseEntry totalKey  = new DatabaseEntry (TOTAL_NUM_RECORDS_KEY.getBytes ("UTF-8"));
        DatabaseEntry totalData = new DatabaseEntry ();
        EntryBinding longBinding = TupleBinding.getPrimitiveBinding (Long.class);
        Long total;
        
        Transaction txn = _theEnvironment.beginTransaction (null, null);
        try {
            if (_countDB.get (txn, totalKey, totalData, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
                txn.commit ();
                total = (Long)longBinding.entryToObject (totalData);
            } else {
                txn.commit ();
                total = new Long (0);
            }
        } catch (DatabaseException dbe) {
            txn.abort ();
            throw dbe;
        }
        
        return total.longValue ();
    }
    
    public boolean cacheEnabled () {
        return true;
    }
    
  
    public void cache (CacheableItem ci) 
        throws DatabaseException, Exception
    {
       throw new UnsupportedOperationException();
    }
    
    public void cache (InputStream is, String context, boolean compression) throws IOException, DatabaseException {
        FastOutputStream baos = new FastOutputStream();
        byte[] buf = new byte[4*1024];
        int numRead;
        while ( (numRead = is.read(buf) ) >= 0) {
            baos.write(buf, 0, numRead);
        }
        
        ByteArrayCacheableItem baci = new ByteArrayCacheableItem (baos.toByteArray(), context, compression);
        
        DatabaseEntry theData = new DatabaseEntry ();
        _cacheItemBinding.objectToEntry(baci, theData);
        
        synchronized(this) {
            _cacheDB.put (null, _theCacheKey, theData);
            notifyAll();
        }
        
    }
    
    public CacheableItem getCachedItem ()
        throws DatabaseException, CacheShutdownException
    {
        CacheableItem ci = null;
        
        Cursor cursor = null;
        Transaction txn = null;
        
        while (ci == null) {
            if (_cacheStop) {
                throw new CacheShutdownException();
            }
            try {
                DatabaseEntry foundKey = new DatabaseEntry ();
                DatabaseEntry foundData = new DatabaseEntry ();
                
                synchronized (this) {
                    txn = _theEnvironment.beginTransaction (null, null);
                    cursor = _cacheDB.openCursor (txn, null);
                
                    OperationStatus retCode = cursor.getNext (foundKey, foundData, LockMode.DEFAULT);
                    
                    if (retCode == OperationStatus.SUCCESS && foundData.getSize() > 0) {
                        cursor.delete ();
                        cursor.close ();
                        txn.commit ();
                        ci = (CacheableItem) _cacheItemBinding.entryToObject (foundData);
                    }  
                    else {
                        cursor.close();
                        txn.commit();
                    }
                    
                    
                   if (ci == null) {
                        try {
                           _cacheItemBinding.resetCounter(); //used to support possible duplicate data
                            wait();
                        } catch (InterruptedException ie) {
                            
                        }
                    }
                }
                
            }
            catch (DatabaseException e) {
                txn.abort ();
                throw e;
            }
            
        }
        return ci;
    }
    
    public void cacheShutdown()
    {
        _cacheStop = true;
        synchronized (this) {
            notifyAll();
        }
    }
    
    public void shutdown () throws Exception {
        
        System.out.println("Provenance Store Backend: Shutting down");
        _theEnvironment.sync ();
        
        _ikToPADB.close();
        _ikDB.close();
        _asserterDB.close();
        _paDB.close ();
        _irDB.close();
        _countDB.close ();
        _cacheDB.close ();
        _subFinishedDB.close();
        _interactionMetadataDB.close();
        
        _theEnvironment.cleanLog();
        _theEnvironment.close ();
    }
    
    public void delete () throws Exception 
    { 
        shutdown();
        File[] filesToDelete = _homeDir.listFiles();
        for (int i = 0; i < filesToDelete.length; i++) {
            filesToDelete[i].delete();
        }
        
        this.init(_homeDir, _sync, _compressed);
    }
    
    /**
     * Prints the time it takes to put something to the db
     * when the method record(...) is called. Callers are expected
     * to appropriately clean-up the provided PrintWriter
     * @param output
     */
    public void enableRecordTimeLog (PrintWriter output) 
    {
        _recordTimeLog = output;
    }
    
    public void disableRecordTimeLog()
    {
        _recordTimeLog = null;
    }
    
    
    private int getLocalViewKind(String viewKind)
        throws UnhandledViewProblem
    {
        int localViewKind;
        if (viewKind.equals(Constants.SENDER_VIEW_TYPE)) {
            localViewKind = SENDER_VIEW;
        } else if (viewKind.equals(Constants.RECEIVER_VIEW_TYPE)){
            localViewKind = RECEIVER_VIEW;
        } else {
            throw new UnhandledViewProblem ("BerkeleyDBJEStorage2: Unhandled viewkind " + viewKind);
        }
        
        return localViewKind;
    }
    
    private void recordPAssertion (long localInteractionKey, 
                                   int localViewKind, 
                                   String localPAssertionId, 
                                   int assertionType,
                                   Element assertion)
        throws DatabaseException
    {
        GPAKWithPAssertionKindLight paLocalIK = 
            new GPAKWithPAssertionKindLight(localInteractionKey, localViewKind, localPAssertionId, assertionType);
    
        DatabaseEntry paLocalIKEnry = new DatabaseEntry();
        _gpakLightBinding.objectToEntry(paLocalIK, paLocalIKEnry);
    
        DatabaseEntry paValue = new DatabaseEntry();
        _elementBinding.objectToEntry(assertion, paValue);
    
        Transaction txn = _theEnvironment.beginTransaction(null, null);
        try {
            _paDB.put (txn, paLocalIKEnry, paValue);
            txn.commit ();
        } catch (DatabaseException dbe) {
            txn.abort ();
            throw dbe;
        }
    }
    
    private void recordInteractionMetaData(long localInteractionKey, int localViewKind, Element assertion)
       throws DatabaseException
    {
        LocalViewKey lvk = new LocalViewKey (localInteractionKey, localViewKind);
        DatabaseEntry lvkEntry = new DatabaseEntry();
        _localViewKeyBinding.objectToEntry(lvk, lvkEntry);
        
        DatabaseEntry iMetaDataEntry = new DatabaseEntry();
        _elementBinding.objectToEntry(assertion, iMetaDataEntry);
            
        Transaction txn = _theEnvironment.beginTransaction(null, null);
        try {
            _interactionMetadataDB.put(txn, lvkEntry, iMetaDataEntry);
            txn.commit();
        } catch (DatabaseException dbe) {
            txn.abort ();
            throw dbe;
        }
    }
    
    private void recordSubmissionFinished (long localInteractionKey, int localViewKind, Element assertion)
        throws DatabaseException
    {
        LocalViewKey lvk = new LocalViewKey (localInteractionKey, localViewKind);
        DatabaseEntry lvkEntry = new DatabaseEntry();
        _localViewKeyBinding.objectToEntry(lvk, lvkEntry);
        
        DatabaseEntry sfEntry = new DatabaseEntry();
        
        Transaction txn = _theEnvironment.beginTransaction(null, null);
        try {
            if (_subFinishedDB.get(txn, lvkEntry, sfEntry, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
                txn.commit();
            } else {
                _elementBinding.objectToEntry(assertion, sfEntry);
                _subFinishedDB.put(txn, lvkEntry, sfEntry);
                txn.commit();
            }
        } catch (DatabaseException dbe) {
            txn.abort ();
            throw dbe;
        }
    }
    
    private int getAssertionType(String kindOfAssertion)
        throws UnhandledAssertionTypeProblem
    {
        int assertionType = -1;
        if (kindOfAssertion.equals(Constants.INTERACTION_PASSERTION_ELEMENT_NAME)) {
            assertionType = INT_PA;
        }
        else if (kindOfAssertion.equals(Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME)) {
            assertionType = AS_PA;
        }
        else if (kindOfAssertion.equals(Constants.RELATIONSHIP_PASSERTION_ELEMENT_NAME)) {
            assertionType = REL_PA;
        }
        else if (kindOfAssertion.equals(Constants.SUBMISSION_FINISHED_ELEMENT_NAME)) {
            assertionType = SF;
        }
        else if (kindOfAssertion.equals(Constants.INTERACTION_METADATA_ELEMENT_NAME)) {
            assertionType = IMDATA;
        } 
        else {
            throw new UnhandledAssertionTypeProblem ("BerkeleyDBJEStorage2: Unhandled assertion type " + kindOfAssertion);
        }
        
        return assertionType;
    }
    
    private void recordAsserter(Element asserter, long localInteractionKey, int localViewKind)
        throws DatabaseException
    {
        LocalViewKey lvk = new LocalViewKey (localInteractionKey, localViewKind);
        DatabaseEntry lvkEntry = new DatabaseEntry();
        _localViewKeyBinding.objectToEntry(lvk, lvkEntry);
        
        DatabaseEntry asserterEntry = new DatabaseEntry();
        
        Transaction txn = _theEnvironment.beginTransaction(null, null);
        try {
            if (_asserterDB.get(txn, lvkEntry, asserterEntry, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
                txn.commit();
            } else {
                _elementBinding.objectToEntry(asserter, asserterEntry);
                _asserterDB.put(txn, lvkEntry, asserterEntry);
                txn.commit();
            }
        } catch (DatabaseException dbe) {
            txn.abort ();
            throw dbe;
        }
    }
    
    /**
     * Given an interaction key this makes sure that the corresponding interaction record
     * exists and returns the local interaction key of the record
     * @param interactionKey
     * @return
     * @throws DatabaseException
     * @throws UnsupportedEncodingException
     */
    private long acquireLocalInteractionKey(InteractionKey interactionKey) 
        throws DatabaseException, UnsupportedEncodingException
    {
        DatabaseEntry ikKey = new DatabaseEntry();
        _interactionKeyBinding.objectToEntry(interactionKey, ikKey);
        
        DatabaseEntry irNum = new DatabaseEntry();
        DatabaseEntry dummyValue = new DatabaseEntry();
        Transaction txn = _theEnvironment.beginTransaction(null, null);
        
        boolean addIK = false;
        try {
            if (_ikDB.get(txn, ikKey, irNum, dummyValue, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
                txn.commit();
            } else {
                addIK = true;
                txn.commit();
            }
        } catch (DatabaseException dbe) {
            txn.abort ();
            throw dbe;
        }
        
        // if the interaction key doesn't exist create a new one by updating the total number
        if (addIK) {
            long total;
            DatabaseEntry totalKey  = new DatabaseEntry (TOTAL_NUM_RECORDS_KEY.getBytes ("UTF-8"));
            DatabaseEntry totalData = new DatabaseEntry ();
            
            txn = _theEnvironment.beginTransaction (null, null);
            try {
                
                if (_countDB.get (null, totalKey, totalData, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
                    total = LongBinding.entryToLong(totalData);
                    total = total + 1;
                } else {
                    total = 1;
                }
                
                // Update the count
                LongBinding.longToEntry(total, totalData);
                _countDB.put (txn, totalKey, totalData);
              
                // Create the key for the database entry that is the key
                // for this interaction record
                long primary = total - 1;
                LongBinding.longToEntry(primary, irNum);
         
                _irDB.put (txn, irNum, ikKey);
                txn.commit();
            } catch (DatabaseException dbe) {
                txn.abort ();
                throw dbe;
            }
        }
        
        long localIK = LongBinding.entryToLong(irNum);
        return localIK;
    }
    
    
    private InteractionRecord getInteractionRecord (DatabaseEntry irNumKey) 
         throws DatabaseException, NoInteractionRecordForKeyProblem, QueryProblem, SAXException, IOException 
    {
        DatabaseEntry ikValue = new DatabaseEntry();
        Transaction txn = _theEnvironment.beginTransaction (null, null);
        try {
            if (_irDB.get (txn, irNumKey, ikValue, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
                txn.commit ();
            } else {
                txn.commit ();
                throw new NoInteractionRecordForKeyProblem ("BerkeleyDBJEStorage2: No interaction record at that index");
            }
        } catch (DatabaseException dbe) {
            txn.abort ();
            throw dbe;
        }
        
        // Generate various forms of interaction keys
        long localInteractionKey = LongBinding.entryToLong(irNumKey);
        String normalisedInteractionKey = (String)_interactionKeyBinding.entryToObject(ikValue);
        InteractionKey interactionKey = _psd.deserialiseInteractionKey(toDOM(normalisedInteractionKey));
        
        // Get views
        View senderView   = getView(SENDER_VIEW, localInteractionKey);
        View receiverView = getView(RECEIVER_VIEW, localInteractionKey);
        
        // Get p-assertions
        DatabaseEntry foundPA = new DatabaseEntry ();
        DatabaseEntry foundLocalGPAK = new DatabaseEntry();
        
        Transaction cursorTxn = _theEnvironment.beginTransaction (null, null);
        SecondaryCursor cursor = _ikToPADB.openSecondaryCursor (cursorTxn, null);
        
        OperationStatus retValue = cursor.getSearchKey(irNumKey, foundLocalGPAK, foundPA, LockMode.DEFAULT);
        
        while (retValue == OperationStatus.SUCCESS) {
            GPAKWithPAssertionKindLight gpakat = (GPAKWithPAssertionKindLight) _gpakLightBinding.entryToObject(foundLocalGPAK); 
            Element elem = (Element) _elementBinding.entryToObject(foundPA);
            
            View view = null;
            if (gpakat.getViewKind() == SENDER_VIEW) {
                view = senderView;
            } 
            else if (gpakat.getViewKind() == RECEIVER_VIEW) {
                view = receiverView;
            } 
            
            if (gpakat.getAssertionKind() == INT_PA) {
                InteractionPAssertion ipa = _psd.deserialiseInteractionPAssertion (elem);
                view.getInteractionPAssertions().add(ipa);
            } 
            else if (gpakat.getAssertionKind() == AS_PA) {
                ActorStatePAssertion aspa = _psd.deserialiseActorStatePAssertion(elem);
                view.getActorStatePAssertions().add(aspa);
            } 
            else if (gpakat.getAssertionKind() == REL_PA) {
                RelationshipPAssertion rel = _psd.deserialiseRelationshipPAssertion(elem);
                view.getRelationshipPAssertions().add(rel);
            }
           
            retValue = cursor.getNextDup (irNumKey, foundLocalGPAK, foundPA, LockMode.DEFAULT);
        }
        
        try {
            cursor.close ();
            cursorTxn.commit ();
        } catch (DatabaseException dbe) {
            cursorTxn.abort();
            throw dbe;
        }
        InteractionRecord iRecord = new InteractionRecord(interactionKey, senderView, receiverView);
      
        return iRecord;
    }
    
    private View getView(int viewKind, long localInteractionKey) 
        throws DatabaseException, QueryProblem
    {
        View view = null;
        LocalViewKey viewKey = new LocalViewKey (localInteractionKey, viewKind);
        DatabaseEntry viewKeyEntry = new DatabaseEntry();
        _localViewKeyBinding.objectToEntry(viewKey, viewKeyEntry);
        
        DatabaseEntry asserterEntry = new DatabaseEntry();
        
        Transaction txn = _theEnvironment.beginTransaction(null, null);
        if (_asserterDB.get(txn, viewKeyEntry, asserterEntry, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
            txn.commit();
            Element asserter = (Element)_elementBinding.entryToObject(asserterEntry);
            view  = new View(new LinkedList(), new LinkedList(), new LinkedList(), new LinkedList(), -1, asserter);
        } else {
            txn.commit();
        }
    
        DatabaseEntry sfEntry = new DatabaseEntry();
        
        txn = _theEnvironment.beginTransaction(null, null);
        if (_subFinishedDB.get(txn, viewKeyEntry, sfEntry, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
            txn.commit();
            Element sfElem = (Element)_elementBinding.entryToObject(sfEntry);
            int sf = _psd.deserialiseNumberOfExpectedMsgs(sfElem);
            view.setNumberOfExpectedPAssertions(sf);
        } else {
            txn.commit();
        }
        

        DatabaseEntry imResult = new DatabaseEntry();
        Transaction imCursorTxn = _theEnvironment.beginTransaction (null, null);
        Cursor imCursor = _interactionMetadataDB.openCursor(imCursorTxn, null);
        OperationStatus retValue = imCursor.getSearchKey(viewKeyEntry, imResult, LockMode.DEFAULT);
        while (retValue == OperationStatus.SUCCESS) {
            Element imElem = (Element) _elementBinding.entryToObject(imResult);
            ExposedInteractionMetadata eim = _psd.deserialiseExposedInteractionMetadata(imElem);
           
            view.getExposedInteractionMetadata().add(eim);
            retValue = imCursor.getNextDup (viewKeyEntry, imResult, LockMode.DEFAULT);
        }
        imCursor.close ();
        imCursorTxn.commit ();
        
        return view;
    }
    
    
    public InteractionRecord findInteractionRecord(InteractionKey key) throws IndexProblem {
        String normalisedIK = _pss.normaliseInteractionKey(key);
        try {
            return getInteractionRecord(normalisedIK);
        } catch (Exception e) {
            throw new IndexProblem("BerkeleyDBJEStorage2 couldn't get interaction key", e);
        }
    }

    public PAssertion findPAssertion(GlobalPAssertionKey gpak) throws IndexProblem {
        
        String normalisedIK = _pss.normaliseInteractionKey(gpak.getInteractionKey());
        DatabaseEntry ikKey = new DatabaseEntry();
        StringBinding.stringToEntry(normalisedIK, ikKey);
        DatabaseEntry irNum = new DatabaseEntry();
        DatabaseEntry data  = new DatabaseEntry();
        
        try {
            Transaction txn = _theEnvironment.beginTransaction(null, null);
            try {
                if (_ikDB.get(txn, ikKey, irNum, data, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
                   txn.commit();
                } else {
                    txn.commit();
                    throw new NoInteractionRecordForKeyProblem ("BerkeleyDBJEStorage2: No interaction record for the given normalised interaction key");
                }
            } catch (DatabaseException dbe) {
                txn.abort();
                throw dbe;
            } 
            
            long localIK = LongBinding.entryToLong(irNum);
            int viewKind = 0;
            if (gpak.getViewKind().equals(Constants.SENDER_VIEW_TYPE)) {
                viewKind = SENDER_VIEW;
            }
            else if (gpak.getViewKind().equals(Constants.RECEIVER_VIEW_TYPE)) {
                viewKind = RECEIVER_VIEW;
            }
            
            
            GPAKWithPAssertionKindLight gpakLightIPA = new GPAKWithPAssertionKindLight (localIK, viewKind, gpak.getLocalPAssertionID(), INT_PA);
            GPAKWithPAssertionKindLight gpakLightASPA = new GPAKWithPAssertionKindLight (localIK, viewKind, gpak.getLocalPAssertionID(), AS_PA);
            GPAKWithPAssertionKindLight gpakLightRelPA = new GPAKWithPAssertionKindLight (localIK, viewKind, gpak.getLocalPAssertionID(), REL_PA);
      
            
            DatabaseEntry gpakKey = new DatabaseEntry();
            DatabaseEntry assertionResult  = new DatabaseEntry();
            
            _gpakLightBinding.objectToEntry(gpakLightIPA, gpakKey);
            txn = _theEnvironment.beginTransaction(null, null);
            if (_paDB.get(txn, gpakKey, assertionResult, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
               txn.commit();
               Element elem = (Element)_elementBinding.entryToObject(assertionResult);
               return _psd.deserialiseInteractionPAssertion(elem);
            }
    
            _gpakLightBinding.objectToEntry(gpakLightASPA, gpakKey);
            if (_paDB.get(txn, gpakKey, assertionResult, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
                txn.commit();
                Element elem = (Element)_elementBinding.entryToObject(assertionResult);
                return _psd.deserialiseActorStatePAssertion(elem);
            }
            
            _gpakLightBinding.objectToEntry(gpakLightRelPA, gpakKey);
            if (_paDB.get(txn, gpakKey, assertionResult, LockMode.DEFAULT) == OperationStatus.SUCCESS) {
                txn.commit();
                Element elem = (Element)_elementBinding.entryToObject(assertionResult);
                return _psd.deserialiseRelationshipPAssertion(elem);
            }
            txn.commit ();
        
        } catch (Exception e) {
            throw new IndexProblem ("BerkeleyDBJEStorage2: Failed in obtaining p-assertion for gpak", e);
        }
        return null;
    }

    public View findView(InteractionKey key, String viewKind) throws IndexProblem {
        InteractionRecord ir = findInteractionRecord(key);
        if (viewKind.equals(Constants.SENDER_VIEW_TYPE)) {
            return ir.getSenderView();
        }
        else if (viewKind.equals(Constants.RECEIVER_VIEW_TYPE)) {
            return ir.getReceiverView();
        }
        return null;
    }


    public List findViews (InteractionKey key, String viewKind) throws IndexProblem {
        List views = new LinkedList ();
        
        views.add (findView (key, viewKind));
        
        return views;
    }

    public View findView (InteractionKey key, String viewKind, Element asserter) throws IndexProblem {
        return findView (key, viewKind);
    }
    
    /**
     * Convenience method for converting serialised XML to a DOM element.
     */
    private synchronized Element toDOM (String serialised) throws SAXException, IOException {
        return _domBuilder.parse (new InputSource (new StringReader (serialised))).getDocumentElement ();
    }
}
