/********************************************************************************
 * Created on 21-Jun-2006
 *
 * Copyright (c) 2005 University of Southampton
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE.
 *******************************************************************************/

package org.pasoa.preserv.storage.je;

import java.io.IOException;

import org.pasoa.preserv.record.CacheableItem;

import com.sleepycat.bind.tuple.TupleBinding;
import com.sleepycat.bind.tuple.TupleInput;
import com.sleepycat.bind.tuple.TupleOutput;
import com.sleepycat.util.FastOutputStream;

public class ByteArrayCachableItemBinding extends TupleBinding
{
    private long counter = 0;
    
    public Object entryToObject(TupleInput ti) {
        String  context    = ti.readString();
        Boolean compressed = ti.readBoolean();
        ti.readLong();
        
        FastOutputStream baos = new FastOutputStream();
        byte[] buf = new byte[4*1024];
        int numRead;
        while ( (numRead = ti.readFast(buf) ) >= 0) {
            baos.writeFast(buf, 0, numRead);
        }
        
        
        ByteArrayCacheableItem ci = new ByteArrayCacheableItem (baos.getBufferBytes(), context, compressed);
        return ci;
    }

    public void objectToEntry(Object obj, TupleOutput to) {
        counter++;
        ByteArrayCacheableItem ci = (ByteArrayCacheableItem)obj;
        to.writeString(ci.getContext());
        to.writeBoolean(ci.isCompressed());
        //HACK ALERT: the following is added to support possible duplicate cached items
        to.writeLong(counter);
        
        try {
            to.write(ci.getBytes());
        } catch (IOException io) {
            System.out.println("StreamCachableItemBinding: Unable to output byte array");
            io.printStackTrace();
        }
       
    }
    
    public void resetCounter()
    {
        counter = 0;
    }

}
