package org.pasoa.preserv.storage.je;

import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.serialiser.PStructureSerialiser;
import org.pasoa.simpledom.DOMWriter;
import org.pasoa.simpledom.NormalisingDOMWriter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class PointerInteractionRecord 
{
    private String  _interactionKey;
    private String  _senderAsserter;
    private int     _senderNumPAs;
    
    private long[]  _senderIGPAKs;
    private long[]  _senderASGPAKs;
    private long[]  _senderRelGPAKs;
  
    private String  _receiverAsserter;
    private int     _receiverNumPAs;
    
    private long[]  _receiverIGPAKs;
    private long[]  _receiverASGPAKs;
    private long[]  _receiverRelGPAKs;
  
    
    public PointerInteractionRecord (InteractionKey ik)
    {
        _senderIGPAKs   = new long[0];
        _senderASGPAKs  = new long[0];
        _senderRelGPAKs = new long[0];
        _senderNumPAs   = -1;
        _senderAsserter = null;
        
        _receiverIGPAKs   = new long[0];
        _receiverASGPAKs  = new long[0];
        _receiverRelGPAKs = new long[0];
        _receiverNumPAs   = -1;
        _receiverAsserter = null;
        
        PStructureSerialiser pss = new PStructureSerialiser();
        _interactionKey = pss.normaliseInteractionKey(ik);
        
    }
    
    public PointerInteractionRecord (String interactionKeyAsXML)
    {
        _senderIGPAKs   = new long[0];
        _senderASGPAKs  = new long[0];
        _senderRelGPAKs = new long[0];
        _senderNumPAs   = -1;
        _senderAsserter = null;
        
        _receiverIGPAKs   = new long[0];
        _receiverASGPAKs  = new long[0];
        _receiverRelGPAKs = new long[0];
        _receiverNumPAs   = -1;
        _receiverAsserter = null;
        
        _interactionKey = interactionKeyAsXML;
        
        
    }
    
    public String getInteractionKey()
    {
        return _interactionKey;
    }
    
    public String getSenderAsserter()
    {
        return _senderAsserter;
    }
    
    public int getSenderNumOfExpectedPAssertions()
    {
        return _senderNumPAs;   
    }
    
    public long[] getSenderRelationshipGPAKs()
    {
        return _senderRelGPAKs;   
    }
    
    public long[] getSenderActorStateGPAKs()
    {
        return _senderASGPAKs;
    }
    
    public long[] getSenderInteractionGPAKs()
    {
        return _senderIGPAKs;   
    }
    
    public void setSenderAsserter (String asserter)
    {
        _senderAsserter = asserter;    
    }
    
    public void setSenderNumOfExpectedPAssertions (int num)
    {
        _senderNumPAs = num;
    }
    
    public void setSenderRelationshipGPAKs(long[] gpaks)
    {
        _senderRelGPAKs = gpaks;   
    }
    
    public void setSenderActorStateGPAKs(long[] gpaks)
    {
        _senderASGPAKs = gpaks;
    }
    
    public void setSenderInteractionGPAKs(long[] gpaks)
    {
        _senderIGPAKs = gpaks;   
    }
    
    public void addSenderInteractionGPAK (long gpak) {
        _senderIGPAKs = addLongToArray (gpak, _senderIGPAKs);
    }
    
    public void addSenderActorStateGPAK (long gpak) {
        _senderASGPAKs = addLongToArray (gpak, _senderASGPAKs);
    }
    
    public void addSenderRelationshipGPAK (long gpak) {
        _senderRelGPAKs = addLongToArray (gpak, _senderRelGPAKs);
    }
    
    
    //Sender getter and setters
    
    public String getReceiverAsserter()
    {
        return _receiverAsserter;
    }
    
    public int getReceiverNumOfExpectedPAssertions()
    {
        return _receiverNumPAs;   
    }
    
    public long[] getReceiveRelationshipGPAKs()
    {
        return _receiverRelGPAKs;   
    }
    
    public long[] getReceiverActorStateGPAKs()
    {
        return _receiverASGPAKs;
    }
    
    public long[] getReceiverInteractionGPAKs()
    {
        return _receiverIGPAKs;   
    }
    
    public void setReceiverAsserter (String asserter)
    {
        _receiverAsserter = asserter;    
    }
    
    public void setReceiverNumOfExpectedPAssertions (int num)
    {
        _receiverNumPAs = num;
    }
    
    public void setReceiverRelationshipGPAKs(long[] gpaks)
    {
        _receiverRelGPAKs = gpaks;   
    }
    
    public void setReceiverActorStateGPAKs(long[] gpaks)
    {
        _receiverASGPAKs = gpaks;
    }
    
    public void setReceiverInteractionGPAKs(long[] gpaks)
    {
        _receiverIGPAKs = gpaks;   
    }
    
    public void addReceiverInteractionGPAK (long gpak) {
        _receiverIGPAKs = addLongToArray (gpak, _receiverIGPAKs);
    }
    
    public void addReceiverActorStateGPAK (long gpak) {
        _receiverASGPAKs = addLongToArray (gpak, _receiverASGPAKs);
    }
    
    public void addReceiverRelationshipGPAK (long gpak) {
        _receiverRelGPAKs = addLongToArray (gpak, _receiverRelGPAKs);
    }
    
    private long[] addLongToArray (long gpak, long[] original) {
        long[] longs = new long[original.length + 1];
        System.arraycopy(original, 0, longs, 0, original.length);
        longs[original.length] = gpak;
        return longs;
    }
}
