package org.pasoa.preserv.storage.je;

import java.io.StringReader;

import org.pasoa.simpledom.DOMWriter;
import org.xml.sax.InputSource;

import com.sleepycat.bind.tuple.TupleBinding;
import com.sleepycat.bind.tuple.TupleInput;
import com.sleepycat.bind.tuple.TupleOutput;

public class PointerInteractionRecordBinding extends TupleBinding {

    
    
    public Object entryToObject(TupleInput ti) {
        
        
        String interactionKey    = ti.readString();
        
        // read in sender stuff
        String senderAsserter    = null;
        boolean isSenderAsserter = ti.readBoolean();
        
        if (isSenderAsserter) {
            senderAsserter = ti.readString();
        }
        
        int    senderNumOfPAs    = ti.readInt();
        int    numSenderIGPAKs   = ti.readInt();
        int    numSenderASGPAKs  = ti.readInt();
        int    numSenderRelGPAKs = ti.readInt();
        
        long[] senderIGPAKs   = new long[numSenderIGPAKs];
        long[] senderASGPAKs  = new long[numSenderASGPAKs];
        long[] senderRelGPAKs = new long[numSenderRelGPAKs];
        
        for (int i = 0; i < numSenderIGPAKs; i++) {
            senderIGPAKs[i] = ti.readLong(); 
        }
        
        for (int a = 0; a < numSenderASGPAKs; a++) {
            senderASGPAKs[a] = ti.readLong();
        }
        
        for (int r = 0; r < numSenderRelGPAKs; r++) {
            senderRelGPAKs[r] = ti.readLong();
        }
        
        // read in receiver stuff
      
        String receiverAsserter    = null;
        boolean isReceiverAsserter = ti.readBoolean();
        
        if (isReceiverAsserter) {
            receiverAsserter = ti.readString();
        }
        
        int    receiverNumOfPAs    = ti.readInt();
        int    numReceiverIGPAKs   = ti.readInt();
        int    numReceiverASGPAKs  = ti.readInt();
        int    numReceiverRelGPAKs = ti.readInt();
        
        long[] receiverIGPAKs   = new long[numReceiverIGPAKs];
        long[] receiverASGPAKs  = new long[numReceiverASGPAKs];
        long[] receiverRelGPAKs = new long[numReceiverRelGPAKs];
        
        
        for (int i = 0; i < numReceiverIGPAKs; i++) {
           receiverIGPAKs[i] = ti.readLong();
        }
        
        for (int a = 0; a < numReceiverASGPAKs; a++) {
            receiverASGPAKs[a] = ti.readLong();
        }
        
        for (int r = 0; r < numReceiverRelGPAKs; r++) {
            receiverRelGPAKs[r] = ti.readLong();
        }
        
        
        // build object
        PointerInteractionRecord pir = new PointerInteractionRecord(interactionKey);
        pir.setSenderAsserter(senderAsserter);
        pir.setSenderNumOfExpectedPAssertions(senderNumOfPAs);
        pir.setSenderInteractionGPAKs(senderIGPAKs);
        pir.setSenderActorStateGPAKs(senderASGPAKs);
        pir.setSenderRelationshipGPAKs(senderRelGPAKs);
        
        pir.setReceiverAsserter(receiverAsserter);
        pir.setReceiverNumOfExpectedPAssertions(receiverNumOfPAs);
        pir.setReceiverInteractionGPAKs(receiverIGPAKs);
        pir.setReceiverActorStateGPAKs(receiverASGPAKs);
        pir.setReceiverRelationshipGPAKs(receiverRelGPAKs);
        
        return pir;
        
    }

    public void objectToEntry(Object obj, TupleOutput to) {
        PointerInteractionRecord pir = (PointerInteractionRecord)obj;
        
        to.writeString(pir.getInteractionKey());
        
        // Serialise the sender stuff
        
        // Insert a boolean which says whether or not there
        // is a sender asserter
        if (pir.getSenderAsserter() == null) {
            to.writeBoolean(false);
        } else {
            to.writeBoolean(true);
            to.writeString(pir.getSenderAsserter());
        }
        to.writeInt(pir.getSenderNumOfExpectedPAssertions());
        long[] senderIGPAKs = pir.getSenderInteractionGPAKs();
        long[] senderASGPAKs = pir.getSenderActorStateGPAKs();
        long[] senderRelGPAKs = pir.getSenderRelationshipGPAKs();
        
        to.writeInt(senderIGPAKs.length);
        to.writeInt(senderASGPAKs.length);
        to.writeInt(senderRelGPAKs.length);
        
        for (int i = 0; i < senderIGPAKs.length; i++) {
            to.writeLong(senderIGPAKs[i]);
        }
        
        for (int a = 0; a < senderASGPAKs.length; a++) {
            to.writeLong(senderASGPAKs[a]);
        }
        
        for (int r = 0; r < senderRelGPAKs.length; r++) {
            to.writeLong(senderRelGPAKs[r]);
        }
        
        // Serialise receiver stuff
        
        if (pir.getReceiverAsserter() == null) {
            to.writeBoolean(false);
        } else {
            to.writeBoolean(true);
            to.writeString(pir.getReceiverAsserter());
        }
        to.writeInt(pir.getReceiverNumOfExpectedPAssertions());
        long[] receiverIGPAKs = pir.getReceiverInteractionGPAKs();
        long[] receiverASGPAKs = pir.getReceiverActorStateGPAKs();
        long[] receiverRelGPAKs = pir.getReceiveRelationshipGPAKs();
        
        to.writeInt(receiverIGPAKs.length);
        to.writeInt(receiverASGPAKs.length);
        to.writeInt(receiverRelGPAKs.length);
        
        for (int i = 0; i < receiverIGPAKs.length; i++) {
            to.writeLong(receiverIGPAKs[i]);
        }
        
        for (int a = 0; a < receiverASGPAKs.length; a++) {
            to.writeLong(receiverASGPAKs[a]);
        }
        
        for (int r = 0; r < receiverRelGPAKs.length; r++) {
            to.writeLong(receiverRelGPAKs[r]);
        }

    }

}
