package org.pasoa.preserv.storage.mysql;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.EOFException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.sql.Blob;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import org.pasoa.preserv.record.Cache;
import org.pasoa.preserv.record.CacheShutdownException;
import org.pasoa.preserv.record.CacheableItem;
import org.pasoa.preserv.storage.je.CachableItemBinding;

/**
 * 
 * @author Andrej Kazakov
 *
 */
public class MySQLCache implements Cache {

//	private MySQLConnector dbI, dbQ;

	private PreparedStatement prepInsert;
	private PreparedStatement prepQuery;
	private PreparedStatement prepDelete;

	public MySQLCache(MySQLConnector dbI, MySQLConnector dbQ) throws SQLException {
//		Debug.err("Cache created");
//		this.dbI = dbI;
//		this.dbQ = dbQ;
		prepInsert = dbI.getPreparedStatement("insert into Cache set context = ?, item = ?, compressed = ?");
		prepQuery = dbQ.getPreparedStatement("select dbCacheID, compressed, context, item from Cache limit 1");
		prepDelete = dbQ.getPreparedStatement("delete from Cache where dbCacheID = ?");
	}

	public boolean cacheEnabled() {
		return true;
	}

	public void cache(InputStream is, String context, boolean compressed) throws Exception {
		//int availible = is.available();
		Debug.msg("" + is.available());
		Debug.msg("" + compressed);
		//StringBuffer buffer = new StringBuffer();
		//GZIPInputStream
		ByteArrayOutputStream accumulator = new ByteArrayOutputStream();
		byte[] buffer = new byte[1024];
		int len;
		while ((len = is.read(buffer)) > 0) {
			accumulator.write(buffer, 0, len);
		}
		Debug.msg("Received input size: " + accumulator.size());
//		Debug.msg(accumulator.toString());
//		GZIPInputStream os = new GZIPInputStream(new ByteArrayInputStream(buffer.toByteArray()));
//		BufferedReader reader = new BufferedReader(new InputStreamReader(os));
//		String line = null;
//		while ((line = reader.readLine()) != null) {
//			Debug.msg(line);
//		}
		cache(context, accumulator.toByteArray(), compressed);
	}

	private synchronized void cache(
			String context,
			byte[] item,
			boolean compressed
	) throws CacheShutdownException, SQLException, NothingInsertedException {
		if (closing) throw new CacheShutdownException();
			prepInsert.setString(1, context);
			prepInsert.setBytes(2, item);
			prepInsert.setString(3, (compressed ? "y" : "n"));
			Debug.msg("MySQLCache insert " + prepInsert);
			if (prepInsert.executeUpdate() != 1) throw new NothingInsertedException("not inserted");
		notifyAll();
	}
	
	public synchronized void cache(CacheableItem ci) throws Exception {
		cache(ci.getContext(), ci.getItem().toString().getBytes(), false);
	}

//	private long count = 0;
	private boolean closing = false;
	
	public synchronized CacheableItem getCachedItem() throws Exception {
//		Debug.msg("Cache getItem() start");
		CacheableItem ci = null;
		long id;
		ResultSet rs = null;
//		synchronized (this) {
//			if (timing) {
//			queryStartTime = System.currentTimeMillis();
//			}
			boolean loop;
			do {
				rs = prepQuery.executeQuery();
				loop = !rs.next();
				if (loop) wait();
				if (closing) throw new CacheShutdownException();
			} while (loop);
		id = rs.getLong(1);
		String compressed = rs.getString(2);
		Blob blob = rs.getBlob(4);
		InputStream is = null;
		Debug.msg("Retrieved from cache " + id + " " + compressed + " " + rs.getString(3));
		try {
		if (compressed.equals("y")) {
			Debug.msg("Getting compressed cache item");
//			ByteArrayOutputStream accumulator = new ByteArrayOutputStream();
//			InputStream in = blob.getBinaryStream();
//			byte[] 
//			int b;
//			while ((b = in.read()) > 0) {
//				accumulator.write(b);
//			}
//			Debug.msg("Size of the received blob " + accumulator.size());
//			is = new GZIPInputStream(new ByteArrayInputStream(accumulator.toByteArray()));
			is = new GZIPInputStream(blob.getBinaryStream());
		} else if (compressed.equals("n")) {
			Debug.msg("Getting not compressed cache item");
			is = blob.getBinaryStream();
		} else {
			throw new Exception ("Invalid compression flag in the cache");
		}
		ByteArrayOutputStream accumulator = new ByteArrayOutputStream();
		byte buffer[] = new byte[1024];
		int len; 
		while ((len = is.read(buffer)) > 0) {
			accumulator.write(buffer, 0, len);
		}
		ci = new CacheableItem(rs.getString(3), new StringBuffer(accumulator.toString()));
		} catch (EOFException ex) {
			ex.printStackTrace();
		}
		
//		synchronized (prepDelete) {
			prepDelete.setLong(1, id);
			prepDelete.execute();
//		}
//	}
//	if (timing) {
//	queryTime += System.currentTimeMillis() - queryStartTime;
//	}
	return ci;
}

	public void resetState() {
		closing = false;
	}
	
	public synchronized void cacheShutdown() {
		closing = true;
		notifyAll();
	}

//private boolean timing;
//private long insertTime;
//private long insertStartTime;
//private long queryTime;
//private long queryStartTime;

//public void startTiming(){
//timing = true;
//insertTime = 0;
//queryTime = 0;
//}

//public String stopTiming(){
//return "CacheInsert\t" + insertTime + "\nCacheQuery\t" + queryTime;
//}

}
