package org.pasoa.preserv.storage.mysql;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.sql.BatchUpdateException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import java.sql.Statement;
import java.util.HashSet;
import java.util.Set;

/**
 * 
 * @author Andrej Kazakov
 *
 */
public class MySQLConnector {

	private Connection connection;
	private String url;
	private String username;
	private String password;
	private Statement statement;
	private Statement batchStatement;
	private String driver = "com.mysql.jdbc.Driver"; 
	
	public MySQLConnector(String url, String username, String password) {
		this.url = url;
		this.username = username;
		this.password = password;
	}
	
	public String getURL() {
		return url;
	}
	
	public void connect() {
		try {
			// Loading the driver for MySQL
			Class.forName(driver);
			// Connectiong to the DB
//			String url= "jdbc:mysql://localhost/preserv";
//			DriverManager.getConnection(url, "root", "aaa");
//			Debug.msg("Connecting to database " + url + " with username "
//					+ username);
			connection = DriverManager.getConnection(url, username, password);
			// In case of an error
			if (connection == null || connection.isClosed()) {
				Debug.err("database " + url + " could not be reached");
				return;
			}
			statement = connection.createStatement();
			batchStatement = connection.createStatement();
			
			Debug.msg("Connection to the DB successfully established " + toString());
//			return connection;
		} catch (SQLException ex) {
			Debug.err(ex, "Error occured while accessing database " + url);
			return;
		}
		catch (ClassNotFoundException ex) {
			Debug.err(ex, "Driver for the mysql database not found");
			return;
		}
	}
	
	
	public boolean isClosed() throws SQLException {
		return connection.isClosed();
	}
	
	public void addBatch(String sql) throws SQLException {
		batchStatement.addBatch(sql);
	}
	
	public int[] executeBatchR() throws SQLException {
		return batchStatement.executeBatch();
	}
	
	public void executeBatch() throws BatchUpdateException, SQLException {
		int [] res = batchStatement.executeBatch();
		for (int i: res)
			if (i == Statement.EXECUTE_FAILED) throw new BatchUpdateException();
		batchStatement.clearBatch();
	}
	
	public void clearBatch() throws SQLException {
		batchStatement.clearBatch();
	}
	
	public PreparedStatement getPreparedStatement(String sqlStatement) throws SQLException {
		return connection.prepareStatement(sqlStatement);
	}
	
	public void disconnect() {
		try {
			statement.close();
			connection.close();
		} catch (SQLException ex) {
			Debug.err(ex, "Connection " + connection + " could not be closed " + toString());
		}
	}
	
	public boolean execute (String sqlStatement) {
		try {
			boolean f;
			if (timing) {
				statementStartTime = System.currentTimeMillis();
				f = statement.execute(sqlStatement);
				statementTime += System.currentTimeMillis() - statementStartTime;
			} else {
				f = statement.execute(sqlStatement);
			}
			return f;
		} catch (SQLException ex) {
			Debug.err(ex, "A query to the DB could not be executed. " + sqlStatement);
		}
		return false;
	}
	
	// return the number of affected rows
	public int executeUpdate (String sqlStatement) {
		try {
			int f;
			if (timing) {
				statementStartTime = System.currentTimeMillis();
				f = statement.executeUpdate(sqlStatement, Statement.RETURN_GENERATED_KEYS);
				statementTime += System.currentTimeMillis() - statementStartTime;
			} else {
				f = statement.executeUpdate(sqlStatement, Statement.RETURN_GENERATED_KEYS);
			}
			return f;
		} catch (SQLException ex) {
			Debug.err(ex, "A query to the DB could not be executed. " + sqlStatement);
		}
		return -1;		
	}
	
	public long executeInsert(String sqlStatement) throws NothingInsertedException {
		if (executeUpdate(sqlStatement) != 1) throw new IllegalArgumentException("The insert has affected not a single row");
		try {
			ResultSet rs = statement.getGeneratedKeys();
//			if (rs.getFetchSize() == 0) throw new NothingInsertedException(sqlStatement);
			if (!rs.next()) throw new NothingInsertedException(sqlStatement);
//			rs.first();
			return rs.getLong(1);
		} catch (SQLException ex) {
			Debug.err(ex, "Could not perform the insertion into the store. " + sqlStatement);
		}
		return -1;
	}
	
	private ResultSet executeQuery (String query, boolean newStatement) {
		try {
			ResultSet f;
			if (timing) {
				queryStartTime = System.currentTimeMillis();
			}
			if (newStatement) {
				statement = connection.createStatement();
			}
			f = statement.executeQuery(query);
			if (timing) {
				queryTime += System.currentTimeMillis() - queryStartTime;
			}
			return f;
			
		} catch (SQLException ex) {
			Debug.err(ex, "A query to the DB could not be performed. " + query);
		}
		return null;
	}
	
	public ResultSet executeQuery(String query){
		return executeQuery (query, true);
	}
	
	public long executeQueryGetKey(String query){
		try {
			ResultSet rs = executeQuery(query, false);
//			if (rs.getFetchSize() == 0) return -1;
			if (!rs.next()) return -1;
//			rs.first();
			return rs.getLong(1);
		} catch (SQLException ex) {
			Debug.err(ex, "Could not retrive a key from the DB. " + query);
		}
		return -1;
	}
	
	public void printContents(File file) {
		String tables[] = new String[] {"InteractionRecords", 
				"PAssertionTypes", 
				"MetaData", 
				"ExposedInteractionMetadata",
				"PAssertions",
				"RelationshipPAssertion_objects",
				"ViewKind",
				"StyledPAssertions",
				"RelationshipPAssertions"
		};
		try {
			FileOutputStream outs = new FileOutputStream(file);
			PrintStream prs = new PrintStream(outs);
			
			for (String tbl: tables) {
				ResultSet rs = executeQuery("select * from " + tbl);
				prs.println("\nTable " + tbl);
				ResultSetMetaData rsm = rs.getMetaData(); 
				int columns = rsm.getColumnCount();
				int row = 0;
				while (rs.next()) {
					prs.println("Row " + (row++));
					for (int i = 1; i <= columns; i++) {
						prs.println(rsm.getColumnName(i) + "\t" + rs.getString(i));
					}
				}
			}
			prs.close();
			outs.close();
		} catch (Exception ex) {
			Debug.err(ex, "Could not print the DB to file");
		}
	}
	
	public static String safeChars(String input) {
		return input.replaceAll("'", "\\'");
	}
	
	private boolean timing = false;
	private long queryTime;
	private long queryStartTime;
	private long statementTime;
	private long statementStartTime;
	
	public void startTiming(){
		timing = true;
		queryTime = 0;
		statementTime = 0; 
	}
	
	public String stopTiming(){
		timing = false;
		return "Queries\t" + queryTime + "\nStatements\t" + statementTime;
	}
	
}
