package org.pasoa.preserv.storage.mysql;

import java.io.IOException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.pasoa.common.Constants;
import org.pasoa.preserv.index.Index;
import org.pasoa.preserv.index.MyIndex;
import org.pasoa.preserv.index.IndexProblem;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.ExposedInteractionMetadata;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.Link;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.pstructure.PAssertion;
import org.pasoa.pstructure.RelationshipPAssertion;
import org.pasoa.pstructure.View;
import org.pasoa.simpledom.NormalisingDOMWriter;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import com.mysql.jdbc.CommunicationsException;

class MySQLIndex implements Index, MyIndex{

	private MySQLConnector db;
	private MySQLRetrieval retrieval;
	private MySQLMaps maps;

	public MySQLIndex(MySQLConnector db, MySQLRetrieval retrieval, MySQLMaps maps) throws SQLException{
		this.db = db;
		this.retrieval = retrieval;
		this.maps = maps;
		initStatements();
	}

	private PreparedStatement prepIRID;
	private PreparedStatement prepPAID;
	private PreparedStatement prepDBAsserterID;

	private void initStatements() throws SQLException {
		
		prepIRID = db.getPreparedStatement("select dbInteractionRecordID from InteractionRecords " +
				"where messageSource = (select dbEndPointID from EndPointReferences where content =?) " +
				"and messageSink = (select dbEndPointID from EndPointReferences where content =?) " +
		"and interactionID = ?");

		prepPAID = db.getPreparedStatement("select dbPAssertionID, passertionTypeID from PAssertions " +
				"where localPAssertionID = ? and dbInteractionRecordID = ? and dbViewKindID = ? and isLocal = 'y'");
		
		prepDBAsserterID = db.getPreparedStatement("select dbAsserterID from Asserters where content = ?");

	}

	public InteractionRecord findInteractionRecord(InteractionKey key) throws IndexProblem{
		try {
			return retrieval.getInteractionRecordByID(getDBInteractionRecordID(key));
		} catch (CommunicationsException ex) {
			throw new ConnectionLostIndexProblem("", ex);
		} catch (SQLException ex) {
			throw new IndexProblem("", ex);
		} catch (SAXException ex) {
			throw new IndexProblem("", ex);
		} catch (IOException ex) {
			throw new IndexProblem("", ex);
		} catch (NotFoundViewKindException ex) {
			throw new IndexProblem("", ex);
		}
	}

	private long getDBInteractionRecordID(InteractionKey key) throws SQLException, IndexProblem {
		long dbInteractionRecordID;
		synchronized(prepIRID) {
			prepIRID.setString(1, NormalisingDOMWriter.writeToString(key.getMessageSource()));
			prepIRID.setString(2, NormalisingDOMWriter.writeToString(key.getMessageSink()));
			prepIRID.setString(3, key.getInteractionID());
			ResultSet rs = prepIRID.executeQuery();
			if (!rs.next()) throw new IndexProblem("InteractionRecord not found");
			dbInteractionRecordID = rs.getLong(1);
		}
		return dbInteractionRecordID;
	}

	public PAssertion findPAssertion(GlobalPAssertionKey gpak) throws IndexProblem {
		try {
			long dbPAssertionID;
			String passertionType;
			String localPAssertionID = gpak.getLocalPAssertionID();
			long dbInteractionRecordID = getDBInteractionRecordID(gpak.getInteractionKey());
			synchronized (prepPAID){
				prepPAID.setString(1, gpak.getLocalPAssertionID());
				prepPAID.setLong(2, dbInteractionRecordID);
				prepPAID.setLong(3, maps.getDBViewKindID(gpak.getViewKind()));
				ResultSet rs = prepPAID.executeQuery();
				if (!rs.next()) throw new IndexProblem ("PAssertion Not Found");
				dbPAssertionID = rs.getLong(1);
//				Debug.msg("PAssertionTypeID : " + rs.getLong(2));
//				Debug.msg("Maps: " + maps);
				passertionType = maps.getPassertionType(rs.getLong(2)).intern();
				
			}
			if (passertionType == Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME) {
				return retrieval.getAPA(localPAssertionID, dbPAssertionID);
			} else if (passertionType == Constants.INTERACTION_PASSERTION_ELEMENT_NAME) {
				return retrieval.getIPA(localPAssertionID, dbPAssertionID);
			} else if (passertionType == Constants.RELATIONSHIP_PASSERTION_ELEMENT_NAME) {
				return retrieval.getRPA(localPAssertionID, dbPAssertionID);
			} else {
				throw new IndexProblem(passertionType + " is not recognised as a type of PAssertion");
			}
		} catch (CommunicationsException ex) {
			throw new ConnectionLostIndexProblem("", ex);
		} catch (SQLException ex) {
			throw new IndexProblem("", ex);
		} catch (SAXException ex) {
			throw new IndexProblem("", ex);
		} catch (IOException ex) {
			throw new IndexProblem("", ex);
		}
	}

	public List<View> findViews(InteractionKey key, String viewKind) throws IndexProblem {
		try {
			long dbInteractionRecordID = getDBInteractionRecordID(key);
			List<View> vs = retrieval.getViews(viewKind, dbInteractionRecordID);
			if (vs.size() == 0) throw new IndexProblem("View not found");
			return vs;
		} catch (CommunicationsException ex) {
			throw new ConnectionLostIndexProblem("", ex);
		} catch (SQLException ex) {
			throw new IndexProblem("", ex);
		} catch (SAXException ex) {
			throw new IndexProblem("", ex);
		} catch (IOException ex) {
			throw new IndexProblem("", ex);
		} catch (NotFoundViewKindException ex) {
			throw new IndexProblem("", ex);
		}

	}

	private long getDBAsserterID(Element asserter) throws SQLException, IndexProblem {
		prepDBAsserterID.setString(1, NormalisingDOMWriter.writeToString(asserter));
		ResultSet rs = prepDBAsserterID.executeQuery();
		if (!rs.first()) throw new IndexProblem("Unrecognised Asserter");
		return rs.getLong(1);
	}
	
	public View findView(InteractionKey key, String viewKind, Element asserter) throws IndexProblem {
		try {
			return retrieval.getView(getDBInteractionRecordID(key), 
					maps.getDBViewKindID(viewKind),
					getDBAsserterID(asserter)
					);
		} catch (CommunicationsException ex) {
			throw new ConnectionLostIndexProblem("", ex);
		} catch (SQLException e) {
			throw new IndexProblem("", e);
		} catch (SAXException e) {
			throw new IndexProblem("", e);
		} catch (IOException e) {
			throw new IndexProblem("", e);
		}
	}

	@Deprecated
	public View findView(InteractionKey key, String viewKind) throws IndexProblem {
		return findViews(key, viewKind).get(0);
	}


}
