package org.pasoa.preserv.storage.mysql;

import java.sql.BatchUpdateException;
import java.sql.SQLException;
import java.sql.Statement;

/**
 * @author Andrej Kazakov
 *
 */
public class MySQLManagement {

	private MySQLConnector db;
	private static String _defaultEngine = "engine=myisam";
	
	
	public MySQLManagement(MySQLConnector db){
		this.db = db;
	}
	public synchronized void delete() throws BatchUpdateException, SQLException{
		recreateTables(_defaultEngine, null);
	}
	public synchronized void delete(String engine) throws BatchUpdateException, SQLException{
		recreateTables(engine, null);
	}
	public synchronized void delete(String engine, String database) throws BatchUpdateException, SQLException{
//		Debug.err("started Delete");
		recreateTables(engine, database);
//		Debug.err("ended Delete");
	}


	/**
	 * @param args
	 * @throws SQLException 
	 * @throws BatchUpdateException 
	 */
	
	private static void printHelp() {
		System.out.println("Creates MySQL preserv database\n\t" +
				"Usage:\t" +
				"java MySQLManagement -p <password> [-u <username>] [-h<host>] [-d <database>]" +
				"Connection parameters for MySQL: " +
				"\t\tpassword - password for an administrator account\n" +
				"\t\tusername - name of an administrator account (default root)\n" +
		"\t\thost - the host for the MySQL server (default jdbc:mysql://localhost)\n" +
		"\t\tdatabse - the name of the database that will be created (default preserv)");
		System.exit(1);
	}

	private final static String _password = "";
	private final static String _username = "root";
	private final static String _host = "jdbc:mysql://localhost";
//	private final static String _database = MySQLStore._sDBName;

	
	public static void main(String[] args) {
		if (args.length == 0) {
			printHelp();
		} else {
			String password = _password;
			String username = _username;
			String host = _host;
			String database = MySQLStore._sDBName;

			for (int i = 0; i < args.length; i++) {
				String arg = args[i].intern();
				if (arg == "-p") {
					password = args[++i];
				} else if (arg == "-h") {
					host = args[++i];
				} else if (arg == "-u") {
					username = args[++i];
				} else if (arg == "-d") {
					database = args[++i];
				} else {
					printHelp();
				}
			}
			if (!host.endsWith("/")) host += "/";
//			Debug.msg("Database " + host);
			MySQLConnector db = new MySQLConnector(host, username, password);
			db.connect();
			MySQLManagement mm = new MySQLManagement(db);
			try {
				mm.recreateDatabase(database);
				mm.recreateTables(_defaultEngine, database);
				mm.recreateUser(database, MySQLStore._sDBUser);
			} catch (BatchUpdateException e) {
				Debug.err(e, "");
			} catch (SQLException e) {
				Debug.err(e, "");
			}
			db.disconnect();
		}
	}

	private synchronized void recreateDatabase(String database) throws BatchUpdateException, SQLException {
		db.addBatch("drop database if exists " + database);
		db.addBatch("create database " + database);

		db.executeBatch();
	}

	private synchronized void recreateTables(String engine, String database) throws BatchUpdateException, SQLException {
//		Debug.msg("Clearing the contents of the store " + db.getURL());
		if (database != null) db.addBatch("use " + database);
//		Debug.msg("Clearing database " + database);
		db.addBatch("drop table if exists EndPointReferences");
		db.addBatch("CREATE TABLE EndPointReferences (" +
				"dbEndPointID bigint unsigned not null unique auto_increment," +
				"content LONGTEXT not null, primary key (content(700))) " + engine);

		db.addBatch("drop table if exists Asserters");
		db.addBatch("CREATE TABLE Asserters (" +
				"dbAsserterID bigint unsigned primary key auto_increment not null," +
				"content LONGTEXT not null, unique (content(700))) " + engine);

		db.addBatch("drop table if exists InteractionRecords ");
		db.addBatch("CREATE TABLE InteractionRecords (" +
				"dbInteractionRecordID bigint unsigned not null unique auto_increment," +
				"messageSource bigint unsigned not null," +
				"messageSink bigint unsigned not null," +
				"interactionID varchar (15000) not null," +
				"isLocal enum('y', 'n') not null default 'n'," +
				"primary key(messageSource, messageSink, interactionID(200))," +
				"index(isLocal)) " + engine);

		db.addBatch("drop table if exists NExpectedPAssertions");
		db.addBatch("CREATE TABLE NExpectedPAssertions (" +
				"dbAsserterID bigint unsigned not null," +
				"dbViewKindID bigint unsigned not null," +
				"dbInteractionRecordID bigint unsigned not null," +
				"numberOfExpectedPAssertions int not null default '-1'," +
				"primary key(dbAsserterID, dbViewKindID, dbInteractionRecordID), " +
				"index (dbViewKindID, dbInteractionRecordID)) " + engine);

		db.addBatch("drop table if exists PAssertionTypes");
		db.addBatch("CREATE TABLE PAssertionTypes(" +
				"passertionTypeID bigint unsigned primary key auto_increment not null," +
				"name varchar(1023) not null, unique (name(700))) " + engine);

		db.addBatch("drop table if exists ExposedInteractionMetadata");
		db.addBatch("CREATE TABLE ExposedInteractionMetadata (" +
				"dbEIMID bigint unsigned unique not null auto_increment," +
				"dbInteractionRecordID bigint unsigned not null," +
				"dbViewKindID bigint unsigned not null," +
				"dbAsserterID bigint unsigned not null," +
				"referenceDBPAssertionID bigint unsigned not null, " +
				"index (referenceDBPAssertionID), " +
				"index (dbInteractionRecordID, dbViewKindID, dbAsserterID)," +
				"index (dbInteractionRecordID, dbViewKindID)" +
				") " + engine);

		db.addBatch("drop table if exists MetaData");
		db.addBatch("CREATE TABLE MetaData (" +
				"dbEIMID bigint unsigned not null," +
				"content LONGTEXT not null, " +
				"index (dbEIMID)) " + engine);

		db.addBatch("drop table if exists PAssertions");
		db.addBatch("CREATE TABLE PAssertions (" +
				"dbPAssertionID bigint unsigned unique not null auto_increment," +
				"dbInteractionRecordID bigint unsigned not null," +
				"localPAssertionID LONGTEXT," +
				"dbViewKindID bigint unsigned not null," +
				"dbAsserterID bigint unsigned null," +
				"passertionTypeID bigint unsigned," +
				"isLocal enum('y', 'n') not null default 'n'," +
				"unique (dbInteractionRecordID, localPAssertionID(700), dbViewKindID, dbAsserterID, isLocal)," +
				"primary key (dbInteractionRecordID, localPAssertionID(700), dbViewKindID, dbAsserterID)," +
				"index (dbInteractionRecordID, dbViewKindID, localPAssertionID(700))," +
				"index (dbInteractionRecordID, localPAssertionID(700), dbViewKindID, isLocal)) " + engine);

		db.addBatch("drop table if exists StyledPAssertions");
		db.addBatch("CREATE TABLE StyledPAssertions(" +
				"dbPAssertionID bigint unsigned not null primary key," +
				"documentationStyle LONGTEXT null, " +
				"fullXMLContent LONGTEXT) " + engine);

		db.addBatch("drop table if exists ViewKind");
		db.addBatch("CREATE TABLE ViewKind (" +
				"dbViewKindID bigint unsigned primary key auto_increment not null," +
				"name varchar(1023) not null, unique (name(700))) " + engine);

		db.addBatch("drop table if exists RelationshipPAssertions");
		db.addBatch("CREATE TABLE RelationshipPAssertions (" +
				"dbPAssertionID bigint unsigned primary key not null," +
				"subjectdbPAssertionID bigint unsigned not null," +
				"subjectDataAccessor LONGTEXT not null," +
				"subjectParameterName varchar(7000)," +
				"relation varchar(7000) not null) " + engine);

		db.addBatch("drop table if exists RelationshipPAssertion_objects");
		db.addBatch("CREATE TABLE RelationshipPAssertion_objects(" +
				"parentDBPAssertionID bigint unsigned not null," +
				"dbPAssertionID bigint unsigned not null," +
				"dataAccessor LONGTEXT null," +
				"parameterName varchar(7000) not null," +
				"objectLink LONGTEXT null," +
				"index (parentDBPAssertionID)) " + engine);

		db.addBatch("drop table if exists Cache");
		db.addBatch("create table Cache(" +
				"dbCacheID bigint unsigned primary key not null auto_increment," +
				"context varchar(700) not null," +
				"item LONGBLOB not null," +
				"compressed enum('y', 'n') not null) " + engine);

		db.executeBatch();
	}

	private synchronized void recreateUser(String database, String user) throws BatchUpdateException, SQLException {
		db.addBatch("use mysql");
		db.addBatch("REVOKE ALL PRIVILEGES, GRANT OPTION ON " + database + ".* FROM " + user + "@localhost");
//		db.addBatch("drop user " + user + "@localhost");
//		Removing the user - may throw an Expcetion
		try {
			db.executeBatch();
		} catch (BatchUpdateException ex) {
//			Ignore the Exception as there may be no user in the first place
		}
		db.addBatch("grant select, update, insert, drop, create on " + database + ".* to preserv@localhost identified by 'QahdEkco22!$^fvjJfr43#fdjewmRTHTr43fd'");
		db.addBatch("grant delete on " + database + ".Cache to preserv@localhost identified by 'QahdEkco22!$^fvjJfr43#fdjewmRTHTr43fd'");

		db.executeBatch();
	}

}

