/**
 * 
 */
package org.pasoa.preserv.storage.mysql;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * @author Andrej Kazakov
 *
 */
class MySQLMaps {

	private MySQLConnector db;
	private Map<String, Long> viewKinds;
	private Map<Long, String> viewKindsRev;
	private Map<String, Long> passertionTypes;
	private Map<Long, String> passertionTypesRev;

	private PreparedStatement prepVK;
	private PreparedStatement prepPT;

	public MySQLMaps (MySQLConnector db) throws SQLException {
		this.db = db;
		viewKinds = new HashMap<String, Long>();
		viewKindsRev = new HashMap<Long, String>();
		passertionTypes = new HashMap<String, Long>();
		passertionTypesRev = new HashMap<Long, String>();
		initStatements();
		reloadMaps();
	}

	private void initStatements() throws SQLException {
		prepVK = db.getPreparedStatement("select dbViewKindID, name from ViewKind");
		prepPT = db.getPreparedStatement("select passertionTypeID, name from PAssertionTypes");
	}

	public long getDBViewKindID(String key){
		Long l = viewKinds.get(key);
		if (l == null) return 0;
		else return l;
	}

	public Long getDBViewKindIDUnsafe(String key) {
		return viewKinds.get(key);
	}

	public void putDBViewKindID(String key, Long value) {
		viewKinds.put(key, value);
		viewKindsRev.put(value, key);
	}

	public void putPAssertionTypeID(String key, Long value) {
		passertionTypes.put(key, value);
		passertionTypesRev.put(value, key);
	}
	
	public String getViewKind (Long key) {
		return viewKindsRev.get(key);
	}

	public long getPassertionTypeID(String key){
		Long l = passertionTypes.get(key);
		if (l == null) return 0;
		else return l;
	}

	public Long getPassertionTypeIDUnsafe(String key){
		return passertionTypes.get(key);
	}

	public String getPassertionType(Long key){
		return passertionTypesRev.get(key);
	}


	public synchronized void reloadMaps() throws SQLException {
		viewKinds.clear();
		viewKindsRev.clear();
		passertionTypes.clear();
		passertionTypesRev.clear();
		loadMap(viewKinds, prepVK);
		loadMap(passertionTypes, prepPT);
		fillRevMap(viewKinds, viewKindsRev);
		fillRevMap(passertionTypes, passertionTypesRev);
	}

	private void fillRevMap(Map<String, Long> map, Map<Long, String> revMap) {
		Set<String> keys = map.keySet();
		for(String key: keys) {
			revMap.put(map.get(key), key);
		}
	}

	private void loadMap(Map<String, Long> map, PreparedStatement prepQ) throws SQLException {
		ResultSet rs = prepQ.executeQuery();
		while (rs.next()) {
			map.put(rs.getString(2), rs.getLong(1));
		}
	}
}
