package org.pasoa.preserv.storage.mysql;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.pasoa.common.Constants;
import org.pasoa.preserv.record.Record;
import org.pasoa.preserv.xquery.QueryProblem;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.ExposedInteractionMetadata;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.Link;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.pstructure.RelationshipPAssertion;
import org.pasoa.pstructure.SubjectID;
import org.pasoa.pstructure.deserialiser.PStructureDeserialiser;
import org.pasoa.pstructure.serialiser.PStructureSerialiser;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.simpledom.DOMWriter;
import org.pasoa.simpledom.NormalisingDOMWriter;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * 
 * @author Andrej Kazakov
 *
 */
public class MySQLRecord implements Record {

	private MySQLConnector db;
	private PStructureDeserialiser pstructDeserialiser = new PStructureDeserialiser();
	private DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);

//	private long dbInteractionRecordID;
//	private long dbViewKindID;
//	private long passertionTypeID;
//	private long dbAsserterID;
//	private long dbPAssertionID;
//	private int sf;

	private MySQLMaps maps;
	public MySQLRecord(MySQLConnector db, MySQLMaps maps) throws SQLException {
		this.db = db;
		this.maps = maps;
		prepareStatements();
	}

	private PreparedStatement prepQueryPAssertionTypeID;
	private PreparedStatement prepInsertPAssertionTypeID;
	private PreparedStatement prepQueryDBViewKindID;
	private PreparedStatement prepInsertDBViewKindID;
	private PreparedStatement prepQueryDBInteractionRecordID;
	private PreparedStatement prepUpdateDBInteractionRecordID;
	private PreparedStatement prepInsertDBInteractionRecordID;
	private PreparedStatement prepQueryDBEndPointID;
	private PreparedStatement prepInsertDBEndPointID;
	private PreparedStatement prepQueryDBAsserterID;
	private PreparedStatement prepInsertDBAsserterID;
	private PreparedStatement prepQueryDBPAssertionIDWithAsserter;
	private PreparedStatement prepUpdateDBPAssertionID;
	private PreparedStatement prepInsertDBPAssertionIDLocal;
	private PreparedStatement prepQueryDBPAssertionIDWithoutAsserter;
	private PreparedStatement prepInsertDBPAssertionIDRemote;

	private PreparedStatement prepInsertIPA;
//	private PreparedStatement prepInsertAPA;
	private PreparedStatement prepInsertRPAHeader;
	private PreparedStatement prepInsertRPAObjects;
	private PreparedStatement prepInsertEIMHeader;
	private PreparedStatement prepInsertEIMObjects;
	private PreparedStatement prepInsertSF;
	private PreparedStatement prepUpdateSF;
	private PreparedStatement prepQuerySF;

	private void prepareStatements() throws SQLException{
		prepQueryPAssertionTypeID = db.getPreparedStatement("select passertionTypeID from PAssertionTypes where name = ?");
		prepInsertPAssertionTypeID = db.getPreparedStatement("insert into PAssertionTypes set name = ?");

		prepQueryDBViewKindID = db.getPreparedStatement("select dbViewKindID from ViewKind where name = ?");
		prepInsertDBViewKindID = db.getPreparedStatement("insert into ViewKind set name = ?");

		prepQueryDBInteractionRecordID = db.getPreparedStatement("select dbInteractionRecordID, isLocal from " +
		"InteractionRecords where messageSource = ? and messageSink = ? and interactionID = ?");
		prepInsertDBInteractionRecordID = db.getPreparedStatement("insert into InteractionRecords" +
		"(messageSource, messageSink, interactionID, isLocal) values (?, ?, ?, ?)");
		prepUpdateDBInteractionRecordID = db.getPreparedStatement("update InteractionRecords set isLocal = 'y' " +
		"where dbInteractionRecordID = ?");

		prepQueryDBEndPointID = db.getPreparedStatement("select dbEndPointID from EndPointReferences where content = ?");
		prepInsertDBEndPointID = db.getPreparedStatement("insert into EndPointReferences set content = ?");

		prepQueryDBAsserterID = db.getPreparedStatement("select dbAsserterID from Asserters where content = ?");
		prepInsertDBAsserterID = db.getPreparedStatement("insert into Asserters set content = ?");


		prepQueryDBPAssertionIDWithAsserter = db.getPreparedStatement("select dbPAssertionID, isLocal from PAssertions " +
		"where localPAssertionID = ? and dbViewKindID = ? and dbAsserterID = ? and dbInteractionRecordID = ?");
		prepUpdateDBPAssertionID = db.getPreparedStatement("update PAssertions set isLocal = 'y', passertionTypeID = ?, " +
		"dbAsserterID = ? where dbPAssertionID = ?");
		prepInsertDBPAssertionIDLocal = db.getPreparedStatement("INSERT INTO PAssertions" +
				"(dbInteractionRecordID, localPAssertionID, dbViewKIndID, dbAsserterID, " +
		"passertionTypeID, isLocal) values (?, ?, ?, ?, ?, 'y')");

		prepQueryDBPAssertionIDWithoutAsserter = db.getPreparedStatement("select dbPAssertionID, isLocal " +
				"from PAssertions where localPAssertionID = ? and " +
		"dbViewKindID = ? and dbInteractionRecordID = ?");
		prepInsertDBPAssertionIDRemote = db.getPreparedStatement("INSERT INTO PAssertions" +
				"(dbInteractionRecordID, localPAssertionID, dbViewKIndID, dbAsserterID, " +
		"passertionTypeID, isLocal) values (?, ?, ?, 0, null , 'n')");

		prepInsertIPA = db.getPreparedStatement("insert into StyledPAssertions(dbPAssertionID, fullXMLContent, documentationStyle) values (?, ?, ?)");
//		prepInsertAPA = db.getPreparedStatement("insert into StyledPAssertions values (?, ?, ?)"); 
		prepInsertRPAHeader = db.getPreparedStatement("insert into RelationshipPAssertions values (?, ?, ?, ?, ?)");
		prepInsertRPAObjects = db.getPreparedStatement("insert into RelationshipPAssertion_objects values (?, ?, ?, ?, ?)");
		prepInsertEIMHeader = db.getPreparedStatement(
				"insert into ExposedInteractionMetadata" +
				"(dbInteractionRecordID, dbViewKindID, dbAsserterID, referenceDBPAssertionID) " +
		"values (?, ?, ?, ?)");
		prepInsertEIMObjects = db.getPreparedStatement("insert into MetaData" +
				"(dbEIMID, content) " +
		"values (?, ?)");

		prepInsertSF = db.getPreparedStatement("insert into NExpectedPAssertions(dbAsserterID, " +
		"dbViewKindID, dbInteractionRecordID, numberOfExpectedPAssertions) values(?, ?, ?, -1)");
		prepUpdateSF = db.getPreparedStatement("update NExpectedPAssertions set numberOfExpectedPAssertions = ? " +
		"where dbAsserterID = ? and dbViewKindID = ? and dbInteractionRecordID = ?");
		prepQuerySF = db.getPreparedStatement("select numberOfExpectedPAssertions from NExpectedPAssertions " +
		"where dbAsserterID = ? and dbViewKindID = ? and dbInteractionRecordID = ?"); 
	}

//	synchronized void clearCache() {
//	_paTypeCache.clear();
//	_viewKindCache.clear();
//	}

	public synchronized void record(GlobalPAssertionKey gpid, Element asserter, 
			Element assertion, String kindOfAssertion) throws UnrecognisedPAssertionTypeException, NothingInsertedException, SQLException, QueryProblem, NumberOfExpectedPAssertionsAlreadyThereException, DatabaseClosedException{

		long dbInteractionRecordID;
		long dbViewKindID;
		long passertionTypeID;
		long dbAsserterID;
		long dbPAssertionID = -1;
		int sf;


//		Checking for the right PAssertionType
		String assertionType = kindOfAssertion.intern();
		if (	assertionType != Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME && 
				assertionType != Constants.INTERACTION_METADATA_ELEMENT_NAME &&
				assertionType != Constants.INTERACTION_PASSERTION_ELEMENT_NAME &&
				assertionType != Constants.SUBMISSION_FINISHED_ELEMENT_NAME &&
				assertionType != Constants.RELATIONSHIP_PASSERTION_ELEMENT_NAME
		) throw new UnrecognisedPAssertionTypeException();

		// Getting the Interaction
		InteractionKey intKey = gpid.getInteractionKey();

		// Getting the ViewKind Object from the pool
		String viewKind = gpid.getViewKind();
//		Debug.msg("Recording viewkind: " + viewKind);
//		if (timing) {
//		headerTimeStart = System.currentTimeMillis();
//		}

		// Getting the dbInteractionRecordID
		dbInteractionRecordID = getDBInteractionRecordID(intKey, true);

		// Getting the dbViewKindID
		dbViewKindID = getDBViewKindID(viewKind);

		// Getting the ID of the type of PAssertion
		passertionTypeID = getPAssertionTypeID(kindOfAssertion);

		// Saving the asserter
		dbAsserterID = getDBAsserterID(NormalisingDOMWriter.writeToString(asserter));

		// Saving the PAssertion
		if (assertionType != Constants.SUBMISSION_FINISHED_ELEMENT_NAME &&
				assertionType != Constants.INTERACTION_METADATA_ELEMENT_NAME
		) {
			dbPAssertionID = getDBPAssertionID(
					dbInteractionRecordID, 
					gpid.getLocalPAssertionID(), 
					dbViewKindID, 
					dbAsserterID,
					passertionTypeID
			);
		}

		sf = getSF(dbAsserterID, dbViewKindID, dbInteractionRecordID);
//		if (timing) {
//		headerTime += System.currentTimeMillis() - headerTimeStart;
//		}

//		if (timing) {
//		bodyTimeStart = System.currentTimeMillis();
//		}
		if (assertionType == Constants.INTERACTION_PASSERTION_ELEMENT_NAME) {
			insertInteractionPAssertion(assertion, dbPAssertionID);
		} else if (assertionType == Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME) {
			insertActorStatePAssertion(assertion, dbPAssertionID);
		} else if (assertionType == Constants.RELATIONSHIP_PASSERTION_ELEMENT_NAME) {
			insertRelationshipPAssertion(assertion, dbPAssertionID, dbInteractionRecordID, dbViewKindID);
		} else if (assertionType == Constants.INTERACTION_METADATA_ELEMENT_NAME) {
			insertExposedInteractionMetadata(dbInteractionRecordID, dbViewKindID, dbAsserterID, assertion);
		} else if (assertionType == Constants.SUBMISSION_FINISHED_ELEMENT_NAME) {
			insertSubmissionFinished(assertion, asserter, viewKind, intKey, dbAsserterID, dbViewKindID, dbInteractionRecordID, sf);
		}
//		if (timing) {
//		bodyTime += System.currentTimeMillis() - bodyTimeStart;
//		}
		dbInteractionRecordID = -1;
		dbViewKindID = -1;
		passertionTypeID = -1;
		dbAsserterID = -1;
		dbPAssertionID = -1;

	}
	private String getElementContent (Element parent, String childname, DOMDeserialiser domDeserialiser) {
		Element contentE = domDeserialiser.single(parent, childname);
		String content = "";
		NodeList nl = contentE.getChildNodes();
		for (int i = nl.getLength() - 1; i > -1; i--) {
			content += DOMWriter.writeToString(nl.item(i));
		}
		return content;
	}
	private void insertInteractionPAssertion(Element assertion, long dbPAssertionID) throws SQLException {
		//	 Deserialising
		InteractionPAssertion ipa = pstructDeserialiser.deserialiseInteractionPAssertion(assertion);

		//	 Parsing
//		String content = getElementContent(ipa.getContent(), "content", domDeserialiser);

		//	 Insert the contents of a PAssertion
//		synchronized (prepInsertIPA) {
		prepInsertIPA.setLong(1, dbPAssertionID);
		prepInsertIPA.setString(2, DOMWriter.writeToString(assertion));
		prepInsertIPA.setString(3, ipa.getDocumentationStyle());
//		long sqlTimeStart = 0;
//		if (timing) sqlTimeStart = System.currentTimeMillis();
		prepInsertIPA.execute();
//		if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
//		}
	}
	private void insertActorStatePAssertion(Element assertion, long dbPAssertionID) throws SQLException {
		//	Deserialising
		ActorStatePAssertion apa = pstructDeserialiser.deserialiseActorStatePAssertion(assertion);

		//	Parsing
//		Element contentFull = apa.getContent();
//		String content = getElementContent(contentFull, "content", domDeserialiser);
		//	String style = domDeserialiser.text(contentFull, "documentationStyle");

		//	Insert the contents of a PAssertion
//		synchronized(prepInsertIPA) {
		prepInsertIPA.setLong(1, dbPAssertionID);
//		prepInsertAPA.setString(2, content);
		prepInsertIPA.setString(2, DOMWriter.writeToString(assertion));
		prepInsertIPA.setString(3, apa.getDocumentationStyle());
//		long sqlTimeStart = 0;
//		if (timing) sqlTimeStart = System.currentTimeMillis();
		prepInsertIPA.execute();
//		if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
//		}
	}

	private void insertRelationshipPAssertion(
			Element assertion, 
			long dbPAssertionID,
			long dbInteractionRecordID,
			long dbViewKindID
	) throws QueryProblem, SQLException, NothingInsertedException, DatabaseClosedException {
		//	Deserialising
		RelationshipPAssertion rpa = pstructDeserialiser.deserialiseRelationshipPAssertion(assertion);
		SubjectID subjectID = rpa.getSubject();

		//	Parsing
		//	none

		//	Saving the relationhipPAssertion
//		synchronized(prepInsertRPAHeader) {
		prepInsertRPAHeader.setLong(1, dbPAssertionID);
		prepInsertRPAHeader.setLong(2, getDBPAssertionIDForSubject(
				subjectID.getLocalPAssertionID(), 
				dbInteractionRecordID, 
				dbViewKindID));
		prepInsertRPAHeader.setString(3, DOMWriter.writeToString(subjectID.getDataAccessor()));
		prepInsertRPAHeader.setString(4, subjectID.getParameterName());
		prepInsertRPAHeader.setString(5, rpa.getRelation());

//		long sqlTimeStart = 0;
//		if (timing) sqlTimeStart = System.currentTimeMillis();
		prepInsertRPAHeader.execute();
//		if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
//		}

		//	Saving the objects
		List<ObjectID> rpaObjects = rpa.getObjectIDs();
		for (ObjectID o: rpaObjects) {
			GlobalPAssertionKey gk = o.getGlobalPAssertionKey();
			Link objLink = o.getLink();
			String objectLink = objLink != null  ? objLink.getStoreURL(): null;
//			objectLink = objectLink != null ? ("'" + objectLink + "'") : "null";
			InteractionKey oik = gk.getInteractionKey();
//			synchronized (prepInsertRPAObjects) {
			prepInsertRPAObjects.setLong(1, dbPAssertionID);
			prepInsertRPAObjects.setLong(2, getDBPAssertionID(getDBInteractionRecordID(oik, false), 
					gk.getLocalPAssertionID(),
					getDBViewKindID(gk.getViewKind())));
			prepInsertRPAObjects.setString(3,DOMWriter.writeToString(o.getDataAccessor()));
			prepInsertRPAObjects.setString(4,o.getParameterName());
			prepInsertRPAObjects.setString(5,objectLink);
//			long sqlTimeStart = 0;
//			if (timing) sqlTimeStart = System.currentTimeMillis();
			prepInsertRPAObjects.execute();
//			if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
//			}
		}
	}
	private void insertExposedInteractionMetadata(
			long dbInteractionRecordID, 
			long dbViewKindID, 
			long dbAsserterID, 
			Element assertion 
//			long dbPAssertionID
	) throws QueryProblem, SQLException, NothingInsertedException, DatabaseClosedException {
		//	Debug.msg(DOMWriter.writeToString(assertion, true));
//		Debug.err(NormalisingDOMWriter.writeToString(assertion));
		ExposedInteractionMetadata eim = pstructDeserialiser.deserialiseExposedInteractionMetadata(assertion);
//		PStructureSerialiser pser = new PStructureSerialiser();
//		Debug.err(NormalisingDOMWriter.writeToString(pser.serialiseExposedInteractionMetadata(eim)));
		GlobalPAssertionKey gpak = eim.getGlobalPAssertionKey();

		// Inserting the header of the ExposedInteractionMetaData
		InteractionKey ik = gpak.getInteractionKey();
		long dbEIMID;
//		synchronized (prepInsertEIMHeader) {
//		prepInsertEIMHeader.setLong(1, dbPAssertionID);
		prepInsertEIMHeader.setLong(1, dbInteractionRecordID);
		prepInsertEIMHeader.setLong(2, dbViewKindID);
		prepInsertEIMHeader.setLong(3, dbAsserterID);
		prepInsertEIMHeader.setLong(4, getDBPAssertionID(getDBInteractionRecordID(ik, false), 
				gpak.getLocalPAssertionID(),
				getDBViewKindID(gpak.getViewKind())));
//		long sqlTimeStart = 0;
//		if (timing) sqlTimeStart = System.currentTimeMillis();
		prepInsertEIMHeader.execute();
//		if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
		ResultSet rs = prepInsertEIMHeader.getGeneratedKeys();
		rs.next();
		dbEIMID = rs.getLong(1);
//		}
		// Inserting the metadata
		List<Element> metadata = eim.getInteractionMetadata();
//		Debug.msg("Metadata size, record: " + metadata.size());
		for (Element el: metadata) {
//			synchronized(prepInsertEIMObjects) {
			prepInsertEIMObjects.setLong(1, dbEIMID); 
			prepInsertEIMObjects.setString(2, DOMWriter.writeToString(el));
//			long sqlTimeStart = 0;
//			if (timing) sqlTimeStart = System.currentTimeMillis();
			prepInsertEIMObjects.execute();
//			if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
//			}
		}
	}	
	private void insertSubmissionFinished(
			Element assertion, 
			Element asserter, 
			String viewKind, 
			InteractionKey intKey,
			long dbAsserterID,
			long dbViewKindID,
			long dbInteractionRecordID,
			int sf
	) throws SQLException, NumberOfExpectedPAssertionsAlreadyThereException {
		int numberOfExpectedPAssertions = pstructDeserialiser.deserialiseNumberOfExpectedMsgs(assertion);
		if (sf != -1) throw new NumberOfExpectedPAssertionsAlreadyThereException("Database: " + sf + " sent: " + numberOfExpectedPAssertions);
//		synchronized(prepUpdateSF) {
		prepUpdateSF.setInt(1, numberOfExpectedPAssertions);
		prepUpdateSF.setLong(2, dbAsserterID);
		prepUpdateSF.setLong(3, dbViewKindID);
		prepUpdateSF.setLong(4, dbInteractionRecordID);
//		long sqlTimeStart = 0;
//		if (timing) sqlTimeStart = System.currentTimeMillis();
		prepUpdateSF.execute();
//		if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
//		}
	}

//	private boolean timing = false;
//	private long headerTime;
//	private long bodyTime;
//	private long headerTimeStart;
//	private long bodyTimeStart;

//	private long sqlTime;

	public void startTiming() {
//		timing = true;
//		headerTime = 0;
//		bodyTime = 0;
//		sqlTime = 0;
////		db.startTiming();
	}

	public Map<String, Long> stopTiming() { //String pref, String suf) {
//		timing = false;
//		HashMap<String, Long> map = new HashMap<String, Long>();
//		map.put("Record Header", headerTime);
//		map.put("Record Body", bodyTime);
//		map.put("Record SQLTiming", sqlTime);
//		map.put("Record JavaTime", headerTime + bodyTime - sqlTime);

//		return map;
		return null;
		/*		pref +  + suf + "\t" + headerTime + "\n" + 
		pref + " Record Body " + suf + "\t" + bodyTime + "\n" +
		pref + " Record SQLTiming " + suf + "\t" + sqlTime + "\n" +
		pref + " Record JavaTime " + suf + "\t" + (headerTime + bodyTime - sqlTime);*/
	}

	private int getSF(
			long dbAsserterID,
			long dbViewKindID,
			long dbInteractionRecordID
	) throws SQLException{
//		synchronized (prepQuerySF) {
		prepQuerySF.setLong(1, dbAsserterID);
		prepQuerySF.setLong(2, dbViewKindID);
		prepQuerySF.setLong(3, dbInteractionRecordID);
//		long sqlTimeStart = 0;
//		if (timing) sqlTimeStart = System.currentTimeMillis();
		ResultSet rs = prepQuerySF.executeQuery();
//		if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
		if (rs.next()) {
			return rs.getInt(1);
		} else {
//			synchronized(prepInsertSF) {
			prepInsertSF.setLong(1, dbAsserterID);
			prepInsertSF.setLong(2, dbViewKindID);
			prepInsertSF.setLong(3, dbInteractionRecordID);
//			sqlTimeStart = 0;
//			if (timing) sqlTimeStart = System.currentTimeMillis();
			prepInsertSF.execute();
//			if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
			return -1;	
		}
//		}
//		}
	}

	private long getIDSingleTextColumn(
			String str, 
			PreparedStatement prepQ, 
			PreparedStatement prepI
	) throws SQLException, DatabaseClosedException{
		long id;
//		synchronized (prepQ) {
			prepQ.setString(1, str);
//			long sqlTimeStart = 0;
//			if (timing) sqlTimeStart = System.currentTimeMillis();
			ResultSet rs = prepQ.executeQuery();
//			if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
//			SQLWarning w = rs.getWarnings();
//			if (db.isClosed()) throw new DatabaseClosedException();
			if (rs.next()) {
				id = rs.getLong(1);
			} else {
				rs.close();
//				synchronized (prepI) {
					prepI.setString(1, str);
//					sqlTimeStart = 0;
//					if (timing) sqlTimeStart = System.currentTimeMillis();
					prepI.executeUpdate();
//					if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
					rs = prepI.getGeneratedKeys();
					rs.next();
					id = rs.getLong(1);
//				}
			}
//		}
		return id;
	}


	private long getIDSingleTextColumn(
			String str, 
			PreparedStatement prepQ, 
			PreparedStatement prepI,
			Map<String, Long> cache
	) throws SQLException, DatabaseClosedException{
//		synchronized (cache) {
//			ResultSet rs2 = db.executeQuery("select dbEndPointID from EndPointReferences where content = '<Address xmlns = \"http://schemas.xmlsoap.org/ws/2004/03/addressing\">http://sender.com</Address>'");
			Long cached = cache.get(str);
			if (cached != null) { 
				return cached.longValue();
			}
			long id = getIDSingleTextColumn(str, prepQ, prepI);
			cache.put(str, id);
			return id;
//		}
	}

	private long getIDSingleTextColumnVK(
			String str, 
			PreparedStatement prepQ, 
			PreparedStatement prepI
	) throws SQLException, DatabaseClosedException{
//		synchronized (maps) {
//			ResultSet rs2 = db.executeQuery("select dbEndPointID from EndPointReferences where content = '<Address xmlns = \"http://schemas.xmlsoap.org/ws/2004/03/addressing\">http://sender.com</Address>'");
			Long cached = maps.getDBViewKindIDUnsafe(str);
			if (cached != null) { 
				return cached.longValue();
			}
			long id = getIDSingleTextColumn(str, prepQ, prepI);
			maps.putDBViewKindID(str, id);
			return id;
//		}
	}


	private long getIDSingleTextColumnPT(
			String str, 
			PreparedStatement prepQ, 
			PreparedStatement prepI
	) throws SQLException, DatabaseClosedException{
			Long cached = maps.getPassertionTypeIDUnsafe(str);
			if (cached != null) { 
				return cached.longValue();
			}
			long id = getIDSingleTextColumn(str, prepQ, prepI);
			maps.putPAssertionTypeID(str, id);
			return id;
	}

//	private int _maxListSize = 20;
/*
	private long getIDSingleTextColumn(
			String str, 
			PreparedStatement prepQ, 
			PreparedStatement prepI,
			Map<String, Long> cache,
			LinkedList<String> list
	) throws SQLException, DatabaseClosedException{
		Long cached = cache.get(str);
		if (cached != null) { 
			return cached.longValue();
		}
//		int ind = list.indexOf(str);
		list.remove(str);
		list.addFirst(str);
		if (list.size() > _maxListSize) {
			String last = list.getLast();
			list.removeLast();
			cache.remove(last);
		}
		long id = getIDSingleTextColumn(str, prepQ, prepI);
		cache.put(str, id);
		return id;
	}
*/
//	private Map<String, Long> _paTypeCache = new HashMap<String, Long>();
	private long getPAssertionTypeID(String kindOfAssertion) throws NothingInsertedException, SQLException, DatabaseClosedException {
		return getIDSingleTextColumnPT(kindOfAssertion, prepQueryPAssertionTypeID, prepInsertPAssertionTypeID);
//		return getIDSingleTextColumn(kindOfAssertion, prepQueryPAssertionTypeID, prepInsertPAssertionTypeID);		
	}

//	private Map<String, Long> _viewKindCache = new HashMap<String, Long>();
	private long getDBViewKindID (String viewKind) throws NothingInsertedException, SQLException, DatabaseClosedException {
		return getIDSingleTextColumnVK(viewKind, prepQueryDBViewKindID, prepInsertDBViewKindID);
//		return getIDSingleTextColumn(viewKind, prepQueryDBViewKindID, prepInsertDBViewKindID);
	}

	private long getDBInteractionRecordID (InteractionKey intKey, boolean isLocal) throws NothingInsertedException, SQLException, DatabaseClosedException {
		long messageSource = getDBEndPointID(NormalisingDOMWriter.writeToString(intKey.getMessageSource()));
		long messageSink = getDBEndPointID(NormalisingDOMWriter.writeToString(intKey.getMessageSink()));
		String interactionID = intKey.getInteractionID();
		long dbInteractionRecordID;

//		synchronized(prepQueryDBInteractionRecordID) {
			prepQueryDBInteractionRecordID.setLong(1, messageSource);
			prepQueryDBInteractionRecordID.setLong(2, messageSink);
			prepQueryDBInteractionRecordID.setString(3, interactionID);

//			long sqlTimeStart = 0;
//			if (timing) sqlTimeStart = System.currentTimeMillis();
			ResultSet rs = prepQueryDBInteractionRecordID.executeQuery();
//			if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;

			if (rs.next()) {
				dbInteractionRecordID = rs.getLong(1);
				if (isLocal && rs.getString(2).equals("n")) {
//					synchronized (prepUpdateDBInteractionRecordID) {
						prepUpdateDBInteractionRecordID.setLong(1, dbInteractionRecordID);
//						sqlTimeStart = 0;
//						if (timing) sqlTimeStart = System.currentTimeMillis();
						prepUpdateDBInteractionRecordID.execute();
//						if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
//					}
				}
				rs.close();
			} else {
//				synchronized(prepInsertDBInteractionRecordID) {
					prepInsertDBInteractionRecordID.setLong(1, messageSource);
					prepInsertDBInteractionRecordID.setLong(2, messageSink);
					prepInsertDBInteractionRecordID.setString(3, interactionID);
					prepInsertDBInteractionRecordID.setString(4, isLocal ? "y" : "n");

//					sqlTimeStart = 0;
//					if (timing) sqlTimeStart = System.currentTimeMillis();
					prepInsertDBInteractionRecordID.executeUpdate();
//					if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;

					rs = prepInsertDBInteractionRecordID.getGeneratedKeys();
					rs.next();
					dbInteractionRecordID = rs.getLong(1);
//				}
			}
//		}
		return dbInteractionRecordID;
	}

	private long getDBEndPointID (String content) throws NothingInsertedException, SQLException, DatabaseClosedException {
		return getIDSingleTextColumn(content, prepQueryDBEndPointID, prepInsertDBEndPointID);
	}

	private long getDBAsserterID (String asserter) throws NothingInsertedException, SQLException, DatabaseClosedException {
		return getIDSingleTextColumn(asserter, prepQueryDBAsserterID, prepInsertDBAsserterID);
	}

	private long getDBPAssertionIDForSubject(
			String localPAssertionID,
			long dbInteractionRecordID,
			long dbViewKindID
	) throws SQLException, NothingInsertedException {
		return getDBPAssertionID(dbInteractionRecordID, localPAssertionID, dbViewKindID);
	}

	private long getDBPAssertionIDForObject(
			long dbInteractionKey, 
			String localPAssertionID, 
			long dbViewKindID,
			long dbInteractionRecordID
	) throws SQLException, NothingInsertedException {

		return getDBPAssertionID(dbInteractionRecordID, localPAssertionID, dbViewKindID);
	}

	private long getDBPAssertionID (
			long dbInteractionRecordID, 
			String localPAssertionID, 
			long dbViewKindID, 
			long dbAsserterID, 
			long passertionTypeID
//			String fullXMLContent, 
//			boolean ignoreLocalPAID
	) throws SQLException, NothingInsertedException{

		ResultSet rs = null;
//		synchronized(prepQueryDBPAssertionIDWithAsserter) {
//			if (!ignoreLocalPAID) {
			prepQueryDBPAssertionIDWithAsserter.setString(1, localPAssertionID);
			prepQueryDBPAssertionIDWithAsserter.setLong(2, dbViewKindID);
			prepQueryDBPAssertionIDWithAsserter.setLong(3, dbAsserterID);
			prepQueryDBPAssertionIDWithAsserter.setLong(4, dbInteractionRecordID);
//			long sqlTimeStart = 0;
//			if (timing) sqlTimeStart = System.currentTimeMillis();
			rs = prepQueryDBPAssertionIDWithAsserter.executeQuery();
//			if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
//			}
			if (rs.next()) {
				long dbPAssertionID = rs.getLong(1); 
				if (rs.getString(2).equals("n")) {
//					synchronized(prepUpdateDBPAssertionID) {
						prepUpdateDBPAssertionID.setLong(1, passertionTypeID);
//						prepUpdateDBPAssertionID.setString(2, fullXMLContent);
						prepUpdateDBPAssertionID.setLong(2, dbAsserterID);
						prepUpdateDBPAssertionID.setLong(3, dbPAssertionID);
//						sqlTimeStart = 0;
//						if (timing) sqlTimeStart = System.currentTimeMillis();
						prepUpdateDBPAssertionID.execute();
//						if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
//					}
				}
				return dbPAssertionID;
			} else {
//				synchronized(prepInsertDBPAssertionIDLocal) {
					if (localPAssertionID == null) localPAssertionID = ""; 
					prepInsertDBPAssertionIDLocal.setLong(1, dbInteractionRecordID); 
					prepInsertDBPAssertionIDLocal.setString(2, localPAssertionID);
					prepInsertDBPAssertionIDLocal.setLong(3, dbViewKindID);
					prepInsertDBPAssertionIDLocal.setLong(4, dbAsserterID);
					prepInsertDBPAssertionIDLocal.setLong(5, passertionTypeID);
//					prepInsertDBPAssertionIDLocal.setString(6, fullXMLContent);
//					sqlTimeStart = 0;
//					if (timing) sqlTimeStart = System.currentTimeMillis();
					prepInsertDBPAssertionIDLocal.execute();
//					if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
					rs = prepInsertDBPAssertionIDLocal.getGeneratedKeys();
					rs.first();
					return rs.getLong(1);
//				}
			}
//		}
	}

	private long getDBPAssertionID (
			long dbInteractionRecordID, 
			String localPAssertionID, 
			long dbViewKindID 
//			boolean ignoreLocalPAID
	) throws SQLException, NothingInsertedException{
		ResultSet rs = null;
//		synchronized(prepQueryDBPAssertionIDWithAsserter) {
//			if (!ignoreLocalPAID) {
			prepQueryDBPAssertionIDWithoutAsserter.setString(1, localPAssertionID);
			prepQueryDBPAssertionIDWithoutAsserter.setLong(2, dbViewKindID);
			prepQueryDBPAssertionIDWithoutAsserter.setLong(3, dbInteractionRecordID);
//			long sqlTimeStart = 0;
//			if (timing) sqlTimeStart = System.currentTimeMillis();
			rs = prepQueryDBPAssertionIDWithoutAsserter.executeQuery();
//			if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
//			}
			if (rs.next()) {
				long dbPAssertionID = rs.getLong(1); 
				return dbPAssertionID;
			} else {
//				synchronized(prepInsertDBPAssertionIDRemote) {
					prepInsertDBPAssertionIDRemote.setLong(1, dbInteractionRecordID);
					prepInsertDBPAssertionIDRemote.setString(2, localPAssertionID);
					prepInsertDBPAssertionIDRemote.setLong(3, dbViewKindID);

//					sqlTimeStart = 0;
//					if (timing) sqlTimeStart = System.currentTimeMillis();
					prepInsertDBPAssertionIDRemote.execute();
//					if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;

					rs = prepInsertDBPAssertionIDRemote.getGeneratedKeys();
					rs.first();
					return rs.getLong(1);
//				}
			}
//		}
	}

}