package org.pasoa.preserv.storage.mysql;

import java.io.IOException;
import java.io.StringReader;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.pasoa.common.Constants;
import org.pasoa.preserv.storage.Retrieval;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.ExposedInteractionMetadata;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.Link;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.pstructure.RelationshipPAssertion;
import org.pasoa.pstructure.View;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * 
 * @author Andrej Kazakov
 *
 */
public class MySQLRetrieval implements Retrieval {

	private MySQLConnector db;

	private PreparedStatement prepGetIK;
	private PreparedStatement prepGetPAH;
	private PreparedStatement prepGetDBAsserterID;
	private PreparedStatement prepGetAsserter;
	private PreparedStatement prepGetSF;
	private PreparedStatement prepGetdbIRID;
	private PreparedStatement prepGetDBEIMID;
	private PreparedStatement prepGetEIMO;
	private PreparedStatement prepGetRPAH;
	private PreparedStatement prepGetRPAO;
	private PreparedStatement prepGetSP;
	private PreparedStatement prepGetCount;
	private MySQLMaps maps;

	public MySQLRetrieval(MySQLConnector db, MySQLMaps maps) throws SQLException {
		this.db = db;
		this.maps = maps;
		initialiseQueries();
	}

	private void initialiseQueries() throws SQLException{
		prepGetIK = db.getPreparedStatement("select sourcet.content as messageSource, sinkt.content as messageSink, interactionID " +
				"from InteractionRecords, EndPointReferences as sourcet, EndPointReferences as sinkt " +
				"where InteractionRecords.messageSource = sourcet.dbEndPointID and " +
				"InteractionRecords.messageSink = sinkt.dbEndPointID and " +
		"InteractionRecords.dbInteractionRecordID = ?");

		prepGetPAH = db.getPreparedStatement("select dbPAssertionID, passertionTypeID, localPAssertionID " +
				"from PAssertions where dbInteractionRecordID = ? and dbViewKindID = ? and " +
		"dbAsserterID = ? and isLocal = 'y'");

		prepGetDBAsserterID = db.getPreparedStatement("select dbAsserterID from NExpectedPAssertions " +
		"where dbViewKindID = ? and dbInteractionRecordID = ?");

		prepGetAsserter = db.getPreparedStatement("select content from Asserters " +
		"where dbAsserterID = ?");

		prepGetSF = db.getPreparedStatement("select numberOfExpectedPassertions" +
				" from NExpectedPAssertions where dbViewKindID = ? " +
		" and dbInteractionRecordID = ? and dbAsserterID = ?");

		prepGetdbIRID = db.getPreparedStatement("select dbInteractionRecordID from InteractionRecords where " +
		"isLocal = 'y' order by dbInteractionRecordID limit ?, 1");

		prepGetDBEIMID = db.getPreparedStatement("select dbEIMID, localPAssertionID, PAssertions.dbViewKindID " +
				"from ExposedInteractionMetadata eims, PAssertions " +
				"where eims.dbInteractionRecordID = ? and eims.dbAsserterID = ? and eims.dbViewKindID = ? and " +
		"PAssertions.dbPAssertionID = eims.referenceDBPAssertionID");

		prepGetEIMO = db.getPreparedStatement("select content from MetaData where dbEIMID = ?");

		prepGetRPAH = db.getPreparedStatement("select subjectDataAccessor, " +
				"subjectParameterName, relation, localPAssertionID " +
				"from RelationshipPAssertions, PAssertions " +
				"where RelationshipPAssertions.dbPAssertionID = ? and " +
		"PAssertions.dbPAssertionID = RelationshipPAssertions.subjectDBPAssertionID");

		prepGetRPAO = db.getPreparedStatement("select dbInteractionRecordID, localPAssertionID, " +
				"dataAccessor, parameterName, objectLink, dbViewKindID " +
				"from RelationshipPAssertion_objects inner join PAssertions using (dbPAssertionID) " +
		"where parentDBPAssertionID = ?");

		prepGetSP = db.getPreparedStatement("select fullXMLContent, documentationStyle " +
		"from StyledPAssertions where dbPAssertionID = ?");

		prepGetCount = db.getPreparedStatement("select count(*) from InteractionRecords where isLocal = 'y'");
	}

	public synchronized InteractionRecord getInteractionRecord(long index) throws NothingFoundException, SQLException, SAXException, IOException, NotFoundViewKindException{
//		long timeStart = 0;
//		if (timing) timeStart = System.currentTimeMillis(); 
		if (index < 0 || index >= _countIRs) throw new NothingFoundException();
		long dbInteractionRecordID = getDBLocalIntID(index);
		InteractionRecord ir = getInteractionRecordByID(dbInteractionRecordID);
//		if (timing) bodyTime += System.currentTimeMillis() - timeStart;
		return ir;
	}

	public synchronized InteractionRecord getInteractionRecordByID(long dbInteractionRecordID) throws SQLException, SAXException, IOException, NotFoundViewKindException{

//		long start = System.currentTimeMillis();
		InteractionRecord ir;
//		synchronized(prepGetIK) {
//		prepGetIK.setLong(1, dbInteractionRecordID);
//		long sqlTimeStart = 0;
//		if (timing) sqlTimeStart = System.currentTimeMillis(); 
//		ResultSet rs = prepGetIK.executeQuery();
//		if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
//		rs.first();

		ir = new InteractionRecord(
				getIK(dbInteractionRecordID),
				getViews(Constants.SENDER_VIEW_TYPE, dbInteractionRecordID),
				getViews(Constants.RECEIVER_VIEW_TYPE, dbInteractionRecordID),
				getViews(Constants.INFERRED_VIEW_TYPE, dbInteractionRecordID)
		);
//		}
//		Debug.msg("Retrieve Total Time\t" + (System.currentTimeMillis() - start));
		return ir;
	}

	private InteractionKey getIK(long dbInteractionRecordID) throws SQLException, SAXException, IOException {
		prepGetIK.setLong(1, dbInteractionRecordID);
		ResultSet rs = prepGetIK.executeQuery();
		rs.first();
		return new InteractionKey(
				string2Element(rs.getString(1)),
				string2Element(rs.getString(2)),
				rs.getString(3)
		);
	}

	List<View> getViews(String strViewKind, long dbInteractionRecordID) throws SQLException, SAXException, IOException, NotFoundViewKindException {

		Long viewKindO = maps.getDBViewKindIDUnsafe(strViewKind);
		if (viewKindO != null) {
			long dbViewKindID = viewKindO;

			prepGetDBAsserterID.setLong(1, dbViewKindID);
			prepGetDBAsserterID.setLong(2, dbInteractionRecordID);

			ResultSet rsAsserter = prepGetDBAsserterID.executeQuery();

			List<View> views = new ArrayList<View>();
			while (rsAsserter.next()) {
				long dbAsserterID = rsAsserter.getLong(1);
				views.add(getView(dbInteractionRecordID, dbViewKindID, dbAsserterID));
			}
			return views; 
		} // Null check on ViewKind map
		else {
			//throw new NotFoundViewKindException(strViewKind);
			// The viewKind has not been recorded yet, but is already queried by the getViews method
			return null;
		}
	}

	View getView(long dbInteractionRecordID, long dbViewKindID, long dbAsserterID) throws SQLException, SAXException, IOException {
		prepGetPAH.setLong(1, dbInteractionRecordID);
		prepGetPAH.setLong(2, dbViewKindID);
		prepGetPAH.setLong(3, dbAsserterID);
		ResultSet rsPA = prepGetPAH.executeQuery();
		List<ActorStatePAssertion> apas = new ArrayList<ActorStatePAssertion>();
		List<InteractionPAssertion> ipas = new ArrayList<InteractionPAssertion>();
		List<RelationshipPAssertion> rpas = new ArrayList<RelationshipPAssertion>();
		while (rsPA.next()) {
			long dbPAssertionID = rsPA.getLong(1);
			String passertionType = maps.getPassertionType(rsPA.getLong(2)).intern();
			String localPAssertionID = rsPA.getString(3);

			if (passertionType == Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME) {
				apas.add(getAPA(localPAssertionID, dbPAssertionID));
			} else if (passertionType == Constants.INTERACTION_PASSERTION_ELEMENT_NAME) {
				ipas.add(getIPA(localPAssertionID, dbPAssertionID));
			} else if (passertionType == Constants.RELATIONSHIP_PASSERTION_ELEMENT_NAME) {
				rpas.add(getRPA(localPAssertionID, dbPAssertionID));
			}

		}

//		Get Asserter
		prepGetAsserter.setLong(1, dbAsserterID);
		ResultSet rsTmp = prepGetAsserter.executeQuery();
		rsTmp.first();
		Element asserterEl = string2Element(rsTmp.getString(1));
		rsTmp.close();

		return new View(
				ipas, 
				apas, 
				rpas,
				getExposedInteractionMetadata(dbViewKindID, dbInteractionRecordID, dbAsserterID),
				getSF(dbViewKindID, dbInteractionRecordID, dbAsserterID),
				asserterEl
		);
	}

	private int getSF(long dbViewKindID, long dbInteractionRecordID, long dbAsserterID) throws SQLException {
		prepGetSF.setLong(1, dbViewKindID);
		prepGetSF.setLong(2, dbInteractionRecordID);
		prepGetSF.setLong(3, dbAsserterID);
		ResultSet rsTmp = prepGetSF.executeQuery();
		rsTmp.first();
		return rsTmp.getInt(1);
	}
	private long getDBLocalIntID(long index) throws SQLException{
//		synchronized (prepGetdbIRID) {
		prepGetdbIRID.setLong(1, index);

//		long sqlTimeStart = 0;
//		if (timing) sqlTimeStart = System.currentTimeMillis(); 
		ResultSet rs = prepGetdbIRID.executeQuery();
//		if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;

		rs.first();
		return rs.getLong(1);
//		}
	}

	private List<ExposedInteractionMetadata> getExposedInteractionMetadata (
			long dbViewKindID, 
			long dbInteractionRecordID, 
			long dbAsserterID
	) throws SQLException, SAXException, IOException {
		prepGetDBEIMID.setLong(1, dbInteractionRecordID);
		prepGetDBEIMID.setLong(2, dbAsserterID);
		prepGetDBEIMID.setLong(3, dbViewKindID);
		ResultSet rsEIM = prepGetDBEIMID.executeQuery();

		InteractionKey ik = getIK(dbInteractionRecordID);

		List<ExposedInteractionMetadata> eims = new ArrayList<ExposedInteractionMetadata>();
		try {
			while (rsEIM.next()) {
				String referenceLP = rsEIM.getString(2);
				long refDBViewKindID = rsEIM.getLong(3);

				prepGetEIMO.setLong(1, rsEIM.getLong(1));

				ResultSet rsM = prepGetEIMO.executeQuery();

				List<Element> metadata = new ArrayList<Element>();
				while (rsM.next()) {
					metadata.add(string2Element(rsM.getString(1)));
				}
				eims.add(new ExposedInteractionMetadata(
						new GlobalPAssertionKey(
								ik, 
								maps.getViewKind(refDBViewKindID),
								referenceLP
						),
						metadata
				));
			}
		} catch (Exception ex) {
			Debug.err(ex, "Failed to query ExposedInteractionMetadata");
		}
		return eims;
//		}
	}

	RelationshipPAssertion getRPA(String localPAssertionID, long dbPAssertionID) throws SQLException, SAXException, IOException {

		// Getting the header of the relationship
		prepGetRPAH.setLong(1, dbPAssertionID);

		ResultSet rsHeader = prepGetRPAH.executeQuery();
//		rsHeader.first();
		if (!rsHeader.first()) {
			ResultSet rsC = db.executeQuery("select count(*) from RelationshipPAssertions where dbPAssertionID = " + dbPAssertionID);
			rsC.first();
			long count1 = rsC.getLong(1);

			try {
				Thread.sleep(7000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			rsC = db.executeQuery("select count(*) from RelationshipPAssertions where dbPAssertionID = " + dbPAssertionID);
			rsC.first();
			long count2 = rsC.getLong(1);

			Debug.err("Could not read R assertion: " + dbPAssertionID + " found with the same ID " + count1 + " and then " + count2);
//			System.exit(1);
		}

		String subjectDataAccessor = rsHeader.getString(1);
		String subjectParameterName = rsHeader.getString(2);
		String relation = rsHeader.getString(3);
		String subjectLocalPAssertionID = rsHeader.getString(4);

		prepGetRPAO.setLong(1, dbPAssertionID);
		ResultSet rsO = prepGetRPAO.executeQuery();
		List <ObjectID> objects = new ArrayList<ObjectID>();
		while (rsO.next()) {
			Link objLink;
			String objLinkStr = rsO.getString(5);
			if (objLinkStr == null) objLink = null;
			else objLink = new Link(objLinkStr); 
			objects.add(new ObjectID(
					new GlobalPAssertionKey(
							getIK(rsO.getLong(1)),
							maps.getViewKind(rsO.getLong(6)),
							rsO.getString(2)
					),
					rsO.getString(4),
					string2Element(rsO.getString(3)),
					objLink
			));


		}
		return new RelationshipPAssertion(
				localPAssertionID,
				subjectLocalPAssertionID,
				string2Element(subjectDataAccessor),
				subjectParameterName,
				relation,
				objects
		);
	}

	ActorStatePAssertion getAPA(String localPAssertionID, long dbPAssertionID) throws SQLException, SAXException, IOException {
		prepGetSP.setLong(1, dbPAssertionID);
		ResultSet rs = prepGetSP.executeQuery();
//		rs.first();
		if (!rs.first()) {
			ResultSet rsC = db.executeQuery("select count(*) from RelationshipPAssertions where dbPAssertionID = " + dbPAssertionID);
			rsC.first();
			long count1 = rsC.getLong(1);

			try {
				Thread.sleep(7000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			rsC = db.executeQuery("select count(*) from RelationshipPAssertions where dbPAssertionID = " + dbPAssertionID);
			rsC.first();
			long count2 = rsC.getLong(1);

			Debug.err("Could not read A assertion: " + dbPAssertionID + " found with the same ID " + count1 + " and then " + count2);
//			System.exit(1);
		}
		return new ActorStatePAssertion(localPAssertionID, rs.getString(2), string2Element(rs.getString(1)));
	}

	InteractionPAssertion getIPA(String localPAssertionID, long dbPAssertionID) throws SQLException, SAXException, IOException {
		prepGetSP.setLong(1, dbPAssertionID);
		ResultSet rs = prepGetSP.executeQuery();
//		rs.first();
		if (!rs.first()) {
			ResultSet rsC = db.executeQuery("select count(*) from RelationshipPAssertions where dbPAssertionID = " + dbPAssertionID);
			rsC.first();
			long count1 = rsC.getLong(1);

			try {
				Thread.sleep(7000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			rsC = db.executeQuery("select count(*) from RelationshipPAssertions where dbPAssertionID = " + dbPAssertionID);
			rsC.first();
			long count2 = rsC.getLong(1);

			Debug.err("Could not read I assertion: " + dbPAssertionID + " found with the same ID " + count1 + " and then " + count2);
//			System.exit(1);
		}

		return new InteractionPAssertion(localPAssertionID, rs.getString(2), string2Element(rs.getString(1)));
	}

	private long _countIRs;

	public synchronized long getNumberOfInteractionRecords() throws SQLException {
//		synchronized (prepGetCount) {
//		long sqlTimeStart = 0;
//		if (timing) sqlTimeStart = System.currentTimeMillis(); 
		ResultSet rs = prepGetCount.executeQuery();
//		if (timing) sqlTime += System.currentTimeMillis() - sqlTimeStart;
		rs.first();
		_countIRs = rs.getLong(1);
		return _countIRs;
//		}
	}

	private static DocumentBuilder docBuilder;

	private static void initBuilder() {
		try {
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			docBuilderFactory.setNamespaceAware(true);
			docBuilder = docBuilderFactory.newDocumentBuilder();
		} catch (ParserConfigurationException ex) {
			throw new RuntimeException(ex);
		}
	}

//	private boolean timing = false;
//	private long bodyTime;

//	private long sqlTime;

	public void startTiming() {
//		timing = true;
//		bodyTime = 0;
//		sqlTime = 0;
////		db.startTiming();
	}

	public Map<String, Long> stopTiming() { //(String pref, String suf) {
//		timing = false;

//		HashMap<String, Long> map = new HashMap<String, Long>();
//		map.put("Retrieve Total", bodyTime);
//		map.put("Retrieve SQLTiming", sqlTime);
//		map.put("Retrieve JavaTime", bodyTime - sqlTime);

//		return map;
		return null;

		/*		return 
		pref + " Retrieve Total " + suf + "\t" + bodyTime + "\n" + 
		pref + " Retrieve SQLTiming " + suf + "\t" + sqlTime + "\n" +
		pref + " Retrieve JavaTime " + suf + "\t" + (bodyTime - sqlTime);*/
	}

	public static Element string2Element (String str) throws SAXException, IOException{
		if (docBuilder == null) initBuilder();
		return docBuilder.parse(new InputSource(new StringReader(str))).getDocumentElement();
	}

}
