package org.pasoa.preserv.storage.mysql;

import java.io.InputStream;
import java.io.PrintStream;
import java.io.UnsupportedEncodingException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.prefs.BackingStoreException;

import javax.xml.parsers.ParserConfigurationException;

import org.pasoa.preserv.index.Index;
import org.pasoa.preserv.index.MyIndex;
import org.pasoa.preserv.index.IndexProblem;
import org.pasoa.preserv.record.CacheableItem;
import org.pasoa.preserv.record.Record;
import org.pasoa.preserv.servlet.management.DefaultInitialConfigImpl;
import org.pasoa.preserv.storage.BackendStore;
import org.pasoa.preserv.storage.InitialConfig;
import org.pasoa.preserv.storage.Retrieval;
import org.pasoa.preserv.xquery.QueryProblem;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.PAssertion;
import org.pasoa.pstructure.View;
import org.w3c.dom.Element;

import com.mysql.jdbc.CommunicationsException;
import com.sleepycat.je.DatabaseException;

/**
 * This class provides the functionality to use MySQL for PreServ.
 * 
 * @author Andrej Kazakov
 *
 */

public class MySQLStore implements Index, MyIndex, BackendStore {

//	private MySQLConnector dbretr, dbcacheI, dbcacheQ, dbrec, dbManagement, dbIndex, dbDebug, dbMaps;
	private MySQLConnector[] dbs;
	final static String _sDBUser = "preserv";
	private final static String _sDBPass = "QahdEkco22!$^fvjJfr43#fdjewmRTHTr43fd";
	final static String _sDBName = "preserv";
	private final static String _sDBHost = "jdbc:mysql://localhost/";
	private final static String _sDBURL = _sDBHost + _sDBName;
	private String dbUser = _sDBUser;
	private String dbPass = _sDBPass;
	private String dbName = _sDBName;
	private String dbHost = _sDBHost;
	private String dbURL = _sDBURL;
	private MySQLRetrieval retrieval;
	private MySQLRecord record;
	private MySQLCache cache;
	private MySQLManagement management;
	private MySQLIndex index;
	private MySQLMaps maps;
	private int n = 8;

//	private static int count = 0;
	
	public MySQLStore () throws SQLException {
		connect();
//		count++;
	}
	
	private boolean checkDBName(String name) {
		return 
		name != null && 
		name.length() != 0 && 
		!name.contains("/") && 
		!name.contains(":") && 
		!name.contains("'") && 
		!name.contains(";");
	}
	
	public MySQLStore (InitialConfig config) throws SQLException {
		dbName = config.getInitParameter("dbName");
		if (dbName == null) dbName =  _sDBName;
		dbHost = config.getInitParameter("dbHost");
		if (dbHost == null) dbHost = _sDBHost;
		
		if (checkDBName(dbName)) {
			dbURL = dbHost + dbName;
			connect(dbURL);
		}
	}

	public MySQLStore (String dbname) throws SQLException {
		if (checkDBName(dbname)) {
			connect(_sDBHost + dbname);
		} else throw new IllegalArgumentException("Database name is not valid " + dbname);
	}

//	public MySQLStore (Preferences pref) throws SQLException {
//		dbName = pref.get("dbName", _sDBName);
//		dbHost = pref.get("dbHost", _sDBHost);
//		if (checkDBName(dbName)) {
//			dbURL = dbHost + dbName;
//			connect(dbURL);
//		}
//	}

//	public Preferences setCurrentSettings (Preferences pref) {
//		pref.put("dbName", dbName);
//		pref.put("dbHost", dbHost);
//		return pref;
//	}
	
	public static InitialConfig getDefaultInitConfig() {
		DefaultInitialConfigImpl conf = new DefaultInitialConfigImpl();
		conf.setInitParameter("dbName", _sDBName);
		conf.setInitParameter("dbHost", _sDBHost);
		return conf;
	}

	public MySQLStore (String dbURL, String dbUser, String dbPass) throws SQLException {
		connect(dbURL, dbUser, dbPass);
//		count++;
	}

	private synchronized void connect(String dbURL) throws SQLException {
		this.dbURL = dbURL;
		connect();
	}
	
//	public static int getNumberOfInstances() {
//		return count;
//	}

	private synchronized void connect(String dbURL, String dbUser, String dbPass) throws SQLException {
		this.dbURL = dbURL;
		this.dbPass = dbPass;
		this.dbUser = dbUser;
		connect();
	}


	private synchronized void connect() throws SQLException{
//		setDefaults(Preferences.systemNodeForPackage(this.getClass()));
				
		Debug.msg("Connecting to the DB " + dbURL);
		
		dbs = new MySQLConnector[n];
		for (int i = 0; i < n; i++) {
			dbs[i] = new MySQLConnector(dbURL, dbUser, dbPass);
			dbs[i].connect();
		}
		
		int cdb = 1;
		maps = new MySQLMaps(dbs[cdb++]);
		retrieval = new MySQLRetrieval(dbs[cdb++], maps);
		record = new MySQLRecord(dbs[cdb++], maps);
		cache = new MySQLCache(dbs[cdb++], dbs[cdb++]);
		management = new MySQLManagement(dbs[cdb++]);
		index = new MySQLIndex(dbs[cdb++], retrieval, maps);
		if (cdb != n) throw new ArrayIndexOutOfBoundsException("Array of Connections malformed");
	}

	public synchronized void reconnect() throws SQLException {
		close();
		connect();
	}

	public static MySQLConnector createDefaultConnection() {
		return new MySQLConnector(_sDBURL, _sDBUser, _sDBPass);
	}

	public synchronized void close() {
		Debug.msg("Closing connection to the DB ");
		retrieval = null;
		record = null;
		cache = null;
		management = null;
		maps = null;
		for (int i = 0; i < n; i++) {
			dbs[i].disconnect();
			dbs[i] = null;
		}
	}

	public MySQLConnector getDB(){
		return dbs[0];
	}

	public void cache(CacheableItem ci) throws Exception {
		try {
			cache.cache(ci);
		} catch (CommunicationsException ex) {
			connectionFailed();
			cache.cache(ci);
		}
	}

	
	
	public void cache(InputStream is, String context, boolean compressed) throws Exception {
		try {
			cache.cache(is, context, compressed);
		} catch (CommunicationsException ex) {
			connectionFailed();
			cache.cache(is, context, compressed);
		}
	}



	private int _reconnectionAttempts = 5;
	private int _reconnectionsAttempted = -1;

	private synchronized void connectionFailed() throws SQLException {
		_reconnectionsAttempted++;
		if (_reconnectionsAttempted < _reconnectionAttempts) {
			try {
				reconnect();
				_reconnectionsAttempted = -1;
			} catch (CommunicationsException ex) {
				connectionFailed();
			}
		}
	}

	public boolean cacheEnabled() {
		return cache.cacheEnabled();
	}

	public CacheableItem getCachedItem() throws Exception {
		try {
			return cache.getCachedItem();
		} catch (CommunicationsException ex) {
			connectionFailed();
			return cache.getCachedItem();
		}
	}

	public synchronized void delete(String engine) throws Exception {
		try {
			management.delete(engine);
			maps.reloadMaps();
		} catch (CommunicationsException ex) {
			connectionFailed();
			management.delete(engine);
			maps.reloadMaps();
		}
	}

	public void delete() throws Exception {
		try {
			management.delete();
			maps.reloadMaps();
			cache.resetState();
		} catch (CommunicationsException ex) {
			connectionFailed();
			management.delete();
			maps.reloadMaps();
			cache.resetState();
		}
	}

	public void shutdown() throws Exception {
		close();
	}

	public InteractionRecord findInteractionRecord(InteractionKey key) throws IndexProblem {
		synchronized(record) {
			try {
				return index.findInteractionRecord(key);
			} catch (ConnectionLostIndexProblem ex) {
				try {
					connectionFailed();
					return index.findInteractionRecord(key);
				} catch (SQLException ex2) {
					throw new IndexProblem ("Reconnection failed", ex2);
				}
			}	
		}
	}

	public PAssertion findPAssertion(GlobalPAssertionKey gpak) throws IndexProblem {
		synchronized(record) {
			try {
		return index.findPAssertion(gpak);
			} catch (ConnectionLostIndexProblem ex) {
				try {
					connectionFailed();
					return index.findPAssertion(gpak);
				} catch (SQLException ex2) {
					throw new IndexProblem ("Reconnection failed", ex2);
				}
			}	

		}
	}

	public View findView(InteractionKey key, String viewKind, Element asserter) throws IndexProblem {
		synchronized(record) {
			try {
		return index.findView(key, viewKind, asserter);
			} catch (ConnectionLostIndexProblem ex) {
				try {
					connectionFailed();
					return index.findView(key, viewKind, asserter);
				} catch (SQLException ex2) {
					throw new IndexProblem ("Reconnection failed", ex2);
				}
			}	
		}
	}

	public List<View> findViews(InteractionKey key, String viewKind) throws IndexProblem {
		synchronized(record) {
			try {
		return index.findViews(key, viewKind);
			} catch (ConnectionLostIndexProblem ex) {
				try {
					connectionFailed();
					return index.findViews(key, viewKind);
				} catch (SQLException ex2) {
					throw new IndexProblem ("Reconnection failed", ex2);
				}
			}
		}
	}

	public InteractionRecord getInteractionRecord(long index) throws Exception {
		synchronized(record) {
//			Debug.msg("Currently " + count + "instances are running");
//			Debug.msg("Retrieval started " + dbs[0].getURL());
			try {		
				return retrieval.getInteractionRecord(index);
			} catch (CommunicationsException ex) {
				connectionFailed();
				return retrieval.getInteractionRecord(index);
			}
		}
	}

	public long getNumberOfInteractionRecords() throws Exception {
		synchronized (record) {
		try {
			return retrieval.getNumberOfInteractionRecords();
		} catch (CommunicationsException ex) {
			connectionFailed();
			return retrieval.getNumberOfInteractionRecords();
		}
		}
	}

	public void record(GlobalPAssertionKey gpid, Element asserter, Element assertion, String kindOfAssertion) throws UnrecognisedPAssertionTypeException, NothingInsertedException, SQLException, QueryProblem, NumberOfExpectedPAssertionsAlreadyThereException, DatabaseClosedException{
//		synchronized (retrieval) {
		synchronized (record) {
//			Debug.msg("Record started " + dbs[0].getURL());
			try {
				record.record(gpid, asserter, assertion, kindOfAssertion);
			} catch (CommunicationsException ex) {
				connectionFailed();
				record.record(gpid, asserter, assertion, kindOfAssertion);
			}
//			Debug.msg("Record finished " + dbs[0].getURL());
		}
	}


	public void startTiming(){
		record.startTiming();
		retrieval.startTiming();
//		cache.startTiming();
	}

	public void stopRecordTiming(Map<String, List<Long>> map, String pref){
		/*		return record.stopTiming(pref, suf) + "\n" + 
		retrieval.stopTiming(pref, suf);*/
//		ArrayList<Map<String, Long>> list = new ArrayList<Map<String, Long>>();

		putTime2Map(record.stopTiming(), map, pref);
		putTime2Map(retrieval.stopTiming(), map, pref);
//		return list;
		// + cache.stopTiming();
	}

	private static void putTime2Map(Map<String, Long> time, Map<String, List<Long>> map, String pref) {
		Set<String> keys = time.keySet();
		for (String key: keys) {
			String k = pref + key;
			if (map.containsKey(k)) {
				map.get(k).add(time.get(key));
			} else {
				ArrayList<Long> list = new ArrayList<Long>();
				list.add(time.get(key));
				map.put(k, list);
			}
		}
	}

	public static void printTimeMap(PrintStream out, Map<String, List<Long>> map) {
		if (map.size() != 0) {
			Set<String> keys = map.keySet();
			int _size = 0;
			for (String key: keys) {
				out.print("\t" + key);
				int size = map.get(key).size();
				if (size > _size) _size = size;
			}

			out.println();
			
			for (int i = 0; i < _size; i++) {
				out.print(i);
				for (String key: keys) {
					List<Long> list = map.get(key);
					out.print("\t");
					if (list.size() != 0)
						out.print(list.get(i));
				}
				out.println();
			}
		}

	}

	public void cacheShutdown() {
		cache.cacheShutdown();
	}

	@Deprecated
	public View findView(InteractionKey key, String viewKind) throws IndexProblem {
		try {
		return index.findView(key, viewKind);
		} catch (ConnectionLostIndexProblem ex) {
			try {
				connectionFailed();
				return index.findView(key, viewKind);
			} catch (SQLException ex2) {
				throw new IndexProblem ("Reconnection failed", ex2);
			}
		}
	}

}
