package org.pasoa.preserv.storage.mysql.test;

import java.net.MalformedURLException;
import java.net.URL;

import junit.framework.TestCase;

import org.pasoa.common.Constants;
import org.pasoa.preserv.index.IndexProblem;
import org.pasoa.preserv.processor.Dispatcher;
import org.pasoa.preserv.record.CacheableItem;
import org.pasoa.preserv.storage.mysql.Debug;
import org.pasoa.preserv.storage.mysql.MySQLCache;
import org.pasoa.preserv.storage.mysql.MySQLConnector;
import org.pasoa.preserv.storage.mysql.MySQLRetrieval;
import org.pasoa.preserv.storage.mysql.MySQLStore;
import org.pasoa.preserv.test.RemoteTestbed;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.Record;
import org.pasoa.storeclient.ClientLib;
import org.pasoa.storeclient.wrappers.java.RecordingActor;
import org.w3c.dom.Element;

/**
 * 
 * @author Andrej Kazakov
 *
 */
public class MySQLCacheTest extends TestCase {

	private MySQLConnector db;
	private MySQLStore cache;
	
	protected void setUp() throws Exception {
		db = MySQLStore.createDefaultConnection();
		db.connect();
		cache = new MySQLStore();
	}

	protected void tearDown() throws Exception {
		cache = null;
		db.disconnect();
		db = null;
	}

	
	public void testCacheSingle() {
		try {
//			db.execute("delete from Cache");
			long count1 = db.executeQueryGetKey("select count(*) from Cache");
			CacheableItem ci = new CacheableItem("some path", new StringBuffer("content of this hgfdfk;jashgfoadhgoeragrhjealgkdfhflshgkdfjhsgfdjhaslfdjs;alfjdlsakgfsda"));
			cache.cache(ci);
			
			long count2 = db.executeQueryGetKey("select count(*) from Cache");
//			Debug.msg(count1 + "\t" + count2);
			assertTrue(count2 - 1L == count1);
			CacheableItem ci2 = cache.getCachedItem();
			count2 = db.executeQueryGetKey("select count(*) from Cache");
			assertTrue(count1 == count2);
			Debug.msg(ci2.getItem().toString());
			Debug.msg(ci.getItem().toString());
			assertTrue(
					ci.getContext().equals(ci2.getContext()) && 
					ci.getItem().toString().equals(ci2.getItem().toString())
			);
		} catch (Exception ex) {
			Debug.err(ex, "");
			fail();
		}
	}

	public void testCompression() throws Exception {
		ExtremeTest et = new ExtremeTest(cache);
		InteractionKey ik = et.composeInteractionKey("sender" + Math.random(), "receiver" + Math.random(), "interactionID" + Math.random());
		String lpaid = "localPAssertionID" + Math.random();
		ActorStatePAssertion apa = et.composeActorStatePA(new String[] {lpaid}).get(0);
		Element asserter = et.composeAsserter("asserter");
		Record record = new Record(apa, ik, Constants.SENDER_VIEW_TYPE, asserter);
		GlobalPAssertionKey gpak = new GlobalPAssertionKey(ik, Constants.SENDER_VIEW_TYPE, lpaid);
		
		try {
			cache.findPAssertion(gpak);
			fail();
		} catch (IndexProblem ex) {
			
		}
		
		ClientLib cl = new ClientLib();
		Debug.msg(cl.makeRecordDocument(record));
		//cl.record(cl.makeRecordDocument(record), new URL("http://localhost:8080/preserv/record"), false);
		cl.record(cl.makeRecordDocument(record), new URL("http://localhost:8080/preserv/record"), true);
		
//		Thread.sleep(500);
		
//		GlobalPAssertionKey gpak2= new GlobalPAssertionKey(new InteractionKey(MySQLRetrieval.string2Element(" <endPointReference>sender0.17273430714241855</endPointReference>"), MySQLRetrieval.string2Element("<endPointReference>receiver0.4230979989614164</endPointReference>"), "interactionID0.6299558009858126"), Constants.SENDER_VIEW_TYPE, "localPAssertionID0.6466768535776527");
		Thread.sleep(100);
		ActorStatePAssertion apa2 = (ActorStatePAssertion)cache.findPAssertion(gpak);
		assertTrue(et.areEqual(apa, apa2));
		
	}
}
