package org.pasoa.preserv.storage.mysql.test;

import static org.junit.Assert.*;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.PriorityQueue;
import java.util.Random;

import junit.framework.TestCase;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.pasoa.common.Constants;
import org.pasoa.preserv.index.IndexProblem;
import org.pasoa.preserv.storage.mysql.DatabaseClosedException;
import org.pasoa.preserv.storage.mysql.Debug;
import org.pasoa.preserv.storage.mysql.MySQLRetrieval;
import org.pasoa.preserv.storage.mysql.MySQLStore;
import org.pasoa.preserv.storage.mysql.NothingInsertedException;
import org.pasoa.preserv.storage.mysql.NumberOfExpectedPAssertionsAlreadyThereException;
import org.pasoa.preserv.storage.mysql.UnrecognisedPAssertionTypeException;
import org.pasoa.preserv.xquery.QueryProblem;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.ExposedInteractionMetadata;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.RelationshipPAssertion;
import org.pasoa.pstructure.View;
import org.pasoa.pstructure.serialiser.PStructureSerialiser;
import org.pasoa.simpledom.NormalisingDOMWriter;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

public class MySQLIndexTest extends TestCase{


	private MySQLStore store;
	private ExtremeTest et;
	private Random random;
	private PStructureSerialiser pser;
//	private Element assertion;

	@Before
	public void setUp() throws Exception {
		store = new MySQLStore();
//		store.delete();
		et = new ExtremeTest(store);
		random = new Random(System.currentTimeMillis());
		pser = new PStructureSerialiser();
//		assertion = MySQLRetrieval.string2Element("<hello> hello </hello>");;
	}

	@After
	public void tearDown() throws Exception {
		store.close();
		store = null;
		et = null;
		random = null;
	}

	@Test
	public void testFindInteractionRecord() throws SAXException, IOException, UnrecognisedPAssertionTypeException, NothingInsertedException, SQLException, QueryProblem, NumberOfExpectedPAssertionsAlreadyThereException, DatabaseClosedException, IndexProblem {
		InteractionKey ik1 = et.composeInteractionKey("sender" + random.nextInt(), "receiver" + random.nextInt(), "interactionID" + random.nextInt());
		InteractionKey ik2 = et.composeInteractionKey("sender" + random.nextInt(), "receiver" + random.nextInt(), "interactionID" + random.nextInt());
		String lpaid1 = "lpaid" + random.nextInt();
		String lpaid2 = "lpaid" + random.nextInt();
		GlobalPAssertionKey gpak1 = new GlobalPAssertionKey(ik1, Constants.RECEIVER_VIEW_TYPE, lpaid1);
		GlobalPAssertionKey gpak2 = new GlobalPAssertionKey(ik2, Constants.RECEIVER_VIEW_TYPE, lpaid2);
		ActorStatePAssertion apa1 = new ActorStatePAssertion(lpaid1, et.docStyle, MySQLRetrieval.string2Element("<hello> hi1 </hello>"));
		ActorStatePAssertion apa2 = new ActorStatePAssertion(lpaid2, et.docStyle, MySQLRetrieval.string2Element("<hello> hi2 </hello>"));
		Element asserter1 = et.composeAsserter("asserter");
		Element asserter2 = et.composeAsserter("asserter");

		boolean cought = true;
		try {
			store.findInteractionRecord(ik1);
			cought = false;
		}catch (IndexProblem e) {
			cought = true;
		}
		assertTrue(cought);

		try {
			store.findInteractionRecord(ik2);
			cought = false;
		}catch (IndexProblem e) {
			cought = true;
		}
		assertTrue(cought);

		store.record(gpak1, asserter1, pser.serialiseActorStatePAssertion(apa1).getDocumentElement(), Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME);
		store.record(gpak2, asserter2, pser.serialiseActorStatePAssertion(apa2).getDocumentElement(), Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME);

		InteractionRecord ir1 = store.findInteractionRecord(ik1);
		assertTrue(compareIRs(ir1, ik1, apa1, asserter1));

		InteractionRecord ir2 = store.findInteractionRecord(ik2);
		assertTrue(compareIRs(ir2, ik2, apa2, asserter2));
	}



	private boolean compareIRs(InteractionRecord ir, InteractionKey ik, final ActorStatePAssertion apa, Element asserter) {
		View view1 = new View(null, new ArrayList(){{add(apa);}}, null, -1, asserter);
		View view2 = new View(null, null, null, -1, null);
		return 
		et.areEqual(ir.getReceiverView(), view1) && 
//		et.areEqual(ir.getSenderView(), view2) && 
		ir.getInteractionKey().equals(ik);
	}

	@Test
	public void testFindPAssertion() throws Exception {
		int n = 6;
		String lpaids[] = new String[n];
		InteractionKey iks[] = new InteractionKey[n];
		GlobalPAssertionKey gpaks[] = new GlobalPAssertionKey[n];
		Element asserters[] = new Element[n];
		for (int i = 0; i < n; i++) {
			iks[i] = et.composeInteractionKey("sender" + random.nextInt(), "receiver" + random.nextInt(), "interactionID" + random.nextInt()); 
			lpaids[i] = "lpaid" + random.nextInt();
			gpaks[i] = new GlobalPAssertionKey(iks[i], Constants.RECEIVER_VIEW_TYPE, lpaids[i]);
			asserters[i] = et.composeAsserter("asserter");
			try {
				store.findPAssertion(gpaks[i]);
				fail();
			}catch (IndexProblem e) {}
		}

		int passsertions = n / 3; 
		ActorStatePAssertion apas[] = new ActorStatePAssertion[passsertions];
		InteractionPAssertion ipas[] = new InteractionPAssertion[passsertions];
		RelationshipPAssertion rpas[] = new RelationshipPAssertion[passsertions];
		int i = -1;
		for (int k = 0; k < passsertions; k++) {
			i++;
			apas[k] = new ActorStatePAssertion(lpaids[i], et.docStyle, et.composeStateBody());
			store.record(gpaks[i], asserters[i], pser.serialiseActorStatePAssertion(apas[k]).getDocumentElement(), Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME);
			i++;
			ipas[k] = new InteractionPAssertion(lpaids[i], et.docStyle, et.composeStateBody());
			store.record(gpaks[i], asserters[i], pser.serialiseInteractionPAssertion(ipas[k]).getDocumentElement(), Constants.INTERACTION_PASSERTION_ELEMENT_NAME);
			i++;
			Map<String, GlobalPAssertionKey[]> map = new HashMap<String, GlobalPAssertionKey[]>();
			map.put(lpaids[i - 1], new GlobalPAssertionKey [] {gpaks[i - 2]});
			rpas[k] = et.composeRelationshipPA(new String[] {lpaids[i]}, map).get(0);
			store.record(gpaks[i], asserters[i], pser.serialiseRelationshipPAssertion(rpas[k]).getDocumentElement(), Constants.RELATIONSHIP_PASSERTION_ELEMENT_NAME);
		}		

		i = -1;
		for (int k = 0; k < passsertions; k++) {
			i++;
			assertTrue(et.areEqual((ActorStatePAssertion)store.findPAssertion(gpaks[i]), apas[k]));
			i++;
			assertTrue(et.areEqual((InteractionPAssertion)store.findPAssertion(gpaks[i]), ipas[k]));
			i++;
			assertTrue(et.areEqual((RelationshipPAssertion)store.findPAssertion(gpaks[i]), rpas[k]));
		}

	}

	private String viewKinds[] = new String[] {
			Constants.SENDER_VIEW_TYPE,
			Constants.RECEIVER_VIEW_TYPE
	};

	@Test
	public void testFindView() throws Exception {
//		Insert 5 Views for the same IK
//		Repeat for 2 IK

		int nv = 5;
		int nik = 5;

		int passertions = nv * nik * viewKinds.length;
		View [][][] views = new View [nik][nv][viewKinds.length];

		et.setMaxSize(2);

		String lpaids[] = new String[passertions];
		InteractionKey iks[] = new InteractionKey[nik];
		GlobalPAssertionKey gpaks[] = new GlobalPAssertionKey[passertions];
		Element asserters[] = new Element[nv];
		for (int i = 0; i < nik; i++) {
			iks[i] = et.composeInteractionKey("sender" + random.nextInt(), "receiver" + random.nextInt(), "interactionID" + random.nextInt());
		}

		for (int i = 0; i < nv; i++) {
			asserters[i] = et.composeAsserter("asserter");
		}


		int pa = -1;
		for (int iik = 0; iik < nik; iik++) {
			for (int iv = 0; iv < nv; iv++) {
				for (int vk = 0; vk < viewKinds.length; vk++) {
					pa++;
					lpaids[pa] = "lpaid" + random.nextInt();
					gpaks[pa] = new GlobalPAssertionKey(iks[iik], viewKinds[vk], lpaids[pa]);
					et.setCurrentInteraciton(iks[iik], viewKinds[vk]);
					views[iik][iv][vk] = et.composeView(asserters[iv], et.generateSize(), et.generateSize(), null, null, viewKinds[vk]);
					try {
						store.findView(iks[iik], viewKinds[vk], asserters[iv]);
						fail("View already in the store");
					} catch (IndexProblem e) {}
					for (ExposedInteractionMetadata eim : (List<ExposedInteractionMetadata>)views[iik][iv][vk].getExposedInteractionMetadata()) {
						if (eim.getInteractionMetadata().size() == 0) {
							Debug.err("Empty EIM");
							System.exit(1);
						}
					}
					et.recordView(views[iik][iv][vk], iks[iik], viewKinds[vk]);
				}
			}

			/*
			List[] vlist = new List[viewKinds.length];
			for (int vk = 0; vk < viewKinds.length; vk++) {
				vlist[vk] = new ArrayList();
				for (int i = 0; i < nv; i++) {
					vlist[vk].add(views[iik][i][0]);
				}
			}
			InteractionRecord ir = new InteractionRecord(iks[iik], vlist[0], vlist[1]);
			et.recordIRAssync(new InteractionRecord[] {ir});
			 */
		}

		for (int iik = 0; iik < nik; iik++) {
			for (int iv = 0; iv < nv; iv++) {
				for (int vk = 0; vk < viewKinds.length; vk++) {
					View view1 = views[iik][iv][vk];
					View view2 = store.findView(iks[iik], viewKinds[vk], asserters[iv]);
					assertTrue(et.areEqual(view1, view2));
				}
			}
		}
	}

	@Test
	public void testFindViews() throws Exception {
//		Set up as previous
//		Get each view separately

		int nv = 5;
		int nik = 5;

		int passertions = nv * nik * viewKinds.length;
		List<View> [][] views = new List [nik][viewKinds.length];

		String lpaids[] = new String[passertions];
		InteractionKey iks[] = new InteractionKey[nik];
		GlobalPAssertionKey gpaks[] = new GlobalPAssertionKey[passertions];
		Element asserters[] = new Element[nv];
		for (int i = 0; i < nik; i++) {
			iks[i] = et.composeInteractionKey("sender" + random.nextInt(), "receiver" + random.nextInt(), "interactionID" + random.nextInt());
		}

		for (int i = 0; i < nv; i++) {
			asserters[i] = et.composeAsserter("asserter");
		}
		et.setMaxSize(2);

		int pa = -1;
		for (int iik = 0; iik < nik; iik++) {
			for (int vk = 0; vk < viewKinds.length; vk++) {
				views[iik][vk] = new ArrayList<View>();
				for (int iv = 0; iv < nv; iv++) {
					pa++;
					lpaids[pa] = "lpaid" + random.nextInt();
					gpaks[pa] = new GlobalPAssertionKey(iks[iik], viewKinds[vk], lpaids[pa]);

					et.setCurrentInteraciton(iks[iik], viewKinds[vk]);
					views[iik][vk].add(et.composeView(asserters[iv], et.generateSize(), et.generateSize(), null, null, viewKinds[vk]));
				}
				try {
					store.findViews(iks[iik], viewKinds[vk]);
					fail("View already in the store");
				} catch (IndexProblem e) {}
				for (View view: views[iik][vk]) {
					et.recordView(view, iks[iik], viewKinds[vk]);
				}
			}
		}

		PriorityQueue<ViewWrapper> pq1 = new PriorityQueue<ViewWrapper>();
		PriorityQueue<ViewWrapper> pq2 = new PriorityQueue<ViewWrapper>();
		for (int iik = 0; iik < nik; iik++) {
			for (int vk = 0; vk < viewKinds.length; vk++) {
				for (View view: views[iik][vk]) {
					if (view == null) throw new NullPointerException();
					pq1.add(new ViewWrapper(view));
				}
				List<View> vs = store.findViews(iks[iik], viewKinds[vk]);
				for (View v: vs) {
					if (v == null) throw new NullPointerException();
					pq2.add(new ViewWrapper(v));
				}
			}
		}
		assertTrue(et.areEqual(pq1, pq2));
	}

	class ViewWrapper implements Comparable{
		private View view;

		public ViewWrapper(View view) {
			this.view = view;
		}

		public boolean equals(Object o) {
			if (!(o instanceof ViewWrapper)) return false;
			ViewWrapper v = (ViewWrapper) o;
			return et.areEqual(view, v.view);
		}

		public int compareTo(Object o) {
			if (!(o instanceof ViewWrapper)) return -1;
			ViewWrapper v = (ViewWrapper)o;
			return NormalisingDOMWriter.writeToString(
					view.getAsserter()).compareTo(
							NormalisingDOMWriter.writeToString(v.view.getAsserter())) + 
							view.getNumberOfExpectedPAssertions() - 
							v.view.getNumberOfExpectedPAssertions();
		}

		public String toString () {
			return "ViewWrapper " + view.getAsserter() + " sf: " + view.getNumberOfExpectedPAssertions();
		}
	}
}

