package org.pasoa.preserv.storage.mysql.test;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import junit.framework.TestCase;
import org.pasoa.common.Constants;
import org.pasoa.preserv.storage.mysql.DatabaseClosedException;
import org.pasoa.preserv.storage.mysql.Debug;
import org.pasoa.preserv.storage.mysql.MySQLConnector;
import org.pasoa.preserv.storage.mysql.MySQLRetrieval;
import org.pasoa.preserv.storage.mysql.MySQLStore;
import org.pasoa.preserv.storage.mysql.NothingInsertedException;
import org.pasoa.preserv.storage.mysql.NumberOfExpectedPAssertionsAlreadyThereException;
import org.pasoa.preserv.storage.mysql.UnrecognisedPAssertionTypeException;
import org.pasoa.preserv.xquery.QueryProblem;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.ExposedInteractionMetadata;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.pstructure.RelationshipPAssertion;
import org.pasoa.pstructure.SubjectID;
import org.pasoa.pstructure.View;
import org.pasoa.pstructure.serialiser.PStructureSerialiser;
import org.pasoa.simpledom.DOMWriter;
import org.pasoa.simpledom.NormalisingDOMWriter;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;


public class MySQLRecordTest extends TestCase {
	
	private MySQLStore store;
	private ExtremeTest et;
	private MySQLConnector db;
	private PStructureSerialiser pser;
	private Random random;
	
	@Override
	protected void setUp() throws Exception {
		 store = new MySQLStore();
		 et = new ExtremeTest(store);
		 db = MySQLStore.createDefaultConnection();
		 db.connect();
		 pser = new PStructureSerialiser();
		 random = new Random(System.currentTimeMillis());
//		 store.delete();
	}

	@Override
	protected void tearDown() throws Exception {
		et = null;
		store.close();
		db.disconnect();
		db = null;
		pser = null;
		random = null;
	}

	public void testPS() throws Exception {
		et.setMaxSize(5);
		InteractionRecord[] irs = new InteractionRecord[] {
				et.composeInteractionRecord(et.composeAsserter("asserter"), et.composeAsserter("asserter"), null, null, null, null),
				et.composeInteractionRecord(et.composeAsserter("asserter"), et.composeAsserter("asserter"), null, null, null, null),
				et.composeInteractionRecord(et.composeAsserter("asserter"), et.composeAsserter("asserter"), null, null, null, null)
		};
		et.recordIRAssync(irs);
		assertTrue(et.areEqual(irs, et.lastInserted(3)));
	}
	
	public void testEIM() throws Exception {
		InteractionKey ik = et.composeInteractionKey("sender" + Math.random(), "sender" + Math.random(), "intID" + Math.random());
		List<Element> met = new ArrayList<Element>();
		met.add(MySQLRetrieval.string2Element("<interactionMetaData xmlns=\"" + Constants.PSTRUCT_NS + "\"><met>metadata1</met></interactionMetaData>"));
		met.add(MySQLRetrieval.string2Element("<interactionMetaData xmlns=\"" + Constants.PSTRUCT_NS + "\"><met>metadata2</met></interactionMetaData>"));
		met.add(MySQLRetrieval.string2Element("<interactionMetaData xmlns=\"" + Constants.PSTRUCT_NS + "\"><met>metadata3</met></interactionMetaData>"));
		ExposedInteractionMetadata eim = new ExposedInteractionMetadata (new GlobalPAssertionKey(ik, Constants.SENDER_VIEW_TYPE, "lpaid" + Math.random()),
				met
		);
		
		Element asserter = et.composeAsserter("asserter");
		
		store.record(
				new GlobalPAssertionKey(ik, Constants.RECEIVER_VIEW_TYPE, "none"), 
				asserter, 
				pser.serialiseExposedInteractionMetadata(eim).getDocumentElement(), 
				Constants.INTERACTION_METADATA_ELEMENT_NAME);
		InteractionRecord ir = et.lastInserted(1)[0];
		assertTrue(et.areEqual(eim, (ExposedInteractionMetadata)ir.getReceiverView().getExposedInteractionMetadata().get(0)));
		
	}
	
	public void testRecord(){
		try {
			InteractionKey[] iks = et.generateIKs(4);
			String[] lpaids = et.generateLocalPAssertionIDs(4);

			InteractionRecord ir = et.composeInteractionRecord(
					et.composeAsserter("sender-random"),
					et.composeAsserter("receiver-random"),
					new GlobalPAssertionKey[][] {
						new GlobalPAssertionKey[] {
								new GlobalPAssertionKey(
										iks[0],
										Constants.SENDER_VIEW_TYPE,
										lpaids[0]
										)
						}
					},
					new GlobalPAssertionKey[][] {
							new GlobalPAssertionKey[] {
									new GlobalPAssertionKey(
											iks[1],
											Constants.RECEIVER_VIEW_TYPE,
											lpaids[1]
											)
							}
						},
					new GlobalPAssertionKey[] {
						new GlobalPAssertionKey(
							iks[2],
							Constants.RECEIVER_VIEW_TYPE,
							lpaids[2]
							)
					},
					new GlobalPAssertionKey[] {
						new GlobalPAssertionKey(
							iks[3],
							Constants.RECEIVER_VIEW_TYPE,
							lpaids[3]
							)
					}
				);

			
		for (int i = 0; i < 4; i++) {
			String cond = "messageSource = (select dbEndPointID from EndPointReferences where content = '" + 
					DOMWriter.writeToString(iks[i].getMessageSource()) + "') and " +
					"messageSink = (select dbEndPointID from EndPointReferences where content = '" + 
					DOMWriter.writeToString(iks[i].getMessageSink()) + "') and " + 
					"interactionID = '" + iks[i].getInteractionID() + "'";
			Debug.msg("select dbInteractionRecordID from InteractionRecords where " + cond);
			if (0 < db.executeQueryGetKey("select dbInteractionRecordID from InteractionRecords where " + cond))
				throw new IllegalArgumentException("Keys aready exist");
		}
		
		et.recordIRAssync(new InteractionRecord[] {ir});		
		
		boolean ok = true;
		
		for (int i = 0; i < 4; i++) {
			String cond = "messageSource = (select dbEndPointID from EndPointReferences where content = '" + 
				DOMWriter.writeToString(iks[i].getMessageSource()) + "') and " +
				"messageSink = (select dbEndPointID from EndPointReferences where content = '" + 
				DOMWriter.writeToString(iks[i].getMessageSink()) + "') and " + 
				"interactionID = '" + iks[i].getInteractionID() + "'";
			ResultSet rs = db.executeQuery("select isLocal from InteractionRecords where " + cond);
			rs.first();
			if (rs.getString(1).equals('y')) {
				ok = false;
				break;
			}
		}
		
		assertTrue(ok);
		
		} catch (Exception ex){
			Debug.err(ex, "");
		}
		
	}
	
	public void testEqualAsserters() {
		try {
			
			Debug.msg("equalAsserters Test");
			Element a1 = et.composeAsserter("asserter");
			Element a2 = et.composeAsserter("asserter");
			et.setMaxSize(2);
		
			InteractionRecord[] irs = new InteractionRecord[2];
			
			irs[0] = et.composeInteractionRecord(
					a1,	a2,
					null, null, null, null
			);
			
			irs [1] = et.composeInteractionRecord(
					a2,	a1,
					null, null, null, null
			);
			
			et.recordIRAssync(irs);
			
			InteractionKey ik1 = irs[0].getInteractionKey();
			InteractionKey ik2 = irs[1].getInteractionKey();
			
			long dbir1ID = db.executeQueryGetKey("select dbInteractionRecordID from InteractionRecords " +
					"where messageSource = (select dbEndPointID from EndPointReferences where content = '" + NormalisingDOMWriter.writeToString(ik1.getMessageSource()) + "') " +
					"and messageSink = (select dbEndPointID from EndPointReferences where content = '" + NormalisingDOMWriter.writeToString(ik1.getMessageSink()) + "') " +
					"and interactionID = '" + ik1.getInteractionID() + "'");
			long dbir2ID = db.executeQueryGetKey("select dbInteractionRecordID from InteractionRecords " +
					"where messageSource = (select dbEndPointID from EndPointReferences where content = '" + NormalisingDOMWriter.writeToString(ik2.getMessageSource()) + "') " +
					"and messageSink = (select dbEndPointID from EndPointReferences where content = '" + NormalisingDOMWriter.writeToString(ik2.getMessageSink()) + "') " +
					"and interactionID = '" + ik2.getInteractionID() + "'");

			String a1s = NormalisingDOMWriter.writeToString(a1);
			String a2s = NormalisingDOMWriter.writeToString(a2);
			
			
			long a1c = db.executeQueryGetKey("select count(*) from Asserters where content = '" + a1s + "'");
//			Debug.msg("" + a1c);
			assertTrue(a1c == 1);
			long a2c = db.executeQueryGetKey("select count(*) from Asserters where content = '" + a2s + "'");
			assertTrue(a2c == 1);
			
			Debug.msg("select COUNT(DISTINCT dbAsserterID) from PAssertions where " +
					"dbViewKindID = (select dbViewKindID from ViewKind where name = '" + Constants.SENDER_VIEW_TYPE + "') and " +
					"dbInteractionRecordID = '" + dbir1ID + "' and dbAsserterID != 0");
			
			assertTrue(1L == db.executeQueryGetKey("select COUNT(DISTINCT dbAsserterID) from PAssertions where " +
					"dbViewKindID = (select dbViewKindID from ViewKind where name = '" + Constants.SENDER_VIEW_TYPE + "') and " +
					"dbInteractionRecordID = '" + dbir1ID + "' and dbAsserterID != 0"
					));
			
			assertTrue(1L == db.executeQueryGetKey("select COUNT(DISTINCT dbAsserterID) from PAssertions where " +
					"dbViewKindID = (select dbViewKindID from ViewKind where name = '" + Constants.RECEIVER_VIEW_TYPE + "') and " +
					"dbInteractionRecordID = '" + dbir1ID + "' and dbAsserterID != 0"
					));

			assertTrue(1L == db.executeQueryGetKey("select COUNT(DISTINCT dbAsserterID) from PAssertions where " +
					"dbViewKindID = (select dbViewKindID from ViewKind where name = '" + Constants.SENDER_VIEW_TYPE + "') and " +
					"dbInteractionRecordID = '" + dbir2ID + "' and dbAsserterID != 0"
					));
			
			assertTrue(1L == db.executeQueryGetKey("select COUNT(DISTINCT dbAsserterID) from PAssertions where " +
					"dbViewKindID = (select dbViewKindID from ViewKind where name = '" + Constants.RECEIVER_VIEW_TYPE + "') and " +
					"dbInteractionRecordID = '" + dbir2ID + "' and dbAsserterID != 0"
					));
			
			et.defaultMaxSize();
		} catch (Exception ex) {
			Debug.err(ex, "");
		}
	}
	
	public void testNotLocalToLocalIR() throws Exception{
		
		InteractionKey ikey1 = et.composeInteractionKey("sender" + random.nextInt(), "receiver" + random.nextInt(), "intID" + random.nextInt());
		InteractionKey ikey2 = et.composeInteractionKey("sender" + random.nextInt(), "receiver" + random.nextInt(), "intID" + random.nextInt());
		InteractionKey ikey3 = et.composeInteractionKey("sender" + random.nextInt(), "receiver" + random.nextInt(), "intID" + random.nextInt());
		
		GlobalPAssertionKey gpak1 = new GlobalPAssertionKey(ikey1, Constants.SENDER_VIEW_TYPE, "localPAID" + random.nextInt());
		GlobalPAssertionKey gpak2 = new GlobalPAssertionKey(ikey2, Constants.SENDER_VIEW_TYPE, "localPAID" + random.nextInt());
		GlobalPAssertionKey gpak3 = new GlobalPAssertionKey(ikey3, Constants.SENDER_VIEW_TYPE, "localPAID" + random.nextInt());
		
		ArrayList<ObjectID> os = new ArrayList<ObjectID>();
		os.add(new ObjectID(
					gpak1, 
					"some-param", 
					MySQLRetrieval.string2Element("<accessor>" + random.nextInt() + "</accessor>"),
					null
				));
		os.add(new ObjectID(
					gpak2, 
					"some-param", 
					MySQLRetrieval.string2Element("<accessor>" + random.nextInt() + "</accessor>"),
					null
				)
		);
		RelationshipPAssertion rpa = new RelationshipPAssertion(
				"relPAlocID" + random.nextInt(),
				new SubjectID (
					"subjectlocID" + random.nextInt(),
					MySQLRetrieval.string2Element("<accessor>" + random.nextInt() + "</accessor>"),
					"parameterName"					
				),
				"some-relation",
				os
				);

		long irsCount2 = db.executeQueryGetKey("select count(*) from InteractionRecords");
		store.record(gpak3, et.composeAsserter("asserter"), pser.serialiseRelationshipPAssertion(rpa).getDocumentElement(), Constants.RELATIONSHIP_PASSERTION_ELEMENT_NAME);
		long irsCount1 = db.executeQueryGetKey("select count(*) from InteractionRecords");
		
		assertTrue (irsCount1 - 3 == irsCount2);
		
		assertTrue(getIsLocalFromIR(ikey1).equals("n"));
		assertTrue(getIsLocalFromIR(ikey2).equals("n"));
		assertTrue(getIsLocalFromIR(ikey3).equals("y"));

		ExposedInteractionMetadata eim1 = new ExposedInteractionMetadata(gpak1, et.composeMetadata());

		store.record(gpak3, et.composeAsserter("asserter"), pser.serialiseExposedInteractionMetadata(eim1).getDocumentElement(), Constants.INTERACTION_METADATA_ELEMENT_NAME);
		irsCount2 = db.executeQueryGetKey("select count(*) from InteractionRecords");
		
		assertTrue(irsCount2 == irsCount1);
		
		assertTrue(getIsLocalFromIR(ikey1).equals("n"));
		assertTrue(getIsLocalFromIR(ikey2).equals("n"));
		assertTrue(getIsLocalFromIR(ikey3).equals("y"));

		ExposedInteractionMetadata eim2 = new ExposedInteractionMetadata(gpak2, et.composeMetadata());

		store.record(gpak3, et.composeAsserter("asserter"), pser.serialiseExposedInteractionMetadata(eim2).getDocumentElement(), Constants.INTERACTION_METADATA_ELEMENT_NAME);
		irsCount2 = db.executeQueryGetKey("select count(*) from InteractionRecords");
		
		assertTrue(irsCount2 == irsCount1);
		
		assertTrue(getIsLocalFromIR(ikey1).equals("n"));
		assertTrue(getIsLocalFromIR(ikey2).equals("n"));
		assertTrue(getIsLocalFromIR(ikey3).equals("y"));
		
		List<ActorStatePAssertion> apas = et.composeActorStatePA(new String[] {"apalpID" + random.nextInt(), "apalpID" + random.nextInt()});
		store.record(gpak1, et.composeAsserter("asserter"), pser.serialiseActorStatePAssertion(apas.get(0)).getDocumentElement(), Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME);
		irsCount2 = db.executeQueryGetKey("select count(*) from InteractionRecords");
		
		assertTrue(irsCount2 == irsCount1);
		
		assertTrue(getIsLocalFromIR(ikey1).equals("y"));
		assertTrue(getIsLocalFromIR(ikey2).equals("n"));
		assertTrue(getIsLocalFromIR(ikey3).equals("y"));

		store.record(gpak2, et.composeAsserter("asserter"), pser.serialiseActorStatePAssertion(apas.get(1)).getDocumentElement(), Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME);
		irsCount2 = db.executeQueryGetKey("select count(*) from InteractionRecords");
		
		assertTrue(irsCount2 == irsCount1);
		
		assertTrue(getIsLocalFromIR(ikey1).equals("y"));
		assertTrue(getIsLocalFromIR(ikey2).equals("y"));
		assertTrue(getIsLocalFromIR(ikey3).equals("y"));

	}
	
	private String getIsLocalFromIR(InteractionKey ik) throws SQLException {
/*		Debug.msg("select isLocal from InteractionRecords where messageSource = " +
		"(select dbEndPointID from EndPointReferences where content = '" + NormalisingDOMWriter.writeToString(ik.getMessageSource()) + "') and " +
		"messageSink = (select dbEndPointID from EndPointReferences where content = '" + NormalisingDOMWriter.writeToString(ik.getMessageSink()) + "') and " +
		"interactionID = '" + ik.getInteractionID() + "'");
*/		ResultSet rs = db.executeQuery("select isLocal from InteractionRecords where messageSource = " +
				"(select dbEndPointID from EndPointReferences where content = '" + NormalisingDOMWriter.writeToString(ik.getMessageSource()) + "') and " +
				"messageSink = (select dbEndPointID from EndPointReferences where content = '" + NormalisingDOMWriter.writeToString(ik.getMessageSink()) + "') and " +
				"interactionID = '" + ik.getInteractionID() + "'");
		rs.first();
		return rs.getString(1);
	}
	
	public void testSameGpaksInRef() throws Exception{
		InteractionKey ikey1 = et.composeInteractionKey("sender" + random.nextInt(), "receiver" + random.nextInt(), "intID" + random.nextInt());
		
		GlobalPAssertionKey gpak1 = new GlobalPAssertionKey(ikey1, Constants.SENDER_VIEW_TYPE, "localPAID" + random.nextInt());
		GlobalPAssertionKey gpak2 = new GlobalPAssertionKey(ikey1, Constants.SENDER_VIEW_TYPE, "localPAID" + random.nextInt());
		GlobalPAssertionKey gpak3 = new GlobalPAssertionKey(ikey1, Constants.SENDER_VIEW_TYPE, "localPAID" + random.nextInt());
		
		ArrayList<ObjectID> os = new ArrayList<ObjectID>();
		
		os.add(new ObjectID(
				gpak1, 
				"some-param", 
				MySQLRetrieval.string2Element("<accessor>" + random.nextInt() + "</accessor>"),
				null
			));
		os.add(new ObjectID(
				gpak2, 
				"some-param", 
				MySQLRetrieval.string2Element("<accessor>" + random.nextInt() + "</accessor>"),
				null
			));
		
		os.add(new ObjectID(
				gpak3, 
				"some-param", 
				MySQLRetrieval.string2Element("<accessor>" + random.nextInt() + "</accessor>"),
				null
			));

		RelationshipPAssertion rpa = new RelationshipPAssertion(
				"relPAlocID" + random.nextInt(),
				new SubjectID (
					"subjectlocID" + random.nextInt(),
					MySQLRetrieval.string2Element("<accessor>" + random.nextInt() + "</accessor>"),
					"parameterName"					
				),
				"some-relation",
				os
		);
		
		long irsCount2 = db.executeQueryGetKey("select count(*) from InteractionRecords");
		long paCount2 = db.executeQueryGetKey("select count(*) from PAssertions");
		long eimCount2 = db.executeQueryGetKey("select count(*) from ExposedInteractionMetadata");
		store.record(gpak3, et.composeAsserter("asserter"), pser.serialiseRelationshipPAssertion(rpa).getDocumentElement(), Constants.RELATIONSHIP_PASSERTION_ELEMENT_NAME);
		long irsCount1 = db.executeQueryGetKey("select count(*) from InteractionRecords");
		long paCount1 = db.executeQueryGetKey("select count(*) from PAssertions");
		long eimCount1 = db.executeQueryGetKey("select count(*) from ExposedInteractionMetadata");
		
		assertTrue (irsCount1 - 1 == irsCount2);
		assertTrue (paCount1 - 4 == paCount2);
		assertTrue (eimCount2 == eimCount1);
		assertTrue(getIsLocalFromIR(ikey1).equals("y"));

		ExposedInteractionMetadata eim1 = new ExposedInteractionMetadata(gpak1, et.composeMetadata());

		store.record(gpak3, et.composeAsserter("asserter"), pser.serialiseExposedInteractionMetadata(eim1).getDocumentElement(), Constants.INTERACTION_METADATA_ELEMENT_NAME);
		irsCount2 = db.executeQueryGetKey("select count(*) from InteractionRecords");
		paCount2 = db.executeQueryGetKey("select count(*) from PAssertions");
		eimCount2 = db.executeQueryGetKey("select count(*) from ExposedInteractionMetadata");
		
		assertTrue(irsCount2 == irsCount1);
		assertTrue(paCount2 == paCount1);
		assertTrue (eimCount2 - 1 == eimCount1);
		paCount1 = paCount2;
		
		assertTrue(getIsLocalFromIR(ikey1).equals("y"));

		ExposedInteractionMetadata eim2 = new ExposedInteractionMetadata(gpak2, et.composeMetadata());

		store.record(gpak3, et.composeAsserter("asserter"), pser.serialiseExposedInteractionMetadata(eim2).getDocumentElement(), Constants.INTERACTION_METADATA_ELEMENT_NAME);
		irsCount2 = db.executeQueryGetKey("select count(*) from InteractionRecords");
		paCount2 = db.executeQueryGetKey("select count(*) from PAssertions");
		eimCount2 = db.executeQueryGetKey("select count(*) from ExposedInteractionMetadata");
		
		assertTrue(irsCount2 == irsCount1);
//		Debug.msg(paCount2 + " - 1 == " + paCount1);
		assertTrue(paCount2 == paCount1);
		assertTrue (eimCount2 - 2 == eimCount1);
		paCount1 = paCount2;
		
		assertTrue(getIsLocalFromIR(ikey1).equals("y"));

		ExposedInteractionMetadata eim3 = new ExposedInteractionMetadata(gpak3, et.composeMetadata());

		store.record(gpak3, et.composeAsserter("asserter"), pser.serialiseExposedInteractionMetadata(eim3).getDocumentElement(), Constants.INTERACTION_METADATA_ELEMENT_NAME);
		irsCount2 = db.executeQueryGetKey("select count(*) from InteractionRecords");
		paCount2 = db.executeQueryGetKey("select count(*) from PAssertions");
		
		assertTrue(irsCount2 == irsCount1);
		assertTrue(paCount2 == paCount1);
		paCount1 = paCount2;
		
		assertTrue(getIsLocalFromIR(ikey1).equals("y"));
		
	}
	
	private static String charFiles[] = new String[] {
//		"E:/Code/preserv/src/org/pasoa/preserv/storage/mysql/test/UTF-8_CharString.txt", 
		"E:/Code/preserv/src/org/pasoa/preserv/storage/mysql/test/ASCII_CharString.txt"
	}; 
	
	public void testSpecificChars() throws Exception{
		for (String fileName : charFiles) {
			FileReader fr = new FileReader(fileName);
			LineNumberReader reader = new LineNumberReader(fr);
			String charStr = reader.readLine();
			Debug.msg(charStr);
			InteractionKey ik = et.composeInteractionKey("sender" + charStr + random.nextInt(), "receiver" + charStr + random.nextInt(), charStr);
			String localPAssertionID = "apalpid" + charStr + random.nextInt();
			GlobalPAssertionKey gpak = new GlobalPAssertionKey(
					ik,
					Constants.SENDER_VIEW_TYPE,
					localPAssertionID
					);
			Element asserter = MySQLRetrieval.string2Element("<asserter>" + charStr + random.nextInt() + "</asserter>");

			ActorStatePAssertion apa = new ActorStatePAssertion(localPAssertionID, et.docStyle, MySQLRetrieval.string2Element("<body>" + charStr + "</body>"));
			Element apaA = pser.serialiseActorStatePAssertion(apa).getDocumentElement();
			store.record(gpak, asserter, apaA, Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME);
			
			localPAssertionID += "ipa";
			InteractionPAssertion ipa = new InteractionPAssertion(localPAssertionID, et.docStyle, MySQLRetrieval.string2Element("<body>" + charStr + "</body>"));
			Element ipaA = pser.serialiseInteractionPAssertion(ipa).getDocumentElement();
			gpak = new GlobalPAssertionKey(
					ik,
					Constants.SENDER_VIEW_TYPE,
					localPAssertionID
					);
			store.record(gpak, asserter, ipaA, Constants.INTERACTION_PASSERTION_ELEMENT_NAME);

			InteractionRecord ir = et.lastInserted(1)[0];
			View view = ir.getSenderView();
			
			Debug.msg(NormalisingDOMWriter.writeToString(((ActorStatePAssertion) view.getActorStatePAssertions().get(0)).getContent()));
			Debug.msg(NormalisingDOMWriter.writeToString(apaA));
			assertTrue(NormalisingDOMWriter.writeToString(((ActorStatePAssertion) view.getActorStatePAssertions().get(0)).getContent()).
					equals(NormalisingDOMWriter.writeToString(apaA)
							));
			assertTrue(NormalisingDOMWriter.writeToString(((InteractionPAssertion) view.getInteractionPAssertions().get(0)).getContent()).
					equals(NormalisingDOMWriter.writeToString(ipaA)
					));
//			localPAssertionID += "rpa";
//			RelationshipPAssertion rpa = new RelationshipPAssertion("ipalpid" + charStr + random.nextInt(), et.docStyle, MySQLRetrieval.string2Element("<body>" + charStr + "</body>"));
//			assertion = pser.serialiseInteractionPAssertion(ipa).getDocumentElement();
//			store.record(gpak, asserter, assertion, Constants.INTERACTION_PASSERTION_ELEMENT_NAME);
		
			reader.close();
			fr.close();
		}
	}
	
	public void testRandomViewName() throws Exception {
		Element a1 = et.composeAsserter("asserter1");
		Element a2 = et.composeAsserter("asserter2");
		Element a3 = et.composeAsserter("asserter3");
		InteractionKey ik = et.composeInteractionKey("sender" + random.nextInt(), "receiver" + random.nextInt(), "" + Math.random());
		String localPAssertionID = "apalpid" + random.nextInt();
		GlobalPAssertionKey gpak = new GlobalPAssertionKey(
				ik,
				"some random view name",
				localPAssertionID
				);

		ActorStatePAssertion apa = new ActorStatePAssertion(localPAssertionID, et.docStyle, MySQLRetrieval.string2Element("<body>fdhskfdhafksd</body>"));
		Element apaA = pser.serialiseActorStatePAssertion(apa).getDocumentElement();
		long c_before = db.executeQueryGetKey("select count(*) from PAssertions");
		store.record(gpak, a1, apaA, Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME);
		store.record(gpak, a2, apaA, Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME);
		store.record(gpak, a3, apaA, Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME);
		long c_after = db.executeQueryGetKey("select count(*) from PAssertions");
		assertTrue(c_before + 3 == c_after);
//		InteractionRecord ir = et.lastInserted(1)[0];
	}
	
	public void testInferredViewKind() throws Exception {
		Element a1 = et.composeAsserter("asserter1");
		Element a2 = et.composeAsserter("asserter2");
		Element a3 = et.composeAsserter("asserter3");
		InteractionKey ik = et.composeInteractionKey("sender" + random.nextInt(), "receiver" + random.nextInt(), "" + Math.random());
		String localPAssertionID = "apalpid" + random.nextInt();
		GlobalPAssertionKey gpak = new GlobalPAssertionKey(
				ik,
				Constants.INFERRED_VIEW_TYPE,
				localPAssertionID
				);

		ActorStatePAssertion apa = new ActorStatePAssertion(localPAssertionID, et.docStyle, MySQLRetrieval.string2Element("<body>fdhskfdhafksd</body>"));
		Element apaA = pser.serialiseActorStatePAssertion(apa).getDocumentElement();
		long c_before = db.executeQueryGetKey("select count(*) from PAssertions");
		store.record(gpak, a1, apaA, Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME);
		store.record(gpak, a2, apaA, Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME);
		store.record(gpak, a3, apaA, Constants.ACTOR_STATE_PASSERTION_ELEMENT_NAME);
		long c_after = db.executeQueryGetKey("select count(*) from PAssertions");
		assertTrue(c_before + 3 == c_after);
		InteractionRecord ir = et.lastInserted(1)[0];
		assertTrue(ir.getReceiverViews().size() == 0);
		assertTrue(ir.getSenderViews().size() == 0);
		assertTrue(ir.getInferredViews().size() + " != 3", ir.getInferredViews().size() == 3);
	}

}

