package org.pasoa.preserv.storage.mysql.test;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

import junit.framework.TestCase;

import org.junit.After;
import org.junit.Before;
import org.pasoa.common.Constants;
import org.pasoa.preserv.storage.mysql.Debug;
import org.pasoa.preserv.storage.mysql.test.MySQLLocalTestBed;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.ExposedInteractionMetadata;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.Link;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.pstructure.Record;
import org.pasoa.pstructure.RecordExposedInteractionMetadata;
import org.pasoa.pstructure.RecordSf;
import org.pasoa.pstructure.RelationshipPAssertion;
import org.pasoa.pstructure.View;
import org.pasoa.pstructure.serialiser.PStructureSerialiser;
import org.pasoa.simpledom.DOMSerialiser;
import org.pasoa.simpledom.DOMWriter;
import org.pasoa.simpledom.NormalisingDOMWriter;
import org.pasoa.simpledom.XMLComparison;
import org.pasoa.util.httpsoap.WSAddressEndpoint;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class PStructureTest extends TestCase{

	public static void main (String args[]) {
		PStructureTest pst = new PStructureTest();
		try {
			pst.testPStructureSupport();
		} catch (Exception ex) {
			Debug.err(ex, "Failed to initialise the test");
		}
	}
	
	
	public PStructureTest() {
		try {
			mtb = new MySQLLocalTestBed();
		} catch (Exception ex) {
			Debug.err(ex, "Failed to initialise the TestBed");
		}
	}
    private static String XML_CONTENT = "<h:hello xmlns:h=\"http://www.hello.com\">hello</h:hello>";
    private static String DATA_ACCESSOR_STR = "<xp:xpath xmlns:xp=\"http://www.pasoa.org/xpath\">/</xp:xpath>";
    
   
    /**
	 * Test to see if an interaction p-assertion can be recorded in both views
	 * 
	 */
    
    private MySQLLocalTestBed mtb;
    
    public MySQLLocalTestBed getLocalTestbed() {
    	return mtb;
    }	
    
    @Before
    public void setUp() throws Exception
    {
         mtb = new MySQLLocalTestBed();
    }
    
    @After
    public void tearDown() throws Exception
    {
//  	  _localTestbed.close();
//  	  _localTestbed.cleanUp();
//        File f = new File(_localTestbed.getPathToStorageDirectory());
        mtb.cleanUp();
//        System.out.println("Size after completion " + this.getSize(f));
      
        
    }
    
	public void testPStructureSupport() throws Exception {
		this.getLocalTestbed().setSynchronousRecording(true);
		WSAddressEndpoint sender         =  new WSAddressEndpoint ("http://sender.com" + Math.random());
		WSAddressEndpoint receiver       =  new WSAddressEndpoint ("http://receiver.com" + Math.random());
		Element           content        =  getLocalTestbed().toDOM(XML_CONTENT);
		InteractionKey    interactionKey =  new InteractionKey (sender.getElement(), receiver.getElement(), "uniqueidof1");
		Record            senderRecord   =  null;
		Record            receiverRecord =  null;
		
//		Recording to the DB
		Debug.msg("Recording to the DB");

		String documentationStyle = "http://www.pasoa.org/schemas/ontologies/DocumentationStyle.Verbatim";
		
		// Create an actor state p-assertion
		Debug.msg("Recording actor state p-assertions");
		String actorStatePaLpid = "aslpid" + Math.random();         
		ActorStatePAssertion aspa = new ActorStatePAssertion (actorStatePaLpid, documentationStyle, content);
		senderRecord   = new Record (aspa, interactionKey, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
		receiverRecord = new Record (aspa, interactionKey, Constants.RECEIVER_VIEW_TYPE,  receiver.getElement ());
		
		this.getLocalTestbed().record(senderRecord);
		this.getLocalTestbed().record(receiverRecord);
		
		Debug.msg("Recording interaction p-assertions");
		String interactionPaLpid = "interactionLpid" + Math.random();
		InteractionPAssertion ipa = new InteractionPAssertion(interactionPaLpid, documentationStyle, content);
		senderRecord   = new Record (ipa, interactionKey, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
		receiverRecord = new Record (ipa, interactionKey, Constants.RECEIVER_VIEW_TYPE,  receiver.getElement ());
		this.getLocalTestbed().record(senderRecord);
		this.getLocalTestbed().record(receiverRecord);
		
		Debug.msg("Recording relationship p-assertions");
		
		String relationshipPaLpid = "rellpid" + Math.random();
		Element dataAccessor = getLocalTestbed().toDOM(DATA_ACCESSOR_STR);
		String relation = "http://www.pasoa.org/schemas/ontologies/relationships/use"; 
		String subjectLpid = interactionPaLpid;
		String subjectParameterName = "http://www.pasoa.org/schemas/ontologies/relationships/usage.user";
		
		String objectLpid = actorStatePaLpid; // points to the actor state
														// p-assertion
		String objectParameterName = "http://www.pasoa.org/schemas/ontologies/relationships/usage.usedItem";
		Link   link = new Link ("http://www.somepstore.com");
		
		GlobalPAssertionKey gpak = new GlobalPAssertionKey(interactionKey, Constants.SENDER_VIEW_TYPE, objectLpid);
		ObjectID objId = new ObjectID (gpak, objectParameterName, dataAccessor, link);
		LinkedList objectIds = new LinkedList();
		objectIds.add (objId);
		objectIds.add (objId);
		
		RelationshipPAssertion rel = new RelationshipPAssertion(relationshipPaLpid, 
		                                                        subjectLpid, 
		                                                        dataAccessor, 
		                                                        subjectParameterName, 
		                                                        relation, 
		                                                        objectIds);
		
		senderRecord = new Record (rel, interactionKey, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
		receiverRecord = new Record (rel, interactionKey, Constants.RECEIVER_VIEW_TYPE,  receiver.getElement ());
		
		this.getLocalTestbed().record(senderRecord);
		this.getLocalTestbed().record(receiverRecord);
		
		Debug.msg("Recording interaction metadata");
		
		DOMSerialiser builder = new DOMSerialiser (Constants.PSTRUCT_NS);
		
		builder.start  ("interactionMetadata");
		builder.single ("someMetadata",  "hello");
		builder.end();
		
		LinkedList ll = new LinkedList();
		ll.add(((Document)builder.getRoot()).getDocumentElement());
		ExposedInteractionMetadata eim = new ExposedInteractionMetadata(gpak, ll);
		
		senderRecord = new RecordExposedInteractionMetadata (eim, interactionKey, Constants.SENDER_VIEW_TYPE, sender.getElement());
		receiverRecord = new RecordExposedInteractionMetadata (eim, interactionKey, Constants.RECEIVER_VIEW_TYPE, receiver.getElement());
		
		this.getLocalTestbed().record(senderRecord);
		this.getLocalTestbed().record(receiverRecord);
		
		int nExpected = 5;
		
		senderRecord = new RecordSf (nExpected, interactionKey, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
		receiverRecord = new RecordSf (nExpected, interactionKey, Constants.RECEIVER_VIEW_TYPE,  receiver.getElement ());
		
		this.getLocalTestbed().record(senderRecord);
		this.getLocalTestbed().record(receiverRecord);
		
		DOMWriter domWriter = new DOMWriter (System.out);
		domWriter.setFormatted(true);
		
//		domWriter.write(this.getLocalTestbed().getStoreContents());
	
		
		Debug.msg("Comparing the original data with the recorded one.");

//		 Getting the record from the DB
		InteractionRecord ir = this.getLocalTestbed().getLastInteractionRecord();

//		XMLComparison xmlComp = new XMLComparison();
		PStructureSerialiser psSerialiser = new PStructureSerialiser();		

		
//		 Receiver
//		boolean f = xmlComp.compare(ir.getReceiverView().getAsserter(), receiverRecord.getAsserter(), true);
//		System.err.println(f);
//		if (xmlComp.compare(ir.getReceiverView().getAsserter(), receiverRecord.getAsserter(), false)) Debug.msg("Receiver \t OK");
		Debug.msg(NormalisingDOMWriter.writeToString(ir.getReceiverView().getAsserter()));		
		Debug.msg(NormalisingDOMWriter.writeToString(receiverRecord.getAsserter()));
		assertTrue ("Receiver", 
				NormalisingDOMWriter.writeToString(
					ir.getReceiverView().getAsserter()
				).equals(NormalisingDOMWriter.writeToString(
					receiverRecord.getAsserter())
				)
			);/* Debug.msg("Receiver \t OK");
		else Debug.msg("Receiver \t ERROR");*/

		
		// Sender
//		f = xmlComp.compare(ir.getSenderView().getAsserter(), senderRecord.getAsserter(), true);
//		System.err.println(f);
//		if (xmlComp.compare(ir.getSenderView().getAsserter(), senderRecord.getAsserter(), false)) Debug.msg("Sender \t OK");
		assertTrue ("Sender", NormalisingDOMWriter.writeToString(
				ir.getSenderView().getAsserter()
			).equals(NormalisingDOMWriter.writeToString(
				senderRecord.getAsserter())
			)
		); 
		/* Debug.msg("Sender \t OK");
		else Debug.msg("Sender \t ERROR");*/

		// InteractionKeys
		assertTrue("InteractionKey", ir.getInteractionKey().equals(interactionKey));
		/*if (ir.getInteractionKey().equals(interactionKey)) Debug.msg("InteractionKey \t OK");
		else Debug.msg("InteractionKey \t ERROR");*/
		
		Debug.msg("Checking the receiver View");
//		checkView(ir.getReceiverView(), psSerialiser, xmlComp, eim, aspa, ipa, rel, receiverRecord.getAsserter(), nExpected);
		checkView(ir.getReceiverView(), psSerialiser, eim, aspa, ipa, rel, nExpected);
		Debug.msg("Checking the sender View");
//		checkView(ir.getSenderView(), psSerialiser, xmlComp, eim, aspa, ipa, rel, senderRecord.getAsserter(), nExpected);
		checkView(ir.getSenderView(), psSerialiser, eim, aspa, ipa, rel, nExpected);
	}
		   
	private void checkView(View view, 
							PStructureSerialiser psSerialiser, 
//							XMLComparison xmlComp,
							ExposedInteractionMetadata eim, 
							ActorStatePAssertion aspa, 
							InteractionPAssertion ipa, 
							RelationshipPAssertion rel,
							int nExpected) {

		// ExposedInteractionMetadata
		List <ExposedInteractionMetadata> dbEimList = view.getExposedInteractionMetadata();
		boolean eimFlag = false;
		for (ExposedInteractionMetadata dbEim: dbEimList) {
			Element originalEimElem = psSerialiser.serialiseExposedInteractionMetadata(eim).getDocumentElement();
			Element receivedEimElem = psSerialiser.serialiseExposedInteractionMetadata(dbEim).getDocumentElement();
			
			/*
			 if (xmlComp.compare(originalEimElem, receivedEimElem, true)) {
				Debug.msg("EIM \t OK");	
			}else 
				Debug.msg("EIM \t ERROR");
			*/
			
//			try {
//			XMLComparison xmlComp = new XMLComparison();
//			boolean f = xmlComp.compare(originalEimElem, receivedEimElem, true);
//			System.err.println(f);
//			} catch (Exception ex) {}
		    String original = NormalisingDOMWriter.writeToString(originalEimElem);
		    String newEimStr = NormalisingDOMWriter.writeToString(receivedEimElem);
//		    Debug.msg(original);
//		    Debug.msg(newEimStr);
		    if (original.equals(newEimStr)) {
		    	eimFlag = true;
		    	break;
		    }
		}
		dbEimList = null;
		
		
		assertTrue("EIM", eimFlag);
		/*if (eimFlag) {
			Debug.msg("ExposedIneractionMetaData \t OK");	
		} else { 
			Debug.msg("ExposedIneractionMetaData \t ERROR");
		}*/
		
		// ActorState
		
		boolean aspaFlag = false;
		List<ActorStatePAssertion> dbActorSList = view.getActorStatePAssertions();
		for (ActorStatePAssertion actorPA: dbActorSList) {
//			try {
//				XMLComparison xmlComp = new XMLComparison();
//				boolean f = xmlComp.compare(psSerialiser.serialiseActorStatePAssertion(aspa).getDocumentElement(), actorPA.getContent(), true);
//				System.err.println(f);
//			} catch (Exception ex) {}

			String orASPA = NormalisingDOMWriter.writeToString(psSerialiser.serialiseActorStatePAssertion(aspa));
			String reASPA = NormalisingDOMWriter.writeToString(actorPA.getContent());
			if (orASPA.equals(reASPA)) {
				aspaFlag = true;
				break;
			}
		}
		
		assertTrue("ASPA", aspaFlag);
		/*if (aspaFlag) {
			Debug.msg("ActorStatePAssertion \t OK");	
		} else { 
			Debug.msg("ActorStatePAssertion \t ERROR");
		}*/
		dbActorSList = null;

		// InteractionPA
		boolean ipaFlag = false;
		List<InteractionPAssertion> dbInterPAList = view.getInteractionPAssertions();
		for (InteractionPAssertion interPA: dbInterPAList) {
//			try {
//				XMLComparison xmlComp = new XMLComparison();
//				boolean f = xmlComp.compare(psSerialiser.serialiseInteractionPAssertion(ipa).getDocumentElement(), interPA.getContent(), true);
//				System.err.println(f);
//			} catch (Exception ex) {}

			String or = NormalisingDOMWriter.writeToString(psSerialiser.serialiseInteractionPAssertion(ipa));
			String re = NormalisingDOMWriter.writeToString(interPA.getContent());
			if (or.equals(re)) {
				ipaFlag = true;
				break;
			}
		}
		
		assertTrue("IPA", ipaFlag);
		/*if (ipaFlag) {
			Debug.msg("InteractionPAssertion \t OK");	
		} else { 
			Debug.msg("InteractionPAssertion \t ERROR");
		}*/
		dbInterPAList = null;
		
		// RelationshipPA
		boolean relFlag = false;
		List <RelationshipPAssertion> dbRelPAList = view.getRelationshipPAssertions();
		for (RelationshipPAssertion relPA: dbRelPAList) {
			String or = NormalisingDOMWriter.writeToString(psSerialiser.serialiseRelationshipPAssertion(rel));
			String re = NormalisingDOMWriter.writeToString(psSerialiser.serialiseRelationshipPAssertion(relPA));
//			Debug.msg(or);
//			Debug.msg(re);
			if (or.equals(re)) {
				relFlag = true;
				break;
			}
		}
		
		assertTrue("RelPA", relFlag);
		/*if (relFlag) {
			Debug.msg("RelationshipPAssertion \t OK");	
		} else { 
			Debug.msg("RelationshipPAssertion \t ERROR");
		}*/
		dbRelPAList = null;
		
		// numberOfExpectedPAssertions
		assertTrue ("NumberOfExpectedPAssertions", view.getNumberOfExpectedPAssertions() == nExpected);
		/*if (view.getNumberOfExpectedPAssertions() == nExpected) {
			Debug.msg("NumberOfExpectedPAssertions \t OK");
		} else {
			Debug.msg("NumberOfExpectedPAssertions \t ERROR");
		}*/
		
//		Debug.msg(eim.getGlobalPAssertionKey().toString());
	}

/*	public static junit.framework.Test suite() {
		return new junit.framework.JUnit4TestAdapter(PStructureSupportTest.class);
	}
		    
	public static void main(String[] args) {
		org.junit.runner.JUnitCore.main("org.pasoa.preserv.test.PStructureSupportTest");
	}*/
}
