package org.pasoa.preserv.storage.mysql.test.scalability;

import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

import org.pasoa.common.Constants;
import org.pasoa.preserv.storage.mysql.test.MySQLLocalTestBedFixture;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.Record;
import org.pasoa.storeclient.ClientLib;
import org.pasoa.util.httpsoap.WSAddressEndpoint;
import org.w3c.dom.Element;

import org.junit.*;
import static org.junit.Assert.*;

public class AsynchSynchSpeedVsPAssertionSizeTest extends MySQLLocalTestBedFixture  
{

    /**
     * Some XML CONTENT
     */
    private static String XML_CONTENT = "<h:hello xmlns:h=\"http://www.hello.com\">hello</h:hello>";
    
    @Test
    public void asynchronousTest() throws Exception
    {
        int         numOfIters      = Integer.valueOf(ScalabilityProps.getProperty("assvpasNumOfIters"));
        String      outputPath      = ScalabilityProps.getProperty("assvpasResultFileAsynch");
     
        runTest (numOfIters, false, outputPath);
        
        assertTrue(true);
        
    }
    
    @Test
    public void synchronousTest() throws Exception
    {
        int         numOfIters      = Integer.valueOf(ScalabilityProps.getProperty("assvpasNumOfIters"));
        String      outputPath      = ScalabilityProps.getProperty("assvpasResultFileSynch");
        runTest(numOfIters, true, outputPath);
        assertTrue(true);
    }
    
    public void runTest(int numOfIters, boolean synch, String outputPath) throws Exception
    {
        PrintWriter out             = new PrintWriter(new BufferedWriter(new FileWriter(outputPath)));        
        
        WSAddressEndpoint sender   = new WSAddressEndpoint ("http://sender.com");
        WSAddressEndpoint receiver = new WSAddressEndpoint ("http://receiver.com");
        String            lpid     = "1";
        
        String            stringContent = XML_CONTENT;
        
        if (synch) {
            getLocalTestbed().setSynchronousRecording(true);
        }
    
        for (int i = 0; i < numOfIters; i++) {
            
            
            String                iid     = Integer.toString (i);  
            InteractionKey        key     = new InteractionKey (sender.getElement (), receiver.getElement (), iid);
            Element               content  = this.getLocalTestbed().toDOM("<c:container xmlns:c=\"http://c.com\">" + stringContent + "</c:container>");
            
            ActorStatePAssertion  aspa    = new ActorStatePAssertion (lpid, content);
            Record                sendRec = new Record (aspa, key, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
          
            ClientLib clientLib = new ClientLib();
            String recDocStr = clientLib.makeRecordDocument(sendRec);
            
            long recordStrSize = recDocStr.length();
            
            long startTime = System.currentTimeMillis();
            
            getLocalTestbed().record(sendRec);
            
            long endTime   = System.currentTimeMillis();
            long elapsed   = endTime - startTime;

            out.println ("" + recordStrSize + " " + elapsed);
            out.flush();
            stringContent = stringContent + XML_CONTENT;
          
            
        }
        
        out.close();
    }
    
    
    public static junit.framework.Test suite() {
        return new junit.framework.JUnit4TestAdapter(AsynchSynchSpeedVsPAssertionSizeTest.class);
    }
    
    public static void main(String[] args) {
        org.junit.runner.JUnitCore.main("org.pasoa.preserv.test.scalability.AsynchSynchSpeedVsPAssertionSizeTest");
    }
}
