package org.pasoa.preserv.storage.mysql.test.scalability;

import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;

import org.pasoa.simpledom.DOMWriter;
import org.pasoa.storeclient.ClientLib;
import org.pasoa.util.httpsoap.WSAddressEndpoint;
import org.pasoa.common.Constants;
import org.pasoa.preserv.storage.mysql.test.MySQLLocalTestBedFixture;
import org.pasoa.preserv.test.LocalTestBedFixture;
import org.pasoa.preserv.test.LocalTestbed;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.PAssertion;
import org.pasoa.pstructure.Record;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.junit.*;
import static org.junit.Assert.*;

public class OverheadTest extends MySQLLocalTestBedFixture {

    /**
     * Some XML CONTENT
     */
    private static String XML_CONTENT = "<h:hello xmlns:h=\"http://www.hello.com\">hello</h:hello>";
    
    private static int INTERACTION = 1;
    private static int ACTOR_STATE = 2;
    private static int RELATION    = 3;
    
    @Before
    public void synchronousStart()
    { 
        getLocalTestbed().setSynchronousRecording(true);  
    }
    
   
    @Test
    public void testOverheadUsingActorStatePAssertions() throws Exception
    {
        int         numOfIters      = Integer.valueOf(ScalabilityProps.getProperty("otNumOfIters"));
        String      outputPath      = ScalabilityProps.getProperty("otResultFileASPA");
        runTest(numOfIters, outputPath, ACTOR_STATE);
        assertTrue(true);
    }
    
    
    @Test
    public void testOverheadUsingInteractionPAssertions() throws Exception
    {
        int         numOfIters      = Integer.valueOf(ScalabilityProps.getProperty("otNumOfIters"));
        String      outputPath      = ScalabilityProps.getProperty("otResultFileIPA");
        runTest(numOfIters, outputPath, INTERACTION);
        assertTrue(true);
    }
   
    public void runTest(int numOfIters, String outputPath, int typeOfPAssertion) throws Exception
    {
        PrintWriter       out      = new PrintWriter(new BufferedWriter(new FileWriter(outputPath)));        
        WSAddressEndpoint sender   = new WSAddressEndpoint ("http://sender.com");
        WSAddressEndpoint receiver = new WSAddressEndpoint ("http://receiver.com");
        String            lpid     = "1";
      
        String            stringContent = XML_CONTENT;
      
        System.out.println("Storing at " + getLocalTestbed().getPathToStorageDirectory());
        getLocalTestbed().enableBackendRecordTimeLogging(out);
        
        for (int i = 0; i < numOfIters; i++) {
            
            
            String         iid = Integer.toString (i);  
            InteractionKey key = new InteractionKey (sender.getElement (), receiver.getElement (), iid);
            PAssertion     pa  = null;
            
            if (typeOfPAssertion == ACTOR_STATE) {
                Element content  = getLocalTestbed().toDOM("<c:container xmlns:c=\"http://c.com\">" + stringContent + "</c:container>");
                        pa       = new ActorStatePAssertion (lpid, content);
            }
            
            if (typeOfPAssertion == INTERACTION) {
                Element content  = getLocalTestbed().toDOM("<c:container xmlns:c=\"http://c.com\">" + stringContent + "</c:container>");
                        pa       = new InteractionPAssertion (lpid, "http://www.pasoa.org/docstyle/Verbatim", content);
            }
            
            Record    sendRec       = new Record (pa, key, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
            ClientLib clientLib     = new ClientLib();
            long      recordStrSize = (clientLib.makeRecordDocument(sendRec)).length();
            

            out.print (i + " " + recordStrSize + " ");
            
            long startTime = System.currentTimeMillis();
            getLocalTestbed().record(sendRec);
            long endTime = System.currentTimeMillis();
            
            long totalTime = endTime - startTime;
            
            out.println(" " + totalTime);
           // stringContent = stringContent + XML_CONTENT;
          
        }
        
        out.close();
       
    }
    
    public static junit.framework.Test suite() {
        return new junit.framework.JUnit4TestAdapter(OverheadTest.class);
    }
    
    public static void main(String[] args) {
        org.junit.runner.JUnitCore.main("org.pasoa.preserv.test.scalability.OverheadTest");
    }

}
