package org.pasoa.preserv.test;

import static org.junit.Assert.assertTrue;

import java.util.LinkedList;

import org.junit.Test;
import org.pasoa.common.Constants;
import org.pasoa.preserv.test.scalability.AsynchSynchSpeedVsDatabaseSizeTest;
import org.pasoa.preserv.test.scalability.ScalabilityProps;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.ExposedInteractionMetadata;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.Link;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.pstructure.Record;
import org.pasoa.pstructure.RecordExposedInteractionMetadata;
import org.pasoa.pstructure.RecordSf;
import org.pasoa.pstructure.RelationshipPAssertion;
import org.pasoa.simpledom.DOMSerialiser;
import org.pasoa.simpledom.DOMWriter;
import org.pasoa.util.httpsoap.WSAddressEndpoint;
import org.w3c.dom.Element;

/**
 * This test determines whether the server supports all the
 * parts of the pstructure
 * @author pgroth
 *
 */
public class PStructureSupportTest extends LocalTestBedFixture 
{
    private static String XML_CONTENT = "<h:hello xmlns:h=\"http://www.hello.com\">hello</h:hello>";
    private static String DATA_ACCESSOR_STR = "<xp:xpath xmlns:xp=\"http://www.pasoa.org/xpath\">/</xp:xpath>";
    
   
    /**
     * Test to see if an interaction p-assertion can be recorded
     * in both views
     * 
     */
    @Test public void testPStructureSupport() throws Exception
    {
        this.getLocalTestbed().setSynchronousRecording(true);
        WSAddressEndpoint sender         =  new WSAddressEndpoint ("http://sender.com");
        WSAddressEndpoint receiver       =  new WSAddressEndpoint ("http://receiver.com");
        Element           content        =  getLocalTestbed().toDOM(XML_CONTENT);
        InteractionKey    interactionKey =  new InteractionKey (sender.getElement(), receiver.getElement(), "uniqueidof1");
        Record            senderRecord   =  null;
        Record            receiverRecord =  null;
        
        
        // Create an actor state p-assertion
        System.out.println("Recording actor state p-assertions");
        String actorStatePaLpid = "aslpid";         
        ActorStatePAssertion aspa = new ActorStatePAssertion (actorStatePaLpid, content);
        senderRecord   = new Record (aspa, interactionKey, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
        receiverRecord = new Record (aspa, interactionKey, Constants.RECEIVER_VIEW_TYPE,  receiver.getElement ());
        this.getLocalTestbed().record(senderRecord);
        this.getLocalTestbed().record(receiverRecord);
        
        System.out.println("Recording interaction p-assertions");
        String interactionPaLpid = "interactionLpid";
        String documentationStyle = "http://www.pasoa.org/schemas/ontologies/DocumentationStyle.Verbatim";
        InteractionPAssertion ipa = new InteractionPAssertion(interactionPaLpid, documentationStyle, content);
        senderRecord   = new Record (ipa, interactionKey, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
        receiverRecord = new Record (ipa, interactionKey, Constants.RECEIVER_VIEW_TYPE,  receiver.getElement ());
        this.getLocalTestbed().record(senderRecord);
        this.getLocalTestbed().record(receiverRecord);
        
        System.out.println("Recording relationship p-assertions");
        
        String relationshipPaLpid = "rellpid";
        Element dataAccessor = getLocalTestbed().toDOM(DATA_ACCESSOR_STR);
        String relation = "http://www.pasoa.org/schemas/ontologies/relationships/use"; 
        String subjectLpid = interactionPaLpid;
        String subjectParameterName = "http://www.pasoa.org/schemas/ontologies/relationships/usage.user";
        
        String objectLpid = actorStatePaLpid; // points to the actor state p-assertion
        String objectParameterName = "http://www.pasoa.org/schemas/ontologies/relationships/usage.usedItem";
        Link   link = new Link ("http://www.somepstore.com");
        
        GlobalPAssertionKey gpak = new GlobalPAssertionKey(interactionKey, Constants.SENDER_VIEW_TYPE, objectLpid);
        ObjectID objId = new ObjectID (gpak, objectParameterName, dataAccessor, link);
        LinkedList objectIds = new LinkedList();
        objectIds.add (objId);
        objectIds.add (objId);
        
        
        
        RelationshipPAssertion rel = new RelationshipPAssertion(relationshipPaLpid, 
                                                                subjectLpid, 
                                                                dataAccessor, 
                                                                subjectParameterName, 
                                                                relation, 
                                                                objectIds);
        
        senderRecord = new Record (rel, interactionKey, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
        receiverRecord = new Record (rel, interactionKey, Constants.RECEIVER_VIEW_TYPE,  receiver.getElement ());
        
        this.getLocalTestbed().record(senderRecord);
        this.getLocalTestbed().record(receiverRecord);
        
        System.out.println("Recording interaction metadata");
        
        DOMSerialiser builder = new DOMSerialiser (Constants.PSTRUCT_NS);
        
        builder.start  ("interactionMetadata");
        builder.single ("someMetadata",  "hello");
        builder.end ();
        
        LinkedList ll = new LinkedList();
        ll.add(builder.getRoot());
        ExposedInteractionMetadata eim = new ExposedInteractionMetadata(gpak, ll);
        
        senderRecord = new RecordExposedInteractionMetadata (eim, interactionKey, Constants.SENDER_VIEW_TYPE, sender.getElement());
        receiverRecord = new RecordExposedInteractionMetadata (eim, interactionKey, Constants.RECEIVER_VIEW_TYPE, sender.getElement());
        
        this.getLocalTestbed().record(senderRecord);
        this.getLocalTestbed().record(receiverRecord);
        
        senderRecord = new RecordSf (5, interactionKey, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
        receiverRecord = new RecordSf (5, interactionKey, Constants.RECEIVER_VIEW_TYPE,  receiver.getElement ());
        
        this.getLocalTestbed().record(senderRecord);
        this.getLocalTestbed().record(receiverRecord);
        
        DOMWriter domWriter = new DOMWriter (System.out);
        domWriter.setFormatted(true);
        
        domWriter.write(this.getLocalTestbed().getStoreContents());
        System.out.println();
        
        assertTrue(true);
    }
    
    public static junit.framework.Test suite() {
        return new junit.framework.JUnit4TestAdapter(PStructureSupportTest.class);
    }
    
    public static void main(String[] args) {
        org.junit.runner.JUnitCore.main("org.pasoa.preserv.test.PStructureSupportTest");
    }
}
