/********************************************************************************
 * Created on 2006 February 28
 *
 * Copyright (c) 2005 University of Southampton
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE.
 *******************************************************************************/
package org.pasoa.preserv.test;

import java.net.URL;
import java.util.Map;
import org.pasoa.common.Constants;
import org.pasoa.preserv.pquery.ProvenanceQueryRequest;
import org.pasoa.pstructure.Record;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.storeclient.ClientLib;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * A testbed implementation that calls a remote Web Service provenance store
 * to record and query.
 */
public class RemoteTestbed extends Testbed {
    private String    _storeURL;
    private URL       _recordURL;
    private URL       _xqueryURL;
    private URL       _pqueryURL;
    private ClientLib _client;
    
    /**
     * Creates a RemoteTestbed that uses given URLs to record, XQuery and do
     * provenance queries.  If you are not intending to perform queries, you can
     * give null for these URLs.
     */
    public RemoteTestbed (String storeURL) throws Exception {
        _client    = new ClientLib ();
        _storeURL  = storeURL;
        _recordURL = new URL (storeURL + "/" + Constants.RECORD_CONTEXT);
        _xqueryURL = new URL (storeURL + "/" + Constants.XQUERY_CONTEXT);
        _pqueryURL = new URL (storeURL + "/" + Constants.PQUERY_CONTEXT);
    }
    
    public String getURL () {
        return _storeURL;
    }

    public Element provenanceQuery (String qdhXPath, String scopeXPath, Map namespaces) throws Exception {
        DOMDeserialiser reader   = new DOMDeserialiser (Constants.SOAP_NS);
        Document        response = _client.provenanceQuery (qdhXPath, scopeXPath, namespaces, _pqueryURL);
        Element         body     = reader.single (response.getDocumentElement (), Constants.SOAP_NS, "Body");
        
        return reader.single (response.getDocumentElement (), Constants.PQUERY_NS, "provenanceQueryResult");
    }
    
    public Element provenanceQuery (ProvenanceQueryRequest query) throws Exception {
        DOMDeserialiser reader   = new DOMDeserialiser (Constants.SOAP_NS);
        Document        response = _client.provenanceQuery (query, _pqueryURL);
        Element         body     = reader.single (response.getDocumentElement (), Constants.SOAP_NS, "Body");
        
        return reader.single (body, Constants.PQUERY_NS, "provenanceQueryResult");
    }

    public void record (Record record) throws Exception {
        _client.record (record, _recordURL);
    }

    public Element xpath (String xpath, String root, String item, Map namespaces) throws Exception {
        return _client.xpath (xpath, root, item, namespaces, _xqueryURL).getDocumentElement ();
    }

    public Element xquery (String xquery) throws Exception {
        DOMDeserialiser reader   = new DOMDeserialiser (Constants.SOAP_NS);
        Document        response = _client.query (xquery, _xqueryURL);
        Element         body     = reader.single (response.getDocumentElement (), Constants.SOAP_NS, "Body");
        
        return reader.single (body, Constants.WSXQUERY_NS, "queryResult");
    }
}
