package org.pasoa.preserv.test;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.pasoa.simpledom.HighlightingDOMWriter;
import org.pasoa.simpledom.NodeListIterator;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;

public class XMLComparison {
    public boolean compare (File expected, Document produced, boolean showdiff) throws ParserConfigurationException, SAXException, IOException {
        DocumentBuilderFactory factory  = DocumentBuilderFactory.newInstance ();
        DocumentBuilder        builder  = factory.newDocumentBuilder ();
        
        return compare (builder.parse (expected), produced, showdiff);
    }
    
    public boolean compare (Document expected, Document produced, boolean showdiff) throws IOException {
        return compare (expected.getDocumentElement (), produced.getDocumentElement (), showdiff);
    }
    
    public boolean compare (Element expected, Element produced, boolean showdiff) throws IOException {
        Collection            mismatches = compare (expected, produced);
        HighlightingDOMWriter writer;
        
        if (mismatches != null && showdiff) {
            writer = new HighlightingDOMWriter (System.err, mismatches);
            System.err.println ("First mismatched document: \n");
            writer.write (expected);
            System.err.println ("\nSecond mismatched document: \n");
            writer.write (produced);
        }
        
        return mismatches == null;
    }
    
    private Collection compare (Node expected, Node produced) {
        Iterator   expectedChildren = new NodeListIterator (expected.getChildNodes ());
        Iterator   producedChildren = new NodeListIterator (produced.getChildNodes ());
        Node       expectedChild;
        Node       producedChild;
        Collection subMismatch;
        
        if (expected.getNodeType () == produced.getNodeType ()
        && equalOrNull (expected.getNamespaceURI (), produced.getNamespaceURI ())
        && equalOrNull (expected.getLocalName (),    produced.getLocalName ())
        && equalOrNull (expected.getNodeValue (),    produced.getNodeValue ())) {
            while (expectedChildren.hasNext ()) {
                expectedChild = (Node) expectedChildren.next ();
                if (empty (expectedChild)) {
                    continue;
                }
                if (!producedChildren.hasNext ()) {
                    return mismatched (expected, produced);
                }
                do {
                    producedChild = (Node) producedChildren.next ();
                } while (producedChildren.hasNext () && empty (producedChild));
                subMismatch = compare (expectedChild, producedChild);
                if (subMismatch != null) {
                    return subMismatch;
                }
            }
            while (producedChildren.hasNext ()) {
                producedChild = (Node) producedChildren.next ();
                if (!empty (producedChild)) {
                    return mismatched (expected, produced);
                }
            }
            return null;
        }
        
        return mismatched (expected, produced);
    }
    
    /**
     * Convenience method returning true only if both objects are null or if they are
     * equal.
     */
    private boolean equalOrNull (String first, String second) {
        return (first == null && second == null) || (first != null && second != null && first.trim ().equals (second.trim ()));
    }
    
    private boolean empty (Node node) {
        return node instanceof Text && node.getNodeValue ().trim ().equals ("");
    }

    private Collection mismatched (Node expected, Node produced) {
        Collection set = new LinkedList ();
        set.add (expected);
        set.add (produced);
        return set;
    }
}
