package org.pasoa.preserv.test.scalability;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;

import org.pasoa.common.Constants;
import org.pasoa.preserv.test.LocalTestBedFixture;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.Record;
import org.pasoa.util.httpsoap.WSAddressEndpoint;
import org.w3c.dom.Element;

import org.junit.*;
import static org.junit.Assert.*;

public class AsynchSynchSpeedVsDatabaseSizeTest extends LocalTestBedFixture
{

    /**
     * Some XML CONTENT
     */
    private static String XML_CONTENT = "<h:hello xmlns:h=\"http://www.hello.com\">hello</h:hello>";
    
    @Test
    public void asynchronousTest() throws Exception
    {
        
        int         numOfIters      = Integer.valueOf(ScalabilityProps.getProperty("assvdsNumOfIters"));
        String      outputPath      = ScalabilityProps.getProperty("assvdsResultFileAsynch");
        PrintWriter out             = new PrintWriter(new BufferedWriter(new FileWriter(outputPath)));        
        String      storageDirPath  = this.getLocalTestbed().getPathToStorageDirectory();
        File        storageDir      = new File (storageDirPath);
        
        System.out.println ("Path to storage: " + getLocalTestbed().getPathToStorageDirectory());
        
        
        WSAddressEndpoint sender   = new WSAddressEndpoint ("http://sender.com");
        WSAddressEndpoint receiver = new WSAddressEndpoint ("http://receiver.com");
        
        
        
        /*BufferedReader br = new BufferedReader (new FileReader ("./testdata.txt"));
        String someContent = "";
        String tmp = "";
        while ((tmp = br.readLine()) != null) {
            someContent = someContent + tmp;
        }
        br.close();*/
        Element           content  = getLocalTestbed().toDOM(XML_CONTENT);
        String            lpid     = "1";
        
        long totalTimeStart = System.currentTimeMillis();
        
        for (int i = 0; i < numOfIters; i++) {
            
            String                iid     = Integer.toString (i);  
            InteractionKey        key     = new InteractionKey (sender.getElement (), receiver.getElement (), iid);
            ActorStatePAssertion  aspa    = new ActorStatePAssertion (lpid, content);
            Record                sendRec = new Record (aspa, key, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
          
            long dbSize    = getSize(storageDir);
            long startTime = System.currentTimeMillis();
            
            getLocalTestbed().record(sendRec);
            
            long endTime   = System.currentTimeMillis();
            long elapsed   = endTime - startTime;
            
            out.println ("" + dbSize + " " + elapsed);
        }
        out.close();
        
        long totalTimeEnd = System.currentTimeMillis();
        
        System.out.println ("Total time for asynchronous record " + (totalTimeEnd - totalTimeStart));
        
        assertTrue(true);
        
    }
    
    @Test
    public void synchronousTest() throws Exception
    {
        getLocalTestbed().setSynchronousRecording(true);
        System.out.println ("Path to storage: " + getLocalTestbed().getPathToStorageDirectory());
        int         numOfIters     = Integer.valueOf(ScalabilityProps.getProperty("assvdsNumOfIters"));
        String      outputPath     = ScalabilityProps.getProperty("assvdsResultFileSynch");
        PrintWriter out            = new PrintWriter(new BufferedWriter(new FileWriter(outputPath)));        
        String      storageDirPath = this.getLocalTestbed().getPathToStorageDirectory();
        File        storageDir     = new File (storageDirPath);

        WSAddressEndpoint     sender   = new WSAddressEndpoint ("http://sender.com");
        WSAddressEndpoint     receiver = new WSAddressEndpoint ("http://receiver.com");
        
        /*BufferedReader br = new BufferedReader (new FileReader ("./testdata.txt"));
        String someContent = "";
        String tmp = "";
        while ((tmp = br.readLine()) != null) {
            someContent = someContent + tmp;
        }
        br.close();*/
        Element               content  = getLocalTestbed().toDOM(XML_CONTENT);
        String                lpid     = "1";
        
        for (int i = 0; i < numOfIters; i++) {
            
            String                iid     = Integer.toString (i);  
            InteractionKey        key     = new InteractionKey (sender.getElement (), receiver.getElement (), iid);
            ActorStatePAssertion  aspa    = new ActorStatePAssertion (lpid, content);
            Record                sendRec = new Record (aspa, key, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
          
            long dbSize    = getSize(storageDir);
            long startTime = System.currentTimeMillis();
            
            getLocalTestbed().record(sendRec);
            
            long endTime   = System.currentTimeMillis();
            long elapsed   = endTime - startTime;
            
            out.println ("" + dbSize + " " + elapsed);
        }
        out.close();
        assertTrue(true);
        
    }
    
    public static junit.framework.Test suite() {
        return new junit.framework.JUnit4TestAdapter(AsynchSynchSpeedVsDatabaseSizeTest.class);
    }
    
    public static void main(String[] args) throws Exception {
        org.junit.runner.JUnitCore.main("org.pasoa.preserv.test.scalability.AsynchSynchSpeedVsDatabaseSizeTest");
        //AsynchSynchSpeedVsDatabaseSizeTest test = new AsynchSynchSpeedVsDatabaseSizeTest();
        //test.setUp();
        //test.asynchronousTest();
        //test.tearDown();
        //test.setUp();
        //test.synchronousTest();
        //test.tearDown();
    }
}
