package org.pasoa.preserv.test.scalability;

import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;

import org.pasoa.simpledom.DOMWriter;
import org.pasoa.util.httpsoap.WSAddressEndpoint;
import org.pasoa.common.Constants;
import org.pasoa.preserv.test.LocalTestBedFixture;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.Record;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.junit.*;

import static org.junit.Assert.*;

public class DatabaseSizeTest extends LocalTestBedFixture {

    /**
     * Some XML CONTENT 61 charcters long
     */
    private static String XML_CONTENT = "<h:hello xmlns:h=\"http://www.hello.com\">hello</h:hello>";
    
    @Before
    public void synchronousStart()
    { 
        getLocalTestbed().setSynchronousRecording(true);  
    }
    
   @Test
    public void testDatabaseSize() throws Exception
    {
        int         numOfIters     = Integer.valueOf(ScalabilityProps.getProperty("dbstNumOfIters"));
        String      outputPath     = ScalabilityProps.getProperty("dbstResultFileDBS");
        PrintWriter out            = new PrintWriter(new BufferedWriter(new FileWriter(outputPath)));
        String      storageDirPath = this.getLocalTestbed().getPathToStorageDirectory();
        File        storageDir     = new File (storageDirPath);
        long        size           = getSize(storageDir);
        
        Element               content  = this.getLocalTestbed().toDOM(XML_CONTENT);
        WSAddressEndpoint     sender   = new WSAddressEndpoint ("http://sender.com");
        WSAddressEndpoint     receiver = new WSAddressEndpoint ("http://receiver.com");
        
        
        
        for (int i = 0; i < numOfIters; i++) {
            
            out.print((i + 1) + " ");
            
            String interactionId          = Integer.toString(i);
            String localPAssertionId      = Integer.toString (i);
            InteractionKey        key     = new InteractionKey (sender.getElement (), receiver.getElement (), interactionId);
            ActorStatePAssertion  aspa    = new ActorStatePAssertion (localPAssertionId, content);
            Record                sendRec = new Record (aspa, key, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
            
            long iterStartTime = System.currentTimeMillis();
            getLocalTestbed().record(sendRec);
            long iterEndTime = System.currentTimeMillis();
            long folderSize = getSize(storageDir);
            
            long queryStartTime = System.currentTimeMillis();
            Document dom = getLocalTestbed().getStoreContents();
            long queryEndTime = System.currentTimeMillis();
            
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            DOMWriter dwriter = new DOMWriter(baos);
            dwriter.write(dom);
            int xmlSize = baos.toByteArray().length;
            baos.close();
            
            long queryTime   = queryEndTime - queryStartTime;
            long iterTime    = iterEndTime - iterStartTime;
            //number of p-assertions, folder size, xmlSize
            out.println(" " + folderSize + " " + xmlSize + " " + iterTime + " " + queryTime);
            
        }
        
        out.close();
        assertTrue(true);
    }
    
    
    @Test
    public void testDatabaseSizeMultipleAssertionsPerIR() throws Exception
    {
        int         numOfIters     = Integer.valueOf(ScalabilityProps.getProperty("dbstNumOfIters"));
        String      outputPath     = ScalabilityProps.getProperty("dbstResultFileDBSOneIR");
        PrintWriter out            = new PrintWriter(new BufferedWriter(new FileWriter(outputPath)));        
        String      storageDirPath = this.getLocalTestbed().getPathToStorageDirectory();
        File        storageDir     = new File (storageDirPath);
        long        size           = getSize(storageDir);
        
        
        
        // Create one interaction key
        WSAddressEndpoint     sender   = new WSAddressEndpoint ("http://sender.com");
        WSAddressEndpoint     receiver = new WSAddressEndpoint ("http://receiver.com");
        InteractionKey        key      = new InteractionKey (sender.getElement (), receiver.getElement (), "1");
        Element               content  = this.getLocalTestbed().toDOM(XML_CONTENT);
        
        // Record i actor state p-assertions into the same interaction record
        // identified by the interaction key, key.
        for (int i = 0; i < numOfIters; i++) {
            out.print((i + 1) + " ");
            
            String localPAssertionId      = Integer.toString (i);
            
            ActorStatePAssertion  aspa    = new ActorStatePAssertion (localPAssertionId, content);
            Record                sendRec = new Record (aspa, key, Constants.SENDER_VIEW_TYPE,  sender.getElement ());
          
            long iterStartTime = System.currentTimeMillis();
            getLocalTestbed().record(sendRec);
            long iterEndTime = System.currentTimeMillis();
            
            long folderSize = getSize(storageDir);
            
            long queryStartTime = System.currentTimeMillis();
            Document dom = getLocalTestbed().getStoreContents();
            long queryEndTime = System.currentTimeMillis();
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            DOMWriter dwriter = new DOMWriter(baos);
           
            dwriter.write(dom);
            int xmlSize = baos.toByteArray().length;
            baos.close();
            
            long queryTime   = queryEndTime - queryStartTime;
            long iterTime    = iterEndTime - iterStartTime;
            
            //number of p-assertions, folder size, xmlSize
            out.println(" " + folderSize + " " + xmlSize + " " + iterTime + " " + queryTime);
        }
        
        out.close();
        assertTrue(true);
    }
    
    public static junit.framework.Test suite() {
        return new junit.framework.JUnit4TestAdapter(DatabaseSizeTest.class);
    }
    
    public static void main(String[] args) {
        org.junit.runner.JUnitCore.main("org.pasoa.preserv.test.scalability.DatabaseSizeTest");
    }

}
