package org.pasoa.preserv.xquery;

import java.io.OutputStream;
import java.util.Map;
import org.pasoa.common.Constants;
import org.pasoa.preserv.factory.PlugIn;
import org.pasoa.preserv.storage.GeneralStorageSystem;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.preserv.storage.StorageSystem;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class XQueryPlugIn implements PlugIn {
    private void checkNotNull (Element node, String namespace, String name) throws QueryProblem {
        if (node == null) {
            throw new QueryProblem ("Cannot parse XQuery request: no {" + namespace + "}" + name + " element");
        }
    }
    
    public Document process (Document request, StorageSystem ss) {
        return process (request, ss, null);
    }
    
    public Document process (Document request, StorageSystem ss, OutputStream response) {
        try {
            System.out.println ("XQueryPlugIn: processing");
            
            DOMDeserialiser reader     = new DOMDeserialiser ();
            Element         envelope   = request.getDocumentElement ();
            Element         body       = reader.single (request, Constants.SOAP_NS, "Body");
            checkNotNull (body, Constants.SOAP_NS, "Body");
            Element         query      = reader.single (body, Constants.WSXQUERY_NS, "query");
            checkNotNull (query, Constants.WSXQUERY_NS, "query");
            Element         xqueryNode = reader.single (query, Constants.WSXQUERY_NS, "xquery");
            checkNotNull (xqueryNode, Constants.WSXQUERY_NS, "xquery");
            String          xquery     = "";
            Document        results;
            /*
            NodeList        children   = xqueryNode.getChildNodes ();
             
            for (int index = 0; index < children.getLength (); index += 1) {
                xquery = xquery + DOMWriter.writeToString (children.item (index));
            }*/
            xquery = xqueryNode.getTextContent ();
            
            System.out.println ("xquery: " + xquery);
            
            if (response == null) {
                results = ss.resolve (xquery);
            } else {
                ((GeneralStorageSystem) ss).resolve (xquery, response);
                results = QueryConstants.errorMsg ("Change of plug-in structure: now use output stream directly");;
            }
            
            //System.out.println ("xquery results: " + DOMWriter.writeToString (results));
            
            return results;
        } catch (QueryProblem problem) {
            problem.printStackTrace ();
            return QueryConstants.errorMsg (problem.getMessage ());
        }
    }
    
    public boolean receiveDispatchesSyncronously () {
        return true;
    }
    
    public String getDefaultResponse () {
        // Must be done synchrounously, so no default response
        return null;
    }
    
    public Map getConfiguration () {
        // TODO Auto-generated method stub
        return null;
    }
    
    public void initialize (Map configuration) {
        // TODO Auto-generated method stub
        
    }
}
