package org.pasoa.preserv.xquery;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.xml.namespace.QName;
import org.pasoa.common.Constants;
import org.pasoa.simpledom.DOMSerialiser;
import org.pasoa.simpledom.SimpleNode;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

public class XQueryProcessor {
    public static Document resolve (String query, Document storeDoc, XQueryResolver engine) throws QueryProblem {
        DOMSerialiser write = new DOMSerialiser (Constants.WSXQUERY_NS);
        Map      spaces    = new HashMap ();
        Map      documents = new HashMap ();
        Map      variables = new HashMap ();
        String   pasoaINS  = "declare namespace " + "psInternal" + " = \"" + Constants.PSTRUCT_NS + "\"; ";
        String   storeVar  = pasoaINS + "declare variable $" + "psInternal" + ":pstruct external ;";
        String   pasoaNS   = "declare namespace " + "ps" + " = \"" + Constants.PSTRUCT_NS + "\"; ";
        String   newProlog = pasoaNS + storeVar;
        Document result;
        Iterator eachAnswer;
        Node     answer;
        
        query = storeVar + query;
        if (query.indexOf ("declare namespace ps ") < 0 && query.indexOf ("declare namespace ps=") < 0) {
            query = pasoaNS + query;
        }
        
        documents.put ("store", storeDoc);
        variables.put (new QName (Constants.PSTRUCT_NS, "pstruct", "ps"), storeDoc.getDocumentElement ());
        System.out.println ("Before the resolve...");
        eachAnswer = engine.resolve (query, documents, variables).iterator ();
        System.out.println ("After the resolve");
        
        write.start ("queryResult");
        while (eachAnswer.hasNext ()) {
            answer = (Node) eachAnswer.next ();
            if (answer instanceof Document) {
                Element element = ((Document) answer).getDocumentElement ();
                if (element instanceof SimpleNode) {
                    write.moveLazily ((SimpleNode) element);
                } else {
                    write.append ((Element) element);
                }
            }
            if (answer instanceof Element) {
                if (answer instanceof SimpleNode) {
                    write.moveLazily ((SimpleNode) answer);
                } else {
                    write.append ((Element) answer);
                }
            }
            if (answer instanceof Text) {
                write.text (((Text) answer).getNodeValue ());
            }
        }
        return (Document) write.getRoot ();
    }
    
    public static void resolve (String query, Document storeDoc, XQueryResolver engine, OutputStream outputStream) throws QueryProblem {
        Map    spaces    = new HashMap ();
        Map    documents = new HashMap ();
        Map    variables = new HashMap ();
        String pasoaINS  = "declare namespace " + "psInternal" + " = \"" + Constants.PSTRUCT_NS + "\"; ";
        String storeVar  = pasoaINS + "declare variable $" + "psInternal" + ":pstruct external ;";
        String pasoaNS   = "declare namespace " + "ps" + " = \"" + Constants.PSTRUCT_NS + "\"; ";
        String newProlog = pasoaNS + storeVar;
        Writer output    = new OutputStreamWriter (outputStream);
        
        query = storeVar + query;
        if (query.indexOf ("declare namespace ps ") < 0 && query.indexOf ("declare namespace ps=") < 0) {
            query = pasoaNS + query;
        }
        
        documents.put ("store", storeDoc);
        variables.put (new QName (Constants.PSTRUCT_NS, "pstruct", "ps"), storeDoc.getDocumentElement ());
        
        try {
            output.write ("<queryResult xmlns=\"" + Constants.WSXQUERY_NS + "\">");
            ((org.pasoa.preserv.xquery.laxquery.XQueryEngine) engine).resolve (query, documents, variables, output);
            output.write ("</queryResult>");
        } catch (IOException ex) {
            throw new QueryProblem ("Problem in outputting", ex);
        }
    }
}
