package org.pasoa.preserv.xquery.ipsixq;

import de.fraunhofer.ipsi.ipsixq.api.impl.IPSIXQDataSource;
import java.io.IOException;
import java.io.StringReader;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xquery.XQConnection;
import javax.xml.xquery.XQDatasource;
import javax.xml.xquery.XQException;
import javax.xml.xquery.XQExpression;
import javax.xml.xquery.XQItemAccessor;
import javax.xml.xquery.XQResultSequence;
import org.pasoa.preserv.xquery.QueryProblem;
import org.pasoa.preserv.xquery.XQueryResolver;
import org.pasoa.simpledom.DOMWriter;
import org.pasoa.simpledom.NormalisingDOMWriter;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class XQueryUtilities implements XQueryResolver {
    private DocumentBuilder _builder;
    
    public XQueryUtilities () throws QueryProblem {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
            factory.setNamespaceAware (true);
            _builder = factory.newDocumentBuilder ();
        } catch (ParserConfigurationException ex) {
            throw new QueryProblem ("XQueryUtilities setup problem", ex);
        }
    }
    
    private List convertSequence (XQResultSequence sequence) throws QueryProblem {
        List converted = new LinkedList ();
        
        try {
            while (!sequence.isAfterLast ()) {
                sequence.next ();
                converted.add (convertItem (sequence));
            }
        } catch (XQException ipsiProblem) {
            // No item
        }
        
        return converted;
    }
    
    private Node convertItem (XQItemAccessor accessor) throws XQException, QueryProblem {
        try {
            /* This complicated process is to get around the fact that IPSI-XQ
             * is not really namespace-aware, but just uses prefixes and xmls:*
             * attributes to work it out on the fly.  The process is:
             *   - Get back the namespace-less DOM version
             *   - Write it to a string containing prefixes and lots of xmlns:* attributes
             *   - Read it in as a namespace-aware DOM
             *   - Write it to a normalised string excluding unncessary prefixes
             *   - Read it back in as nice namespace-aware DOM
             */
            
            Node   original = accessor.getNode ();
            String xml      = DOMWriter.writeToString (original);
            Node   spaced   = _builder.parse (new InputSource (new StringReader (xml)));
            String normal   = NormalisingDOMWriter.writeToString (spaced);
            Node   finalDoc = _builder.parse (new InputSource (new StringReader (normal)));

            return finalDoc;
        } catch (IOException ex) {
            throw new QueryProblem ("Problem extracting XQuery result", ex);
        } catch (SAXException ex) {
            throw new QueryProblem ("Problem extracting XQuery result", ex);
        }
    }
    
    private Map gatherAncestorPrefixes (Node node) {
        NamedNodeMap attrs = node.getAttributes ();
        Node         attr;
        
        System.out.println (node.getNodeName ());
        if (attrs != null) {
            for (int index = 0; index < attrs.getLength (); index += 1) {
                attr = attrs.item (index);
                if (attr.getNodeName ().startsWith ("xmlns")) {
                    System.out.println (node.getNodeName () + ": " + attr.getNodeName () + " = " + attr.getNodeValue ());
                }
            }
        }
        if (node.getParentNode () != null) {
            gatherAncestorPrefixes (node.getParentNode ());
        }
        
        return null;
    }
    
    public List resolve (String query, Map documents, Map variables) throws QueryProblem {
        System.out.println ("IPSIXQ: got a call");
        try {
            //establish connection to IPSI-XQ engine
            XQDatasource     dataSource = new IPSIXQDataSource ();
            XQConnection     connection = dataSource.getConnection ();
            Iterator         eachVar    = variables.keySet ().iterator ();
            QName            varName;
            Object           varValue;
            XQExpression     expression;
            XQResultSequence sequence;
            
            while (eachVar.hasNext ()) {
                varName  = (QName) eachVar.next ();
                varValue = variables.get (varName);
                if (varValue instanceof Node) {
                    connection.bindNode (varName, (Node) varValue);
                }
                if (varValue instanceof String) {
                    connection.bindString (varName, (String) varValue);
                }
                // ...etc.
            }
            
            //create the expression used to carry out the query
            expression = connection.createExpression ();
            System.out.println (query);
            //execute query
            
            return convertSequence (expression.executeQuery (query));
        } catch (Exception ipsiProblem) {
            throw new QueryProblem ("Problem in IPSI-XQ engine", ipsiProblem);
        }
    }
    
    public boolean resolveContainsAny (String query, Map documents, Map variables) throws QueryProblem {
        return resolve (query, documents, variables).size () > 0;
    }
}
