/*
 * AddressAsserterIdentity.java
 *
 * Created on 25 juillet 2006, 20:00
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.pasoa.preview;

import java.util.Vector;
import org.pasoa.common.Constants;
import org.pasoa.preview.utils.PReViewUtilities;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.util.httpsoap.WSAddressEndpoint;
import org.w3c.dom.Element;

/**
 * Implementation of AsserterIdentity Interface for a Address asserter.
 * An Address asserter is like :
 * <asserter>
 *  <wsa:Address>http://one.address</wsa:Address>
 * </asserter>
 * @author tibo
 */
public class AddressAsserterIdentity implements AsserterIdentity {
    
    private Element element;
    private String longName;
    private String shortName;
    
    /**
     * Creates a new instance of AddressAsserterIdentity from an element
     * The given element must be on the the following form :
     * <ps:name>One name</ps:name>
     * <ps:dataName>an other name</ps:dataName>
     * @param elem XML element
     */
    public AddressAsserterIdentity(Element elem) {
        this.element = elem;
        DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);
        this.longName = domDeserialiser.text(this.element, "name");
        //this.longName = domDeserialiser.text(this.element, "Address");
        String dataName = domDeserialiser.text(this.element, "dataName");
        if(dataName.equals("")) {
            //System.out.println("name == null");
            this.shortName = PReViewUtilities.getLastPartURL(this.getLongName()); 
        }
        else {
            this.shortName = dataName;
        }
    }
    
    
    /**
     * Creates a new instance of AddressAsserterIdentity from a String.
     * Used for the "nothing" asserterIdentity.
     * @param elem A String used as shortName
     */
    public AddressAsserterIdentity(String elem) {
        this.element = null;
        this.longName = "";
        this.shortName = elem;
    }
    
    /**
     * Accessor which return the short name of this asserter identity.
     * @return a String
     */
    public String getShortName() {
       return this.shortName;
    }

    /**
     * Accessor which return the long name of this asserter identity.
     * @return a String
     */
    public String getLongName(){
        return  this.longName;
    }
    
    /**
     * Accessor which return the real name of this asserter identity.
     * Used to compare an asserter identity with an asserter in the store.
     * @return a String
     */
    public String getRealName(){
        return  this.longName;
    }
    
    /**
     * Method to compare 2 AddressAsserterIdentity.
     * @param obj an Asserter Identity.
     * @return return true if the two asserter Identity are the same, else return false
     */
    public boolean equals(Object obj) {
        if(obj instanceof AddressAsserterIdentity){
            if(((AddressAsserterIdentity)obj).getRealName().equals(this.getRealName()))
                return true;
        }
        return false;
    }
}
