/*
 * AddressAsserterUtility.java
 *
 * Created on 26 juillet 2006, 14:47
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.pasoa.preview;

import java.util.Vector;
import org.pasoa.common.BestPractice;
import org.pasoa.common.Constants;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.simpledom.DOMWriter;
import org.pasoa.util.httpsoap.WSAddressEndpoint;
import org.w3c.dom.Element;

/**
 * Implementation of AsserterUtility interface to the Address asserter.
 * An Address asserter is like :
 * <asserter>
 *  <wsa:Address>http://one.address</wsa:Address>
 * </asserter>
 * @author root
 */
public class AddressAsserterUtility implements AsserterUtility {
    
    /**
     * Creates a new instance of AddressAsserterUtility
     */
    public AddressAsserterUtility() {
    }
    
    /**
     * Create a new AddressAsserterIdentity from the given XML element, and return it.
     * @param element XML element
     * @return an AddressAsserterIdentity
     */
    public AsserterIdentity newAsserterIdentity (Element element){
        AddressAsserterIdentity aid = new AddressAsserterIdentity(element);
        return aid;
    }
    
    
    /**
     * Create a new "empty" AddressAsserterIdentity, and return it.
     * An "empty" AddressAsserterIdentity is used when there is a missing asserter (or a missing view kind) in an interactionRecord.
     * @return an AddressAsserterIdentity
     */
    public AsserterIdentity newEmptyAsserterIdentity() {
        AddressAsserterIdentity aid = new AddressAsserterIdentity("nothing");
        return aid;
    }
    
    /**
     * Return a string which can get all asserters from the store.
     * @return a String
     */
    public String getAssertersQuery() {
        
        String xQueryString = "declare namespace ap=\"" + BestPractice.ACTOR_PROFILE_NS + "\";" +
                    "for $n in fn:distinct-values(" +
                    "$ps:pstruct/ps:interactionRecord/*/ps:asserter/*/text()) " +
                    "order by $n return <ps:asserter><ps:name>{$n}</ps:name>" +
                    "<ps:dataName>{$n/../../../ps:actorStatePAssertion/ps:content/ap:ActorProfile/ap:name/text()}" +
                    "</ps:dataName></ps:asserter>";
        return xQueryString;
    }
    
    /**
     * Method which return a query to get all the interactions record between the two given asserterIdentity.
     * @param sender an Asserter Identity
     * @param receiver another Asserter Identity
     * @return a String
     */
    public String getInteractionsBetweenQuery(AsserterIdentity sender, AsserterIdentity receiver) {
        String InteractionsQuery;
       
        if(receiver.getRealName().equals("")) {
            InteractionsQuery = "declare namespace wsa=\"" + WSAddressEndpoint.wsaNamespace + "\";" +
                "for $n in $ps:pstruct/ps:interactionRecord[fn:empty(ps:receiver)]" +
                "/ps:sender/ps:asserter[wsa:Address/text()='"+ sender.getRealName() +"']" +
                "/../.. return $n";
        }
        else {
            if(sender.getRealName().equals("")) {
                InteractionsQuery = "declare namespace wsa=\"" + WSAddressEndpoint.wsaNamespace + "\";" +
                     "for $n in $ps:pstruct/ps:interactionRecord[fn:empty(ps:sender)]" +
                    "/ps:sender/ps:receiver[wsa:Address/text()='"+ receiver.getRealName() +"']" +
                    "/../.. return $n";
            }
            else {
        
                InteractionsQuery = "declare namespace wsa=\"" + WSAddressEndpoint.wsaNamespace + "\";" +
                    "for $n in $ps:pstruct/ps:interactionRecord/ps:sender/ps:asserter[wsa:Address/text()='"+ sender.getRealName() +"']" +
                    "/../../ps:receiver/ps:asserter[wsa:Address/text()='"+ receiver.getRealName() +"']/../.. " +
                    "return $n";
            }
        }
        return InteractionsQuery;
    }
    
    /**
     * Method which return a query to get the interaction record with the given interactionID between the two given asserterIdentity.
     * @param sender an Asserter Identity
     * @param receiver another Asserter Identity
     * @param interactionID a String
     * @return a String
     */
    public String getInteractionRecordBetweenQuery(AsserterIdentity sender, AsserterIdentity receiver, String interactionID) {
        String InteractionQuery;
       
        if(receiver.getRealName().equals("")) {
            InteractionQuery = "declare namespace wsa=\"" + WSAddressEndpoint.wsaNamespace + "\";" +
                        "for $n in $ps:pstruct/ps:interactionRecord[fn:empty(ps:receiver)]" +
                        "/ps:sender/ps:asserter[wsa:Address/text()='"+ sender.getRealName() +"']" +
                        "/../../ps:interactionKey[ps:interactionId/text()='"+ interactionID +"']/..  " +
                        "return $n";
        }
        else {
            if(sender.getRealName().equals("")) {
                InteractionQuery = "declare namespace wsa=\"" + WSAddressEndpoint.wsaNamespace + "\";" +
                        "for $n in $ps:pstruct/ps:interactionRecord[fn:empty(ps:sender)]" +
                        "/ps:sender/ps:receiver[wsa:Address/text()='"+ receiver.getRealName() +"']" +
                        "/../../ps:interactionKey[ps:interactionId/text()='"+ interactionID +"']/..  " +
                        "return $n";
            }
            else {
                InteractionQuery = "declare namespace wsa=\"" + WSAddressEndpoint.wsaNamespace + "\";" +
                        "for $n in $ps:pstruct/ps:interactionRecord/ps:sender/ps:asserter[wsa:Address/text()='"+ sender.getRealName() +"']" +
                        "/../../ps:receiver/ps:asserter[wsa:Address/text()='"+ receiver.getRealName() +"']" +
                        "/../../ps:interactionKey[ps:interactionId/text()='"+ interactionID +"']/..  " +
                        "return $n";
            }
        }
        return InteractionQuery;
    }

    /**
     * Method which return a query to get all interaction record where the given asserterIdentity is the sender.
     * @param ai an AsserterIdentity
     * @return a String
     */
    public String getInteractionsWithSenderQuery(AsserterIdentity ai) {
        String InteractionQuery = "declare namespace wsa=\"" + WSAddressEndpoint.wsaNamespace + "\";" +
                        "for $n in $ps:pstruct/ps:interactionRecord/ps:sender/ps:asserter[wsa:Address/text()='"+ ai.getRealName() +"']/../.. " +
                        "return $n";
        return InteractionQuery;
    }

    /**
     * Method which return a query to get all interaction record where the given asserterIdentity is the receiver.
     * @param ai an AsserterIdentity
     * @return a String
     */
    public String getInteractionsWithReceiverQuery(AsserterIdentity ai) {
        String InteractionQuery = "declare namespace wsa=\"" + WSAddressEndpoint.wsaNamespace + "\";" +
                        "for $n in $ps:pstruct/ps:interactionRecord/ps:receiver/ps:asserter[wsa:Address/text()='"+ ai.getRealName() +"']/../.. " +
                        "return $n";
        return InteractionQuery;
    }

    /**
     * Method to find from a XML asserter the corresponding AsserterIdentity.
     * @param elem XML Element
     * @param ActorsNames Vector of AsserterIdentity
     * @return the position of the element into the given table.
     */
    public int findAsserterIdentity(Element elem, Vector ActorsNames) {
        String s;
        DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);
        //Element asserter = domDeserialiser.single(elem, Constants.PSTRUCT_NS, "asserter");
        Element address = domDeserialiser.single(elem, WSAddressEndpoint.wsaNamespace, "Address");
        if(address == null) address = elem;
        for(int i =0 ; i < ActorsNames.size(); i++){
            s = ((AsserterIdentity)ActorsNames.elementAt(i)).getRealName();
            if(address.getTextContent().equals(s))
                return i;

        }
        //}
        return -1;
    }

    /**
     * Method which return a query to get all the interactionsRecord from the store.
     * @return a String
     */
    public String getAllInteractions() {
        String xquery = "declare namespace wsa=\"" + WSAddressEndpoint.wsaNamespace + "\";" +
                "for $sender in $ps:pstruct/ps:interactionRecord/ps:sender/ps:asserter/wsa:Address/text() "+
                "let $receiver := $sender/../../../../ps:receiver/ps:asserter/wsa:Address/text() " +
                "return <ps:interactionRecord><ps:sender><ps:asserter><wsa:Address>{$sender}</wsa:Address></ps:asserter></ps:sender>" +
                "<ps:receiver><ps:asserter><wsa:Address>{$receiver}</wsa:Address></ps:asserter></ps:receiver></ps:interactionRecord>";
				
        return xquery;
    }
    
    /**
     * Method which return a query to get the interactionRecord which the interaction key is same as the ObjectId interactionKey.
     * @param ObjectId an ObjectID
     * @return a String
     */
    public String getInteractionRecord(ObjectID ObjectId){
        String source = "";
        String sink = "";
        String interactionId = "";
//        String viewKind = "";
//        String localPAssertionId = "";
//        String path = "";
        
        DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);
        
        InteractionKey interactionKey = ObjectId.getGlobalPAssertionKey().getInteractionKey();
        Element messageSource = interactionKey.getMessageSource();
        source = messageSource.getTextContent();
        
        Element messageSink = interactionKey.getMessageSink();
        sink = messageSink.getTextContent();
        
        interactionId = interactionKey.getInteractionID();
        
//        Element interactionKey = domDeserialiser.single(ObjectId, Constants.PSTRUCT_NS, "interactionKey");
//        Element messageSource = domDeserialiser.single(interactionKey, Constants.PSTRUCT_NS, "messageSource");
//        source = domDeserialiser.text(messageSource, WSAddressEndpoint.wsaNamespace,"Address");
//        
//        Element messageSink = domDeserialiser.single(interactionKey, Constants.PSTRUCT_NS, "messageSink");
//        sink = domDeserialiser.text(messageSink, WSAddressEndpoint.wsaNamespace,"Address");
//        
//        interactionId = domDeserialiser.text(interactionKey, Constants.PSTRUCT_NS, "interactionId");

//        Element viewK = domDeserialiser.single(ObjectId, Constants.PSTRUCT_NS, "viewKind");
//        viewKind = domDeserialiser.text(viewK, "description");
//        
//        
//        localPAssertionId = domDeserialiser.text(ObjectId,Constants.PSTRUCT_NS, "localPAssertionId");
//   
//        Element dataAccessor = domDeserialiser.single(ObjectId, Constants.PSTRUCT_NS, "dataAccessor");
//        Element tmp = domDeserialiser.single(dataAccessor, Constants.PQUERY_NS, "singleNodeXPath");
//        path = domDeserialiser.text(tmp, Constants.PQUERY_NS,"path");
        
        String xquery = "declare namespace wsa=\"" + WSAddressEndpoint.wsaNamespace + "\"; " +
                "declare namespace pq=\"" + Constants.PQUERY_NS + "\"; " +
                "for $n in $ps:pstruct/ps:interactionRecord/ps:interactionKey/ps:messageSource[wsa:Address/text()='" +source + "']" +
                "/../ps:messageSink[wsa:Address/text()='" + sink +"']" +
                "/../ps:interactionId[text()='" + interactionId + "']" +
                //"/../../*/*/ps:localPAssertionId[text()='" + localPAssertionId + "'] " +
                //"/../../*/ps:relationshipPAssertion/ps:subjectId/ps:dataAccessor/pq:singleNodeXPath/pq:path[text()='" + path + "']/../../../.. " +
                "/../.. " +
                "return $n";
        return xquery;
    } 

}
