/*
 * AsserterUtility.java
 *
 * Created on 26 juillet 2006, 14:36
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.pasoa.preview;

import java.util.Vector;
import org.pasoa.pstructure.ObjectID;
import org.w3c.dom.Element;

/**
 * An Interface which can be implemented to create a new AsserterUtility.
 * This interface is used to give the correct query to the CSL.
 * @author tibo
 */
public interface AsserterUtility {
    
    /**
     * Create a new AsserterIdentity from the given XML element, and return it.
     * @param element XML Element
     * @return an AsserterIdentity
     */
    AsserterIdentity newAsserterIdentity (Element element);
    /**
     * Create a new "empty" AsserterIdentity, and return it.
     * An "empty" AsserterIdentity is used when there is a missing asserter (or a missing view kind) in an interactionRecord.
     * @return an AsserterIdentity
     */
    AsserterIdentity newEmptyAsserterIdentity ();
    /**
     * Return a string which can get all asserters from the store.
     * @return a String
     */
    String getAssertersQuery();
    /**
     * Method which return a query to get all the interactions record between the two given asserterIdentity.
     * @param sender the sender AsserterIdenity
     * @param receiver the receiver AsserterIdentity
     * @return a String which is a query
     */
    String getInteractionsBetweenQuery(AsserterIdentity sender, AsserterIdentity receiver);
    /**
     * Method which return a query to get the interaction record with the given interactionID between the two given asserterIdentity.
     * @param sender The sender Asserteridentity
     * @param receiver the receiver AsserterIdentity
     * @param interactionID The interactionId (a String)
     * @return a String which is a query
     */
    String getInteractionRecordBetweenQuery(AsserterIdentity sender, AsserterIdentity receiver, String interactionID);

    /**
     * Method which return a query to get all interaction record where the given asserterIdentity is the sender.
     * @param ai the sender AsserterIdentity
     * @return a String which is a query
     */
    String getInteractionsWithSenderQuery(AsserterIdentity ai);
    /**
     * Method which return a query to get all interaction record where the given asserterIdentity is the receiver.
     * @param ai the receiver AsserterIdentity
     * @return a String which is a query
     */
    String getInteractionsWithReceiverQuery(AsserterIdentity ai);

    /**
     * Method to find from a XML asserter the corresponding AsserterIdentity.
     * @param elem a XML Element
     * @param ActorsNames A Vector of AsserterIdentity
     * @return an int, which is the position of the xml element in the given Vector
     */
    int findAsserterIdentity(Element elem, Vector ActorsNames);

    /**
     * Method which return a query to get all the interactionsRecord from the store.
     * @return a String which is a query
     */
    String getAllInteractions();
    
    /**
     * Method which return a query to get the interactionRecord which the interaction key is same as the ObjectId interactionKey.
     * @param ObjectId an ObjectID
     * @return a String which is a query
     */
    String getInteractionRecord(ObjectID ObjectId);
    
}
