/*Copyright (c) 2005 University of Southampton
 
 Permission is hereby granted, free of charge,
 to any person obtaining a copy of this
 software and associated documentation files
 (the "Software"), to deal in the Software without
 restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute,
 sublicense, and/or sell copies of the Software, and
 to permit persons to whom the Software is furnished
 to do so, subject to the following conditions:
 
 The above copyright notice and this permission notice
 shall be included in all copies or substantial
 portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY
 OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO
 EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 OR OTHER DEALINGS IN THE SOFTWARE.
 -------------------------------------------------------*/

package org.pasoa.preview;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Vector;
import javax.swing.UIManager;
import org.apache.axis.message.MessageElement;
import org.gridprovenance.client.PortFactory;
import org.gridprovenance.client.ProvenanceService;
import org.gridprovenance.client.XQuery;
import org.gridprovenance.client.XQueryResult;
import org.gridprovenance.client.exception.DataConstructorException;
import org.gridprovenance.client.exception.XQueryException;
import org.gridprovenance.client.impl.PortFactoryPSImpl;
import org.pasoa.common.Constants;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.simpledom.DOMWriter;
import org.pasoa.storeclient.ClientLib;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 *
 * @author tibo
 */

/**
 * Class Controller
 * Third class of the MVC Model for provenance preview :
 *      The Controller
 * It allows the View to be abble to get information to the "good" Model
 * @author tibo
 */
public class Controller {
    
    public String address;
    public ProvenanceService ps;
    public ModelInterface mi;
    public AsserterUtility ai;
    public static boolean debug = true;
    
    /**
     * Creates a new instance of Controller from a given store address.
     * @param add an address of a store
     * @throws java.net.URISyntaxException CSL Exception
     * @throws org.gridprovenance.client.exception.DataConstructorException CSL Exception
     * @throws org.gridprovenance.client.exception.XQueryException CSL Exception
     * @throws java.lang.Exception other Exception
     */
    public Controller (String add) throws URISyntaxException, DataConstructorException, XQueryException, Exception {
        
        //http://prov.ecs.soton.ac.uk:10000/challengestore
        
        //System.out.println(testConnection(add));
        this.address = add;
        String pro = testConnectionPreserv (address);
        //String pro = "Address";
        //String pro = testConnectionProvenance (address);
        System.out.println ("Found asserter type : " + pro);
        if(pro.equals ("Address")){
            this.ai =  new AddressAsserterUtility ();
        } else {
            if(pro.endsWith ("Principal")){
                this.ai = new DNAsserterUtility ();
            } else
                throw new Exception ("Unrecognised asserter type");
        }
        initialiseModel ();
    }
    
    private void initialiseModel () {
        try {
            if (address.startsWith ("pstruct:")) {
                mi = new PreservLocalPStructureFileModel (address.substring (8), ai);
                return;
            }
            if (address.startsWith ("bdb:")) {
                mi = new BerkeleyLocalModel (address.substring (4), ai);
                return;
            }
            //mi = new ProvenanceRemoteStoreModel(ps, ai);
            mi = new PreservRemoteStoreModel (address, ai);
        } catch (Exception problem) {
            problem.printStackTrace ();
            throw new RuntimeException (problem);
        }
    }
    
    /**
     * Creates a new instance of Controller from a store address, and an AsserterUtility.
     * @param add a store address
     * @param au an AsserterUtility
     * @throws java.net.URISyntaxException CSL Exception
     * @throws org.gridprovenance.client.exception.DataConstructorException CSL Exception
     * @throws org.gridprovenance.client.exception.XQueryException CSL Exception
     */
    public Controller (String add, AsserterUtility au) throws URISyntaxException, DataConstructorException, XQueryException {
        this.address = add;
        
        new URI (add);
        PortFactory portFactory = null;
        portFactory = new PortFactoryPSImpl (add);
        //Create a ProvenanceService for recording and xquerying.
        ps = portFactory.newProvenanceService ();
        initialiseModel ();
    }
    
    /**
     * Used to get the current AsserterIdentity of a Controller.
     * @return the current AsserterIdentity Address or DN
     */
    public AsserterUtility getAi () {
        return ai;
    }
    
    
    /**
     * Used to know what kind of asserter we have to used.
     * Used the CSL.
     * @param add a store address
     * @return a String which is the tag name of the first child of te first asserter tag in the first view.
     * @throws java.net.URISyntaxException CSL Exception
     * @throws org.gridprovenance.client.exception.DataConstructorException CSL Exception
     * @throws org.gridprovenance.client.exception.XQueryException CSL Exception
     */
    public String testConnectionProvenance (String add) throws URISyntaxException, DataConstructorException, XQueryException {
        
        new URI (add);
        
        PortFactory portFactory = null;
        portFactory = new PortFactoryPSImpl (add);
        
        //Create a ProvenanceService for recording and xquerying.
        ps = portFactory.newProvenanceService ();
        
        String xQueryString = "for $n in $ps:pstruct/ps:interactionRecord/*/ps:asserter/*[0] return $n";
        //String xQueryString ="for $n in $ps:pstruct return $n";
        XQuery xQueryInstance = ps.newXQuery (xQueryString);
        XQueryResult xqResult = xQueryInstance.execute ();
        
        //xqResult.printOut();
        
        MessageElement[] me = xqResult.getAny ();
        if(me != null){
            Document result = xqResult.getAsDocument ();
            //System.out.println(DOMWriter.writeToString(result, true));
            Node node = result.getDocumentElement ().getFirstChild ();
            return node.getNodeName ();
        } else return null;
    }
    
    /**
     * Used to know what kind of asserter we have to used.
     * Used the Preserv Library.
     * @param add a store address
     * @return a String which is the tag name of the first child of te first asserter tag in the first view.
     * @throws java.net.MalformedURLException URL exception when create a URL from the given address
     * @throws java.lang.Exception other Preserv library exception
     */
    public String testConnectionPreserv (String add) throws MalformedURLException, Exception {
        if (add.startsWith ("pstruct:") || add.startsWith ("bdb:")) {
            return "Principal";
        }
        
        String xQueryString = "for $n in $ps:pstruct/ps:interactionRecord/*/ps:asserter/* return $n";
        
        ClientLib clientLib = new ClientLib ();
        Document result;
        result = clientLib.query (xQueryString, new URL (add + "/xquery"));
        
        DOMDeserialiser domDeserialiser = new DOMDeserialiser (Constants.PSTRUCT_NS);
        Element Body = domDeserialiser.single (result.getDocumentElement (), Constants.SOAP_NS, "Body");
        Element qres = domDeserialiser.single (Body, Constants.WSXQUERY_NS, "queryResult");
        
        Node node = qres.getFirstChild ();
        return node.getNodeName ();
    }
    
    /**
     * Main method of PReView.
     * @param args the command line arguments
     * @throws java.lang.Exception any problem from the thread or runtime
     */
    public static void main (String args[]) throws Exception {
        
        final boolean debug = true;
        
        java.awt.EventQueue.invokeLater (new Runnable () {
            public void run () {
                new View ().setVisible (true);
            }
        });
    }
    
}
