/*
 * DNAsserterUtility.java
 *
 * Created on 26 juillet 2006, 14:47
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.pasoa.preview;

import java.util.Vector;
import org.pasoa.common.BestPractice;
import org.pasoa.common.Constants;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.simpledom.DOMWriter;
import org.pasoa.util.httpsoap.WSAddressEndpoint;
import org.w3c.dom.Element;

/**
 * Implementation of AsserterUtility interface to the DN asserter.
 * An DN asserter is like :
 * <asserter>
 *  <Principal>...something...</Principal>
 * </asserter>
 * @author tibo
 */
public class DNAsserterUtility implements AsserterUtility {
    
    /**
     * Creates a new instance of DNAsserterUtility
     */
    public DNAsserterUtility() {
    }
    
    /**
     * Create a new DNAsserterIdentity from the given XML element, and return it.
     * @param element XML Element
     * @return a new DNAsserterIdentity
     */
    public AsserterIdentity newAsserterIdentity (Element element){
        DNAsserterIdentity aid = new DNAsserterIdentity(element);
        return aid;
    }
    
    
    /**
     * Create a new "empty" DNAsserterIdentity, and return it.
     * An "empty" DNAsserterIdentity is used when there is a missing asserter (or a missing view kind) in an interactionRecord.
     * @return a new empty DNAsserterIdentity
     */
    public AsserterIdentity newEmptyAsserterIdentity() {
        DNAsserterIdentity aid = new DNAsserterIdentity("nothing");
        return aid;
    }
    
    /**
     * Return a string which can get all asserters from the store.
     * @return a query as string
     */
    public String getAssertersQuery() {
       
        String xQueryString = "declare namespace ap=\"" + BestPractice.ACTOR_PROFILE_NS + "\";" +
                    "for $n in fn:distinct-values(" +
                    "$ps:pstruct/ps:interactionRecord/*/ps:asserter/*/text()) " +
                    "order by $n return <ps:asserter><ps:name>{$n}</ps:name>" +
                    "<ps:dataName>{$n/../../../ps:actorStatePAssertion/ps:content/ap:ActorProfile/ap:name/text()}" +
                    "</ps:dataName></ps:asserter>";
        return xQueryString;
    }
    
    /**
     * Method which return a query to get all the interactions record between the two given asserterIdentity.
     * @param sender a DNAsserterIdentity as sender
     * @param receiver a DNAsserterIdentity as receiver
     * @return a query as string
     */
    public String getInteractionsBetweenQuery(AsserterIdentity sender, AsserterIdentity receiver) {
        String InteractionsQuery;
       
        if(receiver.getRealName().equals("nothing")) {
            InteractionsQuery = "for $n in $ps:pstruct/ps:interactionRecord[fn:empty(ps:receiver)]" +
                "/ps:sender/ps:asserter[Principal/text()='"+ sender.getRealName() +"']" +
                "/../.. return $n";
        }
        else {
            if(sender.getRealName().equals("nothing")) {
                InteractionsQuery = "for $n in $ps:pstruct/ps:interactionRecord[fn:empty(ps:sender)]" +
                    "/ps:sender/ps:receiver[Principal/text()='"+ receiver.getRealName() +"']" +
                    "/../.. return $n";
            }
            else {
        
                InteractionsQuery = "for $n in $ps:pstruct/ps:interactionRecord/ps:sender/ps:asserter[Principal/text()='"+ sender.getRealName() +"']" +
                    "/../../ps:receiver/ps:asserter[Principal/text()='"+ receiver.getRealName() +"']/../.. " +
                    "return $n";
            }
        }
        return InteractionsQuery;
    }
    
    /**
     * Method which return a query to get the interaction record with the given interactionID between the two given asserterIdentity.
     * @param sender a DNAsserterIdentity as sender
     * @param receiver a DNAsserterIdentity as receiver
     * @param interactionID a string as the interactionID
     * @return a query as string
     */
    public String getInteractionRecordBetweenQuery(AsserterIdentity sender, AsserterIdentity receiver, String interactionID) {
        String InteractionQuery;
       
        if(receiver.getRealName().equals("nothing")) {
            InteractionQuery = "for $n in $ps:pstruct/ps:interactionRecord[fn:empty(ps:receiver)]" +
                        "/ps:sender/ps:asserter[Principal/text()='"+ sender.getRealName() +"']" +
                        "/../../ps:interactionKey[ps:interactionId/text()='"+ interactionID +"']/..  " +
                        "return $n";
        }
        else {
            if(sender.getRealName().equals("nothing")) {
                InteractionQuery = "for $n in $ps:pstruct/ps:interactionRecord[fn:empty(ps:sender)]" +
                        "/ps:sender/ps:receiver[Principal/text()='"+ receiver.getRealName() +"']" +
                        "/../../ps:interactionKey[ps:interactionId/text()='"+ interactionID +"']/..  " +
                        "return $n";
            }
            else {
                InteractionQuery = "for $n in $ps:pstruct/ps:interactionRecord/ps:sender/ps:asserter[Principal/text()='"+ sender.getRealName() +"']" +
                        "/../../ps:receiver/ps:asserter[Principal/text()='"+ receiver.getRealName() +"']" +
                        "/../../ps:interactionKey[ps:interactionId/text()='"+ interactionID +"']/..  " +
                        "return $n";
            }
        }
        return InteractionQuery;
    }

    /**
     * Method which return a query to get all interaction record where the given asserterIdentity is the sender.
     * @param ai a DNAsserterIdentity as sender
     * @return a query as string
     */
    public String getInteractionsWithSenderQuery(AsserterIdentity ai) {
        String InteractionQuery = "for $n in $ps:pstruct/ps:interactionRecord/ps:sender/ps:asserter[Principal/text()='"+ ai.getRealName() +"']/../.. " +
                        "return $n";
        return InteractionQuery;
    }

    /**
     * Method which return a query to get all interaction record where the given asserterIdentity is the receiver.
     * @param ai a DNAsserterIdentity as receiver
     * @return a query as string
     */
    public String getInteractionsWithReceiverQuery(AsserterIdentity ai) {
        String InteractionQuery = "for $n in $ps:pstruct/ps:interactionRecord/ps:receiver/ps:asserter[Principal/text()='"+ ai.getRealName() +"']/../.. " +
                        "return $n";
        return InteractionQuery;
    }

    /**
     * Method to find from a XML asserter the corresponding AsserterIdentity.
     * @param elem XML Element 
     * @param ActorsNames a vector of AsserterIdentity
     * @return an int as the position of the given AsserterIdentity as XML element in the given Vector of AsserterIdentity 
     */
    public int findAsserterIdentity(Element elem, Vector ActorsNames) {
        String s;
        DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);
        //Element asserter = domDeserialiser.single(elem, Constants.PSTRUCT_NS, "asserter");
        Element principal = domDeserialiser.single(elem, null ,"Principal");
        //System.out.println(DOMWriter.writeToString(principal , true));
        if(principal == null) principal = elem;
        for(int i =0 ; i < ActorsNames.size(); i++){
            s = ((AsserterIdentity)ActorsNames.elementAt(i)).getRealName();
            //System.out.println(s +" ; "+ principal.getTextContent());
            if(principal.getTextContent().equals(s))
                return i;
            
        }
        return -1;
    }

    /**
     * Method which return a query to get all the interactionsRecord from the store.
     * @return a query as string
     */
    public String getAllInteractions() {
        String xquery = "for $sender in $ps:pstruct/ps:interactionRecord/ps:sender/ps:asserter/Principal/text() "+
                "let $receiver := $sender/../../../../ps:receiver/ps:asserter/Principal/text() " +
                "return <ps:interactionRecord><ps:sender><ps:asserter><Principal>{$sender}</Principal></ps:asserter></ps:sender>" +
                "<ps:receiver><ps:asserter><Principal>{$receiver}</Principal></ps:asserter></ps:receiver></ps:interactionRecord>";
        return xquery;
    }

    /**
     * Method which return a query to get the interactionRecord which the interaction key is same as the ObjectId interactionKey.
     * @param ObjectId an ObjectID
     * @return a query as string
     */
    public String getInteractionRecord(ObjectID ObjectId) {
        String source = "";
        String sink = "";
        String interactionId = "";

        DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);
        
        InteractionKey interactionKey = ObjectId.getGlobalPAssertionKey().getInteractionKey();
        Element messageSource = interactionKey.getMessageSource();
        source = messageSource.getTextContent();
        
        Element messageSink = interactionKey.getMessageSink();
        sink = messageSink.getTextContent();
        
        interactionId = interactionKey.getInteractionID();
        
        String xquery = "declare namespace wsa=\"" + WSAddressEndpoint.wsaNamespace + "\"; " +
                "declare namespace pq=\"" + Constants.PQUERY_NS + "\"; " +
                "for $n in $ps:pstruct/ps:interactionRecord/ps:interactionKey/ps:messageSource[wsa:Address/text()='" +source + "']" +
                "/../ps:messageSink[wsa:Address/text()='" + sink +"']" +
                "/../ps:interactionId[text()='" + interactionId + "']" +
                //"/../../*/ps:relationshipPAssertion/ps:subjectId/ps:dataAccessor/pq:singleNodeXPath/pq:path[text()='" + path + "']/../.. " +
                "/../.. " +
                "return $n";
        
        return xquery;
    }
}
