/*Copyright (c) 2005 University of Southampton

 Permission is hereby granted, free of charge,
 to any person obtaining a copy of this
 software and associated documentation files
 (the "Software"), to deal in the Software without
 restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute,
 sublicense, and/or sell copies of the Software, and
 to permit persons to whom the Software is furnished
 to do so, subject to the following conditions:

 The above copyright notice and this permission notice
 shall be included in all copies or substantial
 portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY
 OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO
 EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 OR OTHER DEALINGS IN THE SOFTWARE.
 -------------------------------------------------------*/

package org.pasoa.preview;

import java.util.Vector;
import org.pasoa.pstructure.ObjectID;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * An Interface which can be implemented to create a new ModelInterface.
 * This interface is used to get data from a store.
 * @author tibo
 */
public interface ModelInterface {

    /**
     * Get all actors from yours provenance store
     * @return Vector of AsserterIdenity object as all actors
     * @throws Exception many kind of exception
     */
    Vector getAsserters() throws Exception;
    
    /**
     * Get all interactions between all actors given in argument
     * from yours provenance store
     * @param ActorsNames Vector which contains all AsserterIdentity
     * @return Vector<Vector<int>> which contains all interactions (like a grid)
     * @throws Exception many kind of exception
     */
    Vector<Vector> getInteractions(Vector ActorsNames) throws Exception;
    
    /**
     * Get all interactions between 2 given actors from yours provenance store
     * @param sender Sender asserterIdentity
     * @param receiver Receiver AsserterIdentity
     * @return Vector which contains all interactions between these two actors 
     * whith sender as sender and receiver as receiver
     * @throws Exception many kind of exception
     */
    Vector getAllInteractionsRecordBetween(AsserterIdentity sender, AsserterIdentity receiver) throws Exception;
    
    /**
     * Get interaction record with the given interactionID and between 2 given actors from yours provenance store
     * @param sender Sender AsserterIdentity
     * @param receiver Receiver AsserterIdentity
     * @param interactionID The id of the searched interaction
     * @return Element It is a node which contains the searched interaction between these two actors 
     * whith sender as sender and receiver as receiver and interactionID as the id of the interaction
     * @throws Exception many kind of exception
     */
    Element getInteractionRecordBetween(AsserterIdentity sender, AsserterIdentity receiver, String interactionID) throws Exception;
    
    /**
     * Get all interactions with the given AsserterIdentity is the receiver from yours provenance store
     * @param ai an AsserterIdenity as receiver
     * @return a vector of XML Element which are a list of all the interactionRecord
     * @throws java.lang.Exception many kind of exception from the store
     */
    Vector getAllInteractionReceiverRecord(AsserterIdentity ai) throws Exception;
    /**
     * Get all interactions with the given AsserterIdentity is the sender from yours provenance store
     * @param ai an AsserterIdenity as receiver
     * @throws java.lang.Exception many kind of exception from the store
     * @return a vector of XML Element which are a list of all the interactionRecord
     */
    Vector getAllInteractionSenderRecord(AsserterIdentity ai) throws Exception;     
    /**
     * Get the interactionRecord with the same interactionKey as the given ObjectID from yours provenance store
     * @param ObjectId an ObjectID
     * @return an XML Element
     * @throws java.lang.Exception many kind of exception from the store
     */
    Element getInteractionRecord(ObjectID ObjectId) throws Exception;     
    
    /**
     * A method to get back all the content of a given store
     * @throws java.lang.Exception many kind of exception from the store
     * @return a XML Document
     */
    Document getPStruct()throws Exception;   
    
}
