/*
 * MonUI.java
 *
 * Created on 28 juillet 2006, 13:33
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.pasoa.preview;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import javax.swing.plaf.basic.BasicGraphicsUtils;
import javax.swing.plaf.basic.BasicTabbedPaneUI;

/**
 * a class for vertical tab.
 * @author tibo
 */
public class MonUI extends BasicTabbedPaneUI{
    
    int dx;
    int dy;
    
    public MonUI(){super();}

    /**
     * override the update method
     * @param g a Graphics
     * @param c a JComponent
     */
    public void update(Graphics g, JComponent c) {
        super.update(g, c);
    }
    
    /**
     * to rotate the tab
     * @param tabIndex an int as the index of the tab to rotate
     * @return the rotate image
     */
    private Image getVerticalImage(int tabIndex){
        Icon iconeNormale = tabPane.getIconAt(tabIndex);
        //ImageIcon iconeNormale = null;
        int iconeWidth = (iconeNormale!=null)?iconeNormale.getIconHeight():0;
        int iconeHeight = (iconeNormale!=null)?iconeNormale.getIconWidth():0;
        
        String titre = tabPane.getTitleAt(tabIndex);
        FontMetrics metrics = getFontMetrics();
        int titreWidth = (titre!=null)?metrics.stringWidth(titre):0;
        int titreHeight = (titre!=null)?metrics.getHeight():0;
        int height = iconeWidth+titreWidth+textIconGap+1;
        int width = Math.max(iconeHeight,titreHeight)+5;
        boolean isSelected = (tabPane.getSelectedIndex()==tabIndex);
        int tabPlacement = tabPane.getTabPlacement();
        BufferedImage image = new BufferedImage(width,height,BufferedImage.TYPE_INT_RGB);
        Graphics2D g2 = image.createGraphics();
        if(tabPlacement==JTabbedPane.LEFT){g2.translate(0,height);g2.rotate(-Math.PI/2);}
        else{g2.translate(width,0);g2.rotate(Math.PI/2);}
        Color background = (isSelected)?UIManager.getColor("TabbedPane.selected"):tabPane.getBackgroundAt(tabIndex);
        g2.setColor(background);g2.fillRect(0,0,height,width);
        if(iconeNormale!=null){
            BufferedImage bi = new BufferedImage(iconeNormale.getIconWidth(), iconeNormale.getIconHeight(), BufferedImage.TYPE_INT_ARGB);
            Graphics g = bi.getGraphics();
            iconeNormale.paintIcon(null, g, 0, 0);
            g.dispose();
            g2.drawImage(bi, 0, 0,null);
            iconeNormale.paintIcon(null, g, dx, dy + (height - iconeWidth));
        }
        if(titre!=null){
            g2.setFont(metrics.getFont());
            g2.setColor(tabPane.getForegroundAt(tabIndex));
            int mnemIndex = tabPane.getDisplayedMnemonicIndexAt(tabIndex);
            BasicGraphicsUtils.drawStringUnderlineCharAt(g2,titre, mnemIndex
                ,iconeWidth+textIconGap,metrics.getAscent());
        }
        g2.dispose();
        return image;
    }

    /**
     * simple test to know if the tabs of the JTabbedPane are bottom or top (nothing to do in these case)
     * @return true if the JTabbedPane is Bottom or Top
     * else if it is not
     */
    private boolean isBottomOrTop(){
        int tabPlacement = tabPane.getTabPlacement();
        return tabPlacement==JTabbedPane.TOP||tabPlacement==JTabbedPane.BOTTOM;
    }
    
    /**
     * Calculate the space for the new tab.
     * @param tabPlacement bottom, top, left or right
     * @param tabIndex an int as the index of the current tab
     * @param fontHeight size of the font (height)
     * @return the height of the tab
     */
    protected int calculateTabHeight(int tabPlacement,int tabIndex,int fontHeight){
        if(isBottomOrTop()){
            return super.calculateTabHeight(tabPlacement,tabIndex,fontHeight);
        }
        return super.calculateTabWidth(tabPlacement,tabIndex,getFontMetrics());
    }

    /**
     * Calculate the space for the new tab.
     * @param tabPlacement bottom, top, left or right
     * @param tabIndex an int as the index of the current tab
     * @param metrics size of the font (width)
     * @return the width of the tab
     */
    protected int calculateTabWidth(int tabPlacement,int tabIndex,FontMetrics metrics){
        if(isBottomOrTop()){
            return super.calculateTabWidth(tabPlacement,tabIndex,metrics);
        }
        return getVerticalImage(tabIndex).getWidth(null);
    }
    
    /**
     * paint the text of the tab
     * @param g Graphics
     * @param tabPlacement bottom, top, left or right
     * @param font the Font of the text
     * @param metrics font size of the text
     * @param tabIndex an int as the index of the current tab
     * @param title the title of the tab
     * @param textRect the bounding box of the text
     * @param isSelected a boolean 
     */
    protected void paintText(Graphics g, int tabPlacement, Font font
                            , FontMetrics metrics, int tabIndex, String title
                            , Rectangle textRect, boolean isSelected){
        if(isBottomOrTop()){
            super.paintText(g,tabPlacement,font,metrics,tabIndex,title,textRect,isSelected);
        }else{
            Rectangle r = getTabBounds(tabPane,tabIndex);
            Insets insets = getTabAreaInsets(tabPlacement);
            dx = r.x+insets.top; 
            dy = r.y;
            if(tabPlacement==JTabbedPane.LEFT){dy+=insets.left;}
            else{dy+=insets.right;}
            Image image = getVerticalImage(tabIndex);
            g.drawImage(image,dx,dy,null);
        }
    }

    /**
     * Paint the icon of the tab
     * @param g Graqhics
     * @param tabPlacement bottom, top, left or right
     * @param tabIndex an int as the index of the current tab
     * @param icon the icon to paint
     * @param iconRect the bounding box of the icon
     * @param isSelected a boolean
     */
    protected void paintIcon(Graphics g, int tabPlacement, int tabIndex,
                            Icon icon, Rectangle iconRect, boolean isSelected){
        if(!isBottomOrTop()){return;}
        super.paintIcon(g,tabPlacement,tabIndex,icon,iconRect,isSelected);
    }
}