package org.pasoa.preview;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;
import org.pasoa.common.Constants;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.pstructure.RelationshipPAssertion;
import org.pasoa.pstructure.deserialiser.PStructureDeserialiser;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.simpledom.DOMWriter;
import org.pasoa.storeclient.proxy.StoreProxy;
import org.pasoa.util.httpsoap.WSAddressEndpoint;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class PReServStoreProxyModel implements ModelInterface {
    private StoreProxy      _proxy;
    private AsserterUtility asserterUtility;
    
    public PReServStoreProxyModel (StoreProxy proxy, AsserterUtility asserterUtility) throws MalformedURLException{
        _proxy = proxy;
        this.asserterUtility = asserterUtility;
    }
    
    /**
     * Get all actors from yours provenance store
     * @return a Vector of AsserterIdentity
     * @throws java.lang.Exception Preserv CSL exception
     */
    public Vector getAsserters () throws Exception {
        Document result = _proxy.getXQueryInterface ().resolve (asserterUtility.getAssertersQuery ());
        DOMDeserialiser domDeserialiser = new DOMDeserialiser (Constants.PSTRUCT_NS);
        //Element Body = domDeserialiser.single (result.getDocumentElement (), Constants.SOAP_NS, "Body");
        //Element results = domDeserialiser.single (Body, Constants.WSXQUERY_NS, "queryResult");
        //Iterator asser = domDeserialiser.iterator (results, Constants.PSTRUCT_NS, "asserter");
        Iterator asser = domDeserialiser.iterator (result.getDocumentElement (), Constants.PSTRUCT_NS, "asserter");
        //domDeserialiser.namespace(WSAddressEndpoint.wsaNamespace);
        Element element = null;
        Vector tab = new Vector ();
        //int i = 0;
//        System.out.println(DOMWriter.writeToString(result,true));
        while(asser.hasNext ()){
            element =(Element) asser.next ();
            tab.add (asserterUtility.newAsserterIdentity (element));
        }
        tab.add (asserterUtility.newEmptyAsserterIdentity ());
        return tab;
    }
    
    /**
     * Get all interactions between all actors given in argument
     * from yours provenance store
     * @param ActorsNames a Vector of AsserterIdentity
     * @return Vector<Vector<int>> which contains all interactions (like a grid)
     * @throws java.lang.Exception Preserv CSL exception
     */
    public Vector<Vector> getInteractions (Vector ActorsNames) throws Exception {
        Document result;
        
        
        Vector<Vector> tab = new Vector ();
        
        for (int i =0 ; i < ActorsNames.size () ; i++){
            Vector tmp = new Vector ();
            for (int j =0 ; j < ActorsNames.size () ; j++){
                tmp.add ((int)0);
            }
            tab.add (tmp);
        }
        
        
        result = _proxy.getXQueryInterface ().resolve (asserterUtility.getAllInteractions ());
        DOMDeserialiser domDeserialiser = new DOMDeserialiser (Constants.PSTRUCT_NS);
//        Element Body = domDeserialiser.single (result.getDocumentElement (), Constants.SOAP_NS, "Body");
//        Element qresults = domDeserialiser.single (Body, Constants.WSXQUERY_NS, "queryResult");
        Element qresults = result.getDocumentElement ();
        Iterator ir = domDeserialiser.iterator (qresults ,Constants.PSTRUCT_NS, "interactionRecord");
        Element elem;
        while(ir.hasNext ()){
            elem = (Element)ir.next ();
            
            Element sender = domDeserialiser.single (elem, Constants.PSTRUCT_NS, "sender");
            Element receiver = domDeserialiser.single (elem, Constants.PSTRUCT_NS, "receiver");
            
            Element asserterS = domDeserialiser.single (sender, Constants.PSTRUCT_NS, "asserter");
            Element asserterR = domDeserialiser.single (receiver, Constants.PSTRUCT_NS, "asserter");
            
            //System.out.println(DOMWriter.writeToString(sender, true));
            
            int s = asserterUtility.findAsserterIdentity (asserterS, ActorsNames);
            int r = asserterUtility.findAsserterIdentity (asserterR, ActorsNames);
            //System.out.println(s + ", " + r);
            
            if(s !=-1 && r != -1){
                Vector tmp = tab.elementAt (s);
                
                int nb = Integer.parseInt (tmp.elementAt (r).toString ());
                //System.out.println(nb++);
                tmp.setElementAt (++nb, r);
                
                tab.setElementAt (tmp, s);
            } else {
                throw new Exception ("Asserter didn't find!");
            }
            
        }
        
//        /* Senders */
//        for (int i=0 ; i < ActorsNames.size(); i++){
//            Vector interaction = new Vector();
//            /* Rseceivers */
//            for (int j=0 ; j < ActorsNames.size(); j++){
//
//                result = clientLib.query(
//                        asserterUtility.getInteractionsBetweenQuery(
//                            ((AsserterIdentity)ActorsNames.elementAt(i)),
//                            ((AsserterIdentity)ActorsNames.elementAt(j))),
//                        provenanceStoreQuery);
//
//                DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);
//                Element Body = domDeserialiser.single(result.getDocumentElement(), Constants.SOAP_NS, "Body");
//                Element results = domDeserialiser.single(Body, Constants.WSXQUERY_NS, "queryResult");
//                NodeList nl = results.getElementsByTagNameNS(Constants.PSTRUCT_NS, "interactionRecord");
//                int cpt = nl.getLength();
//
//                if(cpt!=0){
//                    interaction.add(Integer.toString(cpt));
//                }
//                else {
//                    interaction.add("");
//                }
//
//            }
//            tab.add(interaction);
//        }
//
        return tab;
    }
    
    
    /**
     * Get all interactions between 2 given actors from yours provenance store
     * @param sender an AsserterIdentity as sender
     * @param receiver an AsserterIdentity as receiver
     * @return Vector which contains all interactions record (XML Element) between these two actors
     * whith sender as sender and receiver as receiver
     * @throws java.lang.Exception Preserv CSL exception
     */
    public Vector getAllInteractionsRecordBetween (AsserterIdentity sender, AsserterIdentity receiver) throws Exception {
        
        Document result;
        
        /* A xquery for testing only senders interactions */
//        String xquery = "declare namespace wsa=\"" + WSAddressEndpoint.wsaNamespace + "\";" +
//                "for $n in $ps:pstruct//ps:sender/ps:asserter[wsa:Address/text()='"+ sender +"']" +
//                 "/../.. return $n";
        
        result = _proxy.getXQueryInterface ().resolve (asserterUtility.getInteractionsBetweenQuery (sender, receiver));
        DOMDeserialiser domDeserialiser = new DOMDeserialiser (Constants.PSTRUCT_NS);
        //Element Body = domDeserialiser.single (result.getDocumentElement (), Constants.SOAP_NS, "Body");
        //Element results = domDeserialiser.single (Body, Constants.WSXQUERY_NS, "queryResult");
        Element results = result.getDocumentElement ();
        Iterator inteRecord = domDeserialiser.iterator (results, Constants.PSTRUCT_NS, "interactionRecord");
        Element element;
        Element interkey;
        Vector tab = new Vector ();
        while(inteRecord.hasNext ()){
            interkey = domDeserialiser.single ((Element) inteRecord.next (), Constants.PSTRUCT_NS, "interactionKey");
            tab.add (domDeserialiser.text (interkey, "interactionId"));
            //System.out.println(domDeserialiser.text(interkey, "interactionId"));
        }
        return tab;
    }
    
    /**
     * Get interaction record with the given interactionID and between 2 given actors from yours provenance store
     * @param sender AsserterIdentity as sender
     * @param receiver AsserterIdentity as receiver
     * @param interactionID String with the interactionID
     * @return Element It is a node which contains the searched interaction between these two actors
     * whith sender as sender and receiver as receiver and interactionID as the id of the interaction
     * @throws java.lang.Exception Preserv CSL exception
     */
    public Element getInteractionRecordBetween (AsserterIdentity sender, AsserterIdentity receiver, String interactionID) throws Exception {
        Document result;
        
        result = _proxy.getXQueryInterface ().resolve (asserterUtility.getInteractionRecordBetweenQuery (
                sender, receiver, interactionID));
        
        DOMDeserialiser domDeserialiser = new DOMDeserialiser (Constants.PSTRUCT_NS);
        //Element Body = domDeserialiser.single (result.getDocumentElement (), Constants.SOAP_NS, "Body");
        //Element results = domDeserialiser.single (Body, Constants.WSXQUERY_NS, "queryResult");
        Element results = result.getDocumentElement ();
        Element inteRecord = domDeserialiser.single (results, Constants.PSTRUCT_NS, "interactionRecord");
        //System.out.println(DOMWriter.writeToString(inteRecord, true));
        return inteRecord;
    }
    
    
    /**
     * Get all interactions with the given AsserterIdentity is the receiver from yours provenance store
     * @param ai AsserterIdentity as receiver
     * @return Vector which contains all interactions record (XML Element)
     * @throws java.lang.Exception Preserv CSL exception
     */
    public Vector getAllInteractionReceiverRecord (AsserterIdentity ai) throws Exception {
        Document result;
        
        result = _proxy.getXQueryInterface ().resolve (
                asserterUtility.getInteractionsWithReceiverQuery (ai));
        DOMDeserialiser domDeserialiser = new DOMDeserialiser (Constants.PSTRUCT_NS);
        //Element Body = domDeserialiser.single (result.getDocumentElement (), Constants.SOAP_NS, "Body");
        //Element results = domDeserialiser.single (Body, Constants.WSXQUERY_NS, "queryResult");
        Element results = result.getDocumentElement ();
        Iterator inteRecord = domDeserialiser.iterator (results, Constants.PSTRUCT_NS, "interactionRecord");
        Vector tab = new Vector ();
        //int i = 0;
        while(inteRecord.hasNext ()){
            tab.add ((Element) inteRecord.next ());
            //System.out.println(DOMWriter.writeToString((Node) tab.elementAt(i), true));
            //i++;
        }
        return tab;
    }
    
    /**
     * Get all interactions with the given AsserterIdentity is the sender from yours provenance store
     * @param ai AsserterIdentity as sender
     * @throws java.lang.Exception Preserv CSL exception
     * @return Vector which contains all interactions record (XML Element)
     */
    public Vector getAllInteractionSenderRecord (AsserterIdentity ai) throws Exception {
        Document result;
        
        /* A xquery for testing only senders interactions */
//        String xquery = "declare namespace wsa=\"" + WSAddressEndpoint.wsaNamespace + "\";" +
//                "for $n in $ps:pstruct//ps:sender/ps:asserter[wsa:Address/text()='"+ sender +"']" +
//                 "/../.. return $n";
        
        result = _proxy.getXQueryInterface ().resolve (
                asserterUtility.getInteractionsWithSenderQuery (ai));
        DOMDeserialiser domDeserialiser = new DOMDeserialiser (Constants.PSTRUCT_NS);
        //Element Body = domDeserialiser.single (result.getDocumentElement (), Constants.SOAP_NS, "Body");
        //Element results = domDeserialiser.single (Body, Constants.WSXQUERY_NS, "queryResult");
        Element results = result.getDocumentElement ();
        Iterator inteRecord = domDeserialiser.iterator (results, Constants.PSTRUCT_NS, "interactionRecord");
        Vector tab = new Vector ();
        while(inteRecord.hasNext ()){
            tab.add ((Element) inteRecord.next ());
            //System.out.println(domDeserialiser.text(interkey, "interactionId"));
        }
        return tab;
    }
    
    /**
     * Get all the content of the store
     * @return XML Document which contains all the containt of the store
     * @throws java.lang.Exception Preserv CSL exception
     */
    public Document getPStruct () throws Exception {
        Document result;
        
        
        String xquery = "$ps:pstruct";
        
//                "let $m := $j/ps:sender/ps:asserter/* " +
//                "let $n := $j/ps:sender/ps:asserter/* " +
//                "let $k := <ps:interactionRecord><ps:receiver>{$n}</ps:receiver><ps:sender>{$m}</ps:sender></ps:interactionRecord> " +
//                "let $l := fn:distinct-values($k) " +
//                "let $r := fn:count($l) " +
//                "return <result>{$l} - {$r}</result> ";
        result = _proxy.getXQueryInterface ().resolve (xquery);
        
//        String xquery = "for $n in $ps:pstruct/ps:interactionRecord//ps:asserter/* return $n";
//        result = clientLib.query(xquery, provenanceStoreQuery);
//
//
//        DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);
//        Element Body = domDeserialiser.single(result.getDocumentElement(), Constants.SOAP_NS, "Body");
//        Element qres = domDeserialiser.single(Body, Constants.WSXQUERY_NS, "queryResult");
//
//        NodeList nodelist = qres.getChildNodes();
//        for(int i =0 ; i <= nodelist.getLength() ;i++) {
//            Node node = nodelist.item(i);
//            System.out.println(node.getTextContent());
//        }
        
        return result;
    }
    
    /**
     * Get the interactionRecord with the same interactionKey as the given ObjectID from yours provenance store
     * @param ObjectId ObjectID used to get its interactionKey
     * @return XML Element which contains an interactionRecord
     * @throws java.lang.Exception Preserv CSL exception
     */
    public Element getInteractionRecord (ObjectID ObjectId) throws Exception {
        Document result;
        
        result = _proxy.getXQueryInterface ().resolve (
                asserterUtility.getInteractionRecord (ObjectId));
        DOMDeserialiser domDeserialiser = new DOMDeserialiser (Constants.PSTRUCT_NS);
        //Element Body = domDeserialiser.single (result.getDocumentElement (), Constants.SOAP_NS, "Body");
        //Element results = domDeserialiser.single (Body, Constants.WSXQUERY_NS, "queryResult");
        Element results = result.getDocumentElement ();
        Element inteRecord = domDeserialiser.single (results, Constants.PSTRUCT_NS, "interactionRecord");
        return inteRecord;
    }
}
