/*Copyright (c) 2005 University of Southampton

 Permission is hereby granted, free of charge,
 to any person obtaining a copy of this
 software and associated documentation files
 (the "Software"), to deal in the Software without
 restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute,
 sublicense, and/or sell copies of the Software, and
 to permit persons to whom the Software is furnished
 to do so, subject to the following conditions:

 The above copyright notice and this permission notice
 shall be included in all copies or substantial
 portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY
 OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
 LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO
 EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 OR OTHER DEALINGS IN THE SOFTWARE.
 -------------------------------------------------------*/
package org.pasoa.preview;

import com.lowagie.text.DocumentException;
import com.lowagie.text.pdf.DefaultFontMapper;
import com.lowagie.text.pdf.PdfGraphics2D;
import com.lowagie.text.pdf.PdfTemplate;
import java.awt.Toolkit;
import com.lowagie.text.Document;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfWriter;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.MalformedURLException;
import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import java.util.prefs.Preferences;
import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.plaf.basic.BasicTabbedPaneUI;
import javax.swing.table.*;
import java.util.EventObject;
import java.awt.event.KeyEvent;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.pasoa.common.Constants;
import org.pasoa.preview.asserterview.AsserterView;
import org.pasoa.preview.history.HistoryDialog;
import org.pasoa.preview.history.HistoryElement;
import org.pasoa.preview.provenanceview.ProvenanceView;
import org.pasoa.preview.tableview.ColumnHeaderToolTips;
import org.pasoa.preview.tableview.Infobox;
import org.pasoa.preview.tableview.TableView;
import org.pasoa.preview.utils.ErreurDialog;
import org.pasoa.preview.utils.Progression;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.simpledom.DOMWriter;
import org.w3c.dom.Attr;
import org.w3c.dom.CDATASection;
import org.w3c.dom.Comment;
import org.w3c.dom.DOMConfiguration;
import org.w3c.dom.DOMException;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.EntityReference;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.ProcessingInstruction;
import org.w3c.dom.Text;
import org.w3c.dom.UserDataHandler;

/**
 * the main JFrame of PReView
 * @author tibo
 */


public class View extends JFrame {
    
    // Variables declaration - do not modify   
        
    /**
     * a list of address store to be save.
     */
    public static String addressList = "";
    public JFrame jf;
    private MyJTabbedPane jTabbedPane1;
    private JDialog _history;
    public static boolean _historyVisible =false;
    public JRadioButtonMenuItem jrb;
    
    // End of variables declaration       
    /* Must be tested for updating a view... */
    /*jTabbedPane1.getComponentAt(jTabbedPane1.getSelectedIndex());*/
      
    /**
     * Creates new JFrame
     */
    public View() {
        this.jf = this;
        initComponents();
        this._history = new HistoryDialog(this, getJTabbedPane1());
        this._historyVisible = false;
        
    }
    
    /**
     * accessor for the current history dialog
     * @return the current history dialog
     */
    public HistoryDialog getHistoryDialog(){
        return (HistoryDialog) this._history;
    }

    /**
     * accessor for the current JTabbedPane
     * @return the current JTabbedPane
     */
    public MyJTabbedPane getJTabbedPane1() {
        return jTabbedPane1;
    }
                      

    /**
     * initialize all the components and the menu bar
     */
    private void initComponents() {
        //jTabbedPane1 = new javax.swing.JTabbedPane();
        //jTabbedPane1.setUI(new MyTabbedPaneUI());
        jTabbedPane1 = new MyJTabbedPane();
        jTabbedPane1.setOpaque(true);

        setTitle(":: Provenance Preview ::");
        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e){
                if(JOptionPane.showConfirmDialog(jf, "Do you want to exit ?", "exit ?", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION){
                    System.out.println("Bye...");
                    System.exit(0);
                }
            }
        });
        
        JMenuBar mb = createMenuBar();
        setJMenuBar(mb);

        add(jTabbedPane1);
        jf.setPreferredSize(new Dimension(640, 640));
        pack();
        
        
    }

    /**
     * Accessor for the list of address store
     * @return string which contains all the opened stores address
     */
    public String getAddressList() {
        return addressList;
    }

    /**
     * Modifier for the address list
     * @param addressList new value for the list of store address
     */
    public void setAddressList(String addressList) {
        this.addressList = addressList;
    }
    
    /**
     * create the menu bar
     * @return menu bar
     */
      protected JMenuBar createMenuBar() {
        JMenu fileMenu = new JMenu("File"); 
	JMenu storeMenu = new JMenu("Store");
        
        JMenu displayMenu = new JMenu("Display");
        JMenu helpMenu = new JMenu("?");
        
        JMenuItem menuItem;
        
        jrb = new JRadioButtonMenuItem("Show/Hide History"); 
        jrb.setSelected(_historyVisible);
        jrb.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
                if(_historyVisible){
                    //jrb.setSelected(false);
                    _historyVisible = false;
                    _history.setVisible(false);
                }
                else {
                    //jrb.setSelected(true);
                    _historyVisible = true;
                    _history.setVisible(true);
                }
                
            }
        });
       displayMenu.add(jrb);
       
       menuItem = new JMenuItem("Command list");
       menuItem.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent ae) {
               
               String html = "";
               try {
                   BufferedReader in = new BufferedReader(new FileReader("files/help.html"));
                   String line = "";
                   while((line = in.readLine()) != null) {
                       html = html + line + "\n";
                   }
                   //System.out.println(html);
                   HTMLDisplay jd = new HTMLDisplay(jf, html);
                   jd.setVisible(true);
                   in.close();
                   
               } catch (FileNotFoundException ex) {
                   if(Controller.debug)ErreurDialog.afficher(jf, "Problem while loading data", 
                           ex.getMessage(),ex.getStackTrace());
                   else ErreurDialog.afficher(jf, "Problem while loading data", ex.getMessage());
               } catch (IOException ex) {
                   if(Controller.debug)ErreurDialog.afficher(jf, "Problem while loading data", 
                           ex.getMessage(),ex.getStackTrace());
                   else ErreurDialog.afficher(jf, "Problem while loading data", ex.getMessage());
               }
               
           }
       });
       helpMenu.add(menuItem);
       
        menuItem = new JMenuItem("Open store");
        menuItem.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
                OpenStore on = new OpenStore(jf, true, jTabbedPane1, false);
                on.setVisible(true);
            }
        });
       storeMenu.add(menuItem);
        
        menuItem = new JMenuItem("Print pstruct");
        menuItem.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
                OpenStore on = new OpenStore(jf, true, jTabbedPane1, true);
                on.setVisible(true);
            }
        });
        storeMenu.add(menuItem);
        
        menuItem = new JMenuItem("Clear Last Stores");
        menuItem.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
                Preferences prefs = Preferences.userRoot();
                int nb = prefs.getInt("nbStoresAddress", 0);
                prefs.remove("nbStoresAddress");
                for (int i = 0; i < nb ; i++){
                    prefs.remove("StoreAddress" + i);
                }
            }
        });
        storeMenu.add(menuItem);
        
	//fileMenu.addSeparator();
        
        
        menuItem = new JMenuItem("Open");
        menuItem.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
                

            Preferences prefs = Preferences.userRoot();
            JFileChooser jfc = new JFileChooser(prefs.get("OpenPath", "."));
            int result = -1;

            jfc.setFileFilter (
                 new javax.swing.filechooser.FileFilter() {
                     public boolean accept(File f) {
                         if(f.isDirectory()) return true;
                         else {
                             String nom = f.getName() ;
                             return (nom.endsWith(".prev")) ;
                         }
                     }
                     public String getDescription() {
                         return "*.prev" ;
                     }
                 }
             ) ;
            result = jfc.showOpenDialog(jf);
            if(result == jfc.APPROVE_OPTION){

                if(jfc.getSelectedFile().exists()){
                    String path = jfc.getSelectedFile().getAbsolutePath();
                    try {
                        final BufferedReader in = new BufferedReader(new FileReader(path));
                        
                        Runnable go = new Runnable() {
                            public void run() {
                                Progression pgr = new Progression(jf);
                                try {
                                    pgr.startProgress();
                                    String line = "";
                                    while((line = in.readLine()) != null) {
                                        String add  = line;
                                        String type = in.readLine();
                                        if(type != null){
                                            if(type.equals("new")) OpenStore.openNewStoreImpl(jf, add, jTabbedPane1, false);
                                            else OpenStore.openFullStoreImpl(jf, add, jTabbedPane1, type, false);
                                        }
                                    }
                                    pgr.stopProgress();
                                } catch (Exception ex) {
                                    if(Controller.debug)ErreurDialog.afficher(jf, "Problem while loading data", 
                                        ex.getMessage(),ex.getStackTrace());
                                    else ErreurDialog.afficher(jf, "Problem while loading data", ex.getMessage());
                                 pgr.stopProgress();
                                }
                                try {
                                    in.close();
                                } catch (IOException ex) {
                                     if(Controller.debug)ErreurDialog.afficher(jf, "In Exception", 
                                         ex.getMessage(),ex.getStackTrace());
                                     else ErreurDialog.afficher(jf, "In Exception", ex.getMessage());
                                }       
                            }
                        };
                        Thread t = new Thread(go);
                        t.start();
                        prefs.put("OpenPath", path);
                        
                    } catch (FileNotFoundException ex) {
                         if(Controller.debug)ErreurDialog.afficher(jf, "File not found", 
                             ex.getMessage(),ex.getStackTrace());
                         else ErreurDialog.afficher(jf, "File not found", ex.getMessage());
                    } catch (IOException ex) {
                         if(Controller.debug)ErreurDialog.afficher(jf, "In Exception", 
                             ex.getMessage(),ex.getStackTrace());
                         else ErreurDialog.afficher(jf, "In Exception", ex.getMessage());
                    }        
                }
            }
            }
        });
        fileMenu.add(menuItem);
        
        menuItem = new JMenuItem("Save");
        menuItem.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
                
                Preferences prefs = Preferences.userRoot();
                JFileChooser jfc = new JFileChooser(prefs.get("SavePath", "."));
                int result = -1;
                
                jfc.setFileFilter (
                     new javax.swing.filechooser.FileFilter() {
                         public boolean accept(File f) {
                             if(f.isDirectory()) return true;
                             else {
                                 String nom = f.getName() ;
                                 return (nom.endsWith(".prev")) ;
                             }
                         }
                         public String getDescription() {
                             return "*.prev" ;
                         }
                     }
                 ) ;
                result = jfc.showSaveDialog(jf);
                if(result == jfc.APPROVE_OPTION){
                    if(jfc.getSelectedFile().exists()){
                        String path = jfc.getSelectedFile().getAbsolutePath();
                        if(JOptionPane.showConfirmDialog(jf, "Do you want to delete this file ?\n" + path, "delete ?", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION){
                            try {
                                BufferedWriter out = new BufferedWriter(new FileWriter(path));
                                out.write(addressList);
                                out.close();
                                prefs.put("SavePath", path);
                            } catch (FileNotFoundException ex) {
                                 if(Controller.debug)ErreurDialog.afficher(jf, "File not found", 
                                     ex.getMessage(),ex.getStackTrace());
                                 else ErreurDialog.afficher(jf, "File not found", ex.getMessage());
                            } catch (IOException ex) {
                                 if(Controller.debug)ErreurDialog.afficher(jf, "Out Exception", 
                                     ex.getMessage(),ex.getStackTrace());
                                 else ErreurDialog.afficher(jf, "Out Exception", ex.getMessage());
                            } 
                        }
                    }
                }
            }
        });
        fileMenu.add(menuItem);
        
	fileMenu.addSeparator();
        
        fileMenu.add(buildImportMenu());
        fileMenu.add(buildExportMenu());
        
	fileMenu.addSeparator();
        
	menuItem = new JMenuItem("Exit"); 
	menuItem.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		System.out.println("Bye...");
                System.exit(0);
	    }
	});
	fileMenu.add(menuItem); 

	// Create a menu bar
	JMenuBar menuBar = new JMenuBar();
	menuBar.add(fileMenu);
	menuBar.add(storeMenu);
	menuBar.add(displayMenu);
        
        
	// Menu for the look and feels (lafs).
	UIManager.LookAndFeelInfo[] lafs = UIManager.getInstalledLookAndFeels();
	ButtonGroup lafGroup = new ButtonGroup();
	JMenu optionsMenu = new JMenu("Options");

	menuBar.add(optionsMenu);
        menuBar.add(helpMenu);
        
        final JFrame jf = this;
	for(int i = 0; i < lafs.length; i++) {
	    JRadioButtonMenuItem rb = new JRadioButtonMenuItem(lafs[i].getName()); 
	    optionsMenu.add(rb);
	    rb.setSelected(UIManager.getLookAndFeel().getName().equals(lafs[i].getName()));
	    rb.putClientProperty("UIKey", lafs[i]);
	    rb.addItemListener(new ItemListener() {
	        public void itemStateChanged(ItemEvent ae) {
	            JRadioButtonMenuItem rb2 = (JRadioButtonMenuItem)ae.getSource();
	            if(rb2.isSelected()) {
		        UIManager.LookAndFeelInfo info = (UIManager.LookAndFeelInfo) rb2.getClientProperty("UIKey");
                            try {
                                UIManager.setLookAndFeel(info.getClassName());
                                SwingUtilities.updateComponentTreeUI(jf);
                            } catch (ClassNotFoundException ex) {
                                 if(Controller.debug)ErreurDialog.afficher(jf, "Class not found", 
                                     ex.getMessage(),ex.getStackTrace());
                                 else ErreurDialog.afficher(jf, "Class not found", ex.getMessage());
                            } catch (UnsupportedLookAndFeelException ex) {
                                 if(Controller.debug)ErreurDialog.afficher(jf, "Unsupported Look And Feel", 
                                     ex.getMessage(),ex.getStackTrace());
                                 else ErreurDialog.afficher(jf, "Unsupported Look And Feel", ex.getMessage());
                            } catch (InstantiationException ex) {
                                 if(Controller.debug)ErreurDialog.afficher(jf, "unable to set UI", 
                                     ex.getMessage(),ex.getStackTrace());
                                 else ErreurDialog.afficher(jf, "unable to set UI ", ex.getMessage());
                            } catch (IllegalAccessException ex) {
                                 if(Controller.debug)ErreurDialog.afficher(jf, "Illegal Access", 
                                     ex.getMessage(),ex.getStackTrace());
                                 else ErreurDialog.afficher(jf, "Illegal Access", ex.getMessage());
                            }
                            //jTabbedPane1.setUI(new MyTabbedPaneUI());
			 
	            }
	        }
	    });
	    lafGroup.add(rb);
	}
	return menuBar;
    }
      
    
    /**
     * Import menu
     * @return menu
     */
    public JMenu buildImportMenu(){
        
        JMenu ImportMenu = new JMenu("Import"); 
        /* XML */
        JMenuItem xml = new JMenuItem("from a XML file");
        ImportMenu.add(xml);
        xml.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                Preferences prefs = Preferences.userRoot();
                JFileChooser jfc = new JFileChooser(prefs.get("ImportPath", "."));
                int result = -1;
                
                jfc.setFileFilter (
                     new javax.swing.filechooser.FileFilter() {
                         public boolean accept(File f) {
                             if(f.isDirectory()) return true;
                             else {
                                 String nom = f.getName() ;
                                 return (nom.endsWith(".xml")) ;
                             }
                         }
                         public String getDescription() {
                             return "*.xml" ;
                         }
                     }
                 ) ;
                //jfc.setApproveButtonText("Import");
                result = jfc.showOpenDialog(jf);
                if(result == jfc.APPROVE_OPTION){
                    if(jfc.getSelectedFile().exists()){
                        String path = jfc.getSelectedFile().getAbsolutePath();
                        if(JOptionPane.showConfirmDialog(jf, "Do you want to delete this file ?\n" + path, "delete ?", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION){
                            final File f = new File(path);
                            if(jfc.getSelectedFile().exists() && path.endsWith(".xml")){
                                try{
                                    /* Create the provenance view */
                                    Runnable go = new Runnable() {
                                        public void run() {
                                            Progression pgr = new Progression(jf);
                                            try {
                                                pgr.startProgress();
                                                FileInputStream fis = new FileInputStream(f);
                                                DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
                                                dbf.setNamespaceAware(true);
                                                DocumentBuilder docBuilder = dbf.newDocumentBuilder();
                                                org.w3c.dom.Document doc = docBuilder.parse(fis);

                                                JScrollPane provenanceview = null;
                                                DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);
                                                Element Body = domDeserialiser.single(doc.getDocumentElement(), Constants.SOAP_NS, "Body");
                                                Element PV = domDeserialiser.single(Body, Constants.PQUERY_NS, "provenanceQueryResult");
                                                //System.out.println(DOMWriter.writeToString(Body, true));
                                                if (Body != null )provenanceview = new JScrollPane(new ProvenanceView(jf, jTabbedPane1, (Element) PV));
                                                else provenanceview = new JScrollPane(new ProvenanceView(jf, jTabbedPane1, (Element) doc.getDocumentElement()));
                                                jTabbedPane1.addTab(
                                                        "ProvenanceView - "+ f.getName(), 
                                                        null, 
                                                        provenanceview, 
                                                        "ProvenanceView - " + f.getCanonicalPath());

                                                JScrollPane jscrollpane = new JScrollPane();
                                                JTextPane jtextpane = new JTextPane();

                                                jTabbedPane1.setSelectedIndex(jTabbedPane1.getTabCount()-1);
                                                jf.repaint();

                                                pgr.stopProgress();
                                            } catch (Exception ex) {
                                                if(Controller.debug)ErreurDialog.afficher(jf, "Malformed XML file", 
                                                    ex.getMessage(),ex.getStackTrace());
                                                else ErreurDialog.afficher(jf, "Malformed XML file", ex.getMessage());
                                                pgr.stopProgress();
                                            }

                                        }
                                    };
                                    Thread t = new Thread(go);
                                    t.start();

                                }catch(Exception e){
                                     if(Controller.debug)ErreurDialog.afficher(jf, "Error while importing", 
                                         e.getMessage(),e.getStackTrace());
                                     else ErreurDialog.afficher(jf, "Error while importing", e.getMessage());
                                }
                            }
                            else {
                                JOptionPane.showMessageDialog(jf, "Not a correct XML file or file doesn't exist");
                            }
                            prefs.put("ImportPath", f.getAbsolutePath());
                        }
                    }
                }
            }
        });
        return ImportMenu;
    }
      
    /**
     * Export Menu
     * @return menu
     */
      
    public JMenu buildExportMenu(){
        
        
        JMenu ExportMenu = new JMenu("Export"); 
        /* PDF */
        JMenuItem pdf = new JMenuItem("pdf");
        pdf.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                /**/        

                Preferences prefs = Preferences.userRoot();
                JFileChooser jfc = new JFileChooser(prefs.get("ExportPath", "."));
                int result = -1;
                
                jfc.setFileFilter (
                     new javax.swing.filechooser.FileFilter() {
                         public boolean accept(File f) {
                             if(f.isDirectory()) return true;
                             else {
                                 String nom = f.getName() ;
                                 return (nom.endsWith(".pdf")) ;
                             }
                         }
                         public String getDescription() {
                             return "*.pdf" ;
                         }
                     }
                 ) ;
                result = jfc.showSaveDialog(jf);
                if(result == jfc.APPROVE_OPTION){
                    String path = jfc.getSelectedFile().getAbsolutePath();
                    if(!path.endsWith(".pdf")) path += ".pdf";
                    File file = new File(path);
                
                    if(jTabbedPane1.getTabCount() == 0){
                        JOptionPane.showMessageDialog(jf, "You must open first a store or import a XML file");
                    }
                    else {
                        Component c = jTabbedPane1.getComponentAt(jTabbedPane1.getSelectedIndex());

                        if(c instanceof JTabbedPane){
                            //System.out.println("JtabbedPane");
                            Component c1 = ((JTabbedPane)c).getComponentAt(((JTabbedPane)c).getSelectedIndex());
                            if(c1 instanceof JScrollPane){
                                //System.out.println("JScrollPane");
                                JScrollPane jsp = (JScrollPane) c1;
                                if(jsp.getViewport().getComponentCount() == 1){
                                    Component c2 = jsp.getViewport().getComponent(0);
                                    if(c2 instanceof AsserterView){
                                        //System.out.println("AsserterView");
                                        exportView((AsserterView) c2, file);
                                    }
                                    else {
                                        if(c2 instanceof ProvenanceView){
                                            //System.out.println("ProvenanceView");
                                            exportProvenanceView((ProvenanceView) c2, file);
                                        }
                                    }
                                }
                            }
                            else {
                                if(c1 instanceof JSplitPane){
                                    //System.out.println("TableView");
                                    //Separate TableView Infobox ??
                                    exportView(c1, file);
                                }
                            }
                        }
                        else {
                            if(c instanceof JScrollPane){
                                //System.out.println("JScrollPane");
                                JScrollPane jsp = (JScrollPane) c;
                                if(jsp.getViewport().getComponentCount() == 1){
                                    Component c2 = jsp.getViewport().getComponent(0);
                                    if(c2 instanceof ProvenanceView){
                                        //System.out.println("ProvenanceView");
                                        exportProvenanceView((ProvenanceView) c2, file);
                                    }
                                    else {
                                        //System.out.println("PrintStruct...");
                                        exportView(c2, file);
                                    }
                                }
                            }
                            else {
                                exportView(c, file);
                            }
                        }
                    }
                
                    prefs.put("ExportPath", path);
                /**/
                }
                
            }
        });
        /* End PDF */
        
        ExportMenu.add(pdf);
        return ExportMenu;
    
    }
    
    /**
     * create the pdf file from a component with vertor graphics
     * @param av Component to be exported
     * @param file the path
     */
    public void exportView(Component av, File file){
        try{
            Document document = new Document(
                    new com.lowagie.text.Rectangle((int)av.getWidth(),(int)av.getHeight()));
            PdfWriter writer;
            writer = PdfWriter.getInstance(document, new FileOutputStream(file));
            document.open();
            PdfContentByte cb = writer.getDirectContent();
            PdfTemplate tp = cb.createTemplate((int)av.getWidth(),(int)av.getHeight());
            Graphics2D g2d = tp.createGraphics((int)av.getWidth(),(int)av.getHeight(), new DefaultFontMapper());
            av.paint(g2d);

            g2d.dispose();

            cb.addTemplate(tp, 0, 0);

    //                    BufferedImage bi = new BufferedImage(jTabbedPane1.getWidth(),jTabbedPane1.getHeight()
    //                            ,BufferedImage.TYPE_INT_RGB);
    //                    jTabbedPane1.paintAll(bi.createGraphics());
    //                    PdfContentByte cb = writer.getDirectContent();
    //                    com.lowagie.text.Image image = com.lowagie.text.Image.getInstance( Toolkit.getDefaultToolkit()
    //                                                            .createImage( bi.getSource() ), null );
    //
    //                    image.setAbsolutePosition( 0, 0 ); // set image in upper-left quadrant
    ////                    // - reference corner is lower-left
    ////                    image.scaleAbsolute( 300, 300 ); // set image to one-quarter area
    //                    cb.addImage( image );

            document.close();
        
        } catch (FileNotFoundException ex) {
            if(Controller.debug)ErreurDialog.afficher(jf, "Error while exporting", 
                    ex.getMessage(),ex.getStackTrace());
            else ErreurDialog.afficher(jf, "Error while exporting", ex.getMessage());
        } catch (DocumentException ex) {
            if(Controller.debug)ErreurDialog.afficher(jf, "Error while exporting", 
                    ex.getMessage(),ex.getStackTrace());
            else ErreurDialog.afficher(jf, "Error while exporting", ex.getMessage());
        }

    }

    /**
     * create the pdf file from a provenance view with vertor graphics
     * @param _provenanceview a provenance view
     * @param file the path
     */
    public void exportProvenanceView(ProvenanceView _provenanceview, File file){

        double scale = 1.0;
        Dimension d = new Dimension((int)(scale*_provenanceview.tview.getWidth()),
                                    (int)(scale*_provenanceview.tview.getHeight()));
        Document document = new Document(
                        new com.lowagie.text.Rectangle((int)scale*_provenanceview.tview.getWidth(),(int)scale*_provenanceview.tview.getHeight()));
        PdfWriter writer = null;
        try {
            writer = PdfWriter.getInstance(document, new FileOutputStream(file));
            document.open();
            PdfContentByte cb = writer.getDirectContent();
            PdfTemplate tp = cb.createTemplate((int)scale*_provenanceview.tview.getWidth(),(int)scale*_provenanceview.tview.getHeight());
            Graphics2D g2d = tp.createGraphics((int)scale*_provenanceview.tview.getWidth(),(int)scale*_provenanceview.tview.getHeight(), new DefaultFontMapper());

            // set up the display, render, then revert to normal settings
            Point2D p = new Point2D.Double(0,0);
            _provenanceview.tview.zoom(p, scale); // also takes care of damage report
            boolean q = _provenanceview.tview.isHighQuality();
            _provenanceview.tview.setHighQuality(true);
            _provenanceview.tview.paintDisplay(g2d, d);
            _provenanceview.tview.setHighQuality(q);
            _provenanceview.tview.zoom(p, 1/scale); // also takes care of damage report
            g2d.dispose();
            cb.addTemplate(tp, 0, 0);

            document.close();
        } catch (FileNotFoundException ex) {
            if(Controller.debug)ErreurDialog.afficher(jf, "Error while exporting", 
                    ex.getMessage(),ex.getStackTrace());
            else ErreurDialog.afficher(jf, "Error while exporting", ex.getMessage());
        } catch (DocumentException ex) {
            if(Controller.debug)ErreurDialog.afficher(jf, "Error while exporting", 
                    ex.getMessage(),ex.getStackTrace());
            else ErreurDialog.afficher(jf, "Error while exporting", ex.getMessage());
        }

    }
    
}
