/*
 * AsserterView.java
 *
 * Created on 25 juillet 2006, 14:05
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.pasoa.preview.asserterview;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Vector;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import org.pasoa.common.Constants;
import org.pasoa.preview.*;
import org.pasoa.preview.history.HistoryOpenActor;
import org.pasoa.preview.provenanceview.ProvenanceView;
import org.pasoa.preview.utils.ErreurDialog;
import org.pasoa.preview.utils.PReViewUtilities;
import org.pasoa.preview.utils.Progression;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.simpledom.DOMWriter;
import org.pasoa.simpledom.NormalisingDOMWriter;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * the actor view
 * @author tibo
 */
public class AsserterView extends JPanel {
    
    protected Controller control;
    protected AsserterIdentity currentAsserter;
    protected Vector sender;
    protected Vector receiver;
    private List history = new LinkedList();
    /**
     * a list of the drawables objects
     */
    private List drawables = new LinkedList();
    protected int prefWidth;
    protected int prefHeight;
    protected int Width;
    protected int Height;
    protected int actorHeight;
    protected int actorWidth;
    protected int bord;
    
    protected static int actorHeightInit = 20;
    protected static int actorWidthInit = 30;
    protected static int bordInit = 70;
    protected static int betweenInteractionInit = 10;
    protected double scaleX;
    protected double scaleY;
    protected double moveX;
    protected double moveY;
    protected int betweenInteraction;
    
    protected Vector colorRelationship = new Vector();
    protected JFrame jframe;
    protected JTabbedPane jtabbedpane;

    protected Vector assertersNames;
    
    private AffineTransform transform;

    protected JScrollPane infobox;
    
    private static Color actorColor = new Color(150,150,150);

    private boolean first = true;

    private Vector<Vector> interactionsDataGrid;
    
    /**
     * Creates a new instance of AsserterView
     * @param control the controller
     * @param currentAsserter current AsserterIdentity
     * @param assertersNames the vector of all the asserterIdentity
     * @param jf the main JFrame
     * @param jtp the JTabbedPane which contains this view
     * @param infobox a JScrollPane to open a new infobox
     * @param interactionsDataGrid Vector<Vector<int>> which contains all interactions (like a grid)
     * @param is_history if the contructor is called from the history menu
     * @throws java.lang.Exception CSL Exceptions
     */
    public AsserterView(
            Controller control, AsserterIdentity currentAsserter, Vector assertersNames, 
            JFrame jf, JTabbedPane jtp, JScrollPane infobox, Vector<Vector> interactionsDataGrid,
            boolean is_history) throws Exception {
        this.control = control;
        this.currentAsserter = currentAsserter;
        this.assertersNames = assertersNames;
        this.jframe = jf;
        this.jtabbedpane =jtp;
        this.infobox = infobox;
        this.interactionsDataGrid = interactionsDataGrid;
        //System.out.println(currentAsserter.getLongName());
        /* Test */
        setOpaque(true);
        setBackground(Color.WHITE);
        
        this.actorHeight = this.actorHeightInit;
        this.actorWidth = this.actorWidthInit;
        this.scaleX = 1.0;
        this.scaleY = 1.0;
        this.moveX = 0;
        this.moveY = 0;
        this.bord = this.bordInit;
        this.betweenInteraction = this.betweenInteractionInit;
        
        getAsserter();
        
        int max = Math.max(this.sender.size(), this.receiver.size());
    
        scaleX = 3;
        scaleY = 3;

        this.Height = (max * (actorHeight + betweenInteraction) + betweenInteraction);
        //int w = this.jtabbedpane.getWidth() - 2 * bord - 2* actorWidth;
        this.Width = (this.Height > ( 2 * this.bord))? this.bord : this.Height;
        this.prefHeight = this.Height + 2 * this.bord;
        this.prefWidth = this.Width + 2 * this.bord + 2 * this.actorWidth;
        
        fillinDrawables();
        
        new MouseListen(this);
        if(!is_history)((View)jf).getHistoryDialog().addElementHistory(new HistoryOpenActor(control.address, this.currentAsserter));
       
    }
    
    
    /**
     * Creates a new instance of AsserterView
     * @param mX span position X
     * @param mY span position Y
     * @param sX scale value X
     * @param sY scale value Y
     * @param bI space between two grey boxes
     * @param aH asserterBox height
     * @param aW asserterBox Width
     * @param h actor height
     * @param w actor width
     * @param control the controller
     * @param currentAsserter current AsserterIdentity
     * @param assertersNames the vector of all the asserterIdentity
     * @param jf the main JFrame
     * @param jtp the JTabbedPane which contains this view
     * @param infobox a JScrollPane to open a new infobox
     * @param interactionsDataGrid Vector<Vector<int>> which contains all interactions (like a grid)
     * @throws java.lang.Exception CSL Exceptions
     */
    public AsserterView(
            Controller control, AsserterIdentity currentAsserter, 
            Vector assertersNames, JFrame jf, JTabbedPane jtp,
            JScrollPane infobox, double mX, double mY, double sX, double sY,
            int bI, int aH, int aW, int h, int w, Vector<Vector> interactionsDataGrid) throws Exception {
        
        this.control = control;
        this.currentAsserter = currentAsserter;
        this.assertersNames = assertersNames;
        this.jframe = jf;
        this.jtabbedpane =jtp;
        this.infobox = infobox;
        this.interactionsDataGrid = interactionsDataGrid;
        //System.out.println(currentAsserter.getLongName());
        /* Test */
        setOpaque(true);
        setBackground(Color.WHITE);
        
        this.actorHeight = aH;
        this.actorWidth = aW;
        this.scaleX = sX;
        this.scaleY = sY;
        this.moveX = mX;
        this.moveY = mY;
        this.bord = this.bordInit;
        this.betweenInteraction = bI;
        
        getAsserter();
        
        //int max = Math.max(this.sender.size(), this.receiver.size());
    
        int max = Math.max(this.sender.size(), this.receiver.size());
    
        this.Height = (max * (actorHeight + betweenInteraction) + betweenInteraction);
        //this.Height = h;
        //if(this.Height < 1200) 
        
        int tmp = (max * (actorHeightInit + betweenInteractionInit) + betweenInteractionInit);
        tmp = (tmp > this.bord)? this.bord : tmp;
        if(w > tmp) this.Width = w;
        else this.Width = tmp+1;
        //else this.Width = 1200;
        this.prefHeight = this.Height + 2 * this.bord;
        this.prefWidth = this.Width + 2 * this.bord + 2* this.actorWidth;
        
        fillinDrawables();
        
        new MouseListen(this);
        ((View)jf).getHistoryDialog().addElementHistory(new HistoryOpenActor(control.address, this.currentAsserter));
       
    }
    
    /**
     * update the current list of drawable item
     */
    public void reset (){
        this.clear();
        
        fillinDrawables();
    }
    
    /**
     * Paint all the content of the panel
     * @param g Graphics
     */
    public void paint(Graphics g) {
        //super.paint(g);
        
        int tmp = (int) ((g.getFontMetrics().stringWidth(currentAsserter.getShortName())+ 2 * bord) * scaleX );
        this.prefHeight = (int)((this.Height + 2 * this.bord) * scaleY + moveY + 0.5);
        this.prefWidth = (int)((this.Width + 2 * this.actorWidth + 2 + 2 * this.bord) * scaleX + moveX + 0.5);
        if(tmp >= this.prefWidth) this.prefWidth = tmp;
        
        g.clearRect(0, 0, getSize().width, getSize().height);
        Graphics2D g2d = (Graphics2D) g;
        g2d.setStroke(new BasicStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        
        g2d.translate(moveX, moveY);
        g2d.scale(scaleX, scaleY);

        Font f = g2d.getFont();
        g2d.setFont(new Font("Serif", Font.ITALIC, (int)(g2d.getFont().getSize())));
        g2d.drawString(currentAsserter.getShortName(), bord, (int)(bord - (1 * scaleY)));
        g2d.setFont(f);
        g2d.drawRect(bord, bord, Width, Height);

        
        for(Iterator iter = drawables.iterator(); iter.hasNext();){
            IDrawable d = (IDrawable) iter.next();
            if(d.isDrawable())  d.draw(g2d);
        }
        revalidate();

    }

    /**
     * add an object to draw to the draw list
     * @param d a new object to draw
     */
    public void addDrawable(IDrawable d){
        drawables.add(d);
        repaint();
    }
    
    /**
     * Remove one object from the drawable list
     * @param d an object to remove from the drawable list
     */
    public void removeDrawable(IDrawable d){
        drawables.remove(d);
        repaint();
    }
    
    /**
     * clear the drawable list
     */
    public void clear(){
        drawables.clear();
        repaint();
    }
    
    /**
     * set the drawable attribute to all the drawables from the drawables list to the given value
     * @param tf boolean value
     */
    public void setDrawableAll(boolean tf){
	for (Iterator iter = drawables.iterator(); iter.hasNext();) {
            IDrawable element = (IDrawable) iter.next();
            element.setDrawable(tf);
        }
    }
    
    /**
     * set the selected attribute of all the drawables from the drawables list to the given value
     * @param tf boolean value
     */
    public void setSelectedAll(boolean tf){
	for (Iterator iter = drawables.iterator(); iter.hasNext();) {
            IDrawable element = (IDrawable) iter.next();
            element.setSelected(tf);
        }
    }
    
    /**
     * get the a list of the drawables which contains this point
     * @param p a point
     * @return a list of drawables
     */
    public List findDrawables(Point p) {
	List l = new ArrayList();
	for (Iterator iter = drawables.iterator(); iter.hasNext();) {
		IDrawable element = (IDrawable) iter.next();
                
                Graphics2D g2d = (Graphics2D) this.getGraphics();
                boolean stroke;
                if(element instanceof RelationshipElement) stroke = true;
                else stroke = false;
                if(g2d.hit(new Rectangle(new Point(p.x-1, p.y-1), new Dimension (3,3)), element.getShape(), stroke)) {
                //if(element.getShape().contains(p)){
                    l.add(element);
                }
	}
	return l;
    }
    
    /**
     * find all the relationship from an asserter
     * @param rect AsserterDrawable
     * @param select selected value to set the relationship
     * @param paint is drawables ? value to set the relationship
     * @return a list of relatioships
     */
    public List findRelationship(AsserterDrawable rect, boolean select, boolean paint) {
        List l = new ArrayList();
	for (Iterator iter = drawables.iterator(); iter.hasNext();) {
		IDrawable element = (IDrawable) iter.next();
                if(element instanceof RelationshipElement)
                if(((RelationshipElement)element).receiver == rect) {
                    IDrawable send = findInteractionSenderFromRelationship(element);
                    send.setSelected(select);
                    send.setDrawable(paint);
                    element.setSelected(select);
                    element.setDrawable(paint);
                    l.add(element);
		}
                else {
                    if(((RelationshipElement)element).sender == rect){
                        IDrawable recei = findInteractionReceiverFromRelationship(element);
                        recei.setSelected(select);
                        recei.setDrawable(paint);
                        element.setSelected(select);
                        element.setDrawable(paint);
                        l.add(element);
                    }
                }
	}
	return l;
    }
    
    
    /**
     * find all the relationship from a relationship
     * @param rect a relationship drawable
     * @param select selected value to set the relationship
     * @param paint is drawable value to set to the relationship
     * @return a list of relatioships
     */
    public List findRelationship(RelationshipElement rect, boolean select, boolean paint) {
        List l = new ArrayList();
	for (Iterator iter = drawables.iterator(); iter.hasNext();) {
		IDrawable element = (IDrawable) iter.next();
                if(element instanceof RelationshipElement)
                if(((RelationshipElement)element).sender == rect.sender
                        && ((RelationshipElement)element).RelationshipNb == rect.RelationshipNb) {
                    IDrawable recei = findInteractionReceiverFromRelationship(element);
                    recei.setSelected(select);
                    recei.setDrawable(paint);
                    element.setSelected(select);
                    element.setDrawable(paint);
                    l.add(element);
                }
	}
	return l;
    }
    
    /**
     * find all the relationship from an IDrawable
     * @param rect the IDrawable
     * @return a list of relationship
     */
    public List findRelationship(IDrawable rect) {
        List l = new ArrayList();
	for (Iterator iter = drawables.iterator(); iter.hasNext();) {
		IDrawable element = (IDrawable) iter.next();
                if(element instanceof RelationshipElement)
                if(((RelationshipElement)element).receiver == rect || ((RelationshipElement)element).sender == rect){
                    l.add(element);
		}
	}
	return l;
    }
    
    /**
     * find the receiver from a given relationship
     * @param relationship a relationship
     * @return the receiver
     */
    public IDrawable findInteractionReceiverFromRelationship(IDrawable relationship) {
	for (Iterator iter = drawables.iterator(); iter.hasNext();) {
		IDrawable element = (IDrawable) iter.next();
                if(element instanceof AsserterDrawable)
                    if(((RelationshipElement)relationship).receiver == element)
                        return element;
        }
        return null;
    }
    
    /**
     * find the sender from a given relationship
     * @param relationship a relationship
     * @return the sender
     */
    public IDrawable findInteractionSenderFromRelationship(IDrawable relationship) {
	for (Iterator iter = drawables.iterator(); iter.hasNext();) {
		IDrawable element = (IDrawable) iter.next();
                if(element instanceof AsserterDrawable)
                    if(((RelationshipElement)relationship).sender == element)
                        return element;
        }
        return null;
    }
    
    /**
     * test if an area is free (without any IDrawables)
     * @param rect area
     * @return true if it is free
     * else false
     */
    public boolean isFree(Rectangle rect) {
	for (Iterator iter = drawables.iterator(); iter.hasNext();) {
		IDrawable element = (IDrawable) iter.next();
                if(element instanceof AsserterDrawable)
		if(element.getShape().intersects(rect)){
			return false;
		}
	}
	return true;
    }
    
    /**
     * test if the given IDrawable is alone
     * @param drawable a drawable object
     * @return true if the IDrawable is free 
     * else false
     */
    public boolean isAlone(IDrawable drawable) {
	for (Iterator iter = drawables.iterator(); iter.hasNext();) {
		IDrawable element = (IDrawable) iter.next();
                if(element instanceof AsserterDrawable)
		if(!element.equals(drawable) && element.getShape().intersects(drawable.getShape().getBounds())) {
			return false;
		}
	}
	return true;
    }
    
    /**
     * get the asserter from the store
     * @throws java.lang.Exception model Interface exception
     */
    public void getAsserter() throws Exception{
            /* Get interactions with this actor */
        this.sender = this.control.mi.getAllInteractionSenderRecord(this.currentAsserter);
        
        this.receiver = this.control.mi.getAllInteractionReceiverRecord(this.currentAsserter);
        
    }
    
    /**
     * fill in the drawable list
     */
    public void fillinDrawables(){
      
        //setPreferredSize(new Dimension(800, 600));
        Dimension dim = new Dimension(actorWidth,actorHeight);

        if (receiver != null)
        for (int i = 0; i < this.receiver.size(); i++){
            DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);
            Element se = domDeserialiser.single((Element)receiver.elementAt(i), Constants.PSTRUCT_NS, "sender");
            Element asserter = domDeserialiser.single(se, Constants.PSTRUCT_NS, "asserter");
            IDrawable rect;
            if(se != null){
                rect = new ReceiverDrawable(
                        actorColor, 
                        new Point(bord - actorWidth - 1, bord + betweenInteraction + i * (betweenInteraction + actorHeight)), 
                        dim, 
                        (Element) receiver.elementAt(i), 
                        (AsserterIdentity) assertersNames.elementAt(
                            control.getAi().findAsserterIdentity(asserter, assertersNames)));
             
            }
            else {
                rect = new ReceiverDrawable(
                        actorColor, 
                        new Point(bord - actorWidth - 1, bord + betweenInteraction + i * (betweenInteraction + actorHeight)), 
                        dim, 
                        (Element) receiver.elementAt(i), 
                    (AsserterIdentity) assertersNames.elementAt(assertersNames.size()-1));
            }
            addDrawable(rect);
        }

        int colorIndice = 0;
        if (sender != null)
        for (int i = 0; i < this.sender.size(); i++){
            
            
            DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);
            Element recep = domDeserialiser.single((Element) sender.elementAt(i), Constants.PSTRUCT_NS, "receiver");
            Element se = domDeserialiser.single((Element)sender.elementAt(i), Constants.PSTRUCT_NS, "sender");
            Element asserter = domDeserialiser.single(recep, Constants.PSTRUCT_NS, "asserter");
            IDrawable rect;
            
            if(recep != null){
                rect = new SenderDrawable(
                    actorColor, 
                    new Point(Width + bord + 1, bord + betweenInteraction + i * (betweenInteraction + actorHeight)), 
                    dim, 
                    (Element) sender.elementAt(i),
                    (AsserterIdentity) assertersNames.elementAt(
                        control.getAi().findAsserterIdentity(asserter, assertersNames)));
            }
            else {
                rect = new SenderDrawable(
                    actorColor, 
                    new Point(Width + bord + 1, bord + betweenInteraction + i * (betweenInteraction + actorHeight)), 
                    dim, 
                    (Element) sender.elementAt(i),
                    (AsserterIdentity) assertersNames.elementAt(assertersNames.size()-1));
            }
            
            addDrawable(rect);
            
            Iterator rel = domDeserialiser.iterator(se, Constants.PSTRUCT_NS, "relationshipPAssertion");
            
            int nbRelationShip = se.getElementsByTagNameNS(Constants.PSTRUCT_NS, "relationshipPAssertion").getLength();
            int RelationshipNb = 0;
            while(rel.hasNext()){
                Element relationship = (Element) rel.next();
                Iterator obj = domDeserialiser.iterator(relationship, Constants.PSTRUCT_NS, "objectId");

                int nbObjectId = relationship.getElementsByTagNameNS(Constants.PSTRUCT_NS, "objectId").getLength();
                if (first){ 
                    Color selectColor = IDrawable.selectColor;
                    Color color = selectColor;
                    while(color == selectColor || color == Color.WHITE){
                        float r = (float)Math.random();
                        float g = (float)Math.random();
                        float b = (float)Math.random();
                        color = new Color(r, g, b);
                    }
                    colorRelationship.add(color);
                }
                while(obj.hasNext()){
                    Element objectId = (Element) obj.next();
                    String s = NormalisingDOMWriter.writeToString(
                            domDeserialiser.single(objectId, Constants.PSTRUCT_NS, "interactionKey"));
                    if (receiver != null){
                        boolean find = false;
                        int j = 0;
                        while(!find && j < receiver.size()){
                            Element test = domDeserialiser.single((Element)receiver.elementAt(j), Constants.PSTRUCT_NS, "interactionKey");
                            if(s.equals(NormalisingDOMWriter.writeToString(test))){
                                find = true;
                            }
                            else j++;
                        }
                        if(find){
                            if(drawables.get(j) instanceof ReceiverDrawable){
                                int nbConnexion = ((ReceiverDrawable)drawables.get(j)).getNbConnexion() + 1;
                                ((ReceiverDrawable)drawables.get(j)).setNbConnexion(nbConnexion);
                                addDrawable(new RelationshipElement(
                                        (Color)(colorRelationship.elementAt(colorIndice)), rect, (IDrawable)drawables.get(j), relationship, nbRelationShip, RelationshipNb, objectId, nbConnexion));
                            } else {
                                System.err.println ("INTERNAL ERROR in AsserterView.fillinDrawables: Object ID drawable not a ReceiverDrawable");
                            }
                        }
                        else {
                            System.err.println ("No matching interaction found for object key: " + s);
                            System.err.println ("  in relationship with subject key: " + DOMWriter.writeToString (domDeserialiser.single((Element)sender.elementAt(i), Constants.PSTRUCT_NS, "interactionKey"), true));
                            System.err.println ("Relationship not shown");
                            // no find any interactionKey which match
                        }
                    }
                }
                RelationshipNb++;
                colorIndice++;
            }
        }
        
        first = false;
        
        /* End Test */
    
    }
    
    /**
     * go to a new asserter view
     * @param ai AsserterIdentity
     */
    public void goToAsserter(final AsserterIdentity ai){
    
        final JScrollPane asserter = new JScrollPane();

        Runnable go = new Runnable() {
            public void run() {
                Progression pgr = new Progression(jframe);
                try {
                    pgr.startProgress();
                    AsserterView av = new AsserterView(
                            control, ai, assertersNames, jframe, jtabbedpane, infobox, 
                            moveX, moveY,scaleX, scaleY, betweenInteraction, actorHeight, actorWidth, Height, Width, interactionsDataGrid);

                    int removeTab = -1;
                    removeTab = jtabbedpane.indexOfTab(ai.getShortName() + "View");
                    if(removeTab!=-1){
                        jtabbedpane.removeTabAt(removeTab);
                    }

                    asserter.setViewportView(av);
                    jtabbedpane.addTab(
                            ai.getShortName() + "View", 
                            null, 
                            asserter,
                            "This is one Asserter View");
                    jtabbedpane.setSelectedIndex(jtabbedpane.getTabCount()-1);
                    av.requestFocusInWindow();
                    
                    pgr.stopProgress();

                } catch (Exception ex) {
                    if(Controller.debug)ErreurDialog.afficher(jframe, "Unable to initialize Asserterview", 
                        ex.getMessage(),ex.getStackTrace());
                    else ErreurDialog.afficher(jframe, "Unable to initialize Asserterview", ex.getMessage());
                    pgr.stopProgress();
                }

                asserter.repaint();
            }
        };
        Thread t = new Thread(go);
        t.start();
    }
    
    
    /**
     * go to a new provenance view from an objectID
     * @param relationship relationship
     */
    public void goToProvenanceView(final RelationshipElement relationship){
    
        final JScrollPane provenanceview = new JScrollPane();

        Runnable go = new Runnable() {
            public void run() {
                Progression pgr = new Progression(jframe);
                try {
                    pgr.startProgress();
                    ProvenanceView pv = new ProvenanceView(
                            control, jframe, jtabbedpane, infobox, assertersNames,relationship.ObjectId, interactionsDataGrid);

                    int removeTab = -1;
                    removeTab = jtabbedpane.indexOfTab(PReViewUtilities.getLastPartURL(relationship.getObjectId()) + " - Provenance");
                    if(removeTab!=-1){
                        jtabbedpane.removeTabAt(removeTab);
                    }

                    provenanceview.setViewportView(pv);
                    jtabbedpane.addTab(
                            PReViewUtilities.getLastPartURL(relationship.getObjectId()) + " - Provenance", 
                            null, 
                            provenanceview,
                            "This is one Provenance View for "+relationship.getObjectId());
                    jtabbedpane.setSelectedIndex(jtabbedpane.getTabCount()-1);
                    pv.requestFocusInWindow();
                    
                    pgr.stopProgress();

                } catch (Exception ex) {
                    if(Controller.debug)ErreurDialog.afficher(jframe, "Unable to initialize Provenanceview", 
                        ex.getMessage(),ex.getStackTrace());
                    else ErreurDialog.afficher(jframe, "Unable to initialize Provenanceview", ex.getMessage());
                    pgr.stopProgress();
                }

                provenanceview.repaint();
            }
        };
        Thread t = new Thread(go);
        t.start();
    }

    /**
     * go to a new provenance view from an subjectID
     * @param relationship a relationship
     * @param interactionRecord XML element of the interaction record for the relationship
     */
    public void goToProvenanceView(final RelationshipElement relationship, final Element interactionRecord){
    
        final JScrollPane provenanceview = new JScrollPane();

        Runnable go = new Runnable() {
            public void run() {
                Progression pgr = new Progression(jframe);
                try {
                    pgr.startProgress();
                    ProvenanceView pv = new ProvenanceView(
                            control, jframe, jtabbedpane, infobox, assertersNames, interactionRecord, relationship.getElem(), interactionsDataGrid);

                    int removeTab = -1;
                    removeTab = jtabbedpane.indexOfTab(PReViewUtilities.getLastPartURL(relationship.getSubjectId()) + " - Provenance");
                    if(removeTab!=-1){
                        jtabbedpane.removeTabAt(removeTab);
                    }

                    provenanceview.setViewportView(pv);
                    jtabbedpane.addTab(
                            PReViewUtilities.getLastPartURL(relationship.getSubjectId()) + " - Provenance", 
                            null, 
                            provenanceview,
                            "This is one Provenance View for "+relationship.getSubjectId());
                    jtabbedpane.setSelectedIndex(jtabbedpane.getTabCount()-1);
                    pv.requestFocusInWindow();
                    
                    pgr.stopProgress();

                } catch (Exception ex) {
                    if(Controller.debug)ErreurDialog.afficher(jframe, "Unable to initialize Provenanceview", 
                        ex.getMessage(),ex.getStackTrace());
                    else ErreurDialog.afficher(jframe, "Unable to initialize Provenanceview", ex.getMessage());
                    pgr.stopProgress();
                }

                provenanceview.repaint();
            }
        };
        Thread t = new Thread(go);
        t.start();
    }

    /**
     * resize the actor view
     * @return return the size of the actor view
     */
    public Dimension getPreferredSize() {
        return new Dimension (prefWidth, prefHeight);
    }
    

}
