/*
 * MyMenu.java
 *
 * Created on 3 aout 2006, 15:05
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.pasoa.preview.asserterview;

import java.awt.Toolkit;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import javax.imageio.ImageIO;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import org.pasoa.common.Constants;
import org.pasoa.preview.AsserterIdentity;
import org.pasoa.preview.tableview.DrawInfoBox;
import org.pasoa.preview.tableview.TableView;
import org.pasoa.preview.utils.ErreurDialog;
import org.pasoa.preview.utils.Progression;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.simpledom.DOMWriter;
import org.w3c.dom.Element;
import com.lowagie.text.*;
import com.lowagie.text.pdf.*;

/**
 * Create a popup menu for the right click on the actor view
 * @author tibo
 */
public class MyMenu extends JPopupMenu {

    protected IDrawable drawable;
    protected AsserterView canvas;
    
    /**
     * Creates a new instance of MyMenu
     * @param c an actor view
     * @param e the mouse event
     */
    public MyMenu(AsserterView c, MouseEvent e) {
        this.canvas = c;
        
        this.add(buildZoomMenu());

        Point p = new Point();
        p.setLocation(
                    (e.getPoint().getX() - canvas.moveX)/canvas.scaleX,
                    (e.getPoint().getY() - canvas.moveY)/canvas.scaleY);
        List selectedDrawables = canvas.findDrawables(p);
        if (selectedDrawables.size() != 0) {
            drawable = (IDrawable) selectedDrawables.get(0);
            if(drawable instanceof AsserterDrawable) {
                this.add(buildInteractionMenu());
            }
            else {
                this.add(buildRelationshipMenu());
                this.add(buildSubjectMenu());
            }
        }
        
    }
    
    
    public JMenu buildSubjectMenu(){
        
        JMenu SubjectMenu = new JMenu("SubjectId");
        JMenuItem item = new JMenuItem(((RelationshipElement)drawable).getSubjectId());
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                canvas.goToProvenanceView((RelationshipElement)drawable, ((RelationshipElement)drawable).sender.getElem());
            }
        });
        SubjectMenu.add(item);
        return SubjectMenu;
    }
        
    public JMenu buildInteractionMenu(){
        
        final AsserterIdentity tmp = ((AsserterDrawable)drawable).getAi();
        JMenu InteractionMenu = new JMenu("GoTo");
        JMenuItem item = new JMenuItem(tmp.getShortName());
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                canvas.goToAsserter(tmp);
            }
        });
        InteractionMenu.add(item);
        
        DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);
        //Element interactionRecord = domDeserialiser.single(drawable.getElem(), Constants.PSTRUCT_NS, "interactionRecord");
        Element interactionKey = domDeserialiser.single(drawable.getElem(), Constants.PSTRUCT_NS, "interactionKey");
        Element interactionId = domDeserialiser.single(interactionKey, Constants.PSTRUCT_NS, "interactionId");
        
        final String id = interactionId.getTextContent();
        final AsserterIdentity sender;
        final AsserterIdentity receiver;
        if(drawable instanceof SenderDrawable){
            sender = canvas.currentAsserter;
            receiver = tmp;
        }
        else {
            receiver = canvas.currentAsserter;
            sender = tmp;
        }
            
        //String id ="";
        item = new JMenuItem("Interaction("+ id +")");
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                //System.out.println(DOMWriter.writeToString(drawable.getElem(), true));
                int tableTab = canvas.jtabbedpane.indexOfTab(TableView.TABLE_NAME);
                if(tableTab != -1){
                    DrawInfoBox dib = new DrawInfoBox(canvas.infobox, canvas.control, canvas.jframe, false, canvas.jtabbedpane);
                    dib.go(sender, receiver, id);
                
                    canvas.jtabbedpane.setSelectedIndex(tableTab);
                }    
            }
        });
        InteractionMenu.add(item);
        return InteractionMenu;
        
    }
    
    public JMenu buildRelationshipMenu(){
        JMenu RelationshipMenu = new JMenu("ObjectId");
        JMenuItem item = new JMenuItem(((RelationshipElement)drawable).getObjectId());
        item.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                canvas.goToProvenanceView((RelationshipElement)drawable);
            }
        });
        RelationshipMenu.add(item);
        return RelationshipMenu;
    }
    
    public JMenu buildZoomMenu(){
    
        JMenu ZoomMenu = new JMenu("Zoom"); 
        JMenuItem zoomIn = new JMenuItem("Zoom In");
        ZoomMenu.add(zoomIn);
        JMenuItem zoomOut = new JMenuItem("Zoom Out");
        ZoomMenu.add(zoomOut);
        zoomIn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                if(canvas.scaleX <= 100 && canvas.scaleY <= 100){
                    canvas.scaleX += 0.2;
                    canvas.scaleY += 0.2;
                }
                canvas.repaint();   
                canvas.revalidate();
            }
        });
        zoomOut.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                if(canvas.scaleX >= 0.4 && canvas.scaleY >= 0.4){
                    canvas.scaleX -= 0.2;
                    canvas.scaleY -= 0.2;
                }
                canvas.repaint();   
                canvas.revalidate();
            }
        });
        return ZoomMenu;
    
    }
    
}
