/*
 * RelationshipDrawable.java
 *
 * Created on 2 aout 2006, 15:42
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

/*
 * FormDrawable.java
 *
 * Created on 27 juillet 2006, 16:00
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.pasoa.preview.asserterview;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Label;
import java.awt.Paint;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.QuadCurve2D;
import java.awt.geom.Rectangle2D;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTable;
import org.pasoa.common.Constants;
import org.pasoa.preview.utils.PReViewUtilities;
import org.pasoa.simpledom.DOMDeserialiser;
import org.w3c.dom.Element;
import prefuse.util.ui.JCustomTooltip;

/**
 * Implementation of the form drawable to draw the relationship
 * @author tibo
 */
public class RelationshipElement implements IDrawable{
    
    protected IDrawable receiver;
    protected IDrawable sender;
    
    protected Color color;
    protected boolean isSelected;
    protected double x1;
    protected double y1;
    protected double x2;
    protected double y2;
    protected static int thin = 2;
    protected Shape shape;
    protected Element relationship;

    protected double controlX;
    protected double controlY;
    private boolean isDrawable;
    private String name;
    private int nbRelationShip;
    protected int RelationshipNb;
    private boolean firstTime;
    protected Element ObjectId;

    private int nbConnexionReceiver;
    
    /**
     * a new instance of RelationshipElement
     * @param color the line color
     * @param sender asserter identity of the sender
     * @param receiver asserter identity of the receiver
     * @param element XML element of the interaction record
     * @param nbRelationShip number of relationship for the sender
     * @param RelationshipNb position of this relationship on the sender box
     * @param Obj XML Element contains the objectID
     * @param nbConnexion number of connexion on the receiver
     */
    public RelationshipElement(
            Color color, IDrawable sender, IDrawable receiver, Element element, int nbRelationShip, int RelationshipNb, Element Obj, int nbConnexion) {
        this.color = color;
        this.isSelected = false;
        this.isDrawable = true;
        this.sender = sender;
        this.receiver = receiver;
        this.relationship = element;
        this.nbRelationShip = nbRelationShip;
        this.RelationshipNb = RelationshipNb;
        this.ObjectId = Obj;
        this.nbConnexionReceiver = nbConnexion;
        this.firstTime = true;
        
        Rectangle r1 = sender.getShape().getBounds();
        Rectangle r2 = receiver.getShape().getBounds();
        
        x1 = r1.x ;
        if(nbRelationShip == 1){
            y1 = r1.y + (r1.height)/2.0;
        }
        else {
            y1 = r1.y + (((r1.height-4.0)/(nbRelationShip - 1.0)) * (RelationshipNb))+2.0;
        }
        x2 = r2.x + r2.width;
        y2 = r2.y + r2.height/2.0;
        controlX = (x1 + x2) / 2.0;
        controlY = (y1 + y2) / 2.0;
        this.shape = new QuadCurve2D.Double( x1, y1, controlX, controlY, x2, y2);
        DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);
        Element obj = domDeserialiser.single(
                relationship, 
                Constants.PSTRUCT_NS, 
                "relation");
        this.name = PReViewUtilities.getRelationName(obj.getTextContent());
    }

    public boolean isSelected() {
        return this.isSelected;
    }
    
    public void setSelected(boolean tf) {
        this.isSelected = tf;
        
    }
    
    public void setColor(Color c){
        this.color = c;
        
    }
    
    public String getSubjectId(){
        
        DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);
        Element sub = domDeserialiser.single(
                this.relationship, 
                Constants.PSTRUCT_NS, 
                "subjectId");
        Element name = domDeserialiser.single(sub, Constants.PSTRUCT_NS, "parameterName");
        
        String tmp = name.getTextContent();
        return tmp;
    }
    
    public String getObjectId(){
        DOMDeserialiser domDeserialiser = new DOMDeserialiser(Constants.PSTRUCT_NS);

        Element name = domDeserialiser.single(this.ObjectId, Constants.PSTRUCT_NS, "parameterName");
        String tmp = name.getTextContent();
        return tmp;
    }
    
    public void draw (Graphics g){
        if(firstTime) {
            this.firstTime = false;
            //System.out.println(this.nbConnexionReceiver + "/" + ((ReceiverDrawable)this.receiver).getNbConnexion());
            int NbConnexion = ((ReceiverDrawable)this.receiver).getNbConnexion();
            if(NbConnexion > 1){ 
                Rectangle r = receiver.getShape().getBounds();
                y2 = r.y + (((r.height - 4.0)/(NbConnexion -1)) * (nbConnexionReceiver - 1.0) + 2.0);
                controlX = (x1 + x2) / 2.0;
                controlY = (y1 + y2) / 2.0;
                this.shape = new QuadCurve2D.Double( x1, y1, controlX, controlY, x2, y2);
            }
        }
        
        Graphics2D g2d = (Graphics2D) g;
        float [] style = {3f, 2.5f};
        g2d.setStroke(new BasicStroke(thin, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER, 1.0f, style, 0));
        Color c = g2d.getColor();
//        Paint p = g2d.getPaint();
        if(!this.isSelected()) {
            g2d.setColor(color);
//        g2d.setPaint(new GradientPaint(
//                new Point(x1, y1),
//                this.color, 
//                new Point(x2, y2), 
//                Color.LIGHT_GRAY));
            drawName(g2d);
        }
        else {
            
            Font f = g2d.getFont();
            g2d.setFont(new Font("TimeRoman", Font.BOLD, (int)(g2d.getFont().getSize()/3)));
            String tmpString = PReViewUtilities.getLastPartURL(getObjectId());
            g2d.drawString(
                    tmpString,
                    (int) (x2 - (g2d.getFontMetrics().stringWidth(tmpString) + 2)), 
                    (int) (y2 -1+ g2d.getFontMetrics().getHeight()/2));
            tmpString = PReViewUtilities.getLastPartURL(getSubjectId());
            g2d.drawString(
                    tmpString,
                    (int) (x1 + 2), 
                    (int) (y1 - 1 + g2d.getFontMetrics().getHeight()/2));

            g2d.setFont(f);
            drawName(g2d);
            g2d.setColor(selectColor);
            /* end */
        }
    
        g2d.draw(this.shape);
        g2d.setColor(c);
//        g2d.setPaint(p);
        
        //System.out.println(this.nbConnexionReceiver + "/" + ((ReceiverDrawable)this.receiver).getNbConnexion());
        
    }
    
    public void drawName(Graphics2D g2d){

        /* Name of Relationship */
        AffineTransform old = g2d.getTransform();
        AffineTransform at = new AffineTransform(); 

        Font f = g2d.getFont();
        g2d.setFont(new Font("TimeRoman", Font.PLAIN, (int)(g2d.getFont().getSize()/3)));
        double dividende = ((x1 - x2)==0.0)? 1.0 : (x1-x2) ;
        double diviseur = (y1 - y2);
        double total = diviseur/dividende;
        double angle = Math.atan(total);

        double transx = (controlX);
        double transy = (controlY);
        at.setToTranslation(transx,transy);
        g2d.transform(at);
        at.setToRotation(angle);
        g2d.transform(at);
        //g2d.scale(old.getScaleX(), old.getScaleY());
        g2d.drawString(
                name,
                -(g2d.getFontMetrics().stringWidth(name))/2,
                -2);
        g2d.setFont(f);
        g2d.setTransform(old);
    }
    
    public void setDrawable(boolean tf) {
        this.isDrawable = tf;
    }
    
    public boolean isDrawable() {
        return isDrawable;
    }


    public Shape getShape() {
        return this.shape;
    }

    public Element getElem() {
        return this.relationship;
    }
    
}
