/*
 * CollaborationEdgeRenderer.java
 *
 * Created on 8 septembre 2006, 18:05
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.pasoa.preview.provenanceview;

import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import prefuse.Constants;
import prefuse.render.EdgeRenderer;
import prefuse.util.GraphicsLib;
import prefuse.visual.EdgeItem;
import prefuse.visual.VisualItem;

/**
 * a renderer for the collaboration edge 
 * which is abble to display an edge between two time the same actor
 * @author tibo
 */
public class CollaborationEdgeRenderer extends EdgeRenderer{
    
    /**
     * Creates a new instance of CollaborationEdgeRenderer
     * @param i edge form (curv, line...)
     * @param j arrow form (forward, none...)
     */
    public CollaborationEdgeRenderer(int i, int j) {
        super(i,j);
    }

    
    
    /**
     * draw the edge shape
     * @param item the visual item
     * @return shape
     */
    protected Shape getRawShape(VisualItem item) {
        EdgeItem   edge = (EdgeItem)item;
        VisualItem item1 = edge.getSourceItem();
        VisualItem item2 = edge.getTargetItem();

        // create the edge shape
        Shape shape = null;

        if(item1.equals(item2)){
            int type = m_edgeType;

            //getAlignedPoint(m_tmpPoints[0], item1.getBounds(),
            //                Constants.CENTER, Constants.TOP);
            //getAlignedPoint(m_tmpPoints[1], item2.getBounds(),
            //                Constants.CENTER, Constants.TOP);
            m_curWidth = (float)(m_width * getLineWidth(item));

            double x1 = item1.getBounds().getX();
            double y1 = item1.getBounds().getY();
            double w = item1.getBounds().getWidth();
            double h = item1.getBounds().getHeight();
            double minus = (2 * m_curWidth + 20 );
            
                m_tmpPoints[0].setLocation(x1 + w/4, y1);
                m_tmpPoints[1].setLocation(x1 + 3*w/4, y1);

            // create the arrow head, if needed
            EdgeItem e = (EdgeItem)item;
            if ( e.isDirected() && m_edgeArrow != Constants.EDGE_ARROW_NONE ) {
                // get starting and ending edge endpoints
                boolean forward = (m_edgeArrow == Constants.EDGE_ARROW_FORWARD);
                
                Point2D start = null, end = null, tmp = null;
                start = m_tmpPoints[forward?0:1];
                int it = forward?0:1;
                tmp   = (Point2D) m_tmpPoints[it].clone();
                end   = m_tmpPoints[forward?1:0];
                
                start.setLocation(x1 + w , end.getY() - minus);
                // compute the intersection with the target bounding box
                VisualItem dest = forward ? e.getTargetItem() : e.getSourceItem();
                int i = GraphicsLib.intersectLineRectangle(start, end,
                        dest.getBounds(), m_isctPoints);
                if ( i > 0 ) end = m_isctPoints[0];

                // create the arrow head shape
                AffineTransform at = getArrowTrans(start, end, m_curWidth);
                m_curArrow = at.createTransformedShape(m_arrowHead);

                // update the endpoints for the edge shape
                // need to bias this by arrow head size
                Point2D lineEnd = m_tmpPoints[forward?1:0]; 
                lineEnd.setLocation(0, -m_arrowHeight);
                at.transform(lineEnd, lineEnd);
                m_tmpPoints[it].setLocation(tmp);
            } else {
                m_curArrow = null;
            }

            double n1x = m_tmpPoints[0].getX();
            double n1y = m_tmpPoints[0].getY();
            double n2x = m_tmpPoints[1].getX();
            double n2y = m_tmpPoints[1].getY();
            switch ( type ) {
                case Constants.EDGE_TYPE_LINE:          
                    m_line.setLine(n1x, n1y, n2x, n2y);
                    shape = m_line;
                    break;
                case Constants.EDGE_TYPE_CURVE:
                    //getCurveControlPoints(edge, m_ctrlPoints,n1x,n1y,n2x,n2y);
                    m_cubic.setCurve(n1x, n1y,
                                    x1 , n1y - minus,
                                    x1 + w , n2y - minus,
                                    n2x, n2y);
                    shape = m_cubic;
                    break;
                default:
                    throw new IllegalStateException("Unknown edge type");
            }
        }
        else {
            
            int type = m_edgeType;

            getAlignedPoint(m_tmpPoints[0], item1.getBounds(),
                            m_xAlign1, m_yAlign1);
            getAlignedPoint(m_tmpPoints[1], item2.getBounds(),
                            m_xAlign2, m_yAlign2);
            m_curWidth = (float)(m_width * getLineWidth(item));

            // create the arrow head, if needed
            EdgeItem e = (EdgeItem)item;
            if ( e.isDirected() && m_edgeArrow != Constants.EDGE_ARROW_NONE ) {
                // get starting and ending edge endpoints
                boolean forward = (m_edgeArrow == Constants.EDGE_ARROW_FORWARD);
                Point2D start = null, end = null;
                start = m_tmpPoints[forward?0:1];
                end   = m_tmpPoints[forward?1:0];

                // compute the intersection with the target bounding box
                VisualItem dest = forward ? e.getTargetItem() : e.getSourceItem();
                int i = GraphicsLib.intersectLineRectangle(start, end,
                        dest.getBounds(), m_isctPoints);
                if ( i > 0 ) end = m_isctPoints[0];

                // create the arrow head shape
                AffineTransform at = getArrowTrans(start, end, m_curWidth);
                m_curArrow = at.createTransformedShape(m_arrowHead);

                // update the endpoints for the edge shape
                // need to bias this by arrow head size
                Point2D lineEnd = m_tmpPoints[forward?1:0]; 
                lineEnd.setLocation(0, -m_arrowHeight);
                at.transform(lineEnd, lineEnd);
            } else {
                m_curArrow = null;
            }

            
            double n1x = m_tmpPoints[0].getX();
            double n1y = m_tmpPoints[0].getY();
            double n2x = m_tmpPoints[1].getX();
            double n2y = m_tmpPoints[1].getY();
            switch ( type ) {
                case Constants.EDGE_TYPE_LINE:          
                    m_line.setLine(n1x, n1y, n2x, n2y);
                    shape = m_line;
                    break;
                case Constants.EDGE_TYPE_CURVE:
                    getCurveControlPoints(edge, m_ctrlPoints,n1x,n1y,n2x,n2y);
                    m_cubic.setCurve(n1x, n1y,
                                    m_ctrlPoints[0].getX(), m_ctrlPoints[0].getY(),
                                    m_ctrlPoints[1].getX(), m_ctrlPoints[1].getY(),
                                    n2x, n2y);
                    shape = m_cubic;
                    break;
                default:
                    throw new IllegalStateException("Unknown edge type");
            }
        }
        // return the edge shape
        return shape;
    }

    
    /**
     * Determines the control points to use for cubic (Bezier) curve edges. 
     * Override this method to provide custom curve specifications.
     * To reduce object initialization, the entries of the Point2D array are
     * already initialized, so use the <tt>Point2D.setLocation()</tt> method rather than
     * <tt>new Point2D.Double()</tt> to more efficiently set custom control points.
     * @param eitem the EdgeItem we are determining the control points for
     * @param cp array of Point2D's (length >= 2) in which to return the control points
     * @param x1 the x co-ordinate of the first node this edge connects to
     * @param y1 the y co-ordinate of the first node this edge connects to
     * @param x2 the x co-ordinate of the second node this edge connects to
     * @param y2 the y co-ordinate of the second node this edge connects to
     */
    protected void getCurveControlPoints(EdgeItem eitem, Point2D[] cp, 
                    double x1, double y1, double x2, double y2) 
    {
        //System.out.println("Edge to the same actor");
        double dx = x2-x1, dy = y2-y1;      
        cp[0].setLocation(x1+2*dx/3,y1);
        cp[1].setLocation(x2-dx/8,y2-dy/8);
        
    }

    /**
     * get the edge width
     * @param visualItem the edge
     * @return the width
     */
    protected double getLineWidth(VisualItem visualItem) {
        EdgeItem   edge = (EdgeItem)visualItem;
        Double d = (Double) edge.get(CollaborationView.LINE);
        return d;
    }

}
