/*
 * CollaborationView.java
 *
 * Created on 7 septembre 2006, 17:58
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.pasoa.preview.provenanceview;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.SwingConstants;
import org.pasoa.preview.AsserterIdentity;
import org.pasoa.preview.Controller;
import org.pasoa.preview.View;
import org.pasoa.preview.history.HistoryOpenCollaboration;
import org.pasoa.preview.utils.DocumentWrapper;
import prefuse.Visualization;
import prefuse.controls.ControlAdapter;
import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.io.GraphMLReader;
import prefuse.data.query.SearchQueryBinding;
import prefuse.data.search.SearchTupleSet;
import prefuse.util.FontLib;
import prefuse.util.GraphLib;
import prefuse.util.ui.JFastLabel;
import prefuse.util.ui.JSearchPanel;
import prefuse.visual.VisualItem;

/**
 * The collaboration view
 * @author tibo
 */
public class CollaborationView extends JPanel{

    protected Vector         _asserterNames;
    protected Controller     _control;
    protected JFrame         _jframe;
    protected JScrollPane    _infobox;
    protected JTabbedPane    _jtabbedpane;
    protected Set            _currentAsserters;
    protected Vector<Vector> _interactionsDataGrid;

    private static Color    BACKGROUND = Color.WHITE;
    private static Color    FOREGROUND = Color.BLACK;

    protected static String LABEL = "label";
    protected static String INDEX = "index";
    protected static String LINE = "line";
    protected static String ASSERTER_IDENTITY = "asserterIdentity";
    private static String LONGNAME = "longName";
    private static final Schema NODE_SCHEMA = new Schema();
    private static final Schema EDGE_SCHEMA = new Schema();
    private static final String tree = "tree";
    private static final String treeNodes = "tree.nodes";
    private static final String treeEdges = "tree.edges";
    private static final String linear = "linear";


    static {
        EDGE_SCHEMA.addColumn(LINE, Double.class, 1.0);
    }
    static {
        NODE_SCHEMA.addColumn(LABEL, String.class, "");
        NODE_SCHEMA.addColumn(LONGNAME, String.class, "");
        NODE_SCHEMA.addColumn(ASSERTER_IDENTITY, AsserterIdentity.class, null);
        NODE_SCHEMA.addColumn(INDEX, Integer.class, 0);
    }
    
    protected RadialGraphView rview = null;
    private JFastLabel title = new JFastLabel("                 ");
    
    /**/

    /**/
    /**
     * Creates a new instance of CollaborationView
     * @param control the controller
     * @param jframe main jframe
     * @param jtabbedpane jtabbedpane where this view is added
     * @param infobox a ScrollPane to be abble to open a new infobox
     * @param asserters a vector of all the asserter identity
     * @param currentsAsserter a list of the currents asserters (from the provenance view)
     * @param interactionsDataGrid Vector<Vector<int>> of the number of interaction record between each asserter Identity
     * @param rootName the root name of the provenance view
     * @param longName the long name of the root of the provenance view (needed for the history)
     * @param is_history to know if the constructor is called from the history menu
     * @throws java.lang.Exception exception from Model interface
     */
    public CollaborationView(
            Controller control,JFrame jframe,JTabbedPane jtabbedpane,
            JScrollPane infobox, Vector asserters, Set currentsAsserter, 
            Vector<Vector> interactionsDataGrid, String rootName, String longName, boolean is_history) throws Exception {
        
        this._currentAsserters = currentsAsserter;
        this._asserterNames = asserters;
        this._control = control;
        this._jframe = jframe;
        this._jtabbedpane = jtabbedpane;
        this._infobox = infobox;
	this._interactionsDataGrid = interactionsDataGrid;
        
        setOpaque(true);
        setBackground(BACKGROUND);
        
        add(buildRadialGraphView(buildGraph()));
        
        if(!is_history)
            ((View)_jframe).getHistoryDialog().addElementHistory(new HistoryOpenCollaboration(control.address, _currentAsserters ,rootName, longName));
       
    }
    
    
    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2d = (Graphics2D) g;
        g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        
        if(rview != null) {
            Dimension d = new Dimension((int)(_jtabbedpane.getSize().getWidth() - 70) , (int)(_jtabbedpane.getSize().getHeight() - 90));
            title.setSize(new Dimension(d.width, 18));
            title.repaint();
            rview.setSize(d);
            rview.repaint();
        }
        
    }

    public Graph buildGraph() throws Exception{
        
        Graph g = new Graph(true);
        g.getNodeTable().addColumns(NODE_SCHEMA);
        g.getEdgeTable().addColumns(EDGE_SCHEMA);
        
        Node nodes[] = new Node[_currentAsserters.size()];
        int i1 = 0;
        for(Iterator iter = _currentAsserters.iterator(); iter.hasNext(); i1++){
            DocumentWrapper dw = (DocumentWrapper) iter.next();
            int n = _control.getAi().findAsserterIdentity(dw.get_elem(), _asserterNames);
            //System.out.println(((AsserterIdentity)_asserterNames.elementAt(n)).getLongName());
            if(n > -1){
                AsserterIdentity ai = ((AsserterIdentity)_asserterNames.elementAt(n));
                nodes[i1] = g.addNode();
                nodes[i1].setString(LABEL, ai.getShortName());
                nodes[i1].setString(LONGNAME, ai.getLongName());
                nodes[i1].set(ASSERTER_IDENTITY, ai);
                nodes[i1].set(INDEX, n);
            }
            else {
                    throw new Exception("Asserter : "+ dw.get_elemString() +" didn't find!");
            }
        }
        
        for ( int i = 0; i < nodes.length; ++i ) {
            for ( int j = 0; j < nodes.length; ++j ){
                int nb = (Integer) nodes[i].get(INDEX);
                int nb2 = (Integer) nodes[j].get(INDEX);
                int nbInteractions = (Integer) (_interactionsDataGrid.elementAt(nb).elementAt(nb2));
                //System.out.println(i +","+j+","+nb+","+nb2+","+nbInteractions);
                //int tmp = 0;
                //while(tmp < nbInteractions){
                if(nbInteractions > 0) {
                    Edge e = g.addEdge(nodes[i], nodes[j]);
                    // width = nbInteractions * 10 /maxNbInteractions (between 0 and 10);
                    e.set(LINE, (double) nbInteractions);
                }
            }
        }
        return g;
    }

    public JPanel buildRadialGraphView(Graph g){
        rview = new RadialGraphView(g, this);
        Visualization vis = rview.getVisualization();
        
        // create a search panel for the tree map
        SearchQueryBinding sq = new SearchQueryBinding(
             (Table)vis.getGroup(treeNodes), LABEL,
             (SearchTupleSet)vis.getGroup(Visualization.SEARCH_ITEMS));
        JSearchPanel search = sq.createSearchPanel();
        search.setShowResultCount(true);
        search.setBorder(BorderFactory.createEmptyBorder(5,5,4,0));
        search.setFont(FontLib.getFont("Tahoma", Font.PLAIN, 11));
        
        title.setVerticalAlignment(SwingConstants.TOP);
        title.setBorder(BorderFactory.createEmptyBorder(3,0,0,0));
        title.setFont(FontLib.getFont("Tahoma", Font.PLAIN, 12));
        title.setPreferredSize(new Dimension((int)title.getFontMetrics(title.getFont()).stringWidth("                 "), 18));
        title.setBackground(BACKGROUND);
        title.setForeground(FOREGROUND);
        
        
        rview.addControlListener(new ControlAdapter() {
            public void itemEntered(VisualItem item, MouseEvent e) {
                if ( item.canGetString(LONGNAME) )
                    title.setText(item.getString(LONGNAME));
            }
            public void itemExited(VisualItem item, MouseEvent e) {
                title.setText(null);
            }
        });
        
        Box box = new Box(BoxLayout.X_AXIS);
        box.add(Box.createHorizontalGlue());
        box.add(search);
        //box.add(Box.createHorizontalStrut(3));
        box.setBackground(BACKGROUND);
        
        Box box2 = new Box(BoxLayout.X_AXIS);
        box2.add(title);
        box2.setBackground(BACKGROUND);
        
        JPanel panel = new JPanel(new BorderLayout());
        panel.setBackground(BACKGROUND);
        panel.setForeground(FOREGROUND);
        panel.add(rview, BorderLayout.CENTER);
        panel.add(box, BorderLayout.SOUTH);
        panel.add(box2, BorderLayout.NORTH);
        return panel;
        
    }
    
}
