
package org.pasoa.preview.provenanceview;

import java.awt.Cursor;
import java.awt.event.MouseEvent;
import javax.swing.SwingUtilities;

import prefuse.Display;
import prefuse.controls.ControlAdapter;
import prefuse.util.ui.UILib;
import prefuse.visual.VisualItem;


/**
 * copy the prefuse pan control to add the right click for mac user (ctrl + left click)
 * @author tibo
 */
public class MyPanControl extends ControlAdapter {

    private boolean m_panOverItem;
    private int m_xDown, m_yDown;
    private int m_button;
    
    /**
     * Create a new PanControl.
     */
    public MyPanControl() {
        this(LEFT_MOUSE_BUTTON, false);
    }
    
    /**
     * Create a new PanControl.
     * @param panOverItem if true, the panning control will work even while
     * the mouse is over a visual item.
     */
    public MyPanControl(boolean panOverItem) {
        this(LEFT_MOUSE_BUTTON, panOverItem);
    }
    
    /**
     * Create a new PanControl.
     * @param mouseButton the mouse button that should initiate a pan.
     */
    public MyPanControl(int mouseButton) {
        this(mouseButton, false);
    }
    
    /**
     * Create a new PanControl
     * @param mouseButton the mouse button that should initiate a pan. 
     * @param panOverItem if true, the panning control will work even while
     * the mouse is over a visual item.
     */
    public MyPanControl(int mouseButton, boolean panOverItem) {
        m_button = mouseButton;
        m_panOverItem = panOverItem;
    }
    
    // ------------------------------------------------------------------------
    
    public void mousePressed(MouseEvent e) {
        if ( SwingUtilities.isLeftMouseButton(e) && !e.isControlDown() && !e.isShiftDown()) {
            e.getComponent().setCursor(
                Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
            m_xDown = e.getX();
            m_yDown = e.getY();
        }
    }
    
    
    public void mouseDragged(MouseEvent e) {
        if ( SwingUtilities.isLeftMouseButton(e) && !e.isControlDown() && !e.isShiftDown() ) {
            Display display = (Display)e.getComponent();
            int x = e.getX(),   y = e.getY();
            int dx = x-m_xDown, dy = y-m_yDown;
            display.pan(dx,dy);
            m_xDown = x;
            m_yDown = y;
            display.repaint();
        }
    }
    
    public void mouseReleased(MouseEvent e) {
        if ( SwingUtilities.isLeftMouseButton(e) && !e.isControlDown() && !e.isShiftDown() ) {
            e.getComponent().setCursor(Cursor.getDefaultCursor());
            m_xDown = -1;
            m_yDown = -1;
        }
    }
    
    public void itemPressed(VisualItem item, MouseEvent e) {
        if ( m_panOverItem )
            mousePressed(e);
    }

    public void itemDragged(VisualItem item, MouseEvent e) {
        if ( m_panOverItem )
            mouseDragged(e);
    }
    
    public void itemReleased(VisualItem item, MouseEvent e) {
        if ( m_panOverItem )
            mouseReleased(e);
    }
    
} // end of class PanControl
