package org.pasoa.preview.provenanceview;

import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import javax.swing.SwingUtilities;

import prefuse.Display;
import prefuse.controls.AbstractZoomControl;
import prefuse.util.ui.UILib;
import prefuse.visual.VisualItem;


/**
 * copy the prefuse zoom control to add the right click for mac user (ctrl + left click)
 * @author tibo
 */
public class MyZoomControl extends AbstractZoomControl {
    
    private int yLast;
    private Point2D down = new Point2D.Float();
    
    /**
     * Create a new zoom control.
     */
    public MyZoomControl() {
        // do nothing
    }
    
    public void mousePressed(MouseEvent e) {
        if ( (e.isControlDown() && SwingUtilities.isLeftMouseButton(e)) || SwingUtilities.isRightMouseButton(e)) {
            Display display = (Display)e.getComponent();
            if (display.isTranformInProgress()) {
                yLast = -1;
                System.err.println("can't move");
                return;
            }
            display.setCursor(
                Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
            display.getAbsoluteCoordinate(e.getPoint(), down);
            yLast = e.getY();
        }
    }
    
    public void mouseDragged(MouseEvent e) {
        if ( (e.isControlDown() && SwingUtilities.isLeftMouseButton(e)) || SwingUtilities.isRightMouseButton(e) ) {
            Display display = (Display)e.getComponent();
            if (display.isTranformInProgress() || yLast == -1) {
                yLast = -1;
                return;
            }
            
            int y = e.getY();
            int dy = y-yLast;
            double zoom = 1 + ((double)dy) / 100;
            
            int status = zoom(display, down, zoom, true);
            int cursor = Cursor.N_RESIZE_CURSOR;
            if ( status == NO_ZOOM )
                cursor = Cursor.WAIT_CURSOR;
            display.setCursor(Cursor.getPredefinedCursor(cursor));
            
            yLast = y;
        }
    }

    public void mouseReleased(MouseEvent e) {
        if ( (e.isControlDown() && SwingUtilities.isLeftMouseButton(e)) || SwingUtilities.isRightMouseButton(e) ) {
            e.getComponent().setCursor(Cursor.getDefaultCursor());
        }
    }
    
    public void itemPressed(VisualItem item, MouseEvent e) {
        if ( m_zoomOverItem )
            mousePressed(e);
    }

    public void itemDragged(VisualItem item, MouseEvent e) {
        if ( m_zoomOverItem )
            mouseDragged(e);
    }
    
    public void itemReleased(VisualItem item, MouseEvent e) {
        if ( m_zoomOverItem )
            mouseReleased(e);
    }
    
} // end of class ZoomControl
