package org.pasoa.pstructure;

import org.w3c.dom.Element;

/**
 * This class models an ActorStatePAssertion 
 * 
 * @author pgroth, sm
 */
public class ActorStatePAssertion extends PAssertion {
    private Element state;
    private String transform;
    
    /**
     * Construct an actor state PAssertion
     * 
     * @param localPAssertionID the id of the p-assertion
     * @param stateBody a DOM element that represents part of the actor's state
     */
    public ActorStatePAssertion (String localPAssertionID, Element stateBody) {
        super (localPAssertionID);
        state = stateBody;
    }
    
    /**
     * Construct an Actor State PAssertion
     * 
     * @param localPAssertionID the id of the p-assertion
     * @param stateBody a DOM element that represents part of the actor's state
     * @param documentationStyle the documentation style of the stateBody.
     *            this should be a URL
     */
    public ActorStatePAssertion (String localPAssertionID, String documentationStyle, Element stateBody) {
        super (localPAssertionID);
        state = stateBody;
        transform = documentationStyle;
    }
    
    /**
     * An actor state p-assertion contains some element that represents
     * part of an actor's state.
     * 
     * @return an Element that represents part of the actor's state
     */
    public Element getContent () {
        return state;
    }

    /**
     * 
     * The documentation Style for actor's state
     * 
     * @return A String representing the URL for the documentation Style.
     */
    public String getDocumentationStyle()
    {
        return transform;
    }


}
