package org.pasoa.pstructure;

import java.io.Serializable;

/**
 * Represents a global key that accesses a p-assertion.
 *
 * @author pgroth, sm
 */
public class GlobalPAssertionKey implements Serializable {
    private InteractionKey _interactionKey;
    private String         _viewKind;
    private String         _localPAssertionID;
    
    /**
     * Construct a GlobalPAssertionKey, which points to a particular p-assertion
     * uniquely
     *
     * @param interactionRecordID the interaction record where the p-assertion
     *            resides
     * @param viewName the view in the interaction record where the p-assertion
     *            resides
     * @param localPAssertionID the local id of the p-assertion which
     *            distinguishes it from all other p-assertions in that view
     */
    public GlobalPAssertionKey (InteractionKey interactionRecordID,
            String viewName, String localPAssertionID) {
        
        _interactionKey = interactionRecordID;
        _viewKind = viewName;
        _localPAssertionID = localPAssertionID;
    }
    
    /**
     * Checks whether one GlobalPAssertionKey is equal to another. Links are
     * ignored during the comparison.
     */
    public boolean equals (Object other) {
        return other instanceof GlobalPAssertionKey
                && ((GlobalPAssertionKey) other)._interactionKey.equals (_interactionKey)
                && ((GlobalPAssertionKey) other)._viewKind.equals (_viewKind)
                && ((GlobalPAssertionKey) other)._localPAssertionID.equals (_localPAssertionID);
    }
    
    public String getLocalPAssertionID () {
        return _localPAssertionID;
    }
    
    public InteractionKey getInteractionKey () {
        return _interactionKey;
    }
    
    public String getViewKind () {
        return _viewKind;
    }
    
    public int hashCode () {
        return _interactionKey.hashCode () + _viewKind.hashCode () + _localPAssertionID.hashCode ();
    }
    
    public String toString () {
        return "Interaction Key: [" + _interactionKey + "], View Kind: " + _viewKind + ", LPAID: " + _localPAssertionID;
    }
}
