package org.pasoa.pstructure;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;
import org.w3c.dom.Element;

/**
 * Represents a View of an interaction by an actor.
 *
 * @author pgroth
 *
 */
public class View implements Serializable {
    private List    _interactionPAssertions;
    private List    _actorStates;
    private List    _relationshipPAssertions;
    private List    _exposedInteractionMetadata;
    private int     _numExpectedMsgs;
    private Element _asserter;
    
    /**
     * Construct a View
     *
     * @param interactionPAssertions a list of interaction p-assertions
     * @param actorStatePAssertions a list of actor state p-assertions
     * @param relationships a list relationship p-assertions
     * @param numberOfExpectedPAssertions the number of expected p-assertions
     *            for this view
     * @param asserter the asserter of the assertions for the view
     * @param viewLink a reference to the provenance store that contains the
     *            opposite view of the interaction (optional)
     */
    public View (List interactionPAssertions, List actorStatePAssertions,
            List relationships, List exposedInteractionMetadata, int numberOfExpectedPAssertions,
            Element asserter) {
        _interactionPAssertions = interactionPAssertions;
        _actorStates = actorStatePAssertions;
        _relationshipPAssertions = relationships;
        _exposedInteractionMetadata = exposedInteractionMetadata;
        _numExpectedMsgs = numberOfExpectedPAssertions;
        _asserter = asserter;
    }
    
    /**
     * SHOULD BE REMOVED AFTER CHANGE TO 025 SCHEMA
     */
    public View (List interactionPAssertions, List actorStatePAssertions,
            List relationships, int numberOfExpectedPAssertions,
            Element asserter, Link viewLink) {
        this (interactionPAssertions, actorStatePAssertions, relationships,
                new LinkedList (), numberOfExpectedPAssertions, asserter);
    }
    
    /**
     * SHOULD BE REMOVED AFTER CHANGE TO 025 SCHEMA
     */
    public View (List interactionPAssertions, List actorStatePAssertions,
            List relationships, int numberOfExpectedMsgs, Element asserter) {
        this (interactionPAssertions, actorStatePAssertions, relationships, numberOfExpectedMsgs, asserter, null);
    }
    
    public List getActorStatePAssertions () {
        return _actorStates;
    }
    
    public List getExposedInteractionMetadata () {
        return _exposedInteractionMetadata;
    }
    
    public List getInteractionPAssertions () {
        return _interactionPAssertions;
    }
    
    public List getRelationshipPAssertions () {
        return _relationshipPAssertions;
    }
    
    /**
     * If the method returns -1, the number of expected assertions has not yet been
     * set.
     *
     * @return
     */
    public int getNumberOfExpectedPAssertions () {
        return _numExpectedMsgs;
    }
    
    public Element getAsserter () {
        return _asserter;
    }
    
    public void setNumberOfExpectedPAssertions (int total) {
        _numExpectedMsgs = total;
    }
}
