package org.pasoa.pstructure.dom;

import java.util.NoSuchElementException;
import javax.xml.namespace.QName;
import org.pasoa.common.Constants;
import org.pasoa.dommap.ContextItem;
import org.pasoa.dommap.UnderlyingModelAccessor;
import org.pasoa.dommap.UnderlyingModelException;
import org.pasoa.preserv.storage.Retrieval;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.ExposedInteractionMetadata;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.pstructure.RelationshipPAssertion;
import org.pasoa.pstructure.View;
import org.w3c.dom.Document;

public class PStructureJavaModel extends UnderlyingModelAccessor implements PStructureMapConstants {
    private Retrieval _basic;
    
    private static final int[] _requiredContext =
            new int[] {INTERACTIONRECORD, SENDER, RECEIVER,
            SENDER_RELATIONSHIPPASSERTION, RECEIVER_RELATIONSHIPPASSERTION,
            SENDER_RELATIONSHIPPASSERTION_OBJECTID, RECEIVER_RELATIONSHIPPASSERTION_OBJECTID,
            SENDER_EXPOSEDINTERACTIONMETADATA, RECEIVER_EXPOSEDINTERACTIONMETADATA};
    private static final int[] _views  = new int[] {SENDER, RECEIVER};
    private static final int[] _rpas   = new int[] {SENDER_RELATIONSHIPPASSERTION, RECEIVER_RELATIONSHIPPASSERTION};
    private static final int[] _objids = new int[] {SENDER_RELATIONSHIPPASSERTION_OBJECTID, RECEIVER_RELATIONSHIPPASSERTION_OBJECTID};
    private static final int[] _expos  = new int[] {SENDER_EXPOSEDINTERACTIONMETADATA, RECEIVER_EXPOSEDINTERACTIONMETADATA};
    
    private PStructureJavaModel (Retrieval basic) {
        super (_requiredContext);
        _basic = basic;
    }
    
    public Object getContent (int type, int index, ContextItem[] context) throws UnderlyingModelException {
        ObjectID objectID = (ObjectID) getAvailableContextItemData (context, _objids);
        
        if (type == SENDER_RELATIONSHIPPASSERTION_OBJECTID ||
                type == RECEIVER_RELATIONSHIPPASSERTION_OBJECTID) {
            return getPStructureDocument (_basic, OBJECTLINK, context).getDocumentElement ();
        } else {
            return getRepresentation (type, index, context);
        }
    }
    
    public int getContentSize (int type, ContextItem[] context) throws UnderlyingModelException {
        ObjectID objectID = (ObjectID) getAvailableContextItemData (context, _objids);
        
        switch (type) {
            case INTERACTIONRECORD: case SENDER: case RECEIVER:
            case OBJECTLINK_PROVENANCESTOREREF: case VIEWLINK_PROVENANCESTOREREF: 
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY:
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY:
                return 0;
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID:
                if (objectID.getLink () != null) {
                    return 1;
                } else {
                    return 0;
                }
        }
        return 1;
    }

    public static Document getPStructureDocument (Retrieval basic) throws UnderlyingModelException {
        return getPStructureDocument (basic, PSTRUCT, null);
    }

    public static Document getPStructureDocument (Retrieval basic, int rootType, ContextItem[] initialContext) throws UnderlyingModelException {
        return new PStructureMapDocument (new PStructureJavaModel (basic), rootType, initialContext);
    }

    public Object getRepresentation (int type, int index, ContextItem[] context) throws UnderlyingModelException {
        // Some pieces of context that may be present
        InteractionRecord          record   = (InteractionRecord) getContextItemData (context, INTERACTIONRECORD);
        View                       view     = (View) getAvailableContextItemData (context, _views);
        ObjectID                   objectID = (ObjectID) getAvailableContextItemData (context, _objids);
        RelationshipPAssertion     rpa      = (RelationshipPAssertion) getAvailableContextItemData (context, _rpas);
        ExposedInteractionMetadata exposed  = (ExposedInteractionMetadata) getAvailableContextItemData (context, _expos);

        switch (type) {
            case INTERACTIONRECORD:
                try {
                    return _basic.getInteractionRecord (index);
                } catch (Exception problem) {
                    throw new UnderlyingModelException ("Failed to get interaction record", problem);
                }
            case INTERACTIONRECORD_INTERACTIONKEY:
                return record.getInteractionKey ();
            case INTERACTIONRECORD_INTERACTIONKEY_MESSAGESOURCE:
                return record.getInteractionKey ().getMessageSource ();
            case INTERACTIONRECORD_INTERACTIONKEY_MESSAGESINK:
                return record.getInteractionKey ().getMessageSink ();
            case INTERACTIONRECORD_INTERACTIONKEY_INTERACTIONID:
                return record.getInteractionKey ().getInteractionID ();
            case SENDER:
                return record.getSenderView ();
            case RECEIVER:
                return record.getReceiverView ();
            case SENDER_ASSERTER:
            case RECEIVER_ASSERTER:
                return view.getAsserter ();
            case SENDER_INTERACTIONPASSERTION:
            case RECEIVER_INTERACTIONPASSERTION:
                return ((InteractionPAssertion) view.getInteractionPAssertions ().get (index)).getContent ();
            case SENDER_ACTORSTATEPASSERTION:
            case RECEIVER_ACTORSTATEPASSERTION:
                return ((ActorStatePAssertion) view.getActorStatePAssertions ().get (index)).getContent ();
            case SENDER_RELATIONSHIPPASSERTION:
            case RECEIVER_RELATIONSHIPPASSERTION:
                return view.getRelationshipPAssertions ().get (index);
            case SENDER_RELATIONSHIPPASSERTION_LOCALPASSERTIONID:
            case RECEIVER_RELATIONSHIPPASSERTION_LOCALPASSERTIONID:
                return rpa.getLocalPAssertionID ();
            case SENDER_RELATIONSHIPPASSERTION_SUBJECTID:
            case RECEIVER_RELATIONSHIPPASSERTION_SUBJECTID:
                return rpa.getSubject ();
            case SENDER_RELATIONSHIPPASSERTION_RELATION:
            case RECEIVER_RELATIONSHIPPASSERTION_RELATION:
                return rpa.getRelation ();
            case SENDER_RELATIONSHIPPASSERTION_SUBJECTID_LOCALPASSERTIONID:
            case RECEIVER_RELATIONSHIPPASSERTION_SUBJECTID_LOCALPASSERTIONID:
                return rpa.getSubject ().getLocalPAssertionID ();
            case SENDER_RELATIONSHIPPASSERTION_SUBJECTID_DATAACCESSOR:
            case RECEIVER_RELATIONSHIPPASSERTION_SUBJECTID_DATAACCESSOR:
                return rpa.getSubject ().getDataAccessor ();
            case SENDER_RELATIONSHIPPASSERTION_SUBJECTID_PARAMETERNAME:
            case RECEIVER_RELATIONSHIPPASSERTION_SUBJECTID_PARAMETERNAME:
                return rpa.getSubject ().getParameterName ();
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID:
                return rpa.getObjectIDs ().get (index);
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_LOCALPASSERTIONID:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_LOCALPASSERTIONID:
                return objectID.getGlobalPAssertionKey ().getLocalPAssertionID ();
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY:
                return objectID.getGlobalPAssertionKey ().getInteractionKey ();
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY_MESSAGESOURCE:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY_MESSAGESOURCE:
                return objectID.getGlobalPAssertionKey ().getInteractionKey ().getMessageSource ();
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY_MESSAGESINK:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY_MESSAGESINK:
                return objectID.getGlobalPAssertionKey ().getInteractionKey ().getMessageSink ();
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY_INTERACTIONID:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY_INTERACTIONID:
                return objectID.getGlobalPAssertionKey ().getInteractionKey ().getInteractionID ();
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_VIEWKIND:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_VIEWKIND:
                return new QName (Constants.PSTRUCT_NS, objectID.getGlobalPAssertionKey ().getViewKind ());
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_DATAACCESSOR:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_DATAACCESSOR:
                return objectID.getDataAccessor ();
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_PARAMETERNAME:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_PARAMETERNAME:
                return objectID.getParameterName ();
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_VIEWKIND_DESCRIPTION:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_VIEWKIND_DESCRIPTION:
                return objectID.getGlobalPAssertionKey ().getViewKind ();
            case SENDER_EXPOSEDINTERACTIONMETADATA:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA:
                return view.getExposedInteractionMetadata ().get (index);
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY:
                return exposed.getGlobalPAssertionKey ();
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY:
                return exposed.getGlobalPAssertionKey ().getInteractionKey ();
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY_MESSAGESOURCE:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY_MESSAGESOURCE:
                return exposed.getGlobalPAssertionKey ().getInteractionKey ().getMessageSource ();
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY_MESSAGESINK:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY_MESSAGESINK:
                return exposed.getGlobalPAssertionKey ().getInteractionKey ().getMessageSink ();
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY_INTERACTIONID:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY_INTERACTIONID:
                return exposed.getGlobalPAssertionKey ().getInteractionKey ().getInteractionID();    
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_VIEWKIND:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_VIEWKIND:
                return new QName (Constants.PSTRUCT_NS, exposed.getGlobalPAssertionKey ().getViewKind ());
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_VIEWKIND_DESCRIPTION:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_VIEWKIND_DESCRIPTION:
                return exposed.getGlobalPAssertionKey ().getViewKind ();
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_LOCALPASSERTIONID:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_LOCALPASSERTIONID:
                return exposed.getGlobalPAssertionKey ().getLocalPAssertionID ();
            case SENDER_EXPOSEDINTERACTIONMETADATA_INTERACTIONMETADATA:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_INTERACTIONMETADATA:
                return exposed.getInteractionMetadata ().get (index);
            case OBJECTLINK:
                return objectID.getLink ();
            case OBJECTLINK_PROVENANCESTOREREF:
                return objectID.getLink ().getStoreURL ();
            case OBJECTLINK_PROVENANCESTOREREF_ADDRESS:
                return objectID.getLink ().getStoreURL ();
        }

        throw new NoSuchElementException ("Cannot getRepresentation of element of type " + type);
    }
    
    public int[] getRequiredContext () {
        return _requiredContext;
    }
    
    public int getSize (int type, ContextItem[] context) throws UnderlyingModelException {
        // Some pieces of context that may be present
        InteractionRecord          record   = (InteractionRecord) getContextItemData (context, INTERACTIONRECORD);
        View                       view     = (View) getAvailableContextItemData (context, _views);
        ObjectID                   objectID = (ObjectID) getAvailableContextItemData (context, _objids);
        RelationshipPAssertion     rpa      = (RelationshipPAssertion) getAvailableContextItemData (context, _rpas);
        ExposedInteractionMetadata exposed  = (ExposedInteractionMetadata) getAvailableContextItemData (context, _expos);

        switch (type) {
            case PSTRUCT: case INTERACTIONRECORD_INTERACTIONKEY:
            case INTERACTIONRECORD_INTERACTIONKEY_MESSAGESOURCE:
            case INTERACTIONRECORD_INTERACTIONKEY_MESSAGESINK:
            case INTERACTIONRECORD_INTERACTIONKEY_INTERACTIONID:
            case SENDER_ASSERTER:
            case SENDER_RELATIONSHIPPASSERTION_LOCALPASSERTIONID:
            case SENDER_RELATIONSHIPPASSERTION_SUBJECTID:
            case SENDER_RELATIONSHIPPASSERTION_SUBJECTID_LOCALPASSERTIONID:
            case SENDER_RELATIONSHIPPASSERTION_SUBJECTID_PARAMETERNAME:
            case SENDER_RELATIONSHIPPASSERTION_RELATION:
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_LOCALPASSERTIONID:
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY:
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY_MESSAGESINK:
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY_MESSAGESOURCE:
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY_INTERACTIONID:
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_PARAMETERNAME:
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_VIEWKIND:
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_VIEWKIND_DESCRIPTION:
            case RECEIVER_ASSERTER:
            case RECEIVER_RELATIONSHIPPASSERTION_LOCALPASSERTIONID:
            case RECEIVER_RELATIONSHIPPASSERTION_SUBJECTID:
            case RECEIVER_RELATIONSHIPPASSERTION_SUBJECTID_LOCALPASSERTIONID:
            case RECEIVER_RELATIONSHIPPASSERTION_SUBJECTID_PARAMETERNAME:
            case RECEIVER_RELATIONSHIPPASSERTION_RELATION:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_LOCALPASSERTIONID:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY_MESSAGESINK:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY_MESSAGESOURCE:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_INTERACTIONKEY_INTERACTIONID:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_PARAMETERNAME:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_VIEWKIND:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_VIEWKIND_DESCRIPTION:
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY:
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY:
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY_MESSAGESOURCE:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY_MESSAGESOURCE:
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY_MESSAGESINK:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY_MESSAGESINK:
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY_INTERACTIONID:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_INTERACTIONKEY_INTERACTIONID:
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_VIEWKIND:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_VIEWKIND:
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_VIEWKIND_DESCRIPTION:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_VIEWKIND_DESCRIPTION:
            case SENDER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_LOCALPASSERTIONID:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_GLOBALPASSERTIONKEY_LOCALPASSERTIONID:
            case OBJECTLINK: case OBJECTLINK_PROVENANCESTOREREF: case OBJECTLINK_PROVENANCESTOREREF_ADDRESS:
                return 1;
            case SENDER:  case RECEIVER:
            case SENDER_RELATIONSHIPPASSERTION_SUBJECTID_DATAACCESSOR:
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID_DATAACCESSOR:
            case RECEIVER_RELATIONSHIPPASSERTION_SUBJECTID_DATAACCESSOR:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID_DATAACCESSOR:
                return nonNullSize (getRepresentation (type, 0, context));
            case SENDER_INTERACTIONPASSERTION:
            case RECEIVER_INTERACTIONPASSERTION:
                return view.getInteractionPAssertions ().size ();
            case SENDER_ACTORSTATEPASSERTION:
            case RECEIVER_ACTORSTATEPASSERTION:
                return view.getActorStatePAssertions ().size ();
            case SENDER_RELATIONSHIPPASSERTION:
            case RECEIVER_RELATIONSHIPPASSERTION:
                return view.getRelationshipPAssertions ().size ();
            case SENDER_RELATIONSHIPPASSERTION_OBJECTID:
            case RECEIVER_RELATIONSHIPPASSERTION_OBJECTID:
                return rpa.getObjectIDs ().size ();
            case SENDER_EXPOSEDINTERACTIONMETADATA:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA:
                return view.getExposedInteractionMetadata ().size ();
            case SENDER_EXPOSEDINTERACTIONMETADATA_INTERACTIONMETADATA:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_INTERACTIONMETADATA:
                return exposed.getInteractionMetadata ().size ();
            case INTERACTIONRECORD:
                try {
                    return (int) _basic.getNumberOfInteractionRecords ();
                } catch (Exception problem) {
                    throw new UnderlyingModelException ("Failed to get number of interaction records", problem);
                }
/*            case NUMBER_EXPECTED:
                if (view.getNumberOfExpectedPAssertions () > 0) {
                    return 1;
                } else {
                    return 0;
                }*/
        }
        
        return 0;
        //throw new NoSuchElementException ("Cannot getSize of element of type " + type);
    }
    
    public boolean isAlreadyDOM (int type, int index, ContextItem[] context) {
        switch (type) {
            case SENDER_INTERACTIONPASSERTION:
            case RECEIVER_INTERACTIONPASSERTION:
            case SENDER_ACTORSTATEPASSERTION:
            case RECEIVER_ACTORSTATEPASSERTION:
            case SENDER_EXPOSEDINTERACTIONMETADATA_INTERACTIONMETADATA:
            case RECEIVER_EXPOSEDINTERACTIONMETADATA_INTERACTIONMETADATA:
                return true;
            default:
                return false;
        }
    }
    
    private int nonNullSize (Object object) {
        if (object == null) {
            return 0;
        } else {
            return 1;
        }
    }
}
