package org.pasoa.pstructure.dom.util;

import java.util.HashMap;
import org.pasoa.accessors.AccessorException;
import org.pasoa.accessors.DataAccessorRegistry;
import org.pasoa.common.Constants;
import org.pasoa.preserv.xquery.QueryProblem;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.PAssertionDataKey;
import org.pasoa.pstructure.deserialiser.PStructureDeserialiser;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.simpledom.DOMTraverser;
import org.pasoa.accessors.XPath;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class PAssertionDataItemHelper {
    private DataAccessorRegistry _accessors;
    
    public PAssertionDataItemHelper (DataAccessorRegistry accessors) {
        _accessors = accessors;
    }
    
    public GlobalPAssertionKey getGlobalPAssertionKey (Node dataItem) throws QueryProblem {
        return new GlobalPAssertionKey (getInteractionKey (dataItem), getViewKind (dataItem), getLocalPAssertionID (dataItem));
    }
    
    public InteractionKey getInteractionKey (Node dataItem) throws QueryProblem {
        Element record = DOMTraverser.getAncestorNode (dataItem, Constants.PSTRUCT_NS, new String[] {"interactionRecord"});
        Element key    = new DOMDeserialiser (Constants.PSTRUCT_NS).single (record, "interactionKey");
        
        if (record == null) {
            throw new QueryProblem ("Data item is not contained by an interaction record");
        }
        
        return new PStructureDeserialiser ().deserialiseInteractionKey (key);
    }
    
    public String getLocalPAssertionID (Node dataItem) throws QueryProblem {
        Element assertion = DOMTraverser.getAncestorNode (dataItem, Constants.PSTRUCT_NS, new String[] {"interactionPAssertion", "actorStatePAssertion", "relationship"});
        
        return new DOMDeserialiser (Constants.PSTRUCT_NS).text (assertion, "localPAssertionId");
    }
    
    public PAssertionDataKey getPAssertionDataKey (Node dataItem) throws QueryProblem {
        try {
            return new PAssertionDataKey (getGlobalPAssertionKey (dataItem), _accessors.getDataAccessorForItem (dataItem));
        } catch (AccessorException ex) {
            throw new QueryProblem ("Problem in constructing data accessor", ex);
        }
    }
    
    public static XPath getXPathFor (InteractionKey key, boolean fromPStructRoot) {
        HashMap spaces = new HashMap ();
        Element source = key.getMessageSource ();
        Element sink   = key.getMessageSink ();
        String  iid    = key.getInteractionID ();
        XPath   keyPath;
        XPath   sourcePath;
        XPath   sinkPath;
        XPath   rootPath;
        
        spaces.put ("ps", Constants.PSTRUCT_NS);
        keyPath    = new XPath ("/ps:interactionKey[ps:interactionId=\"" + iid + "\"]", spaces);
        sourcePath = XPath.merge (new XPath ("/ps:messageSource", spaces), XPath.completeTraversal (source, spaces), false);
        sinkPath   = XPath.merge (new XPath ("/ps:messageSink",   spaces), XPath.completeTraversal (sink,   spaces), false);
        keyPath    = XPath.merge (keyPath, sourcePath, true);
        keyPath    = XPath.merge (keyPath, sinkPath,   true);
        
        if (fromPStructRoot) {
            rootPath = new XPath ("/ps:pstruct/ps:interactionRecord", spaces);
            rootPath = XPath.merge (rootPath, keyPath, false);
            return rootPath;
        } else {
            return keyPath;
        }
    }
    
    public static XPath getXPathFor (GlobalPAssertionKey gpak) {
        HashMap spaces   = new HashMap ();
        String  lpaid    = gpak.getLocalPAssertionID ();
        String  viewKind = gpak.getViewKind ();
        String  paSearch = "/*[ps:localPAssertionId=\"" + lpaid + "\"]";
        XPath   keyPath  = getXPathFor (gpak.getInteractionKey (), false);
        XPath   path;
        XPath   paPath;
        
        spaces.put ("ps", Constants.PSTRUCT_NS);
        path = new XPath ("/ps:pstruct/ps:interactionRecord", spaces);
        
        if (viewKind.equals (Constants.SENDER_VIEW_TYPE)) {
            paPath = new XPath ("/ps:sender" + paSearch, spaces);
        } else {
            paPath = new XPath ("/ps:receiver" + paSearch, spaces);
        }
        path = XPath.merge (path, keyPath, true);
        path = XPath.merge (path, paPath,  false);
        
        return path;
    }
    
    public XPath getXPathFor (PAssertionDataKey padk) throws QueryProblem {
        Element dataAccessor = padk.getDataAccessor ();
        // Need to include ps:content section?
        
        try {
            return XPath.merge (getXPathFor (padk.getGlobalPAssertionKey ()), _accessors.convertToXPath (dataAccessor), false);
        } catch (AccessorException ex) {
            throw new QueryProblem ("Problem parsing data accessor", ex);
        }
    }
    
    public String getViewKind (Node dataItem) throws QueryProblem {
        Node viewNode = DOMTraverser.getAncestorNode (dataItem, Constants.PSTRUCT_NS, new String[] {"sender", "receiver"});
        
        if (viewNode == null) {
            throw new QueryProblem ("P-assertion data item is not contained in a sender or receiver view node");
        }
        if (viewNode.getLocalName ().equals ("sender")) {
            return Constants.SENDER_VIEW_TYPE;
        } else {
            return Constants.RECEIVER_VIEW_TYPE;
        }
    }
}
