package org.pasoa.pstructure.serialiser;

import java.util.Iterator;
import org.pasoa.common.Constants;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.ExposedInteractionMetadata;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionContext;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.Link;
import org.pasoa.pstructure.ObjectID;
import org.pasoa.pstructure.PAssertionDataKey;
import org.pasoa.pstructure.PHeader;
import org.pasoa.pstructure.Record;
import org.pasoa.pstructure.RecordExposedInteractionMetadata;
import org.pasoa.pstructure.RecordSf;
import org.pasoa.pstructure.RelationshipPAssertion;
import org.pasoa.pstructure.View;
import org.pasoa.simpledom.DOMSerialiser;
import org.pasoa.simpledom.NormalisingDOMWriter;
import org.pasoa.simpledom.SimpleDocument;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Provides convience methods for serialising org.pasoa.common.structure
 * objects to XML.
 *
 * @author sm
 *
 */
public class PStructureSerialiser {
    private static boolean _clientSideLibraryAware = false;
    
    public PStructureSerialiser () {
    }
    
    public String normaliseInteractionKey (InteractionKey ikey) {
        return NormalisingDOMWriter.writeToString (serialiseInteractionKey (ikey));
    }
    
    public Document serialiseActorStatePAssertion (ActorStatePAssertion aspa) {
        return (Document) serialiseActorStatePAssertion (new SimpleDocument (), aspa);
    }
    
    public Node serialiseActorStatePAssertion (Node addTo, ActorStatePAssertion aspa) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PSTRUCT_NS);
        
        builder.start  ("actorStatePAssertion");
        builder.single ("localPAssertionId",  aspa.getLocalPAssertionID ());
        builder.start ("content");
        builder.append (aspa.getContent ());
        builder.end ();
        
        return addTo;
    }
    
    public Document serialiseExposedInteractionMetadata (ExposedInteractionMetadata metadata) {
        return (Document) serialiseExposedInteractionMetadata (new SimpleDocument (), metadata);
    }
    
    public Node serialiseExposedInteractionMetadata (Node addTo, ExposedInteractionMetadata metadata) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PSTRUCT_NS);
        Iterator      eachMd  = metadata.getInteractionMetadata ().iterator ();
        
        builder.start ("exposedInteractionMetaData");
        builder.append (serialiseGlobalPAssertionKey (metadata.getGlobalPAssertionKey ()));
        builder.appendAll (metadata.getInteractionMetadata ());
        
        return addTo;
    }

    public Document serialiseGlobalPAssertionKey (GlobalPAssertionKey gpak) {
        return (Document) serialiseGlobalPAssertionKey (new SimpleDocument (), gpak);
    }
    
    public Node serialiseGlobalPAssertionKey (Node addTo, GlobalPAssertionKey gpak) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PSTRUCT_NS);
        
        builder.start("globalPAssertionKey");
        builder.append (serialiseInteractionKey (gpak.getInteractionKey ()));
        builder.append (serialiseViewKind (gpak.getViewKind ()));
        builder.single ("localPAssertionId", gpak.getLocalPAssertionID ());
        builder.end();
        
        return addTo;
    }
    
    public Node serialiseInteractionContext (Node addTo, InteractionContext context) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PSTRUCT_NS);
        
        builder.start ("interactionContext");
        builder.append (serialiseInteractionKey (context.getInteractionKey ()));
        builder.appendAll (context.getInteractionMetadata ());
        
        return addTo;
    }
    
    public Document serialiseInteractionKey (InteractionKey ikey) {
        return (Document) serialiseInteractionKey (new SimpleDocument (), ikey);
    }
    
    public Node serialiseInteractionKey (Node addTo, InteractionKey ikey) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PSTRUCT_NS);
        
        builder.start  ("interactionKey");
        builder.single ("messageSource", ikey.getMessageSource ());
        builder.single ("messageSink",   ikey.getMessageSink   ());
        builder.single ("interactionId", ikey.getInteractionID ());
        
        return addTo;
    }
    
    public Document serialiseInteractionPAssertion (InteractionPAssertion ipa) {
        return (Document) serialiseInteractionPAssertion (new SimpleDocument (), ipa);
    }
    
    public Node serialiseInteractionPAssertion (Node addTo, InteractionPAssertion ipa) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PSTRUCT_NS);
        
        builder.start  ("interactionPAssertion");
        builder.single ("localPAssertionId",  ipa.getLocalPAssertionID ());
        builder.single ("documentationStyle", ipa.getDocumentationStyle ());
        builder.start ("content");
        builder.append (ipa.getContent ());
        builder.end ();
        
        return addTo;
    }
    
    public Node serialiseInteractionRecord (Node addTo, InteractionRecord record) {
        DOMSerialiser build = new DOMSerialiser (addTo, Constants.PSTRUCT_NS);
        
        build.start ("interactionRecord");
        build.append (serialiseInteractionKey (record.getInteractionKey ()));
        build.append (serialiseView (record.getSenderView (),   Constants.SENDER_VIEW_TYPE));
        build.append (serialiseView (record.getReceiverView (), Constants.RECEIVER_VIEW_TYPE));
        build.end ();
        
        return addTo;
    }
    
    public Document serialiseObjectID (ObjectID objectID) {
        return (Document) serialiseObjectID (new SimpleDocument (), objectID);
    }
    
    public Node serialiseObjectID (Node addTo, ObjectID objectID) {
        DOMSerialiser build = new DOMSerialiser (addTo, Constants.PSTRUCT_NS);
        
        build.start ("objectId");
        build.append (serialiseInteractionKey (objectID.getGlobalPAssertionKey ().getInteractionKey ()));
        build.append (serialiseViewKind (objectID.getGlobalPAssertionKey ().getViewKind ()));
        build.single ("localPAssertionId", objectID.getGlobalPAssertionKey ().getLocalPAssertionID ());
        build.start ("dataAccessor");
        build.append (objectID.getDataAccessor ());
        build.end ();
        build.single ("parameterName", objectID.getParameterName ());
        if (objectID.getLink () != null) {
            build.namespace (Constants.PLINKS_NS);
            build.start ("objectLink");
            build.single ("provenanceStoreRef", objectID.getLink ().getStoreURL ());
        }
        
        return (Document) build.getRoot ();
    }
    
    public Document serialisePAssertionDataKey (PAssertionDataKey padk) {
        return (Document) serialisePAssertionDataKey (new SimpleDocument (), padk);
    }
    
    public Node serialisePAssertionDataKey (Node addTo, PAssertionDataKey padk) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PSTRUCT_NS);
        
        builder.start  ("pAssertionDataKey");
        builder.append (serialiseInteractionKey (padk.getGlobalPAssertionKey ().getInteractionKey ()));
        builder.append (serialiseViewKind (padk.getGlobalPAssertionKey ().getViewKind ()));
        builder.single ("localPAssertionId", padk.getGlobalPAssertionKey ().getLocalPAssertionID ());
        builder.single ("dataAccessor",  padk.getDataAccessor ());
        
        return addTo;
    }
    
    public Document serialiseProvenanceStoreReference (Link link) {
        return (Document) serialiseProvenanceStoreReference (new SimpleDocument (), link);
    }
    
    public Node serialiseProvenanceStoreReference (Node addTo, Link link) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PLINKS_NS);
        
        builder.start ("provenanceStoreRef");
        builder.text (link.getStoreURL ());
        
        return addTo;
    }
    
    public Document serialiseInteractionContext (InteractionContext context) {
        return (Document) serialiseInteractionContext (new SimpleDocument (), context);
    }
    
    public Document serialiseObjectLink (Link link) {
        return (Document) serialiseObjectLink (new SimpleDocument (), link);
    }
    
    public Node serialiseObjectLink (Node addTo, Link link) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PLINKS_NS);
        
        builder.start ("viewLink");
        builder.append (serialiseProvenanceStoreReference (link));
        
        return addTo;
    }
    
    public Document serialisePHeader (PHeader pheader) {
        return (Document) serialisePHeader (new SimpleDocument (), pheader);
    }
    
    public Node serialisePHeader (Node addTo, PHeader pheader) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PHEADER_NS);
        Iterator      eachCtx = pheader.getInteractionContexts ().iterator ();
        
        builder.start ("pheader");
        builder.namespace (Constants.PSTRUCT_NS);
        builder.append (serialiseInteractionKey (pheader.getInteractionKey ()));
        builder.appendAll (pheader.getInteractionMetadata ());
        while (eachCtx.hasNext ()) {
            builder.append (serialiseInteractionContext ((InteractionContext) eachCtx.next ()));
        }
        
        return addTo;
    }
    
    public Document serialiseRelationshipPAssertion (RelationshipPAssertion rpa) {
        return (Document) serialiseRelationshipPAssertion (new SimpleDocument (), rpa);
    }
    
    public Node serialiseRelationshipPAssertion (Node addTo, RelationshipPAssertion rpa) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PSTRUCT_NS);
        Iterator      eachObj = rpa.getObjectIDs ().iterator ();
        ObjectID      object;
        
        builder.start  ("relationshipPAssertion");
        builder.single ("localPAssertionId",  rpa.getLocalPAssertionID ());
        builder.start  ("subjectId");
        builder.single ("localPAssertionId",  rpa.getSubject ().getLocalPAssertionID ());
        builder.single ("dataAccessor",       rpa.getSubject ().getDataAccessor ());
        builder.single ("parameterName",      rpa.getSubject ().getParameterName ());
        builder.end    ();
        builder.single ("relation",           rpa.getRelation ());
        while (eachObj.hasNext ()) {
            object = (ObjectID) eachObj.next ();
            builder.append (serialiseObjectID (object));
        }
        
        return addTo;
    }
    
    public Document serialiseRecordRequest (Record request) {
        return (Document) serialiseRecordRequest (new SimpleDocument (), request);
    }
    
    public Node serialiseRecordRequest (Node addTo, Record request) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PRECORD_NS);
        
        builder.start     ("record");
        builder.start     ("identifiedContent");
        builder.append    (serialiseInteractionKey (request.getInteractionKey ()));
        builder.append    (serialiseViewKind (request.getViewKind ()));
        builder.namespace (Constants.PSTRUCT_NS);
        builder.single    ("asserter",      request.getAsserter ());
        builder.namespace (Constants.PRECORD_NS);
        builder.start     ("content");
        if (request.getPAssertion () instanceof InteractionPAssertion) {
            builder.append (serialiseInteractionPAssertion ((InteractionPAssertion) request.getPAssertion ()));
        }
        if (request.getPAssertion () instanceof ActorStatePAssertion) {
            builder.append (serialiseActorStatePAssertion ((ActorStatePAssertion) request.getPAssertion ()));
        }
        if (request.getPAssertion () instanceof RelationshipPAssertion) {
            builder.append (serialiseRelationshipPAssertion ((RelationshipPAssertion) request.getPAssertion ()));
        }
        if (request instanceof RecordSf) {
            builder.single ("submissionFinished", ((RecordSf) request).getNumberOfExpectedPAssertions ());
        }
        if (request instanceof RecordExposedInteractionMetadata) {
            builder.append(this.serialiseExposedInteractionMetadata(((RecordExposedInteractionMetadata)request).getExposedInteractionMetadata()));
        }
        builder.end();
        
        return addTo;
    }

    public Document serialiseTracerAsInteractionMetadata (String tracer) {
        return (Document) serialiseTracerAsInteractionMetadata (new SimpleDocument (), tracer);
    }
    
    public Node serialiseTracerAsInteractionMetadata (Node addTo, String tracer) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PSTRUCT_NS);
        
        builder.start ("interactionMetaData").single ("tracer", tracer);
        
        return addTo;
    }
    
    public Document serialiseView (View view, String viewKind) {
        return (Document) serialiseView (new SimpleDocument (), view, viewKind);
    }
    
    public Node serialiseView (Node addTo, View view, String viewKind) {
        DOMSerialiser build    = new DOMSerialiser (addTo, Constants.PSTRUCT_NS);
        Iterator      eachIPA  = view.getInteractionPAssertions  ().iterator ();
        Iterator      eachASPA = view.getActorStatePAssertions   ().iterator ();
        Iterator      eachRPA  = view.getRelationshipPAssertions ().iterator ();
        
        if (viewKind.equals (Constants.SENDER_VIEW_TYPE)) {
            build.start ("sender");
        } else {
            build.start ("receiver");
        }
        build.single ("asserter", view.getAsserter ());
        while (eachIPA.hasNext ()) {
            build.append (serialiseInteractionPAssertion ((InteractionPAssertion) eachIPA.next ()));
        }
        while (eachASPA.hasNext ()) {
            build.append (serialiseActorStatePAssertion ((ActorStatePAssertion) eachASPA.next ()));
        }
        while (eachRPA.hasNext ()) {
            build.append (serialiseRelationshipPAssertion ((RelationshipPAssertion) eachRPA.next ()));
        }
        build.end ();
        
        return addTo;
    }
    
    public Document serialiseViewKind (String viewKind) {
        return (Document) serialiseViewKind (new SimpleDocument (), viewKind);
    }
    
    public Node serialiseViewKind (Node addTo, String viewKind) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PSTRUCT_NS);
        
        builder.start  ("viewKind");
        if (!_clientSideLibraryAware) {
            builder.attribute (Constants.XSI_NS, "type",  "ps:" + viewKind);
            builder.attribute ("xmlns:ps",  Constants.PSTRUCT_NS);
        }
        //builder.attribute ("xmlns:xsi", Constants.XSI_NS);
        builder.single ("description", viewKind);
        
        return addTo;
    }
    
    public Document serialiseViewLinkAsInteractionMetadata (Link link) {
        DOMSerialiser builder = new DOMSerialiser (Constants.PSTRUCT_NS);
        
        builder.start  ("interactionMetaData");
        builder.append (serialiseViewLink (link));
        
        return (Document) builder.getRoot ();
    }
    
    public Document serialiseViewLink (Link link) {
        return (Document) serialiseViewLink (new SimpleDocument (), link);
    }
    
    public Node serialiseViewLink (Node addTo, Link link) {
        DOMSerialiser builder = new DOMSerialiser (addTo, Constants.PLINKS_NS);
        
        builder.start ("viewLink");
        builder.append (serialiseProvenanceStoreReference (link));
        
        return addTo;
    }
}
