package org.pasoa.storeclient.pquery;

import java.util.HashMap;
import java.util.Map;
import org.pasoa.common.Constants;
import org.pasoa.preserv.pquery.ProvenanceQueryRequest;
import org.pasoa.preserv.pquery.QueryDataHandle;
import org.pasoa.preserv.pquery.RelationshipTargetFilter;
import org.pasoa.util.httpsoap.WSAddressEndpoint;

/**
 * Convenience class for gradually building a provenance query request document.
 * This class contains two other builders, one for the query data handle, one
 * for the relationship target filter, which should be retrieved and used
 * separately to build each part.  Once these are done, call build() on this
 * class to retrieve the whole request.
 */
public class ProvenanceQueryBuilder {
    private QDHBuilder _qdhBuilder;
    private RTFBuilder _rtfBuilder;
    private Map        _namespaces;
    
    /**
     * Create a new ProvenanceQueryBuilder.
     * 
     * @param namespaces the set of
     *        prefix to namespace mappings (e.g. "ex" -> "http://www.example.com")
     *        that you will use to query inside in the
     *        contents of interaction and actor state p-assertions.
     */
    public ProvenanceQueryBuilder (Map namespaces) {
        _namespaces = namespaces;
        _namespaces.put ("ps",   Constants.PSTRUCT_NS);
        _namespaces.put ("pq",   Constants.PQUERY_NS);
        _namespaces.put ("soap", Constants.SOAP_NS);
        _namespaces.put ("wsa",  WSAddressEndpoint.wsaNamespace);
        _qdhBuilder = new QDHBuilder ();
        _rtfBuilder = new RTFBuilder ();
    }
    
    /**
     * Create a provenance query request - should be called after finishing
     * building the query data handle and relationship target filter.
     */
    public ProvenanceQueryRequest build () {
        return new ProvenanceQueryRequest (new QueryDataHandle (_qdhBuilder.build (), _namespaces, null, null),
                new RelationshipTargetFilter (_rtfBuilder.build (), _namespaces, null));
    }

    /**
     * Get the builder for the query data handle.
     */
    public QDHBuilder getQDHBuilder () {
        return _qdhBuilder;
    }
    
    /**
     * Get the builder for the relationship target filter.
     */
    public RTFBuilder getRTFBuilder () {
        return _rtfBuilder;
    }
    
    public Map getNamespaces () {
        return _namespaces;
    }
}
