package org.pasoa.storeclient.proxy;

import org.pasoa.dommap.UnderlyingModelException;
import org.pasoa.preserv.index.Index;
import org.pasoa.preserv.pquery.ProvenanceQuery;
import org.pasoa.preserv.pquery.ProvenanceQueryEngine;
import org.pasoa.preserv.record.Record;
import org.pasoa.preserv.storage.BackendStore;
import org.pasoa.preserv.storage.GeneralStorageSystem;
import org.pasoa.preserv.storage.StorageSystem;
import org.pasoa.preserv.xquery.Query;
import org.pasoa.preserv.xquery.QueryProblem;
import org.pasoa.pstructure.dom.PStructureJavaModel;
import org.w3c.dom.Document;

public class LocalStoreProxy implements StoreProxy {
    private StorageSystem  _localStore;
    private StoreCatalogue _stores;
    private String         _thisStore;
    private boolean        _locallyDrivePQs;
    
    public LocalStoreProxy (StorageSystem localStore, StoreCatalogue stores,
            String thisStore, boolean locallyDrivenPQs) {
        _localStore      = localStore;
        _stores          = stores;
        _thisStore       = thisStore;
        _locallyDrivePQs = locallyDrivenPQs;
        _stores.registerStore (_thisStore, this);
        _stores.registerDefaultStore (this);
    }
    
    public LocalStoreProxy (BackendStore backend) {
        this (new GeneralStorageSystem (backend, "org.pasoa.preserv.xquery.laxquery.XQueryEngine"),
                new StoreCatalogue (), "http://local.store", false);
    }

    public Index getIndexInterface () {
        return (Index) _localStore.getIndexInterface ();
    }

    public Query getXQueryInterface () {
        return _localStore;
    }

    public Record getRecordInterface () {
        return _localStore;
    }

    public ProvenanceQuery getPQueryInterface () {
        return new ProvenanceQueryEngine (_stores, _thisStore, _locallyDrivePQs);
    }
    
    public Document getStoreContents () throws QueryProblem {
        try {
            return PStructureJavaModel.getPStructureDocument (_localStore.getRetrievalInterface ());
        } catch (UnderlyingModelException backendError) {
            throw new QueryProblem ("Problem with back-end store", backendError);
        }
    }
}
