package org.pasoa.storeclient.proxy;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.pasoa.common.Constants;
import org.pasoa.preserv.storage.BackendStore;
import org.pasoa.preserv.storage.je.BerkeleyDBJEStorage2;
import org.pasoa.pstructure.ActorStatePAssertion;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionPAssertion;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.PAssertion;
import org.pasoa.pstructure.RelationshipPAssertion;
import org.pasoa.pstructure.View;
import org.pasoa.pstructure.deserialiser.PStructureDeserialiser;
import org.pasoa.pstructure.serialiser.PStructureSerialiser;
import org.pasoa.simpledom.DOMDeserialiser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class PStructureFileProxy extends LocalStoreProxy {
    public PStructureFileProxy (String filename) {
        super (uploadPStructure (filename));
    }
    
    private static BackendStore uploadPStructure (String filename) {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance ();
            factory.setNamespaceAware (true);
            DocumentBuilder        builder  = factory.newDocumentBuilder ();
            Document               document = builder.parse (filename);
            
            return uploadPStructure (document.getDocumentElement ());
        } catch (Exception problem) {
            throw new RuntimeException (problem);
        }
    }
    
    private static BackendStore uploadPStructure (Element pstructure) throws Exception {
        BackendStore      backend    = new BerkeleyDBJEStorage2 (createTemporaryDirectory ().toString (), false, true);
        Iterator          eachRecord = new PStructureDeserialiser ().deserialisePStructure (pstructure).iterator ();
        InteractionRecord record;
        InteractionKey    ikey;
        boolean           isSender;
        View              view;
        Element           asserter;
        Iterator          eachPAssertion;
        PAssertion        passertion;
        String            viewKind;
        
        while (eachRecord.hasNext ()) {
            record   = (InteractionRecord) eachRecord.next ();
            ikey     = record.getInteractionKey ();
            isSender = true;
            do {
                if (isSender) {
                    view = record.getSenderView ();
                } else {
                    view = record.getReceiverView ();
                }
                if (view != null) {
                    asserter = view.getAsserter ();
                    for (int passertionType = 1; passertionType <= 3; passertionType += 1) {
                        switch (passertionType) {
                            case 1:
                                eachPAssertion = view.getInteractionPAssertions ().iterator ();
                                break;
                            case 2:
                                eachPAssertion = view.getActorStatePAssertions ().iterator ();
                                break;
                            default:
                                eachPAssertion = view.getRelationshipPAssertions ().iterator ();
                        }
                        while (eachPAssertion.hasNext ()) {
                            passertion = destupid ((PAssertion) eachPAssertion.next ());
                            if (isSender) {
                                viewKind = Constants.SENDER_VIEW_TYPE;
                            } else {
                                viewKind = Constants.RECEIVER_VIEW_TYPE;
                            }
                            if (passertion instanceof InteractionPAssertion) {
                                backend.record (new GlobalPAssertionKey (ikey, viewKind, passertion.getLocalPAssertionID ()),
                                        asserter, new PStructureSerialiser ().serialiseInteractionPAssertion ((InteractionPAssertion) passertion).getDocumentElement (), BackendStore.INTERACTION_PA);
                            }
                            if (passertion instanceof ActorStatePAssertion) {
                                backend.record (new GlobalPAssertionKey (ikey, viewKind, passertion.getLocalPAssertionID ()),
                                        asserter, new PStructureSerialiser ().serialiseActorStatePAssertion ((ActorStatePAssertion) passertion).getDocumentElement (), BackendStore.ACTOR_STATE_PA);
                            }
                            if (passertion instanceof RelationshipPAssertion) {
                                backend.record (new GlobalPAssertionKey (ikey, viewKind, passertion.getLocalPAssertionID ()),
                                        asserter, new PStructureSerialiser ().serialiseRelationshipPAssertion ((RelationshipPAssertion) passertion).getDocumentElement (), BackendStore.RELATIONSHIP_PA);
                            }
                        }
                    }
                }
                isSender = !isSender;
            } while (!isSender);
        }
        
        return backend;
    }
    
    private static File createTemporaryDirectory () throws IOException {
        File temp = File.createTempFile ("database", "");
        temp.delete ();
        temp.mkdir ();
        temp.deleteOnExit ();
        return temp;
    }
    
    
    private static PAssertion destupid (PAssertion passertion) {
        DOMDeserialiser reader = new DOMDeserialiser (Constants.PSTRUCT_NS);
        Element         content;
        
        if (passertion instanceof RelationshipPAssertion) {
            return passertion;
        }
        if (passertion instanceof InteractionPAssertion) {
            content = reader.single (reader.single (((InteractionPAssertion) passertion).getContent (), "content"));
            return new InteractionPAssertion (passertion.getLocalPAssertionID (),
                    ((InteractionPAssertion) passertion).getDocumentationStyle (), content);
        }
        if (passertion instanceof ActorStatePAssertion) {
            content = reader.single (reader.single (((ActorStatePAssertion) passertion).getContent (), "content"));
            return new ActorStatePAssertion (passertion.getLocalPAssertionID (),
                    ((ActorStatePAssertion) passertion).getDocumentationStyle (), content);
        }
        return null;
    }
}
