package org.pasoa.storeclient.proxy;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.Map;
import org.pasoa.common.Constants;
import org.pasoa.preserv.index.Index;
import org.pasoa.preserv.index.IndexProblem;
import org.pasoa.preserv.pquery.PQueryProblem;
import org.pasoa.preserv.pquery.ProvenanceQuery;
import org.pasoa.preserv.pquery.ProvenanceQueryDeserialiser;
import org.pasoa.preserv.pquery.ProvenanceQueryRequest;
import org.pasoa.preserv.pquery.ProvenanceQueryResults;
import org.pasoa.preserv.record.Record;
import org.pasoa.preserv.xquery.Query;
import org.pasoa.preserv.xquery.QueryProblem;
import org.pasoa.pstructure.GlobalPAssertionKey;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.InteractionRecord;
import org.pasoa.pstructure.PAssertion;
import org.pasoa.pstructure.View;
import org.pasoa.pstructure.deserialiser.PStructureDeserialiser;
import org.pasoa.pstructure.dom.util.PAssertionDataItemHelper;
import org.pasoa.simpledom.DOMDeserialiser;
import org.pasoa.storeclient.messages.ProvenanceQueryRequestDocument;
import org.pasoa.storeclient.messages.XQueryRequestDocument;
import org.pasoa.util.httpsoap.DefaultDocumentClient;
import org.pasoa.accessors.XPath;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

public class RemoteStoreProxy implements Index, ProvenanceQuery, Query, Record, StoreProxy {
    private String _storeURL;
    
    public RemoteStoreProxy (String storeURL) {
        _storeURL = storeURL;
    }
    
    private String declareNamspaces (Map namespaces) {
        StringBuffer declares   = new StringBuffer ();
        Iterator     eachPrefix = namespaces.keySet ().iterator ();
        String       prefix;
        
        while (eachPrefix.hasNext ()) {
            prefix = (String) eachPrefix.next ();
            declares.append ("declare namespace ");
            declares.append (prefix);
            declares.append (" = \"");
            declares.append (namespaces.get (prefix));
            declares.append ("\";\n");
        }
        
        return declares.toString ();
    }
    
    public InteractionRecord findInteractionRecord (InteractionKey key) throws IndexProblem {
        try {
            XPath           xpath    = PAssertionDataItemHelper.getXPathFor (key, true);
            String          query    = declareNamspaces (xpath.getNamespaces ()) + "$" + xpath.getPath ().substring (1) + "/..";
            Document        response = getXQueryInterface ().resolve (query);
            DOMDeserialiser soap     = new DOMDeserialiser (Constants.SOAP_NS);
            Element         content  = soap.single (soap.single (soap.single (response.getDocumentElement (), "Body"), Constants.WSXQUERY_NS, "queryResult"));
            
            return new PStructureDeserialiser ().deserialiseInteractionRecord (content);
        } catch (QueryProblem problem) {
            throw new IndexProblem ("Cannot retrieve p-assertion", problem);
        }
    }
    
    public PAssertion findPAssertion (GlobalPAssertionKey gpak) throws IndexProblem {
        try {
            XPath           xpath    = PAssertionDataItemHelper.getXPathFor (gpak);
            String          query    = declareNamspaces (xpath.getNamespaces ()) + "$" + xpath.getPath ().substring (1);
            DOMDeserialiser soap     = new DOMDeserialiser (Constants.SOAP_NS);
            Document        response = getXQueryInterface ().resolve (query);
            Element         content  = soap.single (soap.single (soap.single (response.getDocumentElement (), "Body"), Constants.WSXQUERY_NS, "queryResult"));
            
            return new PStructureDeserialiser ().deserialisePAssertion (content);
        } catch (Exception problem) {
            throw new IndexProblem ("Cannot retrieve p-assertion", problem);
        }
    }
    
    public View findView (InteractionKey key, String viewKind) throws IndexProblem {
        InteractionRecord record = findInteractionRecord (key);
        
        if (viewKind.equals (Constants.SENDER_VIEW_TYPE)) {
            return record.getSenderView ();
        } else {
            return record.getReceiverView ();
        }
    }
    
    public Index getIndexInterface () {
        return this;
    }
    
    public ProvenanceQuery getPQueryInterface () {
        return this;
    }
    
    public Record getRecordInterface () {
        return this;
    }
    
    public Query getXQueryInterface () {
        return this;
    }
    
    private Document invoke (String context, Document requestDoc) throws MalformedURLException, IOException, SAXException {
        URL portURL = new URL (_storeURL + "/" + context);
        
        return new DefaultDocumentClient ().invoke (portURL, requestDoc);
    }
    
    public Document resolve (String query) throws QueryProblem {
if (true) throw new RuntimeException ("Should not be here");
        try {
            return invoke (Constants.XQUERY_CONTEXT, new XQueryRequestDocument (query));
        } catch (MalformedURLException ex) {
            throw new QueryProblem ("Cannot resolve XQuery", ex);
        } catch (SAXException ex) {
            throw new QueryProblem ("Cannot resolve XQuery", ex);
        } catch (IOException ex) {
            throw new QueryProblem ("Cannot resolve XQuery", ex);
        }
    }
    
    public void record (GlobalPAssertionKey gpid, Element asserter, Element assertion, String kindOfAssertion) throws Exception {
        throw new UnsupportedOperationException ();
    }
    
    public ProvenanceQueryResults resolve (ProvenanceQueryRequest request) throws PQueryProblem {
        ProvenanceQueryDeserialiser    parser = new ProvenanceQueryDeserialiser ();
        DOMDeserialiser                soap   = new DOMDeserialiser (Constants.SOAP_NS);
        ProvenanceQueryRequestDocument requestDoc;
        Element                        response;
        
        try {
            requestDoc = new ProvenanceQueryRequestDocument (request);
            response   = soap.single (soap.single (invoke (Constants.PQUERY_CONTEXT, requestDoc).getDocumentElement (), "Body"));
            
            return parser.deserialiseProvenanceQueryResults (response);
        } catch (MalformedURLException ex) {
            throw new PQueryProblem ("Provenance store URL was malformed", ex);
        } catch (SAXException ex) {
            throw new PQueryProblem ("Provenance query response was malformed", ex);
        } catch (IOException ex) {
            throw new PQueryProblem ("Could not send provenance query to provenance store", ex);
        }
    }
}
