package org.pasoa.storeclient.wrappers.java;

import java.net.URL;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.pasoa.common.Constants;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.pstructure.PAssertion;
import org.pasoa.pstructure.Record;
import org.pasoa.storeclient.ClientLib;
import org.w3c.dom.Element;

public abstract class AbstractAsserter implements Asserter {
    private URL       store;
    private ClientLib client;
    private boolean   isSender;
    private List      listeners;

    public AbstractAsserter (URL provenanceStore, boolean isSenderView) {
        store       = provenanceStore;
        client      = new ClientLib ();
        isSender    = isSenderView;
        listeners   = new LinkedList ();
    }
    
    public void addListener (RecordListener newListener) {
        listeners.add (newListener);
    }
    
    protected abstract PAssertion createPAssertion (Element message, String localPAssertionID);
    
    public URL getStoreURL () {
        return store;
    }
    
    public void record (Element asserterID, InteractionKey interaction, Element message, String localPAssertionID) throws Exception {
        Iterator eachListener = listeners.iterator ();
        Record   record;
        
        if (isSender) {
            record = new Record (createPAssertion (message, localPAssertionID), interaction, Constants.SENDER_VIEW_TYPE, asserterID);
        } else {
            record = new Record (createPAssertion (message, localPAssertionID), interaction, Constants.RECEIVER_VIEW_TYPE, asserterID);
        }
        client.record (record, store);
        while (eachListener.hasNext ()) {
            ((RecordListener) eachListener.next ()).recorded (this, record, store);
        }
    }
}
