package org.pasoa.storeclient.wrappers.java;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.pasoa.pstructure.InteractionKey;
import org.pasoa.simpledom.SimpleText;
import org.pasoa.storeclient.wrappers.java.util.TimestampAsserter;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Wraps a service, accessed as an InvokedActor, with a set of p-assertion recorders.
 * The recorders can execute either before or after the service is run.
 */
public class ProvenanceWrapper {
    private List         preRecorders;
    private List         postRecorders;
    private InvokedActor actor;
    private int          localID;
    
    public ProvenanceWrapper (InvokedActor logic) {
        preRecorders  = new LinkedList ();
        postRecorders = new LinkedList ();
        actor         = logic;
        localID       = 1;
    }
    
    public void addPostRecorder (Asserter newPostRecorder) {
        if (!(newPostRecorder instanceof RelationshipAsserter)) {
            postRecorders.add (newPostRecorder);
        }
    }
    
    public void addPreRecorder (Asserter newPreRecorder) {
        preRecorders.add (newPreRecorder);
    }
    
    public InvokedActor getActor () {
        return actor;
    }
    
    public List getPostRecorders () {
        return postRecorders;
    }
    
    public List getPreRecorders () {
        return preRecorders;
    }
    
    public static Node resolveOption (String name, List arguments, Document doc) {
        BufferedReader reader;
        StringBuffer   buffer;
        String         line;
        
        try {
            if (name.equals ("file")) {
                reader = new BufferedReader (new FileReader (arguments.get (0).toString ()));
                buffer = new StringBuffer ();
                line = reader.readLine ();
                while (line != null) {
                    if (line != null) {
                        buffer.append (line);
                    }
                    line = reader.readLine ();
                    if (line != null) {
                        buffer.append ("\n");
                    }
                }
                reader.close ();
                return new SimpleText (doc, buffer.toString ());
            }
            
            throw new NoSuchMethodException ("Option: " + name);
        } catch (Exception problem) {
            throw new RuntimeException (problem);
        }
    }
    
    public void run () {
        Element        identity = actor.getIdentity ();
        Iterator       eachAsserter;
        Element        request;
        Element        response;
        InteractionKey ik;
        
        try {
            request      = actor.getRequest ();
            eachAsserter = preRecorders.iterator ();
            ik           = actor.getRequestID ();
            while (eachAsserter.hasNext ()) {
                ((Asserter) eachAsserter.next ()).record (identity, ik, request, Integer.toString (localID));
                localID += 1;
            }
            new TimestampAsserter (((AbstractAsserter) preRecorders.get (0)).getStoreURL (), false).record (identity, ik, request, Integer.toString (localID));
            
            actor.run ();
            
            response     = actor.getResponse ();
            eachAsserter = postRecorders.iterator ();
            ik           = actor.getResponseID ();
            while (eachAsserter.hasNext ()) {
                ((Asserter) eachAsserter.next ()).record (identity, ik, response, Integer.toString (localID));
                localID += 1;
            }
            new TimestampAsserter (((AbstractAsserter) postRecorders.get (0)).getStoreURL (), false).record (identity, ik, response, Integer.toString (localID));
        } catch (Exception recordProblem) {
            throw new RuntimeException ("Could not record provenance", recordProblem);
        }
    }
}
